/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef FREERTOS_CONFIG_H
#define FREERTOS_CONFIG_H

/*
 * Application specific definitions. These definitions should be adjusted for your
 * particular hardware and application requirements. THESE PARAMETERS ARE
 * DESCRIBED WITHIN THE 'CONFIGURATION' SECTION OF THE FreeRTOS API DOCUMENTATION
 * AVAILABLE ON THE FreeRTOS.org WEB SITE.
 */

#ifndef __ASM__
#include "config.h"
#include "debug.h"

#ifdef DEBUG
/* Define to trap errors during development. */
#define configASSERT(x)  if( x == 0 ) assert_failed((char *)__FILE__, __LINE__);
#endif

#endif /* __ASM__ */

#define configTOTAL_HEAP_SIZE           NEEDED_HEAP_SIZE
#ifdef USE_LOAD_STATISTICS
#define configUSE_IDLE_HOOK             1
#else
#define configUSE_IDLE_HOOK             0
#endif


#define configUSE_PREEMPTION            1
#define configMAX_PRIORITIES            5
#define configUSE_TICK_HOOK             0
#ifdef BOARD_USE_PLL
#define configCPU_CLOCK_HZ              ((unsigned portLONG) PLL_MULTIPLIER * BOARD_EXTERNAL_OSCILLATOR_FREQUENCY)
#else
#define configCPU_CLOCK_HZ              ((unsigned portLONG) BOARD_EXTERNAL_OSCILLATOR_FREQUENCY)
#endif
#define configTICK_RATE_HZ              ((portTickType) 1000)
#define configMINIMAL_STACK_SIZE        ((unsigned portSHORT) 40)
#define configMAX_TASK_NAME_LEN         (12)
#define configUSE_TRACE_FACILITY        1
#define configUSE_STATS_FORMATTING_FUNCTIONS    1
#define configSUPPORT_DYNAMIC_ALLOCATION        1
#define configUSE_16_BIT_TICKS          0
#define configIDLE_SHOULD_YIELD         0
#define configUSE_CO_ROUTINES           0
#define configUSE_MUTEXES               1

#define configMAX_CO_ROUTINE_PRIORITIES (2)
#define configUSE_COUNTING_SEMAPHORES   0
#define configUSE_ALTERNATIVE_API       0
#define configCHECK_FOR_STACK_OVERFLOW  2
#define configUSE_RECURSIVE_MUTEXES     0
#define configQUEUE_REGISTRY_SIZE       50
#define configGENERATE_RUN_TIME_STATS   0
#define configUSE_MALLOC_FAILED_HOOK    0

/*
 * Set the following definitions to 1 to include the API function, or zero to
 * exclude the API function
 */
#define INCLUDE_vTaskPrioritySet            1
#define INCLUDE_uxTaskPriorityGet           0
#define INCLUDE_vTaskDelete                 1
#define INCLUDE_vTaskCleanUpResources       1
#define INCLUDE_vTaskSuspend                0
#define INCLUDE_vTaskDelayUntil             1
#define INCLUDE_vTaskDelay                  1
#define INCLUDE_uxTaskGetStackHighWaterMark 1

/* Use the system definition, if there is one */
#ifdef __NVIC_PRIO_BITS
#define configPRIO_BITS __NVIC_PRIO_BITS
#else
#define configPRIO_BITS 4 /* 32 priority levels */
#endif

/* The priority used by the kernel itself */
#define configKERNEL_INTERRUPT_PRIORITY (31 << (8 - configPRIO_BITS))

/* Sets the highest interrupt priority from which interrupt safe FreeRTOS API functions can be called  - Level 16 */
#define configMAX_SYSCALL_INTERRUPT_PRIORITY  (1 << (8 - configPRIO_BITS))

/* Map CMSIS Handler to FreeRTOS Handler */
#define vPortSVCHandler     SVC_Handler
#define xPortPendSVHandler  PendSV_Handler
#define xPortSysTickHandler SysTick_Handler

#endif /* FREERTOS_CONFIG_H */
