/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include <stdint.h>
#include "user_config.h"

#ifdef __ICCARM__
/* for assert_param() */

#ifdef __TMPM_475__
#include "tx04_common.h"
#elif __TMPM_4KN__
#include "TMPM4Ky.h"
#else
#include "tx03_common.h"
#endif
#define __attribute__(x)
#endif /* __ICCARM__ */

#ifdef __KEIL__
/* for assert_param() */
#include "tx03_common.h"
#define __attribute__(x)
#endif /* __KEIL__ */

#ifdef __GNUC__
#define __packed
#endif

/* Version information */
#define FW_VERSION_MAJOR             3                                          /*!< Major number of the firmware */
#define FW_VERSION_MINOR             10                                         /*!< Minor number of the firmware */

/* Stack sizes & task priorities (lower priority value is higher importance) */
#define INIT_TASK_STACK_SIZE        (200)                                       /*!< Stack Site for Init Process Task */
#define SYSTEM_LOAD_TASK_STACK_SIZE ( 60)                                       /*!< Stack Site for System Load Task */
#define MOTORCONTROL_TASK_STACK_SIZE ( 70)                                      /*!< Stack Site for Vector Engine Task */
#define UI_TASK_STACK_SIZE          ( 90)                                       /*!< Stack Site for User Interface Task */
#define PROTOCOL_TASK_STACK_SIZE    (120)                                       /*!< Stack Site for Serial Protocol Task */
#define CAN_TASK_STACK_SIZE         (120)                                       /*!< Stack Site for CAN Communication Task */
#define STALL_TASK_STACK_SIZE       ( 70)                                       /*!< Stack Site for Stall Detection Task */
#define TURN_TASK_STACK_SIZE        (115)                                       /*!< Stack Site for Turn Task */
#define MOTION_TASK_STACK_SIZE      ( 90)                                       /*!< Stack Site for Motion Task */
#define RGB_LED_TASK_STACK_SIZE     ( 50)                                       /*!< Stack Site for RGB Led Task */
#define PERIPHERAL_TASK_STACK_SIZE  (120)                                       /*!< Stack Site for Peripheral Task */
#ifdef STANDALONE_APP
    #define APP_TASK_STACK_SIZE     (STANDALONE_APP_TASK_STACK_SIZE)			/*!< Stack Site for Application Task */
#endif

#define INIT_TASK_PRIORITY          ((unsigned portBASE_TYPE )  4)              /*!< Task Priority for Init Process */
#define SYSTEM_LOAD_TASK_PRIORITY   ((unsigned portBASE_TYPE )  0)              /*!< Task Priority for System Load */
#define MOTORCONTROL_TASK_PRIORITY  ((unsigned portBASE_TYPE )  4)              /*!< Task Priority for Vector Engine */
#define UI_TASK_PRIORITY            ((unsigned portBASE_TYPE )  0)              /*!< Task Priority for User Interface */
#define PROTOCOL_TASK_PRIORITY      ((unsigned portBASE_TYPE )  1)              /*!< Task Priority for Serial Protocol */
#define CAN_TASK_PRIORITY           ((unsigned portBASE_TYPE )  2)              /*!< Task Priority for CAN Communication */
#define STALL_TASK_PRIORITY         ((unsigned portBASE_TYPE )  3)              /*!< Task Priority for Stall Detection */
#define TURN_TASK_PRIORITY          ((unsigned portBASE_TYPE )  3)              /*!< Task Priority for Turn Task */
#define MOTION_TASK_PRIORITY        ((unsigned portBASE_TYPE )  3)              /*!< Task Priority for Motion Task */
#define RGB_LED_TASK_PRIORITY       ((unsigned portBASE_TYPE )  0)              /*!< Task Priority for Turn Task */
#define PERIPHERAL_TASK_PRIORITY    ((unsigned portBASE_TYPE )  1)              /*!< Task Priority for Peripheral Task */
#define STANDALONE_APP_TASK_PRIORITY ((unsigned portBASE_TYPE )  0)             /*!< Task Priority for Application Task */

/* Interrupt priorities - lower value is higher priority */
#define INTERRUPT_PRIORITY_ERROR     0                                          /*!< Interrupt Priority for External Error Signal */
#define INTERRUPT_PRIORITY_ENCODER   2                                          /*!< Interrupt Priority for Encoder */
#define INTERRUPT_PRIORITY_VE        4                                          /*!< Interrupt Priority for Vector Engine */
#define INTERRUPT_PRIORITY_ADC       3                                          /*!< Interrupt Priority for "Software Vector Engine" - ADC completed */
#define INTERRUPT_PRIORITY_PMD       1                                          /*!< Interrupt Priority for Programable Motor Driver */
#define INTERRUPT_PRIORITY_CAN       6                                          /*!< Interrupt Priority for CAN */
#define INTERRUPT_PRIORITY_PROTOCOL  5                                          /*!< Interrupt Priority for Serial Protocol */
#define INTERRUPT_PRIORITY_SPI       6                                          /*!< Interrupt Priority for SPI */

#ifdef __TMPM_370__
#define MAX_CHANNEL                  2                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     MOTORCONTROL_CHANNEL_0|MOTORCONTROL_CHANNEL_1                  /* M370 has VE channel 0 & 1 available */
#endif /* __TMPM_370__ */

#if ( (defined __TMPM_372__) || (defined __TMPM_373__) || (defined __TMPM_374__) )
#define MAX_CHANNEL                  2                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     MOTORCONTROL_CHANNEL_1                               /* M37_234 has only channel 1 available */
#undef  MOTOR_CHANNEL_0
#endif /* (defined __TMPM_372__) || (defined __TMPM_373__) || (defined __TMPM_372__) */

#ifdef __TMPM_375__
#define MAX_CHANNEL                  2                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     MOTORCONTROL_CHANNEL_1                               /* M375 has VE channel 1 available */
#undef  MOTOR_CHANNEL_0
#endif /* __TMPM_370__ */

#ifdef __TMPM_376__
#define MAX_CHANNEL                  2                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     MOTORCONTROL_CHANNEL_0|MOTORCONTROL_CHANNEL_1                  /* M376 has VE channel 0 & 1 available */
#endif /* __TMPM_370__ */

#ifdef __TMPM_475__
#define MAX_CHANNEL                  2                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     MOTORCONTROL_CHANNEL_1                  /* M376 has VE channel 0 & 1 available */
#endif /* __TMPM_475__ */

#ifdef __TMPM_4K4__
#define MAX_CHANNEL                  2                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     MOTORCONTROL_CHANNEL_1                               /* M4K4 has VE channel 0 & 1 available */
#endif /* __TMPM_4K4__ */

#ifdef __TMPM_4KN__
#define MAX_CHANNEL                  3                                          /*!< Number of vector engine channels */
#define BOARD_AVAILABLE_CHANNELS     (MOTORCONTROL_CHANNEL_0|MOTORCONTROL_CHANNEL_1|MOTORCONTROL_CHANNEL_2)     /* M4K4 has VE channel 0 & 1 & 2 available */
#endif /* __TMPM_4K4__ */


/* Board header files */
#ifdef BOARD_HITEX_M370
#define BOARD_NAME               "Hitex M370"                                   /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_hitex_m370.h"

#undef BOARD_PWR_HEADER_FILE_0
#undef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_0   "hitex_m370_pwr_hv.h"                         /*! Powerboard to be used */
#define BOARD_PWR_HEADER_FILE_1   "hitex_m370_pwr_lv.h"                         /*! Powerboard to be used */

#define USE_KEYPAD                                                              /*!< Activate the keypad input */
#define USE_UI                                                                  /*!< Activate the User Interface */
#define USE_LCD                                                                 /*!< Activate the LCD */
#undef  USE_CAN                                                                 /*!< Disable CAN communication */
#endif /* BOARD_HITEX_M370 */

#ifdef BOARD_M372STK
#define BOARD_NAME               "M372STK"                                      /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_m372stk.h"

#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1   "m374pwr_lv.h"                                /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */

#endif /* BOARD_M372STK */

#ifdef BOARD_M37SIGMA
#define BOARD_NAME                "M37Sigma"                                    /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE  "board_config_m37sigma.h"

#undef USE_LED
#undef  USE_CONFIG_STORAGE_EEPROM
#define USE_CONFIG_STORAGE_FLASH
#undef  USE_CAN
#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1   "m37Sigma_pwr.h"                              /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */

#endif /* BOARD_M37SIGMA */

#ifdef BOARD_M374STK
#define BOARD_NAME               "M374STK"                                      /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_m374stk.h"

#undef USE_EXTERNAL_SPEED_CONTROL

#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1  "m374pwr_lv.h"                                 /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */

#endif /* BOARD_M374STK */

#ifdef BOARD_M375STK
#define BOARD_NAME               "M375STK"                                      /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_m375stk.h"
#undef  USE_CONFIG_STORAGE_EEPROM
#undef  USE_CONFIG_STORAGE
#undef  USE_CAN
#undef  USE_DSO
#undef  USE_ENCODER
#undef  USE_HSDSO
#undef  USE_STALL_DETECT
#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1  "m375stk_pwr.h"                                /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */
#endif /* BOARD_M375STK */

#ifdef BOARD_TMPM475_SK
#define BOARD_NAME               "TMPM475-SK"                                      /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_tmpm475sk.h"
#undef  USE_CONFIG_STORAGE_EEPROM

#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1  "tmpm475sk_pwr.h"                                 /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */

#endif /* BOARD_M374STK */

#ifdef BOARD_M4K4
#define BOARD_NAME               "TMPM4K4"                                      /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_m4k4.h"
#undef  USE_CONFIG_STORAGE_EEPROM

#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1  "m4k4_pwr.h"                                 /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */

#endif /* BOARD_M4K4 */

#ifdef BOARD_M4KN
#define BOARD_NAME               "TMPM4KN"                                      /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_m4kn.h"

#ifndef BOARD_PWR_HEADER_FILE_0
#define BOARD_PWR_HEADER_FILE_0  "m4kn_pwr.h"                                 /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_0 */
#ifndef BOARD_PWR_HEADER_FILE_1
#define BOARD_PWR_HEADER_FILE_1  "m4kn_pwr.h"                                 /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_1 */
#ifndef BOARD_PWR_HEADER_FILE_2
#define BOARD_PWR_HEADER_FILE_2  "m4kn_pwr.h"                                 /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_2 */

#endif /* BOARD_M4KN */

#ifdef BOARD_CLICKER4_M4KN
#define BOARD_NAME               "CLICKER4-TMPM4KN"                             /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_clicker4_m4kn.h"

#ifndef BOARD_PWR_HEADER_FILE_0
#define BOARD_PWR_HEADER_FILE_0  "clicker4_m4kn_pwr.h"                          /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_0 */
#endif /* BOARD_CLICKER4_M4KN */

#ifdef BOARD_SBK_M4KN
#define BOARD_NAME               "SBK-TMPM4KN"                                  /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_sbk_m4kn.h"

#ifndef BOARD_PWR_HEADER_FILE_0
#define BOARD_PWR_HEADER_FILE_0  "sbk_m4kn_pwr.h"                               /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_0 */
#endif /* BOARD_SBK_M4KN */

#ifdef BOARD_M5_IMZ120_M4KN
#define BOARD_NAME               "M5-IMZ120-TMPM4KN"                            /* Name of the Board */
#define BOARD_CONFIG_HEADER_FILE "board_config_m5_imz120_m4kn.h"

#ifndef BOARD_PWR_HEADER_FILE_0
#define BOARD_PWR_HEADER_FILE_0  "eval_m5-imz120r_pwr.h"                        /*! Powerboard to be used */
#endif /* BOARD_PWR_HEADER_FILE_0 */
#endif /* BOARD_SBK_M4KN */

/* Chip header files and information */

#ifdef __TMPM_370__
#define TMPM_UART_HEADER_FILE    "tmpm370_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm370_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm370_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm370_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm370_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm370_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm370_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm370_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm370_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm370_ofd.h"
#define TMPM_HEADER_FILE         "TMPM370.h"
#define MEMORY_SIZE              0x2800
#define PLL_MULTIPLIER           8                                              /* Multiplier of the PLL */
#endif /* __TMPM_370__ */

#ifdef __TMPM_372__
#define TMPM_UART_HEADER_FILE    "tmpm372_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm372_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm372_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm372_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm372_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm372_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm372_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm372_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm372_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm372_ofd.h"
#define TMPM_HEADER_FILE         "TMPM372.h"
#define MEMORY_SIZE              0x1800
#define PLL_MULTIPLIER           8                                              /* Multiplier of the PLL */
#endif /* __TMPM_372__ */

#ifdef __TMPM_373__
#define TMPM_UART_HEADER_FILE    "tmpm373_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm373_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm373_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm373_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm373_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm373_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm373_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm373_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm373_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm373_ofd.h"
#define TMPM_HEADER_FILE         "TMPM373.h"
#define MEMORY_SIZE              0x1800
#define PLL_MULTIPLIER           8                                              /* Multiplier of the PLL */
#endif /* __TMPM_373__ */

#ifdef __TMPM_374__
#define TMPM_UART_HEADER_FILE    "tmpm374_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm374_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm374_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm374_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm374_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm374_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm374_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm374_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm374_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm374_ofd.h"
#define TMPM_HEADER_FILE         "TMPM374.h"
#define MEMORY_SIZE              0x1800
#define PLL_MULTIPLIER           8                                              /* Multiplier of the PLL */
#endif /* __TMPM_374__ */

#ifdef __TMPM_375__
#define TMPM_UART_HEADER_FILE    "tmpm375_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm375_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm375_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm375_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm375_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm375_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm375_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm375_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm375_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm375_ofd.h"
#define TMPM_HEADER_FILE         "TMPM375.h"
#define MEMORY_SIZE              0x1000
#define PLL_MULTIPLIER           4                                              /* Multiplier of the PLL */
#endif /* __TMPM_375__ */

#ifdef __TMPM_376__
#define TMPM_UART_HEADER_FILE    "tmpm376_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm376_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm376_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm376_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm376_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm376_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm376_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm376_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm376_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm376_ofd.h"
#define TMPM_HEADER_FILE         "TMPM376.h"
#define MEMORY_SIZE              0x8000
#define PLL_MULTIPLIER           8                                              /* Multiplier of the PLL */
#endif /* __TMPM_376__ */

#ifdef __TMPM_475__
#define TMPM_UART_HEADER_FILE    "tmpm475_uart.h"
#define TMPM_GPIO_HEADER_FILE    "tmpm475_gpio.h"
#define TMPM_WDT_HEADER_FILE     "tmpm475_wdt.h"
#define TMPM_FC_HEADER_FILE      "tmpm475_fc.h"
#define TMPM_ADC_HEADER_FILE     "tmpm475_adc.h"
#define TMPM_TIMER_HEADER_FILE   "tmpm475_tmrb.h"
#define TMPM_CG_HEADER_FILE      "tmpm475_cg.h"
#define TMPM_WDT_HEADER_FILE     "tmpm475_wdt.h"
#define TMPM_VLTD_HEADER_FILE    "tmpm475_vltd.h"
#define TMPM_OFD_HEADER_FILE     "tmpm475_ofd.h"
#define TMPM_HEADER_FILE         "TMPM475.h"
#define MEMORY_SIZE              0x8000
#define PLL_MULTIPLIER           12                                              /* Multiplier of the PLL */
#endif /* __TMPM_475__ */

#ifdef __TMPM_4K4__
#define TMPM_UART_HEADER_FILE    "txz_uart.h"
#define TMPM_GPIO_HEADER_FILE    "txz_gpio.h"
#define TMPM_WDT_HEADER_FILE     "ipdrv_siwdt.h"
#define TMPM_FC_HEADER_FILE      "fc.h"
#define TMPM_ADC_HEADER_FILE     "txz_adc.h"
#define TMPM_TIMER_HEADER_FILE   "txz_t32a.h"
#define TMPM_CG_HEADER_FILE      "txz_cg.h"
#define TMPM_VLTD_HEADER_FILE    "txz_vltd.h"
#define TMPM_OFD_HEADER_FILE     "ofd.h"
#define TMPM_HEADER_FILE         "TMPM4K4A.h"
#define MEMORY_SIZE              0x8000
#define PLL_MULTIPLIER           12                                              /* Multiplier of the PLL */
#endif /* __TMPM_4K4__ */

#ifdef __TMPM_4KN__
#define BOARD_SPI_HEADER_FILE    "spi_device.h"
//#define TMPM_UART_HEADER_FILE    "txz_uart.h"
#define TMPM_GPIO_HEADER_FILE    "TMPM4Ky_gpio.h"
//#define TMPM_WDT_HEADER_FILE     "ipdrv_siwdt.h"
//#define TMPM_FC_HEADER_FILE      "fc.h"
#define TMPM_ADC_HEADER_FILE     "TMPM4Ky_adc.h"
//#define TMPM_TIMER_HEADER_FILE   "txz_t32a.h"
#define TMPM_CG_HEADER_FILE      "TMPM4Ky_cg.h"
//#define TMPM_VLTD_HEADER_FILE    "txz_vltd.h"
//#define TMPM_OFD_HEADER_FILE     "ofd.h"
#define TMPM_HEADER_FILE         "TMPM4KN.h"
#define TMPM_CG_HEADER_FILE      "TMPM4Ky_cg.h"
#define TMPM_SPI_HEADER_FILE    "TMPM4Ky_spi.h"
#define MEMORY_SIZE              0x8000
#define PLL_MULTIPLIER           16                                              /* Multiplier of the PLL */
#endif /* __TMPM_4K4__ */



/* Needed stack size calculations */
#define TASK_OVERHEAD                  28

#define STACK_SIZE1                     ( 210 )                                  /*!< Scheduler Stack Size */
#define STACK_SIZE2                     (INIT_TASK_STACK_SIZE        + TASK_OVERHEAD)
#ifdef USE_LOAD_STATISTICS
#define STACK_SIZE3                     (SYSTEM_LOAD_TASK_STACK_SIZE + TASK_OVERHEAD)
#else
#define STACK_SIZE3                     0
#endif /* USE_LOAD_STATISTICS */
#ifdef USE_SERIAL_COMMUNICATION
#define STACK_SIZE4                     (PROTOCOL_TASK_STACK_SIZE    + TASK_OVERHEAD)
#else
#define STACK_SIZE4                     0
#endif /* USE_SERIAL_COMMUNICATION */
#ifdef USE_CAN
#define STACK_SIZE5                     (CAN_TASK_STACK_SIZE         + TASK_OVERHEAD)
#else
#define STACK_SIZE5                     0
#endif /* USE_CAN */
#ifdef USE_STALL_DETECT
#define STACK_SIZE6                     (STALL_TASK_STACK_SIZE       + TASK_OVERHEAD)
#else
#define STACK_SIZE6                     0
#endif /* USE_STALL_DETECT */
#ifdef USE_RGB_LED
#define STACK_SIZE7                     (RGB_LED_TASK_STACK_SIZE     + TASK_OVERHEAD)
#else
#define STACK_SIZE7                     0
#endif /* USE_RGB_LED */

#define STACK_SIZE9                     (3*MOTORCONTROL_TASK_PRIORITY + 3*TASK_OVERHEAD)

#ifdef USE_UI
#define STACK_SIZE10                    (UI_TASK_STACK_SIZE          + TASK_OVERHEAD)
#else
#define STACK_SIZE10                    0
#endif /* USE_UI */



#ifdef __TMPM_4KN__
#ifdef USE_TURN_CONTROL
#define STACK_SIZE8                     (3*TURN_TASK_STACK_SIZE      + 3*TASK_OVERHEAD)
#else
#define STACK_SIZE8                     0
#endif /* USE_TURN_CONTROL */

#ifdef USE_MOTION_CONTROL
#define STACK_SIZE11                     (3*MOTION_TASK_STACK_SIZE      + 3*TASK_OVERHEAD)
#else
#define STACK_SIZE11                     0
#endif /* USE_MOTION_CONTROL */

#ifdef STANDALONE_APP
#define STACK_SIZE12                     (APP_TASK_STACK_SIZE      + TASK_OVERHEAD)
#else
#define STACK_SIZE12                     0
#endif

#endif /*  __TMPM_4KN__ */

/* Memory calculations */
#define NEEDED_HEAP_SIZE                (4* (  STACK_SIZE1  \
                                             + STACK_SIZE2  \
                                             + STACK_SIZE3  \
                                             + STACK_SIZE4  \
                                             + STACK_SIZE5  \
                                             + STACK_SIZE6  \
                                             + STACK_SIZE7  \
                                             + STACK_SIZE8  \
                                             + STACK_SIZE9  \
                                             + STACK_SIZE10 \
                                             + STACK_SIZE11 \
                                             + STACK_SIZE12 \
                                             )              \
                                        )

/* Firmware usage sizes for Debug & Release */

#ifdef DEBUG

#ifdef BOARD_HITEX_M370
#define FW_RAM_NEEDED                   4500
#elif  BOARD_M374STK
#define FW_RAM_NEEDED                   3150
#elif  BOARD_M372STK
#define FW_RAM_NEEDED                   3150
#elif  BOARD_M37SIGMA
#define FW_RAM_NEEDED                   3350
#elif  BOARD_M375STK
#define FW_RAM_NEEDED                   3150
#elif  BOARD_TMPM475_SK
#define FW_RAM_NEEDED                   5500
#elif  BOARD_M4KN
#define FW_RAM_NEEDED                   (4500)
#elif  BOARD_SBK_M4KN
#define FW_RAM_NEEDED                   (4500)
#elif  BOARD_CLICKER4_M4KN
#define FW_RAM_NEEDED                   (4500)
#endif

#else

#ifdef BOARD_HITEX_M370
#define FW_RAM_NEEDED                   3800
#elif  BOARD_M374STK
#define FW_RAM_NEEDED                   2900
#elif  BOARD_M372STK
#define FW_RAM_NEEDED                   2900
#elif  BOARD_M37SIGMA
#ifdef SIGMA_PRODUCTION
#define FW_RAM_NEEDED                   3300
#else
#define FW_RAM_NEEDED                   3150
#endif /* SIGMA_PRODUCTION */
#elif  BOARD_M375STK
#define FW_RAM_NEEDED                   2900
#elif  BOARD_TMPM475_SK
#define FW_RAM_NEEDED                   4500
#elif  BOARD_M4KN
#define FW_RAM_NEEDED                   (12*1024)
#elif  BOARD_SBK_M4KN
#define FW_RAM_NEEDED                   (12*1024)
#elif  BOARD_CLICKER4_M4KN
#define FW_RAM_NEEDED                   (12*1024)
#endif

#endif /* DEBUG */

#ifdef USE_DSO
    #ifdef __TMPM_4KN__
        /* Limit the size to avoid huge data transfers */
        #define BOARD_DSO_SIZE                  (4*1024)
    #else
        #define BOARD_DSO_SIZE                  ((MEMORY_SIZE-FW_RAM_NEEDED-NEEDED_HEAP_SIZE)/2)
    #endif /* __TMPM_4KN__ */
#else
#define BOARD_DSO_SIZE                  0
#endif /* USE_DSO */

#if (BOARD_DSO_SIZE < 0)
#error Needed heap size exceeds available memory - please deactivate firmware features in user_config.h
#endif

/* Clock calculations */
#include BOARD_CONFIG_HEADER_FILE

#define INTERNAL_OSCILLATOR_FREQUENCY   9500000

#if   BOARD_CLOCK_GEAR_DIVIDER == CG_DIVIDE_1
#define GEAR_DIVIDER                    1
#elif BOARD_CLOCK_GEAR_DIVIDER == CG_DIVIDE_2
#define GEAR_DIVIDER                    2
#elif BOARD_CLOCK_GEAR_DIVIDER == CG_DIVIDE_4
#define GEAR_DIVIDER                    4
#elif BOARD_CLOCK_GEAR_DIVIDER == CG_DIVIDE_8
#define GEAR_DIVIDER                    8
#elif BOARD_CLOCK_GEAR_DIVIDER == CG_DIVIDE_16
#define GEAR_DIVIDER                    16
#endif

#if   BOARD_PERIPHERIAL_CLOCK_DIVIDER == CG_DIVIDE_1
#define PER_GEAR_DIVIDER                1
#elif BOARD_PERIPHERIAL_CLOCK_DIVIDER == CG_DIVIDE_2
#define PER_GEAR_DIVIDER                2
#elif BOARD_PERIPHERIAL_CLOCK_DIVIDER == CG_DIVIDE_4
#define PER_GEAR_DIVIDER                4
#elif BOARD_PERIPHERIAL_CLOCK_DIVIDER == CG_DIVIDE_8
#define PER_GEAR_DIVIDER                8
#elif BOARD_PERIPHERIAL_CLOCK_DIVIDER == CG_DIVIDE_16
#define PER_GEAR_DIVIDER                16
#endif

#ifdef BOARD_USE_PLL
#define CPU_CLOCK                       ( osc_frequency  \
                                        / GEAR_DIVIDER   \
                                        * PLL_MULTIPLIER)
#else
#define CPU_CLOCK                       ( osc_frequency  \
                                        / GEAR_DIVIDER )
#endif /*  BOARD_USE_PLL */

#define PERIPHERIAL_CLOCK               ( CPU_CLOCK / PER_GEAR_DIVIDER )

/* Checking for consistency */

#ifndef USE_CONFIG_STORAGE
#ifndef USE_INTERNAL_MOTOR_PARAMS
#error EITHER USE_CONFIG_STORAGE OR USE_INTERNAL_MOTOR_PARAMS OR BOTH HAVE TO BE DEFINED!!!
#endif
#endif

#if ( (defined USE_UI) &&  !(defined USE_LCD) )
#error USING THE UI WITHOUT LCD MAKES NO SENSE
#endif

#if ( (defined USE_WDT) && !(defined USE_LOAD_STATISTICS))
#error USING WDT ONLY WITH USE_LOAD_STATISTICS
#endif

#ifdef USE_CONFIG_STORAGE
#if ( (defined USE_CONFIG_STORAGE_FLASH) && (defined USE_CONFIG_STORAGE_EEPROM))
#error Only either USE_CONFIG_STORAGE_FLASH or USE_CONFIG_STORAGE_EEPROM must be defined
#endif
#if ( (!defined USE_CONFIG_STORAGE_FLASH) && (!defined USE_CONFIG_STORAGE_EEPROM))
#error One of USE_CONFIG_STORAGE_FLASH or USE_CONFIG_STORAGE_EEPROM must be defined
#endif
#endif

#define FIX_155_PMD_DELAY        /* PMD registers update done at beginning of VCN0's IRQ handler */

#endif /* _CONFIG_H_ */
