/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdlib.h>
#include "config.h"
#include "FreeRTOS.h"
#include "task.h"
#include "board.h"
#include "units_enums.h"
#include "api.h"
#include "debug.h"
#include "DemoApplication.h"
#include "peripherals_control.h"
#include "hal_gpio.h"
#include "hal_cg.h"
#include "TMPM4KN.h"
#include "motor_control.h"
#include "led.h"
#include BOARD_CONFIG_HEADER_FILE

/***************************************************************************************************************
*                                          SLIDER 2 CLICKER DEMO                                               *
***************************************************************************************************************/
#ifdef DEMO_CLICKER4_SLIDER2

#define DEMO_ROT_DIRECTION_CW               0
#define DEMO_ROT_DIRECTION_CCW              1

/* IO for button press (B5) recognition - simple falling edge IRQ */
#define DEMO_DIRECTION_PORT_CLK             (HAL_CG_CLK_PORT_E)
#define DEMO_DIRECTION_PORT                 (HAL_GPIO_PORT_E)
#define DEMO_DIRECTION_PIN                  (HAL_GPIO_BIT_5)

/* IO for button press (B6) recognition - simple falling edge IRQ */
#define DEMO_START_STOP_PORT_CLK            (HAL_CG_CLK_PORT_U)
#define DEMO_START_STOP_PORT                (HAL_GPIO_PORT_U)
#define DEMO_START_STOP_PIN                 (HAL_GPIO_BIT_3)

/* Used for "same state change" protection */
demo_status actual_schedule = DEMO_STOP;

/* Flag for start/stop detection */
volatile uint8_t StartStopRequested = 0;
volatile uint8_t DirectionChangeRequested = 0;
volatile uint8_t SliderChangeRequested = 0;

/* Flag for direction change */
volatile uint8_t DemoRotDirection = DEMO_ROT_DIRECTION_CW;

/*************************************
*           CONFIG & IRQ             *
*************************************/
#ifdef USE_LED
/*! \brief  vConfigDemoLEDs
*
* Configure the LEDs for demo illumination
*
* @param  None
* @retval none
*/
static void vConfigDemoLEDs(void)
{
    /* LED intial state is off */
    LED_SetState(LED_NO_5, LED_OFF);
    LED_SetState(LED_NO_6, LED_OFF);
}
#endif /* USE_LED */

/*! \brief  vConfigDemoButtons
*
* Configure the I/Os for demo control via push buttons
*
* @param  None
* @retval none
*/
static void vConfigDemoButtons(void)
{
    /* B5 */
    HAL_CG_EnableMiddleSpeedClockA(DEMO_DIRECTION_PORT_CLK);    
    HAL_GPIO_Init(DEMO_DIRECTION_PORT, DEMO_DIRECTION_PIN, HAL_GPIO_INPUT);
   
    /* Input enable PE5 */
    *(volatile uint32_t *) (0x400E0438) |= DEMO_DIRECTION_PIN;
    
    /* B6 */
    HAL_CG_EnableMiddleSpeedClockA(DEMO_START_STOP_PORT_CLK);
    HAL_GPIO_Init(DEMO_START_STOP_PORT, DEMO_START_STOP_PIN, HAL_GPIO_INPUT);
    /* Input enable PU3 */
    *(volatile uint32_t *) (0x400E1038) |= DEMO_START_STOP_PIN;
    
    /* IRQ enable for B5 (PE5) as INT05a, Falling edge */
    TSB_IB->IMC042 = (uint8_t)(0x05);
    NVIC_EnableIRQ(INT05_IRQn);
    
    /* IRQ enable for B6 (PU3) as INT08a, Falling edge */
    TSB_IB->IMC048 = (uint8_t)(0x05);
    NVIC_EnableIRQ(INT08_IRQn);
}

/*! \brief  INT05_IRQHandler
*
* Handler for INT05a/INT05b
*
* @param  None
* @retval none
*/
void INT05_IRQHandler(void)
{
    __IO uint8_t imc = 0;

    do
    {
        /* Clear the "falling edge detection flag" & disable the interrupt */
        TSB_IB->IMC042 = (uint8_t)(0x80);
        /* Dummy read is needed */
        imc = TSB_IB->IMC042;
    }
    while( 0x20 == (imc & 0x20) );
    
    DirectionChangeRequested = 1;
}


/*! \brief  INT08_IRQHandler
*
* Handler for INT08a/INT08b
*
* @param  None
* @retval none
*/
void INT08_IRQHandler(void)
{
    __IO uint8_t imc = 0;

    do
    {
        /* Clear the "falling edge detection flag" & disable the interrupt */
        TSB_IB->IMC048 = (uint8_t)(0x80);
        /* Dummy read is needed */
        imc = TSB_IB->IMC048;
    }
    while( 0x20 == (imc & 0x20) );
    
    /* Active request has been received */
    StartStopRequested = 1;
}


/*************************************
*            GENERIC DEMO            *
*************************************/

/*! \brief  Demo
*
* Triggers execution of particular demo step
*
* @param  new_state: the step to be executed
* @retval None
*/
void Demo(demo_status new_state)
{
    actual_schedule = new_state;
}


/*! \brief  DemoState
*
* Reports the current demo step
*
* @param  None
* @retval current state
*/
demo_status DemoState(void)
{
    return actual_schedule;
}

#ifdef USE_SLIDER_2_CLICKER 
/*! \brief  SliderControl
  *
  * @param  None
  * @retval None
*/
static void SliderControl(void)
{
    static int32_t SliderVal = 0;
    static int32_t OldSliderVal = 0;  

    /* Get the current sldier value */
    SliderVal = Slider_GetValue();  

    /* Filter minor changes of the volateg readings, but always consider 0 (Stop)  */
    if( (SLIDER_2_MIN_STOP_VALUE == SliderVal) || (abs(SliderVal - OldSliderVal) > SLIDER_2_MIN_SPEED_CHANGE) || ( 1 == SliderChangeRequested) )
    {
        /* Map voltage to speed */
        MotorSetValues[0].TargetSpeed = SliderVal * SLIDER_2_SPEED_CONVERSION_FACTOR;
      
        /* Add some RPM to avoid the jerking at low speed if the command is not Stop (0V) */
        if( (SliderVal > SLIDER_2_MAX_STOP_VALUE) && (SliderVal < SLIDER_2_MAX_LOW_SPEED_VALUE) ) 
        {
            MotorSetValues[0].TargetSpeed += SLIDER_2_LOW_SPEED_COMPENSATION_RPM;
        }
        /* Limit the speed */
        if( MotorSetValues[0].TargetSpeed > SLIDER_2_SPEED_LIMIT_RPM )
        {
            MotorSetValues[0].TargetSpeed = SLIDER_2_SPEED_LIMIT_RPM;
        }
        
        if( DEMO_ROT_DIRECTION_CCW == DemoRotDirection )
        {
            MotorSetValues[0].TargetSpeed *= -1;
        }
      
        /* Store the old value for filtering */
        OldSliderVal = SliderVal;
    }
}
#endif /* USE_SLIDER_2_CLICKER */ 


/*! \brief  Demo Application Robotarm AR2
  *
  * @param  pvParameters: None
  * @retval None
*/
void StandaloneApplication(void* pvParameters)
{    
    /* Wait until all task intialization is completed */
    while( InitDone == INIT_NOT_DONE )
    {
        vTaskDelay( 100 / portTICK_RATE_MS );
    }

    /* Configure the MMI */
#ifdef USE_LED
    vConfigDemoLEDs();
#endif /* USE_LED */
    vConfigDemoButtons();
   
    /* Handle any events */
    for(;;)
    {
        vTaskDelay( 100 / portTICK_RATE_MS );
        
        /* Start/stop requested */
        if( 1 == StartStopRequested )                 
        {            
            if( 1 == MotorControl[0].IsStarted )
            {
                /* Stop the motor */
                MotorControl_Stop(0);
                MotorControl[0].IsStarted = 0;
                /* Update the indication and the state */
#ifdef USE_LED
                LED_SetState(LED_NO_6, LED_OFF);
#endif /* USE_LED */
                actual_schedule = DEMO_STOP;                        
            }
            else
            {
                /* Start the motor */
                MotorControl_Start(0);
                MotorControl[0].IsStarted = 1;
                /* Slider speed update is to be enforced */
                SliderChangeRequested = 1;
                /* Update the indication and the state */
#ifdef USE_LED
                LED_SetState(LED_NO_6, LED_ON);
#endif /* USE_LED */
                actual_schedule = DEMO_RUN;
            }
            /* Prevent multiple interrupts per press */
            vTaskDelay( 500 / portTICK_RATE_MS );
            /* Arm the button again */
            StartStopRequested = 0;
            /* Re-enable the IRQ for the button press detection */
            TSB_IB->IMC048 = (uint8_t)(0x05);
        }
        
        /* Direction change requested */
        if( 1 == DirectionChangeRequested )                 
        {
            /* Slider speed update is to be enforced, due to the upcoming direction inversion */
            SliderChangeRequested = 1;
            
            /* Invert the direction */
            if( DEMO_ROT_DIRECTION_CCW == DemoRotDirection )
            {
                /* Update the indication */
#ifdef USE_LED
                LED_SetState(LED_NO_5, LED_OFF);
#endif /* USE_LED */
                /* Revert the direction */
                DemoRotDirection = DEMO_ROT_DIRECTION_CW;
            }
            else
            {
                /* Update the indication */
#ifdef USE_LED
                LED_SetState(LED_NO_5, LED_ON);
#endif /* USE_LED */
                /* Revert the direction */
                DemoRotDirection = DEMO_ROT_DIRECTION_CCW;
            }
            /* Prevent multiple interrupts per press */
            vTaskDelay( 500 / portTICK_RATE_MS );
            /* Arm the button again */
            DirectionChangeRequested = 0;
            /* Re-enable the IRQ for the button press detection */
            TSB_IB->IMC042 = (uint8_t)(0x05);
        }
    
        switch( actual_schedule )
        {                       
            /* Transition from initial/start position to demo/run */
            case DEMO_RUN:      
                /* Start the motor if not running already */
                if( 0 == MotorControl[0].IsStarted )
                { 
                    /* Start the motor */
                    MotorControl_Start(0);
                    MotorControl[0].IsStarted = 1;
                    /* Slider speed update is to be enforced */
                    SliderChangeRequested = 1;
                    /* Update the indication and the state */
#ifdef USE_LED
                    LED_SetState(LED_NO_6, LED_ON);
#endif /* USE_LED */
                }
#ifdef USE_SLIDER_2_CLICKER         
                /* Check for speed and/or direction change */
                SliderControl();   
#endif /* USE_SLIDER_2_CLICKER */                 
                break;

            /* Transition from initial/start or demo/run to stop position */
            case DEMO_STOP:        
              /* Stop the motor */
                MotorControl_Stop(0);
                MotorControl[0].IsStarted = 0;
                /* Update the indication and the state */
#ifdef USE_LED
                LED_SetState(LED_NO_6, LED_OFF);
#endif /* USE_LED */
                actual_schedule = DEMO_IDLE;
                break;
                
            case DEMO_WAKEUP:
            case DEMO_SLEEP:
            case DEMO_IDLE:
            default:
                break;
        }
    }
}
#endif /* DEMO_CLICKER4_SLIDER2 */


