/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include "config.h"
#ifdef USE_SERIAL_COMMUNICATION
//#define TRANSLOG

#include BOARD_CONFIG_HEADER_FILE
#include <stdio.h>
#include "config.h"
#include "hal_uart.h"
#include "led.h"
#include "rgb_led.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"
#include "api.h"
#include "api_internal.h"
#include "protocol.h"
#include "crc8.h"
#include "debug.h"
#include "board.h"
#if defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__
#include TMPM_UART_HEADER_FILE
#else
#include "txz_driver_def.h"
#endif
#include "performance_measurement.h"

__packed struct header
{
  uint8_t xon;
  uint8_t cmd;
};

__packed struct footer
{
  uint8_t crc;
  uint8_t xoff;
};

static uint8_t          g_q[MAX_SIZE_REQ];
static uint8_t          g_a[sizeof(struct header) + MAX_SIZE_RSP + sizeof(struct footer)];

enum transmission_magics
{
  bot = 0x11,
  eot = 0x13,
  crc_error = 0xee,
};

enum rx_state
{
  rx_idle = 0,
  rx_cmd,
  rx_rcv,
  rx_checksum,
  rx_eot,
};

struct rx_machine
{
  enum rx_state state;
  xSemaphoreHandle s;
  uint8_t cmd;
  uint8_t checksum;
  uint8_t *buf;
  int cnt;
  const struct rpc *f;
};

static struct rx_machine g_rx_machine;

enum tx_state
{
  tx_cmd = 0,
  tx_send,
  tx_checksum,
  tx_eot,
  tx_idle
};

struct tx_machine
{
  enum tx_state state;
  xSemaphoreHandle s;
  uint8_t cmd;
  uint8_t checksum;
  uint8_t *buf;
  int len;
  int cnt;
};

static struct tx_machine g_tx_machine;

struct rpc
{
  int (*f) (void*q, void*a);
  int size_q;
  int size_a;
  uint8_t id;
};

#define MAKE_FUNC(ff) \
  { \
    (int(*) (void*, void*)) ff, sizeof(struct ff ## Req), sizeof(struct ff ## Rsp), RPC_##ff \
  }

#define RPC_ConfigDsoLog            0x05
#define RPC_GetDsoLogData           0x06
#define RPC_ConfigureHsDso          0x07
/* Setup the function names that should be called on the different actions */
static const struct rpc functions[] = {
                                  MAKE_FUNC(StartMotor),
                                  MAKE_FUNC(StopMotor),
                                  MAKE_FUNC(GetMotorState),
                                  MAKE_FUNC(StoreParameters),
                                  MAKE_FUNC(ClearParameters),
                                  MAKE_FUNC(ConfigDsoLog),
                                  MAKE_FUNC(GetDsoLogData),
                                  MAKE_FUNC(ConfigureHsDso),
                                  MAKE_FUNC(DoTurn),
                                  MAKE_FUNC(GetOneMotorParameter),
                                  MAKE_FUNC(SetOneMotorParameter),
                                  MAKE_FUNC(GetMotorParameters),
                                  MAKE_FUNC(SetMotorParameters),
                                  MAKE_FUNC(DoLinearMotion),
                                  MAKE_FUNC(GetAbsolutePosition),
                                  MAKE_FUNC(AbortLinearMotion),
                                  MAKE_FUNC(StartMotorTorqueCtrl),
                                  MAKE_FUNC(GetMotorControlMethod),
                                  MAKE_FUNC(SetMotorControlMethod),
                                  MAKE_FUNC(GetEncoderCounter),
                                  MAKE_FUNC(GetFWVersion),
                                  MAKE_FUNC(GetDemoState),
                                  MAKE_FUNC(SetDemoState),
                                  MAKE_FUNC(GetExtendedMotorState),
                                 };

#define NUM_RPC_FUNCTIONS (sizeof(functions)/sizeof(functions[0]))

static void tx_state_machine(void)
{
  struct tx_machine *tx = &g_tx_machine;

  HAL_UART_ClearPendingIRQ(SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_TX);

  switch(tx->state) {
  case tx_cmd:
    HAL_UART_SendData(SERIAL_COMMUNICATION_UART_CHANNEL, tx->cmd);
    tx->state = tx_send;
    tprintf("tx cmd 0x%02x\n", tx->cmd);
    break;
  case tx_send:
    HAL_UART_SendData(SERIAL_COMMUNICATION_UART_CHANNEL, tx->buf[tx->cnt]);
    tprintf("tx buf 0x%02x @ %d\n", tx->buf[tx->cnt], tx->cnt);
    tx->cnt++;
    if(tx->cnt == tx->len) {
      tx->state = tx_checksum;
      break;
    }
    break;
  case tx_checksum:
    HAL_UART_SendData(SERIAL_COMMUNICATION_UART_CHANNEL, tx->checksum);
    tprintf("tx cs 0x%02x\n", tx->checksum);
    tx->state = tx_eot;
    break;
  case tx_eot:
    HAL_UART_SendData(SERIAL_COMMUNICATION_UART_CHANNEL, eot);
    tprintf("tx eot\n");
    tx->state = tx_idle;
    break;
  case tx_idle:
    {
      portBASE_TYPE xHigherPriorityTaskWoken;
      tprintf("tx idle\n");
      HAL_UART_DisableIRQ(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_TX);
      HAL_UART_EnableIRQ(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_RX);
      HAL_UART_SetIRQPriority(SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_RX, ~0);
      xSemaphoreGiveFromISR(tx->s, &xHigherPriorityTaskWoken);
      break;
    }
  default:
    tprintf("tx error\n");
    tx->state = tx_idle;
    break;
  }
}

#if (defined(__ICCARM__)) /* IAR Compiler - */
#pragma diag_suppress = Pe550
#elif defined __KEIL__
#pragma diag_suppress = 550
#endif
static int send_generic (int cmd, uint8_t *buf, int len)
{
  struct tx_machine *tx = &g_tx_machine;
  WorkState w;

  w = HAL_UART_GetBufState(SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_TX);
  assert_param(w == DONE);

  tx->cmd = cmd;
  tx->buf = buf;
  tx->cnt = 0;
  tx->len = len;
  tx->state = tx_cmd;
  tx->checksum = crc_update(cmd, buf, len);

#ifdef TRANSLOG
    if ((tx->cmd != 0x2) && (tx->cmd != 0x09))
    {
      int count;
      printf("Answer : BOT 0x%02x ", tx->cmd);
      for (count = 0;count < tx->len; count ++)
        printf("0x%02x ",*(tx->buf+count));
      printf("0x%02x EOT\n",tx->checksum);
    }
#endif

  HAL_UART_SetIRQPriority(SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_TX, INTERRUPT_PRIORITY_PROTOCOL);
  HAL_UART_EnableIRQ(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_TX);
  HAL_UART_SendData(SERIAL_COMMUNICATION_UART_CHANNEL, bot);
  xSemaphoreTake(tx->s, portMAX_DELAY);
  return 0;
}
#if (defined(__ICCARM__)) /* IAR Compiler - */
#pragma diag_default = Pe550
#elif defined __KEIL__
#endif

static int send_error (int ret)
{
  uint8_t buf = ret;
  return send_generic(crc_error, &buf, sizeof(buf));
}

uint8_t global_rx_error_count = 0;

static void err_state_machine(void)
{
  uint32_t error = 0;
  
/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32UART1ERRPrevTime = u32UART1ERRStartTime;
    /* Get the new start time */
    u32UART1ERRStartTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32UART1ERRLoopOccuranceRate = u32UART1ERRStartTime - u32UART1ERRPrevTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */
    
  global_rx_error_count ++;

  struct rx_machine *rx = &g_rx_machine;
  struct tx_machine *tx = &g_tx_machine;

  /* Check the errors one by one */
  do
  {
    error = HAL_UART_GetError(SERIAL_COMMUNICATION_UART_CHANNEL);

    /* Overrun upon reception */
    if( (OVRERR & error) )
    {
      /* Disable the reception and transmission control */
      HAL_UART_SetRxTx(SERIAL_COMMUNICATION_UART_CHANNEL, 0, 1);

      /* Read out any remaining data in the RX FIFO */
      while( 0 != HAL_UART_GetRXFIFOLevel(SERIAL_COMMUNICATION_UART_CHANNEL) )
      {
        HAL_UART_GetReceivedData(SERIAL_COMMUNICATION_UART_CHANNEL);
      }
      /* Clear the read/write pointer of the receive FIFO */
      HAL_UART_ClearRXFIFO(SERIAL_COMMUNICATION_UART_CHANNEL);
      HAL_UART_ClearError(SERIAL_COMMUNICATION_UART_CHANNEL, OVRERR);
      portBASE_TYPE xHigherPriorityTaskWoken;
      xSemaphoreGiveFromISR(rx->s, &xHigherPriorityTaskWoken);
      xSemaphoreGiveFromISR(tx->s, &xHigherPriorityTaskWoken);
      /* reset state machines */
      g_rx_machine.state = rx_idle;
      /* Re-enable the reception control */
      HAL_UART_SetRxTx(SERIAL_COMMUNICATION_UART_CHANNEL, 1, 1);
    }
    /* All others need to be cleared at least (only) */
    if( (~OVRERR) & error )
    {
      HAL_UART_ClearError(SERIAL_COMMUNICATION_UART_CHANNEL, error);
    }
  }
  while( error );

  
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32UART1ERRStopTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32UART1ERRProcessingTime = u32UART1ERRStopTime - u32UART1ERRStartTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */  
}

static void rx_state_machine(void)
{
  uint8_t b = HAL_UART_GetReceivedData(SERIAL_COMMUNICATION_UART_CHANNEL);

  struct rx_machine *rx = &g_rx_machine;
  int i;
  
/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32UART1RXPrevTime = u32UART1RXStartTime;
    /* Get the new start time */
    u32UART1RXStartTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32UART1RXLoopOccuranceRate = u32UART1RXStartTime - u32UART1RXPrevTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */

  HAL_UART_ClearPendingIRQ(SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_RX);
  rprintf("rx %02x\n", b);

  switch(rx->state)
  {
  case rx_idle:
    if (b == bot)
    {
      rx->state = rx_cmd;
      break;
    }
    rprintf("rx bot\n");
    break;
  case rx_cmd:
    rx->cmd = b;
    for (i = 0; i < NUM_RPC_FUNCTIONS; i++)
    {
      if (rx->cmd == functions[i].id)
      {
        rx->f = &functions[i];
        break;
      }
    }
    if (i == NUM_RPC_FUNCTIONS)
    {
      rx->state = rx_idle;
      rprintf("rx cmd not found\n");
      break;
    }
    rx->state = rx_rcv;
    rx->cnt = 0;
    rx->buf = &g_q[0];
    break;
  case rx_rcv:
    rx->buf[rx->cnt] = b;
    rx->cnt++;
    if (rx->cnt < rx->f->size_q)
      break;
    rx->state = rx_checksum;
    break;
  case rx_checksum:
    rx->checksum = b;
    rx->state = rx_eot;
    break;
  case rx_eot:
    rx->state = rx_idle;
    if (b == eot)
    {
      portBASE_TYPE xHigherPriorityTaskWoken;
      HAL_UART_DisableIRQ(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_RX);
      xSemaphoreGiveFromISR(rx->s, &xHigherPriorityTaskWoken);
      break;
    }
    rprintf("rx eot\n");
    break;
  default:
    /* something bad has happenend */
    assert_param(0);
    break;
  }
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32UART1RXStopTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32UART1RXProcessingTime = u32UART1RXStopTime - u32UART1RXStartTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */
}

/*! \brief  Initialize the UART
  *
  * @param  None
  * @retval None
*/
static void uart_init (void)
{
  HAL_UART_Init(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_RXTX,SPEED_115200,8,NO_PARITY,1,NO_FLOW_CTRL);
  HAL_UART_RegisterIRQCallback (SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_TX , tx_state_machine );
  HAL_UART_RegisterIRQCallback (SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_RX , rx_state_machine );
  HAL_UART_RegisterIRQCallback (SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_ERR , err_state_machine );
}

/*! \brief  Initialize the Protocol
  *
  * @param  None
  * @retval Success
*/
static int protocol_init (void)
{
  struct rx_machine *rx = &g_rx_machine;
  struct tx_machine *tx = &g_tx_machine;

  vSemaphoreCreateBinary(rx->s);                                                /* create semaphore for receive */
  vSemaphoreCreateBinary(tx->s);                                                /* create semaphore for transmit */
  xSemaphoreTake(rx->s, 0);                                                     /* take semaphore for receive  directly that task has to wait */
  xSemaphoreTake(tx->s, 0);                                                     /* take semaphore for transmit directly that task has to wait */

  uart_init();                                                                  /* initialize the uart */

  return 0;
}

/*! \brief  Handle Question / Answer
  *
  * Handle one Question or Answer from Communication Protocol
  *
  * @param  None
  * @retval Success
*/

static int protocol_handle_one_q_a (void)
{
  struct rx_machine *rx = &g_rx_machine;

  const struct rpc *f;
  int ret;
  uint8_t checksum;

  /* enable to be sure */
  HAL_UART_SetIRQPriority(SERIAL_COMMUNICATION_UART_CHANNEL, HAL_UART_RX, INTERRUPT_PRIORITY_PROTOCOL);
  HAL_UART_EnableIRQ(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_RX);
  HAL_UART_EnableIRQ(SERIAL_COMMUNICATION_UART_CHANNEL,HAL_UART_ERR);

  ret = xSemaphoreTake(rx->s, 1000 / portTICK_RATE_MS);
  if(ret == pdFALSE) {
    rprintf("timeout @ xSemaphoreTake() @ RX\n");
    return -1;
  }

  checksum = crc_update(rx->cmd, rx->buf, rx->cnt);

  if (checksum != rx->checksum) {
    dprintf("rx cs mismatch\n");
    send_error(ret);
    return -1;
  }

  f = rx->f;
  ret = f->f(rx->buf, &g_a[0]);

  if(ret)
  {
    dprintf("function %d failed @ error %d\n", rx->cmd, ret);
    send_error(ret);
    return -1;
  }

#ifdef TRANSLOG
  if ((rx->cmd != 0x2) && (rx->cmd != 0x09))
  {
    int count;
    printf("Request: BOT 0x%02x ", rx->cmd);
    for (count = 0;count < rx->cnt; count ++)
      printf("0x%02x ",*rx->buf++);
    printf("0x%02x EOT\n",rx->checksum);
  }
#endif

  /* send the regular answer */
  ret = send_generic(rx->cmd, &g_a[0], f->size_a);
  if(ret)
  {
    dprintf("send_generic() failed @ error %d\n", ret);
    return -1;
  }

  return 0;
}

/*! \brief  Protocol Task
  *
  * @param  pvParameters: None
  * @retval None
*/
void ProtocolTask( void *pvParameters )
{
  while (InitDone==INIT_NOT_DONE)
    vTaskDelay( 100 / portTICK_RATE_MS );

  protocol_init();

  for( ;; )
  {
    protocol_handle_one_q_a();                                                  /* Handle one question or answer */
#ifdef USE_LED
    LED_Toggle(LED_SIGNAL_SERIAL_COMMUNICATION_RUNNING);                        /* Tollgle LED for signal working to outside */
#endif
#ifdef USE_RGB_LED
    RGB_LED_ToggleValue(LED_RGB_GREEN);
#endif /* USE_RGB_LED */
  }
}

#endif
