/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifdef BOARD_CLICKER4_M4KN

#include "config.h"
#include BOARD_CONFIG_HEADER_FILE
#include "hal_gpio.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "hal_amp_cmp.h"
#include "hal_spi.h"
#include "spi_device.h"

#include "max14890e.h"
#include "drv8323.h"
#include "led.h"

#include "hal_pmd.h"
#include TMPM_HEADER_FILE
#if defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__ || defined __TMPM_4KN__
#include TMPM_ADC_HEADER_FILE
#endif
#include "hal_adc.h"
#include "hal_cg.h"
#include "motor_control.h"
#include "string.h"

/* Board revision in the format Major.0, e.g. 0x10 would mean Revision 16.0 */
static uint8_t u8SBK_Board_Revision = 0x01;


#ifdef USE_SLIDER_2_CLICKER  
int32_t Slider_GetValue(void)
{
  ADC_Result result;
  static int32_t SliderPosition = 0;
  
  /* Get the result of the prvious single conversion */
  result = ADC_GetConvertResult(SLIDER_ADC0, SLIDER_REG0);
  SliderPosition = (result.Bit.ADResult >> 4);
  
  /* Start another conversion */
  ADC_Start(SLIDER_ADC0, ADC_TRG_SINGLE);
  
  return SliderPosition;
}

void Slider_Configure(void)
{
  /* RSL = LOW to have 4.096V */
  HAL_CG_EnableMiddleSpeedClockA(RSL_SLIDER_PORT_CLK);
  HAL_GPIO_Init(RSL_SLIDER_PORT, RSL_SLIDER_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(RSL_SLIDER_PORT, RSL_SLIDER_PORT_BIT, SLIDER_2_CLICKER_RSL_4096);
  
  /* Enable the MCP1501 #SHDN = HIGH */
  HAL_CG_EnableMiddleSpeedClockA(MCP_SLIDER_PORT_CLK);
  HAL_GPIO_Init(MCP_SLIDER_PORT, MCP_SLIDER_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(MCP_SLIDER_PORT, MCP_SLIDER_PORT_BIT, SLDIER_2_CLICKER_MCP1501_REF_DISABLED);
  
  /* PORT J4 - ADC input AIN04C */
  HAL_CG_EnableMiddleSpeedClockA(VO_SLIDER_PORT_CLK);
  HAL_GPIO_Init(VO_SLIDER_PORT, VO_SLIDER_PORT_BIT, HAL_GPIO_INPUT);
  
  /* ADC Clock - Unit C */
  HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_ADC_UC);
  HAL_CG_SetAdcClkSupply(SLIDER_ADC0, ENABLE);
  ADC_SetClk(SLIDER_ADC0, ADC_SAMPLING_PERIOD_2, ADC_SCLK_2); 
  
  /* Software triggered conversion without interrupt */
  ADC_SetSWTrg(SLIDER_ADC0, SLIDER_REG0, TRG_DISABLE(AIN_SLIDER));
   /* Enable ADC Unit */
  ADC_Enable(SLIDER_ADC0, ADC_CLKMOD_160M, ADC_REFSEL_AIN, ADC_RCUT_NORMAL);
  
  /* Start single conversion */
  ADC_Start(SLIDER_ADC0, ADC_TRG_SINGLE);
}
#endif /* USE_SLIDER_2_CLICKER */ 


#ifdef USE_LED

/*! \brief LED SelfTest
  *
  * Test the LED operation on the board
  *
  * @param  none
  *
  * @retval none
*/
void LED_SelfTest(void)
{
  LED_SetState(LED_NO_1, LED_ON);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_SetState(LED_NO_1, LED_OFF);
  
  LED_SetState(LED_NO_2, LED_ON);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_SetState(LED_NO_2, LED_OFF);
  
  LED_SetState(LED_NO_3, LED_ON);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_SetState(LED_NO_3, LED_OFF);
  
  LED_SetState(LED_NO_4, LED_ON);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_SetState(LED_NO_4, LED_OFF);
  
  LED_SetState(LED_NO_5, LED_ON);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_SetState(LED_NO_5, LED_OFF);
  
  LED_SetState(LED_NO_6, LED_ON);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_SetState(LED_NO_6, LED_OFF);
  
  vTaskDelay( 300 / portTICK_RATE_MS );
  
  LED_Toggle(LED_NO_1);
  LED_Toggle(LED_NO_2);
  LED_Toggle(LED_NO_3);
  LED_Toggle(LED_NO_4);
  LED_Toggle(LED_NO_5);
  LED_Toggle(LED_NO_6);
  vTaskDelay( 300 / portTICK_RATE_MS );
  LED_Toggle(LED_NO_1);
  LED_Toggle(LED_NO_2);
  LED_Toggle(LED_NO_3);
  LED_Toggle(LED_NO_4);
  LED_Toggle(LED_NO_5);
  LED_Toggle(LED_NO_6);
}
#endif /* USE_LED */

/*! \brief SPI Init
  *
  * Initalize the SPI bus
  *
  * @param  spi_speed: only for legacy compilation
  *
  * @retval none
*/
void SPI_DeviceInit(uint32_t speed)
{
  /* configure GPIO ports for SPI interface */

  /* configure PC2 used as SPI0 CS0 */
  HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_2, HAL_GPIO_SPI_OUTPUT);
  /* configure PC3 used as SPI0 MISO */
  HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_3, HAL_GPIO_SPI_INPUT);
  /* configure PC4 used as SPI0 MOSI */
  HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_4, HAL_GPIO_SPI_OUTPUT);
  /* configure PC5 used as SPI0 CLOCK */
  HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_5, HAL_GPIO_SPI_OUTPUT);  
  /* configure PC6 used as SPI0 CS1 */
  HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_6, HAL_GPIO_SPI_OUTPUT);

}


/**
  * @brief  function to write one register into MCP23S08
  * @param  Register address of the register to be written
  *         see MCP23S08.h all defines with ..._REGISTER
  * @param  Data for the register
  * @retval None
  */
uint8_t DRV8323_Configure (void)
{
    DRV8323_AddrData data, dataRcvd;
    DRV8323_RETURN retResult;

    // Write Drive control register
    // Set 6xPWM mode. 1PWM_COM and 1PWM_DIR and BRAKE not used.
    /* drv.write_DCR(DIS_CPUV_EN, DIS_GDF_EN, OTW_REP_DIS, PWM_MODE_6X, PWM_1X_COM_SYNC, PWM_1X_DIR_0, COAST_NRM, 0x0, 0x0); */
    memset (&data, 0, sizeof(DRV8323_AddrData));
    data.drvCtrl.dis_cpuv = DIS_CPUV_ENA;
    data.drvCtrl.dis_gdf =  DIS_GDF_ENA;
    data.drvCtrl.otw_rep =  OTW_REP_NO_REPORT;
    data.drvCtrl.pwm_mode = PWM_MODE_6;
    data.drvCtrl.pwm_com1 = PWM_COM1_SYNCHRON;
    data.drvCtrl.pwm_dir1 = PWM_DIR1_DISABLE;
    data.drvCtrl.coast    = COAST_NO_HI_Z_STATE;
    data.drvCtrl.brake    = BRAKE_NO_1PWM_MODE;
    data.drvCtrl.clr_flt  = CLR_FLT_OFF;

    retResult = DRV8323_WriteDrvCtrlRegister (data.drvCtrl);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    retResult = DRV8323_ReadDrvCtrlRegister (&dataRcvd.drvCtrl);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    if (0 != memcmp (&data.drvCtrl, &dataRcvd.drvCtrl, sizeof(dataRcvd.drvCtrl)))
    {
      return DRV8323_INIT_ERROR;
    }

    /* drv.write_HSR( LOCK_OFF, IDRIVEP_HS_330MA, IDRIVEN_HS_520MA); */
    memset (&data, 0, sizeof(DRV8323_AddrData));
    data.gateDrvHs.idrivep_hs = IDRIVEP_330;
    data.gateDrvHs.idriven_hs = IDRIVEN_520;
    data.gateDrvHs.lock = LOCK_SETTINGS_DIS;

    retResult = DRV8323_WriteGateDrvHsRegister (data.gateDrvHs);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    retResult = DRV8323_ReadGateDrvHsRegister (&dataRcvd.gateDrvHs);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    if (0 != memcmp (&data.gateDrvHs, &dataRcvd.gateDrvHs, sizeof(dataRcvd.gateDrvHs)))
    {
      return DRV8323_INIT_ERROR;
    }

    /* drv.write_LSR( 0x0, TDRIVE_4000NS, IDRIVEP_LS_330MA, IDRIVEN_LS_520MA); */
    memset (&data, 0, sizeof(DRV8323_AddrData));
    data.gateDrvLs.cbc = CBC_FAULT_NO;
    data.gateDrvLs.tdrive = TDRIVE_4000;
    data.gateDrvLs.idriven_ls =IDRIVEN_520;
    data.gateDrvLs.idrivep_ls = IDRIVEP_330;

    retResult = DRV8323_WriteGateDrvLsRegister (data.gateDrvLs);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    retResult = DRV8323_ReadGateDrvLsRegister (&dataRcvd.gateDrvLs);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    if (0 != memcmp (&data.gateDrvLs, &dataRcvd.gateDrvLs, sizeof(dataRcvd.gateDrvLs)))
    {
      return DRV8323_INIT_ERROR;
    }

    /* drv.write_OCPCR(TRETRY_4MS, DEADTIME_400NS, OCP_RETRY, OCP_DEG_4US, VDS_LVL_1_5); */
    memset (&data, 0, sizeof(DRV8323_AddrData));
    data.ocpCtrl.tretry = TRETRY_4;
    data.ocpCtrl.dead_time = DEAD_TIME_400;
    data.ocpCtrl.ocp_mode = OCP_MODE_AUTO_RETRY_FAULT;
    data.ocpCtrl.ocp_deg = OCP_DEG_4;
    data.ocpCtrl.vds_lvl = VDS_LVL_1_5;

   retResult = DRV8323_WriteOcpRegister (data.ocpCtrl);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    retResult = DRV8323_ReadOcpCtrlRegister (&dataRcvd.ocpCtrl);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    if (0 != memcmp (&data.ocpCtrl, &dataRcvd.ocpCtrl, sizeof(dataRcvd.ocpCtrl)))
    {
      return DRV8323_INIT_ERROR;
    }

    /* drv.write_CSACR(CSA_FET_SP, VREF_DIV_2, LS_REF_SH_SP, CSA_GAIN_20, DIS_SEN_EN, CSA_CAL_A_NRM, CSA_CAL_B_NRM, CSA_CAL_C_NRM, SEN_LVL_1_0); */
    memset (&data, 0, sizeof(DRV8323_AddrData));
    data.csaCtrl.csa_fet = CSA_FET_SP;
    data.csaCtrl.vref_div = VREF_DIV_VREF_2;
    data.csaCtrl.ls_ref = LS_REF_SH_SP;
    data.csaCtrl.csa_gain = CSA_GAIN_20;
    data.csaCtrl.dis_sen = DIS_SEN_OVERC_EN;
    data.csaCtrl.csa_cal_a = CSA_CAL_AMP_NORMAL;
    data.csaCtrl.csa_cal_b = CSA_CAL_AMP_NORMAL;
    data.csaCtrl.csa_cal_c = CSA_CAL_AMP_NORMAL;
    data.csaCtrl.sen_lvl = SEN_LVL_1_0;

    retResult = DRV8323_WriteCsaCtrlRegister (data.csaCtrl);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    retResult = DRV8323_ReadCsaCtrlCtrlRegister (&dataRcvd.csaCtrl);

    if (DRV8323_SUCCESS != retResult)
    {
      return retResult;
    }

    if (0 != memcmp (&data.csaCtrl, &dataRcvd.csaCtrl, sizeof(dataRcvd.csaCtrl)))
    {
      return DRV8323_INIT_ERROR;
    }
    return DRV8323_SUCCESS;
}

uint8_t GATE_DRIVER_Init (MotorParameters *motorParameters)
{
  /* gate driver configuration is not yet part of the motor parameter value, but is prepared for future use */
  if (BOARD_AVAILABLE_CHANNELS & MOTORCONTROL_CHANNEL_0)
  {
    if (DRV8323_Configure () != 0)
    {
      return 1;
    }
  }

  return 0;
}


static uint8_t DiffSingleEndedReceiverInit (uint8_t motorId, uint8_t encoderReceiver, uint8_t encoderMode )
{  
  if (ENCODER_RS_422 == encoderMode)
  {
      if (0 != MAX14890E_Configure (MAX14890E_RS_422))
      {
        return 1;
      }
  }
  
  if (ENCODER_D_HTL == encoderMode)
  {
      if(0 != MAX14890E_Configure (MAX14890E_D_HTL))
      {
        return 1;
      }
  }

  if (ENCODER_TTL == encoderMode)
  {
      if (0 != MAX14890E_Configure (MAX14890E_TTL))
      {
        return 1;
      }
  }
  
  if (ENCODER_SE_HTL == encoderMode)
  {
     if (0 != MAX14890E_Configure (MAX14890E_SE_HTL))
     {
        return 1;
     }
  }
  
  return 0;
}

uint8_t ENCODER_RECEIVER_Init (MotorParameters *motorParameters)
{

  uint8_t loop, index, returnValue;

  index = 0;

  /* Only channel 0 available with Clicker 4 board */
  for (loop =1; loop < MOTORCONTROL_CHANNEL_1; loop <<= 1)
  {
     if (BOARD_AVAILABLE_CHANNELS & loop )
     {
        switch (motorParameters[index].EncoderReceiver)
        {
            case ENCODER_RESOLVER_RECEIVER:
              /* not yet implemented */
              /*  ResolverReceiverInit(); */
              break;
            case ENCODER_DIFFERENTIAL_RECEIVER:
            case ENCODER_SINGLE_ENDED_RECEIVER:
              returnValue = DiffSingleEndedReceiverInit (index, motorParameters->EncoderReceiver, motorParameters->EncoderReceiverMode);
              if (returnValue != 0)
              {
                return returnValue;
              }
              break;
            default:  /* no encoder receiver */
              break;
        }
      }
      index++;
  }

  return 0;
}

uint8_t BOARD_Detect_Revision(void)
{
  return u8SBK_Board_Revision;
}

#endif   /* BOARD_CLICKER4_M4KN */

