/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdint.h>
#include "config.h"
#include "hal_adc.h"
#include "hal_ofd.h"
#include "hal_wdt.h"
#include "motor_control.h"
#include BOARD_CONFIG_HEADER_FILE
#include "gain.h"
#include "led.h"

#if defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__
#include TMPM_CG_HEADER_FILE
#endif

#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
#ifdef USE_HV_COMMUNICATION
#define HV_COMM 1
#endif /* USE_HV_COMMUNICATION */
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_0 */

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
#ifdef USE_HV_COMMUNICATION
#define HV_COMM 1
#endif /* USE_HV_COMMUNICATION */
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_1 */

#ifdef BOARD_PWR_HEADER_FILE_2
#include BOARD_PWR_HEADER_FILE_2
#ifdef USE_HV_COMMUNICATION
#define HV_COMM 1
#endif /* USE_HV_COMMUNICATION */
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_2 */


#include "hv_serial_communication.h"
#include "pga.h"
#include "board.h"
#include "eeprom.h"
#include "hal_pmd.h"
#include "motor_control.h"
#include "i2c_master_bitbanging.h"
#include "external_speed_control.h"
#include "hv_serial_communication.h"
#include "temperature_control.h"
#include "rgb_led.h"
#include "hal_spi.h"
#include "keypad.h"
#include "rgb_led.h"
#include "lcd_LCM_S02004DSx.h"
#include "units_enums.h"


__weak void    KEYPAD_Init(void){}
__weak void    LCD_Init(void){}
__weak void    HV_SerialCommunicationInit(void){}
__weak void    I2C_Init(void){}
__weak void    RGB_LED_Init(void){}
__weak uint8_t BOARD_Detect_Revision(){return 0;}
__weak void    SPI_DeviceInit(uint32_t);
__weak void    LED_Init (void);
__weak void    LED_SelfTest(void);
__weak void    BOARD_SetupGain(void){};
__weak uint8_t ENCODER_RECEIVER_Init (MotorParameters *motorParameters);
__weak uint8_t GATE_DRIVER_Init (MotorParameters *motorParameters);

uint8_t   external_clock_working = 1;                                           /* flag for external clock working */
uint8_t   BoardRevision          = 0;                                           /* Board revision as global variable */
uint8_t   BoardHasLED            = 1;                                           /* Board revision as global variable */
volatile uint8_t   InitDone      = INIT_NOT_DONE;                               /* Sync flag */
uint32_t  osc_frequency          = 0;                                           /* global Oscillator frequency for calculations */
FWVersion FirmwareVersion;

#ifdef USE_WDT
/* Watchdog default config */
static const halWdtInitTypeDef configWDT =
{
 HAL_WDT_DETECT_TIME_EXP_25,
 HAL_WDT_WDOUT,
};
#endif

/*! \brief  Setup the Watchdog
  *
  * @param  None
  * @retval None
*/
void BOARD_SetupWDT(void)
{
#ifndef __TXZ__
#ifdef USE_WDT
  HAL_WDT_Init(&configWDT);
  HAL_WDT_WriteClear();
  HAL_WDT_Enable();
#else
  HAL_WDT_Disable();
#endif /* USE_WDT */
#endif /* __TXZ__ */
}

/*! \brief  Setup the Oscillator Frequency Detector
  *
  * @param  None
  * @retval None
*/
void BOARD_SetupOFD(void)
{
#define REFERENCE_CLOCK_MAX     10500000
#define REFERENCE_CLOCK_MIN      8500000

#ifdef USE_OFD
  uint32_t max_value_pll =  ((uint64_t)(osc_frequency * PLL_MULTIPLIER * 11 / 10) * (2<<6)) / ( 4 * REFERENCE_CLOCK_MIN);
  uint32_t min_value_pll =  ((uint64_t)(osc_frequency * PLL_MULTIPLIER *  9 / 10) * (2<<6)) / ( 4 * REFERENCE_CLOCK_MAX);
  uint32_t max_value     =  ((uint64_t)(osc_frequency                  * 11 / 10) * (2<<6)) / ( 4 * REFERENCE_CLOCK_MIN);
  uint32_t min_value     =  ((uint64_t)(osc_frequency                  *  9 / 10) * (2<<6)) / ( 4 * REFERENCE_CLOCK_MAX);

  HAL_OFD_SetRegWriteMode(HAL_OFD_ENABLE);
  HAL_OFD_SetDetectionFrequency(HAL_OFD_PLL_ON , max_value_pll, min_value_pll);
  HAL_OFD_SetDetectionFrequency(HAL_OFD_PLL_OFF, max_value,     min_value);
  HAL_OFD_Enable();
  HAL_OFD_SetRegWriteMode(HAL_OFD_DISABLE);
#endif /* USE_OFD */
}

/*! \brief  Setup the CPU Clocks
  *
  * @param  None
  * @retval None
*/

void BOARD_SetupClocks(void)
{
#if defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__
  CG_SetPhiT0Src(BOARD_PERIPHERIAL_CLOCK_SOURCE);
  if (CG_SetPhiT0Level(BOARD_PERIPHERIAL_CLOCK_DIVIDER) != SUCCESS)
    for(;;);
  CG_SetFgearLevel(BOARD_CLOCK_GEAR_DIVIDER);

#ifdef BOARD_USE_EXTERNAL_OSCILLATOR
#if (!defined __TMPM_370__ && !defined __TMPM_475__)
  CG_SetPortM(CG_PORTM_AS_HOSC);
  if (CG_SetFosc(CG_FOSC_OSC1, ENABLE) != SUCCESS)
    external_clock_working = 0;

  if (external_clock_working == 1)
  {
    osc_frequency = BOARD_EXTERNAL_OSCILLATOR_FREQUENCY;

    CG_SetFoscSrc(CG_FOSC_OSC1);
    if (CG_SetFosc(CG_FOSC_OSC2, DISABLE) != SUCCESS)
      for(;;);
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC1, 0xfff);                               /* Maximum time */
  }
  else
    osc_frequency = INTERNAL_OSCILLATOR_FREQUENCY;
#else
    osc_frequency = BOARD_EXTERNAL_OSCILLATOR_FREQUENCY;
#endif /* !defined __TMPM_370__ && !defined __TMPM_475__ */

#ifdef BOARD_USE_PLL
  if (CG_SetPLL(ENABLE) != SUCCESS)
    for(;;);
  CG_StartWarmUp();
  while (CG_GetWarmUpState() != DONE );
  CG_SetFcSrc(CG_FC_SRC_FPLL);
#endif /* BOARD_USE_PLL */

#endif /* BOARD_USE_EXTERNAL_OSCILLATOR */
  
#else
  osc_frequency = BOARD_EXTERNAL_OSCILLATOR_FREQUENCY;
#endif /* defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__ */
}

/*! \brief  Setup the Board Hardware
  *
  * @param  None
  * @retval None
*/
void BOARD_SetupHW(void)
{
#ifdef USE_LED
  uint8_t i;
#endif /* USE_LED */

  FirmwareVersion.fw_version[0]=FW_VERSION_MAJOR;
  FirmwareVersion.fw_version[1]=FW_VERSION_MINOR;

  SPI_DeviceInit(10);                                                   /* Init SPI Channel */
  BoardRevision = BOARD_Detect_Revision();
  BoardHasLED = 1;

#ifdef BOARD_PWR_HEADER_FILE_0                                                  /* Set up Board dependant values */
#include BOARD_PWR_HEADER_FILE_0
  ChannelValues[0].Deadtime                        = BOARD_DEAD_TIME;
  ChannelValues[0].BootstrapDelay                  = BOARD_BOOTSTRAP_DELAY;
  ChannelValues[0].GainCurrentMeasure              = BOARD_GAIN_CURRENT_MEASURE;
  ChannelValues[0].MeasurementType                 = BOARD_MEASUREMENT_TYPE;
  ChannelValues[0].SensitivityVoltageMeasure.value = BOARD_SENSITIVITY_VOLTAGE_MEASURE_VALUE;
  ChannelValues[0].SensitivityVoltageMeasure.unit  = BOARD_SENSITIVITY_VOLTAGE_MEASURE_UNIT;
  ChannelValues[0].SensitivityCurrentMeasure.value = BOARD_SENSITIVITY_CURRENT_MEASURE_VALUE;
  ChannelValues[0].SensitivityCurrentMeasure.unit  = BOARD_SENSITIVITY_CURRENT_MEASURE_UNIT;

#ifdef BOARD_CURRENT_MEASURE_DIRECTION
  ChannelValues[0].MeasurementDirection             = BOARD_CURRENT_MEASURE_DIRECTION;
#endif /* BOARD_CURRENT_MEASURE_DIRECTION */
  ChannelValues[0].PolL                             = BOARD_POLL;
  ChannelValues[0].PolH                             = BOARD_POLH;
#include "pwr_undefine.h"

#endif /* BOARD_PWR_HEADER_FILE_0 */

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
  ChannelValues[1].Deadtime                         = BOARD_DEAD_TIME;
  ChannelValues[1].BootstrapDelay                   = BOARD_BOOTSTRAP_DELAY;
  ChannelValues[1].GainCurrentMeasure               = BOARD_GAIN_CURRENT_MEASURE;
  ChannelValues[1].MeasurementType                  = BOARD_MEASUREMENT_TYPE;
  ChannelValues[1].SensitivityVoltageMeasure.value  = BOARD_SENSITIVITY_VOLTAGE_MEASURE_VALUE;
  ChannelValues[1].SensitivityVoltageMeasure.unit   = BOARD_SENSITIVITY_VOLTAGE_MEASURE_UNIT;
  ChannelValues[1].SensitivityCurrentMeasure.value  = BOARD_SENSITIVITY_CURRENT_MEASURE_VALUE;
  ChannelValues[1].SensitivityCurrentMeasure.unit   = BOARD_SENSITIVITY_CURRENT_MEASURE_UNIT;

#ifdef BOARD_CURRENT_MEASURE_DIRECTION
  ChannelValues[1].MeasurementDirection             = BOARD_CURRENT_MEASURE_DIRECTION;
#endif /* BOARD_CURRENT_MEASURE_DIRECTION */
  ChannelValues[1].PolL                             = BOARD_POLL;
  ChannelValues[1].PolH                             = BOARD_POLH;
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_1 */

#ifdef BOARD_PWR_HEADER_FILE_2
#include BOARD_PWR_HEADER_FILE_2
  ChannelValues[2].Deadtime                         = BOARD_DEAD_TIME;
  ChannelValues[2].BootstrapDelay                   = BOARD_BOOTSTRAP_DELAY;
  ChannelValues[2].GainCurrentMeasure               = BOARD_GAIN_CURRENT_MEASURE;
  ChannelValues[2].MeasurementType                  = BOARD_MEASUREMENT_TYPE;
  ChannelValues[2].SensitivityVoltageMeasure.value  = BOARD_SENSITIVITY_VOLTAGE_MEASURE_VALUE;
  ChannelValues[2].SensitivityVoltageMeasure.unit   = BOARD_SENSITIVITY_VOLTAGE_MEASURE_UNIT;
  ChannelValues[2].SensitivityCurrentMeasure.value  = BOARD_SENSITIVITY_CURRENT_MEASURE_VALUE;
  ChannelValues[2].SensitivityCurrentMeasure.unit   = BOARD_SENSITIVITY_CURRENT_MEASURE_UNIT;

#ifdef BOARD_CURRENT_MEASURE_DIRECTION
  ChannelValues[2].MeasurementDirection             = BOARD_CURRENT_MEASURE_DIRECTION;
#endif /* BOARD_CURRENT_MEASURE_DIRECTION */
  ChannelValues[2].PolL                             = BOARD_POLL;
  ChannelValues[2].PolH                             = BOARD_POLH;
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_2 */

  BOARD_SetupGain();                                                            /* Set up the gain for current measure */
#ifdef USE_LED
  LED_Init();                                                                   /* Init LED Ports */
#endif /* USE_LED */  
  KEYPAD_Init();                                                                /* Init Keypad Port */
  LCD_Init();                                                                   /* Init LCD */
  HV_SerialCommunicationInit();
  I2C_Init();
  RGB_LED_Init();

  if (external_clock_working == 0)
  {
#ifdef USE_LED
    for (i=0;i<4;i++)
      LED_SetState(i,1);
#endif /* USE_LED */
#ifdef USE_RGB_LED
    RGB_LED_SetValue(LED_RGB_YELLOW);
#endif /* USE_RGB_LED */
  }

}

static void OverVoltageConfig(uint8_t channel)
{
#ifdef HV_COMM
#ifdef USE_SW_OVER_UNDER_VOLTAGE_DETECTION
  HV_Communication_OverUndervoltageDetect(channel);
#endif /* USE_SW_OVER_UNDER_VOLTAGE_DETECTION */
#else
  HAL_ADC_OverUndervoltageDetect(channel);
#endif /* HV_COMM */
}

void BOARD_SetupHW2(void)
{
  uint8_t channel;
  
  for(channel = 0; channel < MAX_CHANNEL; channel++ )
  {
    if( CHANNEL_IS_USED == ChannelUsageConfiguration[channel] )
    {
      /* Configure the ADC & PMD for the channel */
      HAL_ADC_Init(channel, (CURRENT_MEASUREMENT)ChannelValues[channel].MeasurementType);            
      HAL_PMD_HWInit(channel);

#ifdef USE_TEMPERATURE_CONTROL
      /* Temperature control */
      TEMPERATURE_ConfigureADCforTemperature(channel);
#endif /* USE_TEMPERATURE_CONTROL */  
    
      switch( channel )
      {
        case 0:
#ifndef BOARD_VDC_CHANNEL_0_VALUE
          OverVoltageConfig(channel);
#endif /* !defined BOARD_VDC_CHANNEL_0 */          
          break;
        case 1:
#ifndef BOARD_VDC_CHANNEL_1_VALUE
          OverVoltageConfig(channel);
#endif /* !defined BOARD_VDC_CHANNEL_1 */            
          break;

#if defined __TMPM_4KN__          
        case 2:
#ifndef BOARD_VDC_CHANNEL_2_VALUE
          OverVoltageConfig(channel);
#endif /* !defined BOARD_VDC_CHANNEL_2 */            
          break;  
#endif /* defined __TMPM_4KN__ */        
        default:
          assert_param(0);
          break;
      }
    }
  }

  /* Initialise gate driver if applied */
  if( 0 != GATE_DRIVER_Init (MotorParameterValues) ) 
  {      
     printf("Gate driver initialization failed\n");
  }
  else
  {
     printf("Gate driver initialization OK\n");
  }
  
  /* Initialise encoder receiver device if applied */
  if( 0 != ENCODER_RECEIVER_Init (MotorParameterValues) )
  {
     printf("Encoder receiever initialization failed\n");
  }
  else
  {
     printf("Encoder receiever initialization OK\n");
  }
  
#ifdef USE_LED  
  LED_SelfTest();
#endif /* USE_LED */
 
/* Use the Slider 2 Clicker */  
#ifdef USE_SLIDER_2_CLICKER  
  Slider_Configure();
#endif /* USE_SLIDER_2_CLICKER */  

  InitDone=INIT_DONE;                                                           /* Allow other tasks to access the HW */

#ifdef USE_EXTERNAL_SPEED_CONTROL
  EXTERNAL_SPEED_CONTROL_Init();
#endif /* USE_EXTERNAL_SPEED_CONTROL */
}

#if (defined USE_WDT) && (!(defined USE_LOAD_STATISTICS))
/*! \brief  Idle handler
  *
  * Overloading function for FreeRTOS idle handler
  *
  * @param  None
  * @retval None
*/
void vApplicationIdleHook( void )
{
  WDT_WriteClearCode();
}
#endif /* (defined USE_WDT) && (!(defined USE_LOAD_STATISTICS)) */


#ifndef __TXZ__

#if defined __TMPM_370__  || defined __TMPM_376__
/* Interrupt program for 3 Shunt measure on Channel A */
const PMD_TrgProgINTTypeDef  TrgProgINT_3ShuntA =
{
  PMD_INTADPDA,
  PMD_INTADPDA,
  PMD_INTADPDA,
  PMD_INTADPDA,
  PMD_INTADPDA,
  PMD_INTADPDA,
};
#endif /* defined __TMPM_370__  || defined __TMPM_376__ */

/* Interrupt program for 3 Shunt measure on Channel B */
const PMD_TrgProgINTTypeDef  TrgProgINT_3ShuntB =
{
  PMD_INTADPDB,
  PMD_INTADPDB,
  PMD_INTADPDB,
  PMD_INTADPDB,
  PMD_INTADPDB,
  PMD_INTADPDB,
};

#if defined __TMPM_370__  || defined __TMPM_376__
/* Interrupt program for 1 Shunt measure on Channel A */
const PMD_TrgProgINTTypeDef  TrgProgINT_1ShuntA =
{
  PMD_INTNONE,
  PMD_INTADPDA,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
};
#endif /* defined __TMPM_370__  || defined __TMPM_376__ */

/* Interrupt program for 1 Shunt measure on Channel B */
const PMD_TrgProgINTTypeDef  TrgProgINT_1ShuntB =
{
  PMD_INTNONE,
  PMD_INTADPDB,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
};

#if defined __TMPM_370__  || defined __TMPM_376__
/* Interrupt program for 2 Sensor measure on Channel A */
const PMD_TrgProgINTTypeDef TrgProgINT_2SensorA =
{
  PMD_INTADPDA,
  PMD_INTADPDA,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
};
#endif /* defined __TMPM_370__  || defined __TMPM_376__ */

/* Interrupt program for 2 Sensor measure on Channel B */
const PMD_TrgProgINTTypeDef TrgProgINT_2SensorB =
{
  PMD_INTADPDB,
  PMD_INTADPDB,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
  PMD_INTNONE,
};

/* PMDTrigger 0 for 3 Shunt measure on Channel A */
const PMD_TrgTypeDef PMDTrigger0_3PhaseA =
{
  PMD_PROG0,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_VDC1
};

/* PMDTrigger 1 for 3 Shunt measure on Channel A */
const PMD_TrgTypeDef PMDTrigger1_3PhaseA =
{
  PMD_PROG1,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_ENABLE,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_VDC1
};

/* PMDTrigger 2 for 3 Shunt measure on Channel A */
const PMD_TrgTypeDef PMDTrigger2_3PhaseA =
{
  PMD_PROG2,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_ENABLE,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_VDC1
};

/* PMDTrigger 3 for 3 Shunt measure on Channel A */
const PMD_TrgTypeDef PMDTrigger3_3PhaseA =
{
  PMD_PROG3,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_ENABLE,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_VDC1
};

/* PMDTrigger 4 for 3 Shunt measure on Channel A */
const PMD_TrgTypeDef PMDTrigger4_3PhaseA =
{
  PMD_PROG4,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_ENABLE,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_VDC1
};

/* PMDTrigger 5 for 3 Shunt measure on Channel A */
const PMD_TrgTypeDef PMDTrigger5_3PhaseA =
{
  PMD_PROG5,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_VDC1
};

/* PMDTrigger 0 for 1 Shunt measure on Channel A */
#ifdef AIN_1PHASE_CURRENT
const PMD_TrgTypeDef PMDTrigger0_1PhaseA =
{
  PMD_PROG0,
  VE_PHASE_NONE,
  VE_PHASE_ENABLE,
  VE_PHASE_NONE,
  VE_PHASE_NONE,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT
};
#endif

/* PMDTrigger 1 for 1 Shunt measure on Channel A */
#ifdef AIN_1PHASE_CURRENT
const PMD_TrgTypeDef PMDTrigger1_1PhaseA =
{
  PMD_PROG1,
  VE_PHASE_ENABLE,
  VE_PHASE_NONE,
  VE_PHASE_NONE,
  VE_PHASE_ENABLE,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_VDC1
};
#endif

#if (defined AIN_2PHASE_W) && (defined AIN_2PHASE_V)
/* PMDTrigger 0 for 2 Sensor measure on Channel A */
const PMD_TrgTypeDef PMDTrigger0_2PhaseA =
{
  PMD_PROG0,
  VE_PHASE_W,
  VE_PHASE_V,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_2PHASE_W,
  AIN_2PHASE_V,
  AIN_2PHASE_V,
  AIN_VDC1
};

/* PMDTrigger 1 for 2 Sensor measure on Channel A */
const PMD_TrgTypeDef PMDTrigger1_2PhaseA =
{
  PMD_PROG1,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_2PHASE_V,
  AIN_2PHASE_W,
  AIN_2PHASE_V,
  AIN_VDC1
};
#endif /* BOARD_HITEX_M370 */

/* PMDTrigger 0 for 3 Shunt measure on Channel B */
const PMD_TrgTypeDef PMDTrigger0_3PhaseB =
{
  PMD_PROG0,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_VDC1
};

/* PMDTrigger 1 for 3 Shunt measure on Channel B */
const PMD_TrgTypeDef PMDTrigger1_3PhaseB =
{
  PMD_PROG1,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_ENABLE,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_VDC1
};

/* PMDTrigger 2 for 3 Shunt measure on Channel B */
const PMD_TrgTypeDef PMDTrigger2_3PhaseB =
{
  PMD_PROG2,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_ENABLE,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_VDC1
};

/* PMDTrigger 3 for 3 Shunt measure on Channel B */
const PMD_TrgTypeDef PMDTrigger3_3PhaseB =
{
  PMD_PROG3,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_ENABLE,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_VDC1
};

/* PMDTrigger 4 for 3 Shunt measure on Channel B */
const PMD_TrgTypeDef PMDTrigger4_3PhaseB =
{
  PMD_PROG4,
  VE_PHASE_U,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_ENABLE,
  AIN_3PHASE_U,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_VDC1
};

/* PMDTrigger 5 for 3 Shunt measure on Channel B */
const PMD_TrgTypeDef PMDTrigger5_3PhaseB =
{
  PMD_PROG5,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_3PHASE_V,
  AIN_3PHASE_W,
  AIN_3PHASE_U,
  AIN_VDC1
};

/* PMDTrigger 0 for 1 Shunt measure on Channel B */
#ifdef AIN_1PHASE_CURRENT
const PMD_TrgTypeDef PMDTrigger0_1PhaseB =
{
  PMD_PROG0,
  VE_PHASE_NONE,
  VE_PHASE_ENABLE,
  VE_PHASE_NONE,
  VE_PHASE_NONE,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT
};
#endif

/* PMDTrigger 1 for 1 Shunt measure on Channel B */
#ifdef AIN_1PHASE_CURRENT
const PMD_TrgTypeDef PMDTrigger1_1PhaseB =
{
  PMD_PROG1,
  VE_PHASE_ENABLE,
  VE_PHASE_NONE,
  VE_PHASE_NONE,
  VE_PHASE_ENABLE,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_1PHASE_CURRENT,
  AIN_VDC1
};
#endif

#if (defined AIN_2PHASE_W) && (defined AIN_2PHASE_V)
/* PMDTrigger 0 for 2 Sensor measure on Channel B */
const PMD_TrgTypeDef PMDTrigger0_2PhaseB =
{
  PMD_PROG0,
  VE_PHASE_W,
  VE_PHASE_V,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_2PHASE_W,
  AIN_2PHASE_V,
  AIN_2PHASE_V,
  AIN_VDC1
};

/* PMDTrigger 1 for 2 Sensor measure on Channel B */
const PMD_TrgTypeDef PMDTrigger1_2PhaseB =
{
  PMD_PROG1,
  VE_PHASE_V,
  VE_PHASE_W,
  VE_PHASE_U,
  VE_PHASE_ENABLE,
  AIN_2PHASE_V,
  AIN_2PHASE_W,
  AIN_2PHASE_V,
  AIN_VDC1
};
#endif

#endif
