/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _BOARD_CLICKER4_M4KN_H_
#define _BOARD_CLICKER4_M4KN_H_

/* Clock setup */
#define BOARD_USE_EXTERNAL_OSCILLATOR           1
#define BOARD_EXTERNAL_OSCILLATOR_FREQUENCY     10000000                        /* Oscillator Frequency in MHz */

#define BOARD_USE_PLL                           1                               /* Usage of PLL - 0:OFF 1:ON */


typedef enum
{
  SPI_DEVICE_IO_EXPANDER,
  SPI_DEVICE_DIFF_ENC_CH0,
  SPI_DEVICE_DIFF_ENC_CH1,
  SPI_DEVICE_DIFF_ENC_CH2,
  SPI_DEVICE_SE_ENC_CH0,
  SPI_DEVICE_SE_ENC_CH1,
  SPI_DEVICE_SE_ENC_CH2,
  SPI_DEVICE_RESOLVER_ENC_CH0,
  SPI_DEVICE_RESOLVER_ENC_CH1,
  SPI_DEVICE_RESOLVER_ENC_CH2,
  SPI_DEVICE_GATE_DRV_CH0,
  SPI_DEVICE_GATE_DRV_CH1,
  SPI_DEVICE_GATE_DRV_CH2,
} SPI_DeviceSelect;


static const unsigned int gaintable[]={10};
typedef enum
{
  GAIN_1,                                                                       /* Gain of +1 */
} PGA_GAIN;

/* Gain settings */
#define BOARD_GAIN_CURRENT_MEASURE              GAIN_1

/* Analog input port for current measurement with 3-shunt */
#define AIN0_3PHASE_U                           ADC_AIN18
#define AIN0_3PHASE_V                           ADC_AIN17
#define AIN0_3PHASE_W                           ADC_AIN16
#define AIN0_VDC                                ADC_AIN9

/* Analog input port for current measurement with 1-shunt */
#define AIN0_1PHASE                             ADC_AIN16

/* 
   Although the phase measurement registers of channels 
   1 & 2 are not used, these must be defined - use the 
   channel 0 definition to avoid any electrical issues
*/
#define AIN1_3PHASE_U                           ADC_AIN18
#define AIN1_3PHASE_V                           ADC_AIN17
#define AIN1_3PHASE_W                           ADC_AIN16
#define AIN1_VDC                                ADC_AIN9

#define AIN2_3PHASE_U                           ADC_AIN18
#define AIN2_3PHASE_V                           ADC_AIN17
#define AIN2_3PHASE_W                           ADC_AIN16
#define AIN2_VDC                                ADC_AIN9

/* Information for SW Over/Undervoltage Detection */
#define VDC_MEASURE0_REG                        ADC_REG5

/* 
   Although the VDC measurement registers of channels 1 & 2 
   are not used, these must be defined  - use the 
   channel 0 definition to avoid any electrical issues
*/
#define VDC_MEASURE1_REG                        ADC_REG5
#define VDC_MEASURE2_REG                        ADC_REG5

#define VDC_MEASURE_TIMES                       5

/* Analog input port for temperature measurement */
#define AIN0_TEMPERATURE                        ADC_AIN13
/* 
   Although the temperature measurement registers of channels 
   1 & 2 are not used, these must be defined  - use the 
   channel 0 definition to avoid any electrical issues
*/
#define AIN1_TEMPERATURE                        ADC_AIN13
#define AIN2_TEMPERATURE                        ADC_AIN13

/* Information for Temperature Control */
/* All temperature measurements on ADC Unit A */
#define TEMPERATURE_ADC0                        TSB_ADA
#define TEMPERATURE_REG0                        ADC_REG6

/* Specification of serial port for serial protocol task */
#define SERIAL_COMMUNICATION_UART_CHANNEL       0

/* Specification of serial port for HS-DSO task */
#define HSDSO_COMMUNICATION_UART_CHANNEL        1

/* SPI Channel used on the Board for Gate Driver */
#define BOARD_SPI_CHANNEL                       0

#define LED_NO_1_PORT                           HAL_GPIO_PORT_K
#define LED_NO_1_PORT_BIT                       HAL_GPIO_BIT_1
#define LED_NO_2_PORT                           HAL_GPIO_PORT_K
#define LED_NO_2_PORT_BIT                       HAL_GPIO_BIT_2
#define LED_NO_3_PORT                           HAL_GPIO_PORT_K
#define LED_NO_3_PORT_BIT                       HAL_GPIO_BIT_3
#define LED_NO_4_PORT                           HAL_GPIO_PORT_J
#define LED_NO_4_PORT_BIT                       HAL_GPIO_BIT_5
#define LED_NO_5_PORT                           HAL_GPIO_PORT_V
#define LED_NO_5_PORT_BIT                       HAL_GPIO_BIT_1
#define LED_NO_6_PORT                           HAL_GPIO_PORT_V
#define LED_NO_6_PORT_BIT                       HAL_GPIO_BIT_0

/* LED definitions in case of USE_LED is enabled. Please make sure that led.c or any other implementation is included in the build  */
typedef enum
{ 
  LED_NO_1 = 1,
  LED_NO_2 = 2,
  LED_NO_3 = 3,
  LED_NO_4 = 4,
  LED_NO_5 = 5,
  LED_NO_6 = 6
} LED_NO;

/* Signaling of different states - not used at first stage */
#define LED_SIGNAL_CONFIG_READ                  LED_NO_1                        /* Led Number for signaling config successful read from EEPROM */
#define LED_SIGNAL_SERIAL_COMMUNICATION_RUNNING LED_NO_2                        /* Led Number for signaling serial communication protocol active */
#define LED_SIGNAL_VE_RUN_BASE                  LED_NO_3                        /* Led base Number for VE is in FOC */


/* Information for Slider Control */
/* All mikroBUS slots are connected to ADC Unit C */
#define SLIDER_ADC0                             TSB_ADC
#define SLIDER_REG0                             ADC_REG0

/* Maximum voltage level selection */
#define SLIDER_2_CLICKER_RSL_2048               1
#define SLIDER_2_CLICKER_RSL_4096               0

/* Michrochip MCP1501 can be used to provide refernce voltage for ADC calibration */
#define SLDIER_2_CLICKER_MCP1501_REF_ENABLED    0
#define SLDIER_2_CLICKER_MCP1501_REF_DISABLED   1

/* mikroBUS definitions */
#define MIKROBUS_SLOT1                          1
#define MIKROBUS_SLOT2                          2
#define MIKROBUS_SLOT3                          3
#define MIKROBUS_SLOT4                          4

/* mikroBUS definitions */
typedef enum {
  CLCIKER4_MIKROBUS_SLOT1 = MIKROBUS_SLOT1,
  CLCIKER4_MIKROBUS_SLOT2 = MIKROBUS_SLOT2,
  CLCIKER4_MIKROBUS_SLOT3 = MIKROBUS_SLOT3,
  CLCIKER4_MIKROBUS_SLOT4 = MIKROBUS_SLOT4
} CLCIKER4_MIKROBUS_SLOT_NO;

#define SLIDER_2_CLICKER_SLOT                   MIKROBUS_SLOT4

/* mikorBus Slot Selection */
#if (SLIDER_2_CLICKER_SLOT == MIKROBUS_SLOT1)
/* mikorBus Slot #1 */
  /* RSL output - level selection */
  #define RSL_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_U
  #define RSL_SLIDER_PORT                       HAL_GPIO_PORT_U
  #define RSL_SLIDER_PORT_BIT                   HAL_GPIO_BIT_2
  /* MCP1501 output - reference voltage enable / normal operation */
  #define MCP_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_G
  #define MCP_SLIDER_PORT                       HAL_GPIO_PORT_G
  #define MCP_SLIDER_PORT_BIT                   HAL_GPIO_BIT_0
  /* VO input - potentiometer voltage */
  #define VO_SLIDER_PORT_CLK                    HAL_CG_CLK_PORT_J
  #define VO_SLIDER_PORT                        HAL_GPIO_PORT_J
  #define VO_SLIDER_PORT_BIT                    HAL_GPIO_BIT_1
  /* Used ADC input */ 
  #define AIN_SLIDER                            ADC_AIN1
#elif (SLIDER_2_CLICKER_SLOT == MIKROBUS_SLOT2)
/* mikorBus Slot #2 */
  /* RSL output - level selection */
  #define RSL_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_U
  #define RSL_SLIDER_PORT                       HAL_GPIO_PORT_U
  #define RSL_SLIDER_PORT_BIT                   HAL_GPIO_BIT_4
  /* MCP1501 output - reference voltage enable / normal operation */
  #define MCP_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_G
  #define MCP_SLIDER_PORT                       HAL_GPIO_PORT_G
  #define MCP_SLIDER_PORT_BIT                   HAL_GPIO_BIT_1
  /* VO input - potentiometer voltage */
  #define VO_SLIDER_PORT_CLK                    HAL_CG_CLK_PORT_J
  #define VO_SLIDER_PORT                        HAL_GPIO_PORT_J
  #define VO_SLIDER_PORT_BIT                    HAL_GPIO_BIT_2
  /* Used ADC input */ 
  #define AIN_SLIDER                            ADC_AIN2
#elif (SLIDER_2_CLICKER_SLOT == MIKROBUS_SLOT3)
  #define RSL_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_A
  #define RSL_SLIDER_PORT                       HAL_GPIO_PORT_A
  #define RSL_SLIDER_PORT_BIT                   HAL_GPIO_BIT_3
  /* MCP1501 output - reference voltage enable / normal operation */
  #define MCP_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_B
  #define MCP_SLIDER_PORT                       HAL_GPIO_PORT_B
  #define MCP_SLIDER_PORT_BIT                   HAL_GPIO_BIT_7
  /* VO input - potentiometer voltage */
  #define VO_SLIDER_PORT_CLK                    HAL_CG_CLK_PORT_J
  #define VO_SLIDER_PORT                        HAL_GPIO_PORT_J
  #define VO_SLIDER_PORT_BIT                    HAL_GPIO_BIT_3
  /* Used ADC input */ 
  #define AIN_SLIDER                            ADC_AIN3
#elif (SLIDER_2_CLICKER_SLOT == MIKROBUS_SLOT4)
  #define RSL_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_A
  #define RSL_SLIDER_PORT                       HAL_GPIO_PORT_A
  #define RSL_SLIDER_PORT_BIT                   HAL_GPIO_BIT_4 
  /* MCP1501 output - reference voltage enable / normal operation */
  #define MCP_SLIDER_PORT_CLK                   HAL_CG_CLK_PORT_C
  #define MCP_SLIDER_PORT                       HAL_GPIO_PORT_C
  #define MCP_SLIDER_PORT_BIT                   HAL_GPIO_BIT_7
  /* VO input - potentiometer voltage */
  #define VO_SLIDER_PORT_CLK                    HAL_CG_CLK_PORT_J
  #define VO_SLIDER_PORT                        HAL_GPIO_PORT_J
  #define VO_SLIDER_PORT_BIT                    HAL_GPIO_BIT_4
  /* Used ADC input */ 
  #define AIN_SLIDER                            ADC_AIN4
#endif /* MIKROBUS_SLOT4 */

#ifdef DEMO_CLICKER4_SLIDER2
  /* The input volatge ranges from 0 ro RSL (4.096 by default). The speed  */
  #define SLIDER_2_SPEED_CONVERSION_FACTOR        20
  /* Filter the fluctuation in the ADC measurement results */
  #define SLIDER_2_MIN_SPEED_CHANGE               2

  /* Stop region */
  #define SLIDER_2_MIN_STOP_VALUE                 0
  #define SLIDER_2_MAX_STOP_VALUE                 0

  /* Low speed jerking avoidance */
  #define SLIDER_2_MAX_LOW_SPEED_VALUE            10
  #define SLIDER_2_LOW_SPEED_COMPENSATION_RPM     200

  /* Maximal speed */
  #define SLIDER_2_SPEED_LIMIT_RPM                4000
#endif /* DEMO_CLICKER4_SLIDER2 */

extern uint8_t BOARD_Detect_Revision(void);
extern void LED_SelfTest(void);

#ifdef USE_SLIDER_2_CLICKER
  extern void Slider_Configure(void);
  extern int32_t Slider_GetValue(void);
#endif /* USE_SLIDER_2_CLICKER */  

#endif /* _BOARD_CLICKER4_M4KN_H_ */
