/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _BOARD_M4K4_H_
#define _BOARD_M4K4_H_

/* Clock setup */
#define BOARD_USE_EXTERNAL_OSCILLATOR           1
#define BOARD_EXTERNAL_OSCILLATOR_FREQUENCY     10000000                        /* Oscillator Frequency in MHz */

#define BOARD_USE_PLL                           1                               /* Usage of PLL - 0:OFF 1:ON */

#define SPI_USE_CS_PORT_EXTENSION               (1)                             /* Usage of chip selection extension logical circuit */

typedef enum
{
  SPI_DEVICE_IO_EXPANDER,
  SPI_DEVICE_DIFF_ENC_CH0,
  SPI_DEVICE_DIFF_ENC_CH1,
  SPI_DEVICE_DIFF_ENC_CH2,
  SPI_DEVICE_SE_ENC_CH0,
  SPI_DEVICE_SE_ENC_CH1,
  SPI_DEVICE_SE_ENC_CH2,
  SPI_DEVICE_RESOLVER_ENC_CH0,
  SPI_DEVICE_RESOLVER_ENC_CH1,
  SPI_DEVICE_RESOLVER_ENC_CH2,
  SPI_DEVICE_GATE_DRV_CH0,
  SPI_DEVICE_GATE_DRV_CH1,
  SPI_DEVICE_GATE_DRV_CH2,
} SPI_DeviceSelect;

typedef enum {
  SPI1_CS_00 =0,
  SPI1_CS_01,
  SPI1_CS_02,
  SPI1_CS_03,
  SPI1_CS_04,
  SPI1_CS_05,
  SPI1_CS_06,
  SPI1_CS_07,
  SPI1_CS_08,
  SPI1_CS_09,
  SPI1_CS_0A,
  SPI1_CS_0B,
  SPI1_CS_0C,
  SPI1_CS_0D,
  SPI1_CS_0E,
  SPI1_CS_0F,
}SPI_ChipSelect;


#define SPI_ENC0_CS_CH_MOTOR_0                  (SPI1_CS_00)                     /* chip select channel for encoder 0 and motor channel 0 */
#define SPI_ENC1_CS_CH_MOTOR_0                  (SPI1_CS_01)                     /* chip select channel for encoder 1 and motor 0 */
#define SPI_DRV_CS_CH_MOTOR_0                   (SPI1_CS_02)                     /* chip select channel for smart gate driver motor 0 */

#define SPI_ENC0_CS_CH_MOTOR_1                  (SPI1_CS_03)                     /* chip select channel for encoder 0 and motor channel 1 */
#define SPI_ENC1_CS_CH_MOTOR_1                  (SPI1_CS_04)                     /* chip select channel for encoder 1 and motor channel 1 */
#define SPI_DRV_CS_CH_MOTOR_1                   (SPI1_CS_05)                     /* chip select channel for smart gate driver motor channel 1 */

#define SPI_ENC0_CS_CH_MOTOR_2                  (SPI1_CS_06)                     /* chip select channel for encoder 0 and motor channel 2 */
#define SPI_ENC1_CS_CH_MOTOR_2                  (SPI1_CS_07)                     /* chip select channel for encoder 1 and motor channel 2 */
#define SPI_DRV_CS_CH_MOTOR_2                   (SPI1_CS_08)                     /* chip select channel for smart gate driver motor channel 2 */

static const unsigned int gaintable[]={10};
typedef enum
{
  GAIN_1,                                                                       /* Gain of +1 */
} PGA_GAIN;

/* Gain settings */
#define BOARD_GAIN_CURRENT_MEASURE              GAIN_1

/* Analog input port for current measurement with 3-shunt */
#define AIN0_3PHASE_U                           ADC_AIN18
#define AIN0_3PHASE_V                           ADC_AIN17
#define AIN0_3PHASE_W                           ADC_AIN16
#define AIN0_VDC                                ADC_AIN9

/* 
   Although the phase measurement registers of channels 
   1 & 2 are not used, these must be defined - use the 
   channel 0 definition to avoid any electrical issues
*/
#define AIN1_3PHASE_U                           ADC_AIN0
#define AIN1_3PHASE_V                           ADC_AIN1
#define AIN1_3PHASE_W                           ADC_AIN2
#define AIN1_VDC                                ADC_AIN3

#define AIN2_3PHASE_U                           ADC_AIN3
#define AIN2_3PHASE_V                           ADC_AIN4
#define AIN2_3PHASE_W                           ADC_AIN5
#define AIN2_VDC                                ADC_AIN1

/* Analog input port for current measurement with 1-shunt */
#define AIN0_1PHASE                             ADC_AIN16

/* all on ADC A */
#define AIN0_TEMPERATURE                        ADC_AIN13
#define AIN1_TEMPERATURE                        ADC_AIN14
#define AIN2_TEMPERATURE                        ADC_AIN15

/* Information for SW Over/Undervoltage Detection */
#define VDC_MEASURE0_REG                        ADC_REG5
#define VDC_MEASURE1_REG                        ADC_REG5
#define VDC_MEASURE2_REG                        ADC_REG5
#define VDC_MEASURE_TIMES                       5

/* Information for Temperature Control */
#define TEMPERATURE_ADC1                        TSB_ADB
#define TEMPERATURE_REG1                        ADC_REG3

/* Specification of serial port for serial protocol task */
/* Specification of serial port for serial protocol task */
#define SERIAL_COMMUNICATION_UART_CHANNEL       1

#define HSDSO_COMMUNICATION_UART_CHANNEL        0

/* SPI Channel used on the Board for Gate Driver */
#define BOARD_SPI_CHANNEL                       1

typedef enum
{ LED_NO_0,
  LED_NO_1,
  LED_NO_2,
  LED_NO_3,
} LED_NO;

/* Signaling of different states */
#define LED_SIGNAL_CONFIG_READ                  LED_NO_0                        /* Led Number for signaling config succesful read from EEProm */
#define LED_SIGNAL_SERIAL_COMMUNICATION_RUNNING LED_NO_1                        /* Led Number for signaling serial communication protocol active */
#define LED_SIGNAL_VE_RUN_BASE                  LED_NO_1                        /* Led base Number for VE is in FOC */
#define LED_SIGNAL_xxxxxxxxxxxxxxxxxxxxxxxx     LED_NO_3                        /* Led Number for signaling  */


uint8_t BOARD_Detect_Revision           (void);

#endif /* _BOARD_M4K4_H_ */
