/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _BOARD_SBK_M4K4_H_
#define _BOARD_SBK_M4K4_H_

/* Clock setup */
#define BOARD_USE_EXTERNAL_OSCILLATOR           1                               /* Crystal ECS-100-8-33-10.000MHZ */
#define BOARD_EXTERNAL_OSCILLATOR_FREQUENCY     10000000                        /* Oscillator Frequency in MHz */

#define BOARD_USE_PLL                           1                               /* Usage of PLL - 0:OFF 1:ON */

static const unsigned int gaintable[]={10};
typedef enum
{
  GAIN_1,                                                                       /* Gain of +1 */
} PGA_GAIN;

/* Gain settings */
#define BOARD_GAIN_CURRENT_MEASURE              GAIN_1

/* Analog input port for current measurement with 3-shunt */
#define AIN0_3PHASE_U                           ADC_AIN18
#define AIN0_3PHASE_V                           ADC_AIN17
#define AIN0_3PHASE_W                           ADC_AIN16

/* Analog input port for current measurement with 1-shunt */
#define AIN0_1PHASE                             ADC_AIN16

/* 
   Although the phase measurement registers of channels 
   1 & 2 are not used, these must be defined - use the 
   channel 0 definition to avoid any electrical issues
*/
#define AIN1_3PHASE_U                           ADC_AIN18
#define AIN1_3PHASE_V                           ADC_AIN17
#define AIN1_3PHASE_W                           ADC_AIN16

#define AIN2_3PHASE_U                           ADC_AIN18
#define AIN2_3PHASE_V                           ADC_AIN17
#define AIN2_3PHASE_W                           ADC_AIN16

/* Analog input port for DC voltage measurement */
#define AIN0_VDC                                ADC_AIN9
/* 
   Although the VDC measurement registers of channels 1 & 2 
   are not used, these must be defined - use the 
   channel 0 definition to avoid any electrical issues
*/
#define AIN1_VDC                                ADC_AIN9
#define AIN2_VDC                                ADC_AIN9

/* Information for SW Over/Undervoltage Detection */
#define VDC_MEASURE0_REG                        ADC_REG5
/* 
   Although the VDC measurement registers of channels 1 & 2 
   are not used, these must be defined  - use the 
   channel 0 definition to avoid any electrical issues
*/
#define VDC_MEASURE1_REG                        ADC_REG5
#define VDC_MEASURE2_REG                        ADC_REG5

#define VDC_MEASURE_TIMES                       5

/* Analog input port for temperature measurement */
#define AIN0_TEMPERATURE                        ADC_AIN13
/* 
   Although the temperature measurement registers of channels 
   1 & 2 are not used, these must be defined  - use the 
   channel 0 definition to avoid any electrical issues
*/
#define AIN1_TEMPERATURE                        ADC_AIN13
#define AIN2_TEMPERATURE                        ADC_AIN13


/* Information for Temperature Control */
/* All temperature measurements on ADC Unit A */
#define TEMPERATURE_ADC0                        TSB_ADA
/* All temperature measurements on ADC Unit A */
#define TEMPERATURE_REG0                        ADC_REG6

/* Specification of serial port for the communciation protocol task */
#define SERIAL_COMMUNICATION_UART_CHANNEL       0

/* Specification of serial port for HS-DSO task */
#define HSDSO_COMMUNICATION_UART_CHANNEL        1

/* LED defintions in case of USE_LED is enabled. Please make sure that led.c or any other implementation is incldued in the build  */
typedef enum
{ 
  LED_NO_0,
  LED_NO_1,
  LED_NO_2,
  LED_NO_3,
  LED_NO_4,
  LED_NO_5,
} LED_NO;

/* Signaling of different states - not used at first stage */
#define LED_SIGNAL_CONFIG_READ                  LED_NO_0                        /* Led Number for signaling config succesful read from EEProm */
#define LED_SIGNAL_SERIAL_COMMUNICATION_RUNNING LED_NO_1                        /* Led Number for signaling serial communication protocol active */
#define LED_SIGNAL_VE_RUN_BASE                  LED_NO_4                        /* Led Number for VE is in FOC */
#define LED_SIGNAL_EMERGENCY                    LED_NO_5                        /* Led Number for signaling emergency */

/* Board revision */
extern uint8_t BOARD_Detect_Revision(void);

#endif /* _BOARD_SBK_M4K4_H_ */
