/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include "config.h"

#ifdef BOARD_CLICKER4_M4KN
#ifdef USE_LED

#include "hal_gpio.h"
#include "led.h"


/*! \brief LED Init
  *
  * Initialize the LED access
  *
  * @retval None
*/
void LED_Init(void)
{  
  HAL_GPIO_Init(LED_NO_1_PORT, LED_NO_1_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(LED_NO_1_PORT, LED_NO_1_PORT_BIT, LED_OFF);
  HAL_GPIO_Init(LED_NO_2_PORT, LED_NO_2_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(LED_NO_2_PORT, LED_NO_2_PORT_BIT, LED_OFF);
  HAL_GPIO_Init(LED_NO_3_PORT, LED_NO_3_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(LED_NO_3_PORT, LED_NO_3_PORT_BIT, LED_OFF);
  HAL_GPIO_Init(LED_NO_4_PORT, LED_NO_4_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(LED_NO_4_PORT, LED_NO_4_PORT_BIT, LED_OFF);
  HAL_GPIO_Init(LED_NO_5_PORT, LED_NO_5_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(LED_NO_5_PORT, LED_NO_5_PORT_BIT, LED_OFF);
  HAL_GPIO_Init(LED_NO_6_PORT, LED_NO_6_PORT_BIT, HAL_GPIO_OUTPUT);
  HAL_GPIO_SetBit(LED_NO_6_PORT, LED_NO_6_PORT_BIT, LED_OFF);
}

/*! \brief LED Toggle
  *
  * Toggle an LED
  *
  * @param  led:  LED Number
  *
  * @retval None
*/
void LED_Toggle(unsigned char led)
{                     
  unsigned char current_state = LED_OFF;
  unsigned char new_state = LED_ON;
  
  current_state = LED_GetState(led);
  new_state = (~current_state & 0x01);
  LED_SetState(led, new_state);
}

/*! \brief LED set state
  *
  * Set the state of an LED
  *
  * @param  led:   LED Number
  * @param  state: State to set
  *
  * @retval None
*/
void LED_SetState(unsigned char led, unsigned char state)
{
  switch( led )
  {
    case LED_NO_1:
      HAL_GPIO_SetBit(LED_NO_1_PORT, LED_NO_1_PORT_BIT, state);
      break;
    
    case LED_NO_2:
      HAL_GPIO_SetBit(LED_NO_2_PORT, LED_NO_2_PORT_BIT, state);
      break;
  
    case LED_NO_3:
      HAL_GPIO_SetBit(LED_NO_3_PORT, LED_NO_3_PORT_BIT, state);
      break;
      
    case LED_NO_4:
      HAL_GPIO_SetBit(LED_NO_4_PORT, LED_NO_4_PORT_BIT, state);
      break;
    
    case LED_NO_5:
      HAL_GPIO_SetBit(LED_NO_5_PORT, LED_NO_5_PORT_BIT, state);
      break;
  
    case LED_NO_6:
      HAL_GPIO_SetBit(LED_NO_6_PORT, LED_NO_6_PORT_BIT, state);
      break;
  }
}

/*! \brief LED get state
  *
  * Get the state of an LED
  *
  * @param  led:   LED Number
  *
  * @retval actual state
*/
unsigned char LED_GetState(unsigned char led)
{
  unsigned char state = LED_OFF;                                                      /* actual state of LED */

  switch( led )
  {
    case LED_NO_1:
      state = HAL_GPIO_GetBit(LED_NO_1_PORT, LED_NO_1_PORT_BIT);
      break;
    
    case LED_NO_2:
      state = HAL_GPIO_GetBit(LED_NO_2_PORT, LED_NO_2_PORT_BIT);
      break;
  
    case LED_NO_3:
      state = HAL_GPIO_GetBit(LED_NO_3_PORT, LED_NO_3_PORT_BIT);
      break;
      
    case LED_NO_4:
      state = HAL_GPIO_GetBit(LED_NO_4_PORT, LED_NO_4_PORT_BIT);
      break;
    
    case LED_NO_5:
      state = HAL_GPIO_GetBit(LED_NO_5_PORT, LED_NO_5_PORT_BIT);
      break;
  
    case LED_NO_6:
      state = HAL_GPIO_GetBit(LED_NO_6_PORT, LED_NO_6_PORT_BIT);
      break;
  }

  return state;
}
#endif /* USE_LED */
#endif /* BOARD_CLICKER4_M4KN */
