/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef __DRV8323_H_
#define __DRV8323_H_

#include <stdint.h>
#include <hal_spi.h>


/* definitions for Fault Status 1 & 2 Register */

/* for all bit values in goth regsiters use these defines to differ between errors/warnings and no errors */
#define NO_FAULT     (0x00)		/* 1b = Charge pump UVLO fault is disabled             */
#define FAULT        (0x01)		/* 0b = Charge pump UVLO fault is enabled              */

/* definitions for Driver Conrol Register */

#define DIS_CPUV_ENA        (0x00)		/* 0b = Charge pump UVLO fault is enabled              */
#define DIS_CPUV_DIS        (0x01)		/* 1b = Charge pump UVLO fault is disabled             */

#define DIS_GDF_ENA         (0x00)		/* 0b = Gate drive fault is enabled                    */
#define DIS_GDF_DIS         (0x01)		/* 1b = Gate drive fault is disabled                   */

#define OTW_REP_NO_REPORT   (0x00)		/* 0b = OTW is not reported on nFAULT or the FAULT bit */
#define OTW_REP_REPORT      (0x01)		/* 1b = OTW is reported on nFAULT and the FAULT bit    */

#define  PWM_MODE_6            (0x00)  /* 00b = 6x PWM Mode         */
#define  PWM_MODE_3            (0x01)  /* 01b = 3x PWM mode         */
#define  PWM_MODE_1            (0x02)  /* 10b = 1x PWM mode         */
#define  PWM_MODE_INDEPENDANT  (0x03)  /* 11b = Independent PWM mod */

#define PWM_COM1_SYNCHRON    (0x00)  /* 0b = 1x PWM mode uses synchronous rectification */
#define PWM_COM1_ASYNCHRON   (0x01)  /* 1b = 1x PWM mode uses asynchronous rectification (diode freewheeling)*/

#define PWM_DIR1_DISABLE     (0x00)  /* In 1x PWM mode this bit is ORed with the INHC (DIR) input */
#define PWM_DIR1_ENABLE      (0x01)  /* In 1x PWM mode this bit is ORed with the INHC (DIR) input */

#define COAST_NO_HI_Z_STATE (0x00)   /* no Hi-Z state for all MOSFETs */
#define COAST_HI_Z_STATE    (0x01)   /* put all MOSFETs in the Hi-Z state */

#define BRAKE_1PWM_MODE     (0x01)   /* no turn on all three low-side MOSFETs */
#define BRAKE_NO_1PWM_MODE  (0x00)   /* turns  all three low-side MOSFETs in 1xPWM mode. */

#define CLR_FLT_OFF         (0x00)   /* Write a 1 to this bit to clear latched fault bits.*/
#define CLR_FLT_ON          (0x01)   /* This bit automatically resets after being written.*/

/* definitions for Gate Driver HS Register */

#define LOCK_SETTINGS_ENA                 (0x06)  /*  lock the settings by ignoring further register writes except to these bits and address 0x02 bits 0-2.*/
#define LOCK_SETTINGS_DIS                 (0x03)  /*  Value for unlock all registers */

#define IDRIVEP_10   (0x00)    /* 0000b = 10 mA   */
#define IDRIVEP_30   (0x01)	/* 0001b = 30 mA   */
#define IDRIVEP_60   (0x02)	/* 0010b = 60 mA   */
#define IDRIVEP_80   (0x03)	/* 0011b = 80 mA   */
#define IDRIVEP_120  (0x04)	/* 0100b = 120 mA  */
#define IDRIVEP_140  (0x05)	/* 0101b = 140 mA  */
#define IDRIVEP_170  (0x06)	/* 0110b = 170 mA  */
#define IDRIVEP_190  (0x07)	/* 0111b = 190 mA  */
#define IDRIVEP_260  (0x08)	/* 1000b = 260 mA  */
#define IDRIVEP_330  (0x09)	/* 1001b = 330 mA  */
#define IDRIVEP_370  (0x0A)	/* 1010b = 370 mA  */
#define IDRIVEP_440  (0x0B)	/* 1011b = 440 mA  */
#define IDRIVEP_570  (0x0C)	/* 1100b = 570 mA  */
#define IDRIVEP_680  (0x0D)	/* 1101b = 680 mA  */
#define IDRIVEP_820  (0x0E)	/* 1110b = 820 mA  */
#define IDRIVEP_1000 (0x0F)	/* 1111b = 1000 mA */

#define IDRIVEN_20    (0x00)  /* 0000b = 20 mA   */
#define IDRIVEN_60    (0x01)	/* 0001b = 60 mA   */
#define IDRIVEN_120   (0x02)	/* 0010b = 120 mA  */
#define IDRIVEN_160   (0x03)	/* 0011b = 160 mA  */
#define IDRIVEN_240   (0x04)	/* 0100b = 240 mA  */
#define IDRIVEN_280   (0x05)	/* 0101b = 280 mA  */
#define IDRIVEN_340   (0x06)	/* 0110b = 340 mA  */
#define IDRIVEN_380   (0x07)	/* 0111b = 380 mA  */
#define IDRIVEN_520   (0x08)	/* 1000b = 520 mA  */
#define IDRIVEN_660   (0x09)	/* 1001b = 660 mA  */
#define IDRIVEN_740   (0x0A)	/* 1010b = 740 mA  */
#define IDRIVEN_880   (0x0B)	/* 1011b = 880 mA  */
#define IDRIVEN_1140  (0x0C)	/* 1100b = 1140 mA */
#define IDRIVEN_1360  (0x0D)	/* 1101b = 1360 mA */
#define IDRIVEN_1640  (0x0E)	/* 1110b = 1640 mA */
#define IDRIVEN_2000  (0x0F)	/* 1111b = 2000 mA */

/* definitions for Gate Driver LS Register */

#define CBC_FAULT_NO  (0x00) 		/* Cycle-by cycle operation: No fault  */
#define CBC_FAULT     (0x01) 		/* Cycle-by cycle operation: Fault*/

#define TDRIVE_500  (0x00) 		/* 00b = 500-ns peak gate-current drive time  */
#define TDRIVE_1000 (0x01) 		/* 01b = 1000-ns peak gate-current drive time */
#define TDRIVE_2000 (0x02) 		/* 10b = 2000-ns peak gate-current drive time */
#define TDRIVE_4000 (0x03) 		/* 11b = 4000-ns peak gate-current drive time */

/* NOTE: for IDRIVEP_LS and IDRIVEN_LS use the same definition as deifined already for Gate Drive HS Resister */


/* definitions for OCP Control Register */

#define TRETRY_4     (0x00)		/* 0b = VDS_OCP and SEN_OCP retry time is 4 ms */
#define TRETRY_50	 (0x01)		/* 1b = VDS_OCP and SEN_OCP retry time is 50 s */

#define DEAD_TIME_50            (0x00)    /* 00b = 50-ns dead time         */
#define DEAD_TIME_100           (0x01)    /* 01b = 100-ns dead time        */
#define DEAD_TIME_200           (0x02)    /* 10b = 200-ns dead time        */
#define DEAD_TIME_400           (0x03)    /* 11b = 400-ns dead time        */

#define OCP_MODE_LATCHED_FAULT     	 (0x00)	/* 00b = Overcurrent causes a latched fault                 */
#define OCP_MODE_AUTO_RETRY_FAULT  	 (0x01)	/* 01b = Overcurrent causes an automatic retrying fault     */
#define OCP_MODE_REPORT_NO_ACTION  	 (0x02)	/* 10b = Overcurrent is report only but no action is taken  */
#define OCP_MODE_NO_REPORT_NO_ACTION (0x03)	/* 11b = Overcurrent is not reported and no action is taken */

#define OCP_DEG_2       (0x00)  /* 00b = Overcurrent deglitch time of 2 s  */
#define OCP_DEG_4       (0x01)  /* 01b = Overcurrent deglitch time of 4 s  */
#define OCP_DEG_6       (0x02)  /* 10b = Overcurrent deglitch time of 6 s  */
#define OCP_DEG_8       (0x03)  /* 11b = Overcurrent deglitch time of 8 s  */

#define VDS_LVL_0_06    (0x00)  /* 0000b = 0.06 V   */
#define VDS_LVL_0_13    (0x01)  /* 0001b = 0.13 V   */
#define VDS_LVL_0_2     (0x02)  /* 0010b = 0.2 V    */
#define VDS_LVL_0_26    (0x03)  /* 0011b = 0.26 V   */
#define VDS_LVL_0_31    (0x04)  /* 0100b = 0.31 V   */
#define VDS_LVL_0_45    (0x05)  /* 0101b = 0.45 V   */
#define VDS_LVL_0_53    (0x06)  /* 0110b = 0.53 V   */
#define VDS_LVL_0_6     (0x07)  /* 0111b = 0.6 V    */
#define VDS_LVL_0_68    (0x08)  /* 1000b = 0.68 V   */
#define VDS_LVL_0_75    (0x09)  /* 1001b = 0.75 V   */
#define VDS_LVL_0_94    (0x0A)  /* 1010b = 0.94 V   */
#define VDS_LVL_1_13    (0x0B)  /* 1011b = 1.13 V   */
#define VDS_LVL_1_3     (0x0C)  /* 1100b = 1.3 V    */
#define VDS_LVL_1_5     (0x0D)  /* 1101b = 1.5 V    */
#define VDS_LVL_1_7     (0x0E)  /* 1110b = 1.7 V    */
#define VDS_LVL_1_88    (0x0F)  /* 1111b = 1.88 V   */


/* definitions for CSA Control Register */

#define CSA_FET_SP	        (0x00)	/* 0b = Current sense amplifier positive input is SPx	               */
#define CSA_FET_SH	        (0x01)	/* 1b = Current sense amplifier positive input is SHx                  */
#define VREF_DIV_VREF 0     (0x00)	/* 0b = Current sense amplifier reference voltage is VREF              */
#define VREF_DIV_VREF_2     (0x01)	/* 1b = Current sense amplifier reference voltage is VREF divided by 2 */
#define LS_REF_SH_SP        (0x00)  /* 0b = VDS_OCP for the low-side MOSFET is measured across SHx to SPx  */
#define LS_REF_SH_SN        (0x01)  /* 1b = VDS_OCP for the low-side MOSFET is measured across SHx to SNx  */
#define CSA_GAIN_50         (0x00)  /* 0b = 5-V/V current sense amplifier gain                             */
#define CSA_GAIN_10         (0x01)  /* 01b = 10-V/V current sense amplifier gain                           */
#define CSA_GAIN_20         (0x02)  /* 10b = 20-V/V current sense amplifier gain                           */
#define CSA_GAIN_40         (0x03)  /* 11b = 40-V/V current sense amplifier gain                           */
#define DIS_SEN_OVERC_EN    (0x00)  /* 0b = Sense overcurrent fault is enabled                             */
#define DIS_SEN_OVERC_DIS   (0x01)  /* 1b = Sense overcurrent fault is disabled                            */
#define CSA_CAL_AMP_NORMAL  (0x00)  /* 0b = Normal current sense amplifier X operation                     */
#define CSA_CAL_AMP_SHORT   (0x01)  /* 1b = Short inputs to current sense amplifier X for offset calibration */
#define SEN_LVL_0_25        (0x00)  /* 00b = Sense OCP 0.25 V                                              */
#define SEN_LVL_0_5	        (0x01)	/* 01b = Sense OCP 0.5 V                                               */
#define SEN_LVL_0_75        (0x02)	/* 10b = Sense OCP 0.75 V                                              */
#define SEN_LVL_1_0	        (0x03)	/* 11b = Sense OCP 1 V                                                 */

typedef struct
{
    uint16_t vds_lc:1;                                                          /* Logic OR of FAULT status registers. Mirrors nFAULT pin.    */
    uint16_t vds_hc:1;                                                          /* Indicates VDS monitor overcurrent fault condition          */
    uint16_t vds_lb:1;                                                          /* Indicates gate drive fault condition                       */
    uint16_t vds_hb:1;                                                          /* Indicates undervoltage lockout fault condition             */
    uint16_t vds_la:1;                                                          /* Indicates overtemperature shutdown                         */
    uint16_t vds_ha:1;                                                          /* Indicates VDS overcurrent fault on the A high-side MOSFET  */
    uint16_t otsd :1;                                                           /* Indicates VDS overcurrent fault on the A low-side MOSFET   */
    uint16_t uvlo :1;                                                           /* Indicates VDS overcurrent fault on the B high-side MOSFET  */
    uint16_t gdf  :1;                                                           /* Indicates VDS overcurrent fault on the B low-side MOSFET   */
    uint16_t vds_ocp:1;                                                         /* Indicates VDS overcurrent fault on the C high-side MOSFET  */
    uint16_t fault :1;                                                          /* Indicates VDS overcurrent fault on the C low-side MOSFET   */
    uint16_t reserved:5;
}DRV8323_FaultStatus1;

typedef struct
{
    uint16_t vgs_lc:1;                                                          /* Indicates overcurrent on phase A sense amplifier (DRV8323xS) */
    uint16_t vgs_hc:1;                                                          /* Indicates overcurrent on phase B sense amplifier (DRV8323xS) */
    uint16_t vgs_lb:1;                                                          /* Indicates overcurrent on phase C sense amplifier (DRV8323xS) */
    uint16_t vgs_hb:1;                                                          /* Indicates overtemperature warning */
    uint16_t vgs_la:1;                                                          /* Indicates charge pump undervoltage fault condition       */
    uint16_t vgs_ha:1;                                                          /* Indicates gate drive fault on the A high-side MOSFET     */
    uint16_t otw_cpuv:1;                                                        /* Indicates gate drive fault on the A low-side MOSFET      */
    uint16_t sc_oc:1;                                                           /* Indicates gate drive fault on the B high-side MOSFET     */
    uint16_t sb_oc:1;                                                           /* Indicates gate drive fault on the B low-side MOSFET      */
    uint16_t sa_oc:1;                                                           /* Indicates gate drive fault on the C high-side MOSFET     */
    uint16_t reserved:5;                                                        /* Indicates gate drive fault on the C low-side MOSFET      */
}DRV8323_FaultStatus2;

typedef struct
{
    uint16_t clr_flt:1;
    uint16_t brake:1;
    uint16_t coast:1;
    uint16_t pwm_dir1:1;
    uint16_t pwm_com1:1;
    uint16_t pwm_mode:2;
    uint16_t otw_rep:1;
    uint16_t dis_gdf:1;
    uint16_t dis_cpuv:1;
    uint16_t reserved_10:1;
    uint16_t reserved:5;
}DRV8323_DrvCtrl;


typedef struct
{
    uint16_t idriven_hs:4;
    uint16_t idrivep_hs:4;
    uint16_t lock:3;
    uint16_t reserved:5;
}DRV8323_GateDrvHs;

typedef struct
{
    uint16_t idriven_ls:4;
    uint16_t idrivep_ls:4;
    uint16_t tdrive:2;
    uint16_t cbc:1;
    uint16_t reserved:5;
}DRV8323_GateDrvLs;

typedef struct
{
    uint16_t vds_lvl:4;
    uint16_t ocp_deg:2;
    uint16_t ocp_mode:2;
    uint16_t dead_time:2;
    uint16_t tretry :1;
    uint16_t reserved:5;
}DRV8323_OcpCtrl;

typedef struct
{
    uint16_t sen_lvl:2;
    uint16_t csa_cal_c:1;
    uint16_t csa_cal_b:1;
    uint16_t csa_cal_a:1;
    uint16_t dis_sen:1;
    uint16_t csa_gain:2;
    uint16_t ls_ref:1;
    uint16_t vref_div:1;
    uint16_t csa_fet:1;
    uint16_t reserved:5;
}DRV8323_CsaCtrl;


typedef struct
{
    DRV8323_FaultStatus1 faultStatus1;
    DRV8323_FaultStatus2 faultStatus2;
    DRV8323_DrvCtrl      drvCtrl;
    DRV8323_GateDrvHs	 gateDrvHs;
    DRV8323_GateDrvLs	 gateDrvLs;
    DRV8323_CsaCtrl      csaCtrl;
    DRV8323_OcpCtrl		 ocpCtrl;
}DRV8323_AddrData;


typedef enum {
  DRV8323_SUCCESS               = 0,                                          /*!< Operation mode of differential encoder is differential HTL  */
  DRV8323_SPI_ACCESS_ERROR      = 1,                                          /*!< Operation mode of differential encoder is RS-242  */
  DRV8323_FAULT_ERROR           = 2,                                          /*!< Operation mode of differential encoder is RS-242  */
  DRV8323_INIT_ERROR            = 3,                                          /*!< Operation mode of differential encoder is RS-242  */
} DRV8323_RETURN;

DRV8323_RETURN DRV8323_ReadFaultStatus1Register(DRV8323_FaultStatus1 *data);
DRV8323_RETURN DRV8323_ReadFaultStatus2Register(DRV8323_FaultStatus2 *data);
DRV8323_RETURN DRV8323_ReadDrvCtrlRegister(DRV8323_DrvCtrl *data);
DRV8323_RETURN DRV8323_WriteDrvCtrlRegister(DRV8323_DrvCtrl data);
DRV8323_RETURN DRV8323_ReadGateDrvHsRegister(DRV8323_GateDrvHs *data);
DRV8323_RETURN DRV8323_WriteGateDrvHsRegister(DRV8323_GateDrvHs data);
DRV8323_RETURN DRV8323_ReadGateDrvLsRegister(DRV8323_GateDrvLs *data);
DRV8323_RETURN DRV8323_WriteGateDrvLsRegister(DRV8323_GateDrvLs data);
DRV8323_RETURN DRV8323_ReadOcpCtrlRegister(DRV8323_OcpCtrl *data);
DRV8323_RETURN DRV8323_WriteOcpRegister(DRV8323_OcpCtrl data);
DRV8323_RETURN DRV8323_ReadCsaCtrlCtrlRegister(DRV8323_CsaCtrl *data);
DRV8323_RETURN DRV8323_WriteCsaCtrlRegister(DRV8323_CsaCtrl data);
#endif /* __DRV8323_ */
