/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef __MAX14890E_H_
#define __MAX14890E_H_

#include <stdint.h>
#include <hal_spi.h>


/* struct for Change Configuration with Detailed Fault Readback SPI Command */
typedef struct
{
  uint16_t fltr: 1;
  uint16_t d3di: 1;
  uint16_t d2di: 1;
  uint16_t ysngl: 1;
  uint16_t yttl: 1;
  uint16_t zdi: 1;
  uint16_t zsngl: 1;
  uint16_t zttl: 1;
  uint16_t bsngl: 1;
  uint16_t bttl: 1;
  uint16_t asngl: 1;
  uint16_t attl: 1;
  uint16_t hith: 1;
  uint16_t opcode:3;
}MAX14890E_CmdChangeCfg;

/* struct for Change Configuration with Detailed Fault Readback SPI Command */
typedef struct
{
  uint16_t reserved:13;
  uint16_t opcode: 3;
}MAX14890E_CmdFaultAndRecStatus;

/* struct for Read current fault output and receiver outputs SPI Command */
typedef struct
{
  uint16_t reserved:13;
  uint16_t opcode: 3;
}MAX14890E_CmdCurrentCfg;

/* struct for Read detailed fault status SPI Command */
typedef struct
{
  uint16_t reserved:13;
  uint16_t opcode: 3;
}MAX14890E_CmdDetailedFaultStatus;

/* struct for Change Configuration with Detailed Fault Readback SPI event */
typedef struct
{
  uint16_t d3hft: 1;
  uint16_t d3lft: 1;
  uint16_t d2hft: 1;
  uint16_t d2lft: 1;
  uint16_t ydflt: 1;
  uint16_t diyflt: 1;
  uint16_t yflt: 1;
  uint16_t zdflt: 1;
  uint16_t zflt: 1;
  uint16_t zflt_neg: 1;
  uint16_t bdflt: 1;
  uint16_t bflt: 1;
  uint16_t bdflt_neg: 1;
  uint16_t adflt: 1;
  uint16_t aflt: 1;
  uint16_t aflt_neg: 1;
}MAX14890E_EventDetailedFaultStatus;

/* struct for Current Fault and Receiver Status Readback SPI event */
typedef struct
{
  uint16_t l03: 1;
  uint16_t l02: 1;
  uint16_t y0: 1;
  uint16_t z0: 1;
  uint16_t b0: 1;
  uint16_t a0: 1;
  uint16_t d3fault: 1;
  uint16_t d2fault: 1;
  uint16_t yfault: 1;
  uint16_t zfault: 1;
  uint16_t bfault: 1;
  uint16_t afault: 1;
  uint16_t dummyZero: 1;
  uint16_t reserved: 3;
}MAX14890E_EventCurFaultRecStatus;

/* struct for receiver configuration readback SPI event */
typedef struct
{
  uint16_t fltr: 1;
  uint16_t d3di: 1;
  uint16_t d2di: 1;
  uint16_t ysngl: 1;
  uint16_t yttl: 1;
  uint16_t zdi: 1;
  uint16_t zsngl: 1;
  uint16_t zttl: 1;
  uint16_t bsngl: 1;
  uint16_t bttl: 1;
  uint16_t asngl: 1;
  uint16_t attl: 1;
  uint16_t hith: 1;
  uint16_t opcode: 3;
}MAX14890E_EventRecCfg;

/* union containing all available commands */
typedef union {
  MAX14890E_CmdChangeCfg               cmdChangeCfg;
  MAX14890E_CmdDetailedFaultStatus     cmdDetailedFault;
  MAX14890E_CmdFaultAndRecStatus       cmdCurrentFault;
  MAX14890E_CmdCurrentCfg              cmdReceiverCfg;
}MAX14890E_Cmd;

/* union containing all available events */
typedef union {
  MAX14890E_EventDetailedFaultStatus     evtDetailedFault;
  MAX14890E_EventCurFaultRecStatus       evtCurrentFault;
  MAX14890E_EventRecCfg                  evtReceiverCfg;
}MAX14890E_Event;

/* available receiver modes */
typedef enum {
  MAX14890E_D_HTL              = 0,                                          /*!< Operation mode of differential encoder is differential HTL  */
  MAX14890E_RS_422             = 1,                                          /*!< Operation mode of differential encoder is RS-242  */
  MAX14890E_SE_HTL             = 2,                                          /*!< Operation mode of differential encoder is single ended HTL  */
  MAX14890E_TTL                = 3,                                          /*!< Operation mode of differential encoder is TTL  */
  MAX14890E_MODE_SIZE          = 4,
} MAX14890E_RECEIVE_MODE;

/* list of all used return codes */
typedef enum {
  MAX14890E_SUCCESS               = 0,                                          /*!< Operation was successful  */
  MAX14890E_SPI_ACCESS_ERROR      = 1,                                          /*!< SPI access error  */
  MAX14890E_FAULT_ERROR           = 2,                                          /*!< Receiver reports fault error, during initialisation  */
  MAX14890E_INIT_ERROR            = 3,                                          /*!< Init failed, because the recived register data doesn't match written data */
} MAX14890E_RETURN;

/*--------------------------------------------------*/
/**
  * @brief  MAX14890E_Configure -> initialization depending of connected encoder
  * @param  MAX14890E_RECEIVE_MODE select the operation mode of the reciver#
                 - MAX14890E_D_HTL
                 - MAX14890E_RS_422
                 - MAX14890E_SE_HTL
                 - MAX14890E_TTL
  * @return -MAX14890E_RETURN
  * @retval -
  * @note   It changes the configuration register and and check the new value settings.
  *         The Fault status register will also read and checked, In case an fault bit is set the
  *         function returns with a FAUlt error.
  */
/*--------------------------------------------------*/
MAX14890E_RETURN MAX14890E_Configure (MAX14890E_RECEIVE_MODE mode);

/*--------------------------------------------------*/
/**
  * @brief  MAX14890E_ChangeCfg modifies configuration of MYX14890E
  * @param  struct MAX14890E_CmdChangeCfg
  * @return -MAX14890E_RETURN
  * @retval -
  * @note   This function only writes a new configuration it doesn't check the parameter
  *         afterwards. This function performs only 16-bit SPI write acces (only one SPI command cycle)
  */
/*--------------------------------------------------*/
MAX14890E_RETURN MAX14890E_ChangeCfg (MAX14890E_CmdChangeCfg data);

/*--------------------------------------------------*/
/**
  * @brief  MAX14890E_ReadFaultAndRecStatus reads the current fault output and receiver outputs
  * @param  struct MAX14890E_EventCurFaultRecStatus
  * @return -MAX14890E_RETURN
  * @retval -
  * @note   This function performs two 16-bit SPI write accesess (two SPI command cycles)
  */
/*--------------------------------------------------*/
MAX14890E_RETURN MAX14890E_ReadFaultAndRecStatus (MAX14890E_EventCurFaultRecStatus *data);

/*--------------------------------------------------*/
/**
  * @brief  MAX14890E_ReadCurrentConfiguration reads read current configuration
  * @param  struct MAX14890E_EventRecCfg
  * @return -MAX14890E_RETURN
  * @retval -
  * @note   This function performs two 16-bit SPI write accesess (two SPI command cycles)
  */
/*--------------------------------------------------*/
MAX14890E_RETURN MAX14890E_ReadCurrentConfiguration (MAX14890E_EventRecCfg *data);

/*--------------------------------------------------*/
/**
  * @brief  MAX14890E_ReadFaultAndRecStatus reads the detailed fault status
  * @param  struct MAX14890E_EventDetailedFaultStatus
  * @return -MAX14890E_RETURN
  * @retval -
  * @note   This function performs two 16-bit SPI write accesess (two SPI command cycles)
  */
/*--------------------------------------------------*/
MAX14890E_RETURN MAX14890E_ReadDetailedFaultStatus (MAX14890E_EventDetailedFaultStatus *data);

#endif /* __MAX14890E_ */
