/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef __MCP3S08_H_
#define __MCP3S08_H_

#include <stdint.h>
#include "hal_spi.h"

/* defines for direction configuration  for all GPIOs */
#define  IODIR_ALL_INPUT        (0xFF)
#define  IODIR_ALL_OUTPUT       (0x00)

/* defines for level output for all GPIOs */
#define  IODIR_ALL_HIGH        (0xFF)
#define  IODIR_ALL_LOW         (0x00)

typedef enum {
  MCP23S08_SUCCESS               = 0,                                          /*!< Operation mode of differential encoder is differential HTL  */
  MCP23S08_SPI_ACCESS_ERROR      = 1,                                          /*!< Operation mode of differential encoder is RS-242  */
} MCP23S08_RETURN;

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetDirectionRegister sets the GPIO direction register
  * @param  direction value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   It writes the direction value to the direction register.
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetDirectionRegister  (uint8_t  direction);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetDirectionRegister sets the GPIO direction register
  * @param  direction value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   It writes the direction value to the direction register.
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadDirectionRegister (uint8_t* direction);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   It data the direction value to the direction register.
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetPolarityRegister   (uint8_t  data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   It data the direction value to the direction register.
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadPolarityRegister  (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetIrqOnChangeCtrlRegister sets the GPIO direction register
  * @param  direction value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetIrqOnChangeCtrlRegister  (uint8_t data) ;
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   It data the direction value to the direction register.
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadIrqOnChangeCtrlRegister (uint8_t* data) ;

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetDfltValueRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetDfltValueRegister  (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadDfltValueRegister (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetIrqCtrlRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetIrqCtrlRegister    (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   It data the direction value to the direction register.
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadIrqCtrlRegister   (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetCfgRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetCfgRegister        (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadCfgRegister       (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPullUpCfgRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetPullUpCfgRegister  (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadPullUpCfgRegister (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetDirectionRegister sets the GPIO direction register
  * @param  direction value
  * @return -MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetIrqFlagRegister    (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetDirectionRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadIrqFlagRegister   (uint8_t* data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetIrqCaptureRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetIrqCaptureRegister  (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the GPIO direction register
  * @param  data value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadIrqCaptureRegister (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetGeneralPurposeRegister sets the GPIO direction register
  * @param  direction value
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetGeneralPurposeRegister  (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister sets the General Purpose register
  * @param  pointer to the General Purpose data
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadGeneralPurposeRegister (uint8_t* data);

/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetOutputLatchRegister sets the Output Latch register
  * @param  Output Latch data
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_SetOutputLatchRegister  (uint8_t data);
/*--------------------------------------------------*/
/**
  * @brief  MCP23S08_SetPolarityRegister reads the Output Latch register
  * @param  pointer to the Outut Latch data
  * @return MCP23S08_RETURN
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MCP23S08_RETURN MCP23S08_ReadOutputLatchRegister (uint8_t* data);
#endif /* __MCP2515_H_ */
