/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include "config.h"

#if (defined USE_LED && defined LED_NO_0_PORT)

#include "FreeRTOS.h"
#include "task.h"
#include "debug.h"
#include "led.h"
#include "hal_gpio.h"

#include BOARD_CONFIG_HEADER_FILE

#define INIT_LED(NUMBER) \
  { \
      HAL_GPIO_Init(NUMBER ## _PORT, NUMBER ## _PORT_BIT, GPIO_OUTPUT); \
      LED_SetState(NUMBER ,LED_OFF); \
  }

#define LED_TOGGLE(NUMBER) \
  { \
	case NUMBER :\
      HAL_GPIO_SetBit(NUMBER ## _PORT, NUMBER ## _PORT_BIT, (~HAL_GPIO_GetBit(NUMBER ## _PORT, NUMBER ## _PORT_BIT) & 0x01)); \
      break; \
  }

#define LED_SET_STATE(NUMBER,VALUE) \
  { \
	case NUMBER :\
      HAL_GPIO_SetBit(NUMBER ## _PORT, NUMBER ## _PORT_BIT, VALUE);\
      break; \
  }

#define LED_GET_STATE(NUMBER) \
  { \
	case NUMBER :\
          state = ~HAL_GPIO_GetBit(NUMBER ## _PORT, NUMBER ## _PORT_BIT); \
      break; \
  }

/*! \brief LED Init
  *
  * Initialize the LED access
  *
  * @retval None
*/
void LED_Init(void)
{  
#ifdef LED_NO_0_PORT    
  INIT_LED(LED_NO_0)
#endif    
#ifdef LED_NO_1_PORT    
  INIT_LED(LED_NO_1)
#endif    
#ifdef LED_NO_2_PORT    
  INIT_LED(LED_NO_2)
#endif    
#ifdef LED_NO_3_PORT    
  INIT_LED(LED_NO_3)
#endif    
}

/*! \brief LED Toggle
  *
  * Toggle an LED
  *
  * @param  led:  LED Number
  *
  * @retval None
*/
void LED_Toggle(unsigned char led)
{
  switch(led)
  {
#ifdef LED_NO_0_PORT    
    LED_TOGGLE(LED_NO_0)	  
#endif    
#ifdef LED_NO_1_PORT    
    LED_TOGGLE(LED_NO_1)
#endif    
#ifdef LED_NO_2_PORT    
    LED_TOGGLE(LED_NO_2)
#endif    
#ifdef LED_NO_3_PORT    
    LED_TOGGLE(LED_NO_3)	  
#endif    
  }
}

/*! \brief LED set state
  *
  * Set the state of an LED
  *
  * @param  led:   LED Number
  * @param  state: State to set
  *
  * @retval None
*/
void LED_SetState(unsigned char led, unsigned char state)
{
  switch(led)
  {
#ifdef LED_NO_0_PORT    
    LED_SET_STATE(LED_NO_0,state)
#endif    
#ifdef LED_NO_1_PORT    
    LED_SET_STATE(LED_NO_1,state)
#endif    
#ifdef LED_NO_2_PORT    
    LED_SET_STATE(LED_NO_2,state)
#endif    
#ifdef LED_NO_3_PORT    
    LED_SET_STATE(LED_NO_3,state)
#endif    
  }
}

/*! \brief LED get state
  *
  * Get the state of an LED
  *
  * @param  led:   LED Number
  *
  * @retval actual state
*/
unsigned char LED_GetState(unsigned char led)
{
  unsigned char state = 0;                                                      /* actual state of LED */

  switch(led)
  {
#ifdef LED_NO_0_PORT    
    LED_GET_STATE(LED_NO_0)
#endif    
#ifdef LED_NO_1_PORT    
    LED_GET_STATE(LED_NO_1)
#endif    
#ifdef LED_NO_2_PORT    
    LED_GET_STATE(LED_NO_2)
#endif    
#ifdef LED_NO_3_PORT    
    LED_GET_STATE(LED_NO_3)
#endif    
  }

  return state;
}
#endif
