/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <string.h>

#include "config.h"

#include "FreeRTOS.h"
#include "task.h"

#include "MAX14890E.h"
#include "hal_spi.h"
#include BOARD_CONFIG_HEADER_FILE
#include BOARD_SPI_HEADER_FILE


#define  CMD_READ_DETAILED_FAULT    (0x00)                                        /* command opcode for read detailed fault status */
#define  CMD_READ_FAULT_AND_OUT     (0x02)                                        /* command opcode for read the current fault amd receiver outputs */
#define  CMD_CHANGE_CFG             (0x05)                                        /* command opcode for change configuration */
#define  CMD_READ_CFG               (0x07)                                        /* command opcode for read configuration */

#define  ENABLE_FILTER              (0x01)
#define  DISABLE_FILTER             (0x00)

#define  FAULT                      (0x01)
#define  FAULT_NO                   (0x00)

#define  FILTER_CFG                 (ENABLE_FILTER)                             /* fault IRQ not used, nevertheless enbale filter */

typedef struct {
    MAX14890E_RECEIVE_MODE recOperation;
    uint8_t xSngl;
    uint8_t xTTL;
}MAX14890E_RxaRxbMapping;


const MAX14890E_RxaRxbMapping rxaRxbMappingTable[] =
{
    { MAX14890E_D_HTL  , 0 , 0 },
    { MAX14890E_RS_422 , 0 , 1 },
    { MAX14890E_SE_HTL , 1 , 0 },
    { MAX14890E_TTL    , 1 , 1 },
};

typedef struct {
    MAX14890E_RECEIVE_MODE recOperation;
    uint8_t zttl;
    uint8_t zsngl;
    uint8_t zdi;
}MAX14890E_RxzMapping;

const MAX14890E_RxzMapping rxzMappingTable[] =
{
    { MAX14890E_D_HTL  , 0, 0 , 0 },
    { MAX14890E_SE_HTL , 0, 0 , 1 },
    { MAX14890E_RS_422 , 1, 0 , 0 },
    { MAX14890E_TTL    , 1, 1 , 0 },
  /*{ ENCODER_DIFF_DI     ,  1, 1 , 1 },*/
};

typedef struct {
    MAX14890E_RECEIVE_MODE recOperation;
    uint8_t yttl;
    uint8_t ysngl;
}MAX14890E_RxyMapping;


MAX14890E_RxyMapping rxyRxbMappingTable[] =
{
    { MAX14890E_D_HTL  , 0 , 0 },
    { MAX14890E_RS_422 , 0 , 1 },
    { MAX14890E_SE_HTL , 1 , 0 },
    { MAX14890E_TTL    , 1 , 1 },
};

typedef struct {
    MAX14890E_RECEIVE_MODE recOperation;
    uint8_t d_di;
}MAX14890E_RxDxMapping;


MAX14890E_RxDxMapping rxdxRxbMappingTable[] =
{
    { MAX14890E_TTL, 0},
   /* { ENCODER_DIFF_DI, 0}, */
};

#ifdef BOARD_CLICKER4_M4KN
    #define   MAX14890E_SPI_CHANNEL     (0)
    #define   MAX14890E_SPI_CS          (1)
#else
    #define   MAX14890E_SPI_CHANNEL     (1)
    #define   MAX14890E_SPI_CS          (0)
#endif /* BOARD_CLICKER4_M4KN */

#define   MAX14890E_SPI_TRANSFER_MODE   (1)                                     /*!< This member configures the SPI mode: 1= master, 0 = slave */
#define   MAX14890E_SPI_SPEED           (HAL_SPI_1_MHZ)                         /*!< speed for SPI port */
#define   MAX14890E_SPI_DATA_LENGTH     (HAL_SPI_DATA_LENGTH_16)
#define   MAX14890E_SPI_CK_EDGE         (HAL_SPI_CK_1ST_EDGE)
#define   MAX14890E_SPI_CK_IDLE         (HAL_SPI_CK_IDLE_LOW)

/* SPI configuration settings */
const HAL_SPI_CfgData max14890eSpiCfgData = {
  MAX14890E_SPI_CHANNEL      ,
  MAX14890E_SPI_CS           ,
  MAX14890E_SPI_TRANSFER_MODE,
  MAX14890E_SPI_SPEED        ,
  MAX14890E_SPI_DATA_LENGTH  ,
  MAX14890E_SPI_CK_EDGE      ,
  MAX14890E_SPI_CK_IDLE      ,
  (void*)0                   ,       /* callbacks for async calls not used */
  (void*)0                   ,       /* callbacks for async calls not used */
};

typedef enum
{
    MAX14890E_WRITE,
    MAX14890E_WRITE_READ,
}MAX14890E_Access;

/**
  * @brief  function to read one register from MCP23S08
  * @param  Register address of the register to be read
  *         see MCP23S08.h all defines with ..._REGISTER
  * @retval SPI_Rx_Buffer[0] read register
  */
static MAX14890E_RETURN MAX14890E_ReadWriteCommand(MAX14890E_Cmd *sendData, MAX14890E_Event *receivedData, MAX14890E_Access access)
{
  HAL_SPI_TransRec_t transferReceive;   /* value of register */
  uint16_t dummyData;


  if (HAL_SPI_NO_ERR != HAL_SPI_Init(&max14890eSpiCfgData))
  {
    return MAX14890E_SPI_ACCESS_ERROR;
  }

  transferReceive.trx16.p_data = (uint16_t*)sendData;
  transferReceive.trx16.num = 1;

  /* first spi command cycle: send data */
  if (HAL_SPI_NO_ERR != HAL_SPI_MasterWriteBytes(MAX14890E_SPI_CHANNEL, &transferReceive))
  {
    return MAX14890E_SPI_ACCESS_ERROR;
  }

  /* set pointer to dummy buffer */
  transferReceive.trx16.p_data = &dummyData;

  /* dummy read to clear receive buffer */
  if (HAL_SPI_NO_ERR != HAL_SPI_MasterReadBytes (MAX14890E_SPI_CHANNEL, &transferReceive))
  {
    return MAX14890E_SPI_ACCESS_ERROR;
  }

  /* second command sending is only for reading data */
  if (access != MAX14890E_WRITE)
  {
      /* send same cmd second time to read response */
      transferReceive.trx16.p_data = (uint16_t*)sendData;
      /* second SPI command cycle: received data */
      if (HAL_SPI_NO_ERR != HAL_SPI_MasterWriteBytes(MAX14890E_SPI_CHANNEL, &transferReceive))
      {
        return MAX14890E_SPI_ACCESS_ERROR;
      }

      /* check if buffer for response is available */
      if (receivedData != (void *)0)
      {
          /* set pointer to receive buffer */
          transferReceive.trx16.p_data = (uint16_t*) receivedData;
      }
      else
      {
        /* set pointer to dummy buffer */
        transferReceive.trx16.p_data = &dummyData;
      }

      if (HAL_SPI_NO_ERR != HAL_SPI_MasterReadBytes (MAX14890E_SPI_CHANNEL, &transferReceive))
      {
        return MAX14890E_SPI_ACCESS_ERROR;
      }
  }
  return MAX14890E_SUCCESS;
}

MAX14890E_RETURN MAX14890E_Configure (MAX14890E_RECEIVE_MODE mode)
{
  uint8_t loop;
  MAX14890E_Cmd  cmd;  /* value of register */
  MAX14890E_Event event;

  for (loop = 0; loop < MAX14890E_MODE_SIZE; loop++)
  {
    if (mode == rxaRxbMappingTable[loop].recOperation)
    {
        cmd.cmdChangeCfg.attl   = rxaRxbMappingTable[loop].xTTL;
        cmd.cmdChangeCfg.asngl  = rxaRxbMappingTable[loop].xSngl;
        cmd.cmdChangeCfg.bttl   = rxaRxbMappingTable[loop].xTTL;
        cmd.cmdChangeCfg.bsngl  = rxaRxbMappingTable[loop].xSngl;
        cmd.cmdChangeCfg.yttl   = rxaRxbMappingTable[loop].xTTL;
        cmd.cmdChangeCfg.ysngl  = rxaRxbMappingTable[loop].xSngl;

        cmd.cmdChangeCfg.yttl   = 0;
        cmd.cmdChangeCfg.ysngl  = 0;
    }
    if (mode == rxzMappingTable[loop].recOperation)
    {
        cmd.cmdChangeCfg.zttl   = rxzMappingTable[loop].zttl;
        cmd.cmdChangeCfg.zsngl  = rxzMappingTable[loop].zsngl;
        cmd.cmdChangeCfg.zdi    = rxzMappingTable[loop].zdi;
    }
  }

  cmd.cmdChangeCfg.hith= 0;     /* disable the single ended HTL thresholds */
  cmd.cmdChangeCfg.d2di = 0;   /* not used ->configure them as TTL input */
  cmd.cmdChangeCfg.d3di = 0;   /* not used ->configure them as TTL input */
  cmd.cmdChangeCfg.fltr = FILTER_CFG;    /* not used ->enable filter */

  if (MAX14890E_SUCCESS != MAX14890E_ChangeCfg(cmd.cmdChangeCfg))
  {
    return MAX14890E_SPI_ACCESS_ERROR;
  }

  if (MAX14890E_SUCCESS != MAX14890E_ReadDetailedFaultStatus((MAX14890E_EventDetailedFaultStatus*)&event))
  {
    return MAX14890E_SPI_ACCESS_ERROR;
  }

  /* check the response for errors */
  switch (mode)
  {
    /* differential modes */
    case MAX14890E_D_HTL:
    case MAX14890E_RS_422:
      if (( event.evtDetailedFault.adflt != FAULT_NO) ||
          ( event.evtDetailedFault.bdflt != FAULT_NO) ||
          ( event.evtDetailedFault.zdflt != FAULT_NO) ||
          ( event.evtDetailedFault.ydflt != FAULT_NO))
      {
        return MAX14890E_FAULT_ERROR;
      }
      break;

    /* single ended modes */
    case MAX14890E_SE_HTL:
    case MAX14890E_TTL:
      if (( event.evtDetailedFault.d3hft    != FAULT_NO) ||
          ( event.evtDetailedFault.d3lft    != FAULT_NO) ||
          ( event.evtDetailedFault.d2hft    != FAULT_NO) ||
          ( event.evtDetailedFault.d2lft    != FAULT_NO) ||
          ( event.evtDetailedFault.diyflt   != FAULT_NO) ||
          ( event.evtDetailedFault.yflt     != FAULT_NO) ||
          ( event.evtDetailedFault.zflt     != FAULT_NO) ||
          ( event.evtDetailedFault.zflt_neg != FAULT_NO) ||
          ( event.evtDetailedFault.bflt     != FAULT_NO) ||
          ( event.evtDetailedFault.bdflt_neg!= FAULT_NO) ||
          ( event.evtDetailedFault.aflt     != FAULT_NO) ||
          ( event.evtDetailedFault.aflt_neg != FAULT_NO))
      {
        return MAX14890E_FAULT_ERROR;
      }
      break;

    default:
      return MAX14890E_FAULT_ERROR;
      break;
  }


  if (MAX14890E_SUCCESS != MAX14890E_ReadCurrentConfiguration((MAX14890E_EventRecCfg*)&event))
  {
    return MAX14890E_SPI_ACCESS_ERROR;
  }

  if (( cmd.cmdChangeCfg.fltr   != event.evtReceiverCfg.fltr) ||
      ( cmd.cmdChangeCfg.d3di   != event.evtReceiverCfg.d3di) ||
      ( cmd.cmdChangeCfg.d2di   != event.evtReceiverCfg.d2di) ||
      ( cmd.cmdChangeCfg.ysngl  != event.evtReceiverCfg.ysngl)||
      ( cmd.cmdChangeCfg.yttl   != event.evtReceiverCfg.yttl) ||
      ( cmd.cmdChangeCfg.zdi    != event.evtReceiverCfg.zdi ) ||
      ( cmd.cmdChangeCfg.zsngl  != event.evtReceiverCfg.zsngl)||
      ( cmd.cmdChangeCfg.zttl   != event.evtReceiverCfg.zttl) ||
      ( cmd.cmdChangeCfg.bsngl  != event.evtReceiverCfg.bsngl)||
      ( cmd.cmdChangeCfg.bttl   != event.evtReceiverCfg.bttl) ||
      ( cmd.cmdChangeCfg.asngl  != event.evtReceiverCfg.asngl)||
      ( cmd.cmdChangeCfg.attl   != event.evtReceiverCfg.attl) ||
      ( cmd.cmdChangeCfg.hith   != event.evtReceiverCfg.hith))

  {
    return MAX14890E_INIT_ERROR;
  }
  return MAX14890E_SUCCESS;
}


/**
  * @brief  function to write one register into MCP23S08
  * @param  Register address of the register to be written
  *         see MCP23S08.h all defines with ..._REGISTER
  * @param  Data for the register
  * @retval None
  */


MAX14890E_RETURN MAX14890E_ChangeCfg (MAX14890E_CmdChangeCfg data)
{
  /* reserved field is for opcode */
  data.opcode = CMD_CHANGE_CFG;

  return MAX14890E_ReadWriteCommand((MAX14890E_Cmd*) &data, (void*)0, MAX14890E_WRITE);
}

MAX14890E_RETURN MAX14890E_ReadFaultAndRecStatus (MAX14890E_EventCurFaultRecStatus *data)
{
  MAX14890E_CmdFaultAndRecStatus       curFault = {0};

  /* reserved field is for opcode */
  curFault.opcode = CMD_READ_FAULT_AND_OUT;

  return  MAX14890E_ReadWriteCommand((MAX14890E_Cmd*)&curFault, (MAX14890E_Event*) data, MAX14890E_WRITE_READ);
}

MAX14890E_RETURN MAX14890E_ReadCurrentConfiguration (MAX14890E_EventRecCfg *data)
{
  MAX14890E_CmdCurrentCfg  cfg = {0};

  /* reserved field is for opcode */
  cfg.opcode = CMD_READ_CFG;

  return  MAX14890E_ReadWriteCommand((MAX14890E_Cmd*)&cfg, (MAX14890E_Event*)data, MAX14890E_WRITE_READ);
}


MAX14890E_RETURN MAX14890E_ReadDetailedFaultStatus (MAX14890E_EventDetailedFaultStatus *data)
{
  MAX14890E_CmdDetailedFaultStatus     dtlFault = {0};

  /* reserved field is for opcode */
  dtlFault.opcode = CMD_READ_DETAILED_FAULT;

  return  MAX14890E_ReadWriteCommand((MAX14890E_Cmd*)&dtlFault,(MAX14890E_Event*) data, MAX14890E_WRITE_READ);
}

