/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <string.h>

#include "config.h"

#include "FreeRTOS.h"
#include "task.h"

#include "MCP23S08.h"
#include "hal_spi.h"
#include BOARD_CONFIG_HEADER_FILE
#include BOARD_SPI_HEADER_FILE


#define   MCP23S08_SPI_ADDRESS         (0x40)
#define   MCP23S08_READ_FLAG           (0x01)
#define   MCP23S08_WRITE_FLAG          (0x00)

/* SPI confifguration settings */
#define   MCP23S08_SPI_CHANNEL         (1)
#define   MCP23S08_SPI_CS              (1)
#define   MCP23S08_SPI_TRANSFER_MODE   (1)                                     /*!< This member configures the SPI mode: 1= master, 0 = slave */
#define   MCP23S08_SPI_SPEED           (HAL_SPI_10_MHZ)                         /*!< speed for SPI port */
#define   MCP23S08_SPI_DATA_LENGTH     ((HAL_SPI_DATA_LENGTH_24))
#define   MCP23S08_SPI_CK_EDGE         (HAL_SPI_CK_2ND_EDGE)
#define   MCP23S08_SPI_CK_IDLE         (HAL_SPI_CK_IDLE_HI )

typedef enum
{
  IODIR_REGISTER  = 0x00,
  IPOL_REGISTER   = 0x01,
  GPINTEN_REGISTER= 0x02,
  DEFVAL_REGISTER = 0x03,
  INTCON_REGISTER = 0x04,
  IOCON_REGISTER  = 0x05,
  GPPU_REGISTER   = 0x06,
  INTF_REGISTER   = 0x07,
  INTCAP_REGISTER = 0x08,
  GPIO_REGISTER   = 0x09,
  OLAT_REGISTER   = 0x0A
} MCP23S08_REGISTERS;


const HAL_SPI_CfgData mcp32S08SpiCfgData = {
  MCP23S08_SPI_CHANNEL      ,
  MCP23S08_SPI_CS           ,
  MCP23S08_SPI_TRANSFER_MODE,
  MCP23S08_SPI_SPEED        ,
  MCP23S08_SPI_DATA_LENGTH  ,
  MCP23S08_SPI_CK_EDGE      ,
  MCP23S08_SPI_CK_IDLE      ,
  (void*) 0 ,             /* callbacks for async calls not used */
  (void*) 0 ,             /* callbacks for async calls not used */
};

/**
  * @brief  function to read one register from MCP23S08
  * @param  Register address of the register to be read
  *         see MCP23S08.h all defines with ..._REGISTER
  * @retval SPI_Rx_Buffer[0] read register
  */
static MCP23S08_RETURN MCP23S08_ReadRegister(MCP23S08_REGISTERS address, uint8_t* regValue)
{
    uint32_t value;                                                            /* value of register */
    HAL_SPI_TransRec_t transferReceive;                                           /* value of register */

    if (HAL_SPI_NO_ERR != HAL_SPI_Init(&mcp32S08SpiCfgData))
    {
      return MCP23S08_SPI_ACCESS_ERROR;
    }

    value = ((MCP23S08_SPI_ADDRESS  | MCP23S08_READ_FLAG) << 16) | (address << 8);

    transferReceive.trx32.p_data = &value;
    transferReceive.trx32.num = 1;

    if (HAL_SPI_NO_ERR != HAL_SPI_MasterWriteBytes(MCP23S08_SPI_CHANNEL, &transferReceive))
    {
      return  MCP23S08_SPI_ACCESS_ERROR;;
    }
    /* dummy read to clear receive buffer */
    if (HAL_SPI_NO_ERR != HAL_SPI_MasterReadBytes (MCP23S08_SPI_CHANNEL, &transferReceive))
    {
      return  MCP23S08_SPI_ACCESS_ERROR;;
    }

    *regValue = (uint8_t)(transferReceive.trx32.p_data[0] & 0xFF);

    return MCP23S08_SUCCESS;
}

/**
  * @brief  function to write one register into MCP23S08
  * @param  Register address of the register to be written
  *         see MCP23S08.h all defines with ..._REGISTER
  * @param  Data for the register
  * @retval None
  */
static MCP23S08_RETURN MCP23S08_WriteRegister(uint8_t addr, uint8_t regValue)
{
    uint32_t value;                                                            /* value to transmit over SPI*/
    HAL_SPI_TransRec_t transferReceive;

    if (HAL_SPI_NO_ERR != HAL_SPI_Init(&mcp32S08SpiCfgData))
    {
      return MCP23S08_SPI_ACCESS_ERROR;
    }

    value = ((MCP23S08_SPI_ADDRESS | MCP23S08_WRITE_FLAG) << 16) | (addr << 8 ) | regValue;

    transferReceive.trx32.p_data = &value;
    transferReceive.trx32.num = 1;

    if (HAL_SPI_NO_ERR != HAL_SPI_MasterWriteBytes(MCP23S08_SPI_CHANNEL, &transferReceive))
    {
      return MCP23S08_SPI_ACCESS_ERROR;
    }
    /* dummy read to clear receive buffer */
    HAL_SPI_MasterReadBytes (MCP23S08_SPI_CHANNEL, &transferReceive);

    return MCP23S08_SUCCESS;
}


MCP23S08_RETURN MCP23S08_SetDirectionRegister(uint8_t direction)
{
  return MCP23S08_WriteRegister(IODIR_REGISTER, direction);
}

MCP23S08_RETURN MCP23S08_ReadDirectionRegister (uint8_t* direction)
{
  return MCP23S08_ReadRegister(IODIR_REGISTER, direction);
}

MCP23S08_RETURN  MCP23S08_SetPolarityRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(IPOL_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadPolarityRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(IPOL_REGISTER, data);
}

MCP23S08_RETURN  MCP23S08_SetIrqOnChangeCtrlRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(GPINTEN_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadIrqOnChangeCtrlRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(GPINTEN_REGISTER, data);
}

MCP23S08_RETURN  MCP23S08_SetDfltValueRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(DEFVAL_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadDfltValueRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(DEFVAL_REGISTER, data);
}

MCP23S08_RETURN  MCP23S08_SetIrqCtrlRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(INTCON_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadIrqCtrlRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(INTCON_REGISTER, data);
}
MCP23S08_RETURN  MCP23S08_SetCfgRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(IOCON_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadCfgRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(IOCON_REGISTER, data);
}

MCP23S08_RETURN  MCP23S08_SetPullUpCfgRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(GPPU_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadPullUpCfgRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(GPPU_REGISTER, data);
}

MCP23S08_RETURN  MCP23S08_SetIrqFlagRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(INTF_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadIrqFlagRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(INTF_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_SetIrqCaptureRegister (uint8_t data)
{
  return MCP23S08_WriteRegister(INTCAP_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadIrqCaptureRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(INTCAP_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_SetGeneralPurposeRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(GPIO_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadGeneralPurposeRegister (uint8_t* data)
{
  return MCP23S08_ReadRegister(GPIO_REGISTER, data);
}

MCP23S08_RETURN  MCP23S08_SetOutputLatchRegister(uint8_t data)
{
  return MCP23S08_WriteRegister(OLAT_REGISTER, data);
}

MCP23S08_RETURN MCP23S08_ReadOutputLatchRegister(uint8_t* data)
{
  return MCP23S08_ReadRegister(OLAT_REGISTER, data);
}