/** 
 *******************************************************************************
 * @file    TMPM4Ky_cg.h
 * @brief   This file provides all the functions prototypes for CG driver.
 * @version 
 * $Date:: 
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __CG_H
#define __CG_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup CG CG
 *  @brief    CG Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Exported_define CG Exported Define
 *  @{
 */
/** 
 *  @defgroup CG_CGOSCCR_OSCF  fosc high-speed oscillator Select Status.
 *  @brief    fosc high-speed oscillator Select Status.
 *  @{
 */
#define CG_CGOSCCR_OSCF_IHOSC           ((uint32_t)0x00000000)    /*!< IHOSC    */
#define CG_CGOSCCR_OSCF_EHOSC           ((uint32_t)0x00000200)    /*!< EHOSC    */

#define CG_CLOCKA_MS_TSA32_CH03_BIT     ((uint32_t)((0x1)<<31))
#define CG_CLOCKA_MS_CH02_BIT           ((uint32_t)((0x1)<<30))
#define CG_CLOCKA_MS_CH01_BIT           ((uint32_t)((0x1)<<29))
#define CG_CLOCKA_MS_CH00_BIT           ((uint32_t)((0x1)<<28))
#define CG_CLOCKA_MS_CAN_BIT            ((uint32_t)((0x1)<<27))
#define CG_CLOCKA_MS_I2C_CH1_BIT        ((uint32_t)((0x1)<<26))
#define CG_CLOCKA_MS_I2C_CH0_BIT        ((uint32_t)((0x1)<<25))
#define CG_CLOCKA_MS_UART_CH3_BIT       ((uint32_t)((0x1)<<24))
#define CG_CLOCKA_MS_UART_CH2_BIT       ((uint32_t)((0x1)<<23))
#define CG_CLOCKA_MS_UART_CH1_BIT       ((uint32_t)((0x1)<<22))
#define CG_CLOCKA_MS_UART_CH0_BIT       ((uint32_t)((0x1)<<21))   
#define CG_CLOCKA_MS_TSPI_CH1_BIT       ((uint32_t)((0x1)<<20))   
#define CG_CLOCKA_MS_TSPI_CH0_BIT       ((uint32_t)((0x1)<<19))
#define CG_CLOCKA_MS_PORT_W_BIT         ((uint32_t)((0x1)<<18))
#define CG_CLOCKA_MS_PORT_V_BIT         ((uint32_t)((0x1)<<17))
#define CG_CLOCKA_MS_PORT_U_BIT         ((uint32_t)((0x1)<<16))
#define CG_CLOCKA_MS_PORT_T_BIT         ((uint32_t)((0x1)<<15))
#define CG_CLOCKA_MS_PORT_R_BIT         ((uint32_t)((0x1)<<14))
#define CG_CLOCKA_MS_PORT_P_BIT         ((uint32_t)((0x1)<<13))
#define CG_CLOCKA_MS_PORT_N_BIT         ((uint32_t)((0x1)<<12))
#define CG_CLOCKA_MS_PORT_M_BIT         ((uint32_t)((0x1)<<11))   
#define CG_CLOCKA_MS_PORT_L_BIT         ((uint32_t)((0x1)<<10))   
#define CG_CLOCKA_MS_PORT_K_BIT         ((uint32_t)((0x1)<<9))
#define CG_CLOCKA_MS_PORT_J_BIT         ((uint32_t)((0x1)<<8))
#define CG_CLOCKA_MS_PORT_H_BIT         ((uint32_t)((0x1)<<7))
#define CG_CLOCKA_MS_PORT_G_BIT         ((uint32_t)((0x1)<<6))
#define CG_CLOCKA_MS_PORT_F_BIT         ((uint32_t)((0x1)<<5))
#define CG_CLOCKA_MS_PORT_E_BIT         ((uint32_t)((0x1)<<4))
#define CG_CLOCKA_MS_PORT_D_BIT         ((uint32_t)((0x1)<<3))
#define CG_CLOCKA_MS_PORT_C_BIT         ((uint32_t)((0x1)<<2))
#define CG_CLOCKA_MS_PORT_B_BIT         ((uint32_t)((0x1)<<1))   
#define CG_CLOCKA_MS_PORT_A_BIT         ((uint32_t)(0x1))   

#define CG_CLOCKB_MS_SIWDT_BIT          ((uint32_t)((0x1)<<31))
#define CG_CLOCKB_MS_NBDIF_BIT          ((uint32_t)((0x1)<<30))
#define CG_CLOCKB_MS_DMAC_BIT           ((uint32_t)((0x1)<<17))
#define CG_CLOCKB_MS_TRGSEL_BIT         ((uint32_t)((0x1)<<16))
#define CG_CLOCKB_MS_TRM_BIT            ((uint32_t)((0x1)<<15))
#define CG_CLOCKB_MS_OFD_BIT            ((uint32_t)((0x1)<<14))
#define CG_CLOCKB_MS_RAMP_CH1_BIT       ((uint32_t)((0x1)<<13))
#define CG_CLOCKB_MS_AVE_CH0_BIT        ((uint32_t)((0x1)<<12))
#define CG_CLOCKB_MS_APMD_CH2_BIT       ((uint32_t)((0x1)<<11))   
#define CG_CLOCKB_MS_APMD_CH1_BIT       ((uint32_t)((0x1)<<10))   
#define CG_CLOCKB_MS_APMD_CH0_BIT       ((uint32_t)((0x1)<<9))
#define CG_CLOCKB_MS_AENC_CH2_BIT       ((uint32_t)((0x1)<<8))
#define CG_CLOCKB_MS_AENC_CH1_BIT       ((uint32_t)((0x1)<<7))
#define CG_CLOCKB_MS_AENC_CH0_BIT       ((uint32_t)((0x1)<<6))
#define CG_CLOCKB_MS_OAMP_BIT           ((uint32_t)((0x1)<<5))
#define CG_CLOCKB_MS_ADC_C_BIT          ((uint32_t)((0x1)<<4))
#define CG_CLOCKB_MS_ADC_B_BIT          ((uint32_t)((0x1)<<3))
#define CG_CLOCKB_MS_ADC_A_BIT          ((uint32_t)((0x1)<<2))
#define CG_CLOCKB_MS_T32_CH5_BIT        ((uint32_t)((0x1)<<1))   
#define CG_CLOCKB_MS_T32_CH4_BIT        ((uint32_t)(0x1))   

#define CG_CLOCKA_HS_RAMP_CH0_BIT       ((uint32_t)((0x1)<<1))   
#define CG_CLOCKA_HS_CRC_BIT            ((uint32_t)(0x1))   

/**
 *  @}
 */ /* End of group CG_CGOSCCR_OSCF */
/**
 *  @}
 */ /* End of group CG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Exported_define CG Exported Define
 *  @{
 */
/* no define */
/**
 *  @}
 */ /* End of group CG_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Exported_typedef CG Exported Typedef
 *  @{
 */
/** 
 * @brief  CG member.
*/
/*----------------------------------*/
typedef struct
{
    TSB_CG_TypeDef       *p_instance;     /*!< Registers base address.           */
} cg_t;

/**
 *  @}
 */ /* End of group CG_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Exported_functions CG Exported Functions
 *  @{
 */
uint32_t    cg_get_phyt0(cg_t *p_obj);
Result  cg_ihosc_enable(cg_t *p_obj);
Result  cg_ihosc_disable(cg_t *p_obj);
Result cg_ehosc_enable(cg_t *p_obj);
Result cg_ehosc_disable(cg_t *p_obj);
uint32_t cg_fosc_status_get(cg_t *p_obj);
Result cg_middle_clock_a_enable(cg_t *p_obj, uint32_t enableBits);
Result cg_middle_clock_b_enable(cg_t *p_obj, uint32_t enableBits);

/**
 *  @}
 */ /* End of group CG_Exported_functions */

/**
 *  @}
 */ /* End of group CG */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __CG_H */


