/* THE SOURCE CODE AND ITS
* RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
* WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
* AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
* INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
* DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
* INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
* MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
* TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
* LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
* FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
* LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
* OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
* ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
* ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
* FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
* OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
*/
#include <stdint.h>
#include <string.h>
#include "api.h"
#include "hal_cg.h"
#include "TMPM4Ky_adc.h"
#include "motor_control.h"
#include "fwmathlib.h"

#define cPMD0_PREGS				(PMD_REG0_3)
#define cPMD1_PREGS				(PMD_REG0_3)
#define cPMD2_PREGS				(PMD_REG0_3)
/*--------------------------------------------------------------------------------------*/
#define cPMD0_PREGS_3SHUNT		((cPMD0_PREGS << 0U) | (cPMD0_PREGS << 4U) | (cPMD0_PREGS << 8U) | (cPMD0_PREGS << 12U) | (cPMD0_PREGS << 16U) | (cPMD0_PREGS << 20U))
#define cPMD0_PREGS_1SHUNT 		((cPMD0_PREGS << 0U) | (cPMD0_PREGS << 4U) | (cPMD0_PREGS << 8U))
#define cPMD0_ADREG_OFFSET		(cPMD0_PREGS * 4)

#define cPMD1_PREGS_3SHUNT		((cPMD1_PREGS << 0U) | (cPMD1_PREGS << 4U) | (cPMD1_PREGS << 8U) | (cPMD1_PREGS << 12U) | (cPMD1_PREGS << 16U) | (cPMD1_PREGS << 20U))
#define cPMD1_PREGS_1SHUNT 		((cPMD1_PREGS << 0U) | (cPMD1_PREGS << 4U) | (cPMD1_PREGS << 8U))
#define cPMD1_ADREG_OFFSET		(cPMD1_PREGS * 4)

#define cPMD2_PREGS_3SHUNT		((cPMD2_PREGS << 0U) | (cPMD2_PREGS << 4U) | (cPMD2_PREGS << 8U) | (cPMD2_PREGS << 12U) | (cPMD2_PREGS << 16U) | (cPMD2_PREGS << 20U))
#define cPMD2_PREGS_1SHUNT 		((cPMD2_PREGS << 0U) | (cPMD2_PREGS << 4U) | (cPMD2_PREGS << 8U))
#define cPMD2_ADREG_OFFSET		(cPMD2_PREGS * 4)

    typedef enum {
        ADC_CMPCR_0 = 0U,
        ADC_CMPCR_1 = 1U
    } ADC_CMPCRx;
#define IS_ADC_CMPCRx(param)        ((param) <= ADC_CMPCR_1)

    typedef enum {
        ADC_LARGER_THAN_CMP_REG = 0U,
        ADC_SMALLER_THAN_CMP_REG = 1U
    } ADC_CmpCondition;
#define IS_ADC_CMPCONDITION(param)   ((param) <= ADC_SMALLER_THAN_CMP_REG)
    
/** 
  * @brief  ADC Monitor Init Structure definition
  */
    typedef struct {
        ADC_CMPCRx CMPCRx;      /*!< Select ADC_CMPCR_0 for ADxCMPCR0 or ADC_CMPCR_1 for ADxCMPCR1 */
        ADC_REGx ResultREGx;    /*!< Select which result register among ADC_REG0 to ADC_REG11 will be used */
        uint32_t CmpTimes;      /*!< How many times will Comparison be counted, range from 1 to 16 */
        ADC_CmpCondition Condition;     /*!< Conditon to compare ADxREGm with ADxCMPn. (x = A, B; m = 0 to 11; n = 0, 1) */
        uint32_t CmpValue;      /*!< Comparison value to be set in ADxCMP0 or ADxCMP1, max value is 4095 */
    } ADC_MonitorTypeDef;

#define IS_ADC_CMPTIMES(param)  (((param) >= 1U) && ((param) <= 16U))
#define IS_ADC_CMPVALUE_12BIT(param)   ((param) <= 4095U)

void     HAL_ADC_SetMonitor(TSB_AD_TypeDef * ADx, ADC_MonitorTypeDef * Monitor);
void     HAL_ADC_DisableMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx CMPCRx);
void     HAL_ADC_EnableMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx CMPCRx);

/* Trigger configuration for Undervoltage Channel A */
static ADC_MonitorTypeDef UndervoltageA = { ADC_CMPCR_0,
                                            VDC_MEASURE0_REG,
                                            VDC_MEASURE_TIMES,
                                            ADC_SMALLER_THAN_CMP_REG,
                                            0};
/* Trigger configuration for Overvoltage Channel A */
static ADC_MonitorTypeDef OvervoltageA  = { ADC_CMPCR_1,
                                            VDC_MEASURE0_REG,
                                            VDC_MEASURE_TIMES,
                                            ADC_LARGER_THAN_CMP_REG,
                                            0};

/* Trigger configuration for Undervoltage Channel B */
static ADC_MonitorTypeDef UndervoltageB = { ADC_CMPCR_0,
                                            VDC_MEASURE1_REG,
                                            VDC_MEASURE_TIMES,
                                            ADC_SMALLER_THAN_CMP_REG,
                                            0};
/* Trigger configuration for Overvoltage Channel B */
static ADC_MonitorTypeDef OvervoltageB  = { ADC_CMPCR_1,
                                            VDC_MEASURE1_REG,
                                            VDC_MEASURE_TIMES,
                                            ADC_LARGER_THAN_CMP_REG,
                                            0};

/* Trigger configuration for Undervoltage Channel C */
static ADC_MonitorTypeDef UndervoltageC = { ADC_CMPCR_0,
                                            VDC_MEASURE2_REG,
                                            VDC_MEASURE_TIMES,
                                            ADC_SMALLER_THAN_CMP_REG,
                                            0};
/* Trigger configuration for Overvoltage Channel C */
static ADC_MonitorTypeDef OvervoltageC  = { ADC_CMPCR_1,
                                            VDC_MEASURE2_REG,
                                            VDC_MEASURE_TIMES,
                                            ADC_LARGER_THAN_CMP_REG,
                                            0};

/**
  * @brief  Set the monitor function of the specified ADC unit and enable it.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *     TSB_ADA, TSB_ADB, TSB_ADC
  * @param  Monitor: The structure containing ADC Monitor configuration
  *         Refer to the members of ADC_MonitorTypeDef in .h file for more detail usage.
  * @retval None
  */
void HAL_ADC_SetMonitor(TSB_AD_TypeDef * ADx, ADC_MonitorTypeDef * Monitor)
{
    uint32_t tmp = 0U;
    uint32_t tmpComp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_POINTER_NOT_NULL(Monitor));
    assert_param(IS_ADC_CMPCRx(Monitor->CMPCRx));
    assert_param(IS_ADC_REGx(Monitor->ResultREGx));
    assert_param(IS_ADC_CMPTIMES(Monitor->CmpTimes));
    assert_param(IS_ADC_CMPCONDITION(Monitor->Condition));
    assert_param(IS_ADC_CMPVALUE_12BIT(Monitor->CmpValue));

    tmpComp = Monitor->CmpValue << 4U;  /* 12bit mode: use bit15 to bit4 */

    tmp |= (uint32_t) Monitor->ResultREGx;
    tmp |= (uint32_t) Monitor->Condition << 5U;
    tmp |= (uint32_t) (Monitor->CmpTimes - 1U) << 8U;

    if (Monitor->CMPCRx == ADC_CMPCR_0) {
        ADx->CMPCR0 = tmp;
        ADx->CMP0 = tmpComp;
    } else {
        ADx->CMPCR1 = tmp;
        ADx->CMP1 = tmpComp;
    }
}

/**
  * @brief  Disable the monitor function of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *     TSB_ADA, TSB_ADB, TSB_ADC
  * @param  CMPCRx: Select which compare control register will be used.
  *   This parameter can be one of the following values:
  *     ADC_CMPCR_0, ADC_CMPCR_1
  * @retval None
  */
void HAL_ADC_DisableMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx CMPCRx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_CMPCRx(CMPCRx));

    if (CMPCRx == ADC_CMPCR_0) {
        tmp = ADx->CMPEN;
        ADx->CMPEN = tmp & 2;
    } else {
        tmp = ADx->CMPEN;
        ADx->CMPEN = tmp & 1;
    }
}

/**
  * @brief  Enable the monitor function of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *     TSB_ADA, TSB_ADB, TSB_ADC
  * @param  CMPCRx: Select which compare control register will be used.
  *   This parameter can be one of the following values:
  *     ADC_CMPCR_0, ADC_CMPCR_1
  * @retval None
  */
void HAL_ADC_EnableMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx CMPCRx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_CMPCRx(CMPCRx));

    if (CMPCRx == ADC_CMPCR_0) {
        tmp = ADx->CMPEN;
        ADx->CMPEN = tmp | (1<<0);
    } else {
        tmp = ADx->CMPEN;
        ADx->CMPEN = tmp | (1<<1);
    }
}


void INTADACP0_IRQHandler(void)
{
  HAL_ADC_DisableMonitor(TSB_ADA,ADC_CMPCR_0);
  MotorErrorField[0].Error |= VE_SWUNDERVOLTAGE;
  MotorControl[0].ActualStage.main = Stage_Emergency;
}

void INTADACP1_IRQHandler(void)
{
  HAL_ADC_DisableMonitor(TSB_ADA,ADC_CMPCR_1);
  MotorErrorField[0].Error |= VE_SWOVERVOLTAGE;
  MotorControl[0].ActualStage.main = Stage_Emergency;
}

void INTADBCP0_IRQHandler(void)
{
  HAL_ADC_DisableMonitor(TSB_ADB,ADC_CMPCR_0);
  MotorErrorField[1].Error |= VE_SWUNDERVOLTAGE;
  MotorControl[1].ActualStage.main = Stage_Emergency;
}

void INTADBCP1_IRQHandler(void)
{
  HAL_ADC_DisableMonitor(TSB_ADB,ADC_CMPCR_1);
  MotorErrorField[1].Error |= VE_SWOVERVOLTAGE;
  MotorControl[1].ActualStage.main = Stage_Emergency;
}

void INTADCCP0_IRQHandler(void)
{
  HAL_ADC_DisableMonitor(TSB_ADC,ADC_CMPCR_0);
  MotorErrorField[2].Error |= VE_SWUNDERVOLTAGE;
  MotorControl[2].ActualStage.main = Stage_Emergency;
}

void INTADCCP1_IRQHandler(void)
{
  HAL_ADC_DisableMonitor(TSB_ADC,ADC_CMPCR_1);
  MotorErrorField[2].Error |= VE_SWOVERVOLTAGE;
  MotorControl[2].ActualStage.main = Stage_Emergency;
}


/*! \brief  Setup ADC Compare registers
*
* @param  channel_number:  channel to control
* @retval None
*/
void HAL_ADC_OverUndervoltageDetect(uint8_t channel_number)
{
  ADC_Result         result;                                                    /* ADC measurement value */

  switch (channel_number)
  {
  case 0:
    UndervoltageA.CmpValue = 0x0;
    OvervoltageA.CmpValue  = 0xfff;

    if (SystemValues[channel_number].SwUndervoltage != 0)
    {
      uint16_t cmp_value;

      cmp_value = 0xfff * SystemValues[channel_number].SwUndervoltage
                        * ChannelValues[channel_number].SensitivityVoltageMeasure.value
                        / 5
                        / pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit);

      if (cmp_value > 0xfff)
        cmp_value = 0xfff;

      UndervoltageA.CmpValue = cmp_value;
      HAL_ADC_SetMonitor(TSB_ADA, &UndervoltageA);
      HAL_ADC_EnableMonitor(TSB_ADA, ADC_CMPCR_0);
      NVIC_EnableIRQ(INTADACP0_IRQn);
    }
    else
      NVIC_DisableIRQ(INTADACP0_IRQn);

    if (SystemValues[channel_number].SwOvervoltage != 0)
    {
      uint16_t cmp_value;

      cmp_value = 0xfff * SystemValues[channel_number].SwOvervoltage
                        * ChannelValues[channel_number].SensitivityVoltageMeasure.value
                        / 5
                        / pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit);

      if (cmp_value > 0xfff)
        cmp_value = 0xfff;

      OvervoltageA.CmpValue = cmp_value;
      HAL_ADC_SetMonitor(TSB_ADA, &OvervoltageA);
      HAL_ADC_EnableMonitor(TSB_ADA, ADC_CMPCR_1);
      NVIC_EnableIRQ(INTADACP1_IRQn);
    }
    else
      NVIC_DisableIRQ(INTADACP1_IRQn);

    ADC_Start(TSB_ADA,ADC_TRG_CONTINUE);

    /* Even if the TD says Level Check - it's only reacting of edge
       do a manual check of the value */
    result=ADC_GetConvertResult(TSB_ADA, VDC_MEASURE0_REG);

    if (result.Bit.ADResult < UndervoltageA.CmpValue)
      INTADACP0_IRQHandler();

    if (result.Bit.ADResult > OvervoltageA.CmpValue)
      INTADACP1_IRQHandler();

   break;

  case 1:
    UndervoltageB.CmpValue = 0x0;
    OvervoltageB.CmpValue  = 0xfff;

    if (SystemValues[channel_number].SwUndervoltage != 0)
    {
      uint16_t cmp_value;

      cmp_value = 0xfff * SystemValues[channel_number].SwUndervoltage
                        * ChannelValues[channel_number].SensitivityVoltageMeasure.value
                        / 5
                        / pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit);


      if (cmp_value > 0xfff)
        cmp_value = 0xfff;

      UndervoltageB.CmpValue = cmp_value;
      HAL_ADC_SetMonitor(TSB_ADB, &UndervoltageB);
      HAL_ADC_EnableMonitor(TSB_ADB, ADC_CMPCR_0);
      NVIC_EnableIRQ(INTADBCP0_IRQn);
    }
    else
      NVIC_DisableIRQ(INTADBCP0_IRQn);

    if (SystemValues[channel_number].SwOvervoltage != 0)
    {
      uint16_t cmp_value;

      cmp_value = 0xfff * SystemValues[channel_number].SwOvervoltage
                        * ChannelValues[channel_number].SensitivityVoltageMeasure.value
                        / 5
                        / pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit);

      if (cmp_value > 0xfff)
        cmp_value = 0xfff;

      OvervoltageB.CmpValue = cmp_value;
      HAL_ADC_SetMonitor(TSB_ADB, &OvervoltageB);
      HAL_ADC_EnableMonitor(TSB_ADB, ADC_CMPCR_1);
      NVIC_EnableIRQ(INTADBCP1_IRQn);
    }
    else
      NVIC_DisableIRQ(INTADBCP1_IRQn);

    ADC_Start(TSB_ADB,ADC_TRG_CONTINUE);

    /* Even if the TD says Level Check - it's only reacting of edge
       do a manual check of the value */
    result=ADC_GetConvertResult(TSB_ADB, VDC_MEASURE1_REG);

    if (result.Bit.ADResult < UndervoltageB.CmpValue)
      INTADBCP0_IRQHandler();

    if (result.Bit.ADResult > OvervoltageB.CmpValue)
      INTADBCP1_IRQHandler();

    break;
    
  case 2:
    UndervoltageC.CmpValue = 0x0;
    OvervoltageC.CmpValue  = 0xfff;

    if (SystemValues[channel_number].SwUndervoltage != 0)
    {
      uint16_t cmp_value;

      cmp_value = 0xfff * SystemValues[channel_number].SwUndervoltage
                        * ChannelValues[channel_number].SensitivityVoltageMeasure.value
                        / 5
                        / pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit);


      if (cmp_value > 0xfff)
        cmp_value = 0xfff;

      UndervoltageC.CmpValue = cmp_value;
      HAL_ADC_SetMonitor(TSB_ADC, &UndervoltageC);
      HAL_ADC_EnableMonitor(TSB_ADC, ADC_CMPCR_0);
      NVIC_EnableIRQ(INTADCCP0_IRQn);
    }
    else
      NVIC_DisableIRQ(INTADCCP0_IRQn);

    if (SystemValues[channel_number].SwOvervoltage != 0)
    {
      uint16_t cmp_value;

      cmp_value = 0xfff * SystemValues[channel_number].SwOvervoltage
                        * ChannelValues[channel_number].SensitivityVoltageMeasure.value
                        / 5
                        / pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit);

      if (cmp_value > 0xfff)
        cmp_value = 0xfff;

      OvervoltageC.CmpValue = cmp_value;
      HAL_ADC_SetMonitor(TSB_ADC, &OvervoltageC);
      HAL_ADC_EnableMonitor(TSB_ADC, ADC_CMPCR_1);
      NVIC_EnableIRQ(INTADCCP1_IRQn);
    }
    else
      NVIC_DisableIRQ(INTADCCP1_IRQn);

    ADC_Start(TSB_ADC,ADC_TRG_CONTINUE);

    /* Even if the TD says Level Check - it's only reacting of edge
       do a manual check of the value */
    result=ADC_GetConvertResult(TSB_ADC, VDC_MEASURE2_REG);

    if (result.Bit.ADResult < UndervoltageC.CmpValue)
      INTADCCP0_IRQHandler();

    if (result.Bit.ADResult > OvervoltageC.CmpValue)
      INTADCCP1_IRQHandler();

    break;
    
  default:
    assert_param(0);
    break;
  }
}

/*! \brief  Get VDC voltage value
  *
  * @param  channel_number:  channel read
  * @retval None
*/
uint16_t HAL_ADC_GetVDC(uint8_t channel_number)
{
  ADC_Result result;                                                    /* ADC measurement value */

  memset(&result,0,sizeof(result));

  switch (channel_number)
  {
  case 0:
    result=ADC_GetConvertResult(TSB_ADA, VDC_MEASURE0_REG);
   break;
   
  case 1:
    ADC_SetSWTrg(TSB_ADB, VDC_MEASURE1_REG, TRG_ENABLE(AIN1_VDC));
    ADC_Start(TSB_ADB, ADC_TRG_SINGLE);
    while (ADC_GetConvertState(TSB_ADB,ADC_TRG_SINGLE)==BUSY);
    result=ADC_GetConvertResult(TSB_ADB, VDC_MEASURE1_REG);
    break;
 
  case 2:
    ADC_SetSWTrg(TSB_ADC, VDC_MEASURE2_REG, TRG_ENABLE(AIN2_VDC));
    ADC_Start(TSB_ADC, ADC_TRG_SINGLE);
    while (ADC_GetConvertState(TSB_ADC,ADC_TRG_SINGLE)==BUSY);
    result=ADC_GetConvertResult(TSB_ADC, VDC_MEASURE2_REG);
    break;
    
  default:
    assert_param(0);  
    break;
  }

  return result.Bit.ADResult;
}

void INTADAPDA_IRQHandler(void)
{
}

/*! \brief  Initialize the ADC
*
* @param  channel_number:  channel to configure
* @param  mesurement_type: selects between 1-shunt, 3-shunt or 2 sensor mode
* @retval None
*/
void HAL_ADC_Init(uint8_t channel_number, CURRENT_MEASUREMENT mesurement_type)
{
  TSB_AD_TypeDef* ADx;
  uint32_t data32;
  uint32_t OldState = 0;

  switch (channel_number)
  {
  case 0:
    ADx= TSB_ADA;
    OldState = TSB_CG_SPCLKEN_ADCKEN0;
    break;
  case 1:
    ADx= TSB_ADB;
    OldState = TSB_CG_SPCLKEN_ADCKEN1;
    break;
  case 2:
    ADx= TSB_ADC;
    OldState = TSB_CG_SPCLKEN_ADCKEN2;
    break;
  default:
    assert_param(0);
    break;
  }

  /***** PMD TRG Start mode set *****/

  HAL_CG_SetFcSupplyPeriph((HAL_CG_FC_PERIPH_DNF_A | HAL_CG_FC_PERIPH_DNF_B | HAL_CG_FC_PERIPH_DNF_C), ENABLE);
  
  /* No need to call the function if ADC clock supply is already enabled.
     This is to avoid indefinite wait (for ADC conversion to be stopped) inside function   
     during Deadtime setting from PC tool */
  if(OldState != ENABLE)
  {
      HAL_CG_SetAdcClkSupply(ADx, ENABLE);
  }

  switch (channel_number)
  {
  case 0:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_PORT_L);
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_ADC_UA);
    TSB_CG_SPCLKEN_ADCKEN0 = 0x00000001;
    break;

  case 1:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_PORT_L);
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_ADC_UA);
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_PORT_K);
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_ADC_UB);
    TSB_CG_SPCLKEN_ADCKEN1 = 0x00000001;
    break;

  case 2:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_PORT_L);
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_ADC_UA);
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_PORT_J);
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_ADC_UC);
    TSB_CG_SPCLKEN_ADCKEN2 = 0x00000001;
    break;

  default:
    assert_param(0);
    break;
  }

  switch (mesurement_type)
  {
  case CURRENT_SHUNT_3:
    switch (channel_number)
    {
    case 0:
      ADx->PSEL0  = TRG_ENABLE(PMD_PROG0);		/* PMD0 TRG0:BProg0 at Sector0	3-shunt */
      ADx->PSEL1  = TRG_ENABLE(PMD_PROG1);		/* PMD0 TRG1:BProg1 at Sector1	3-shunt */
      ADx->PSEL2  = TRG_ENABLE(PMD_PROG2);		/* PMD0 TRG2:BProg2 at Sector2	3-shunt */
      ADx->PSEL3  = TRG_ENABLE(PMD_PROG3);		/* PMD0 TRG3:BProg3 at Sector3	3-shunt */
      ADx->PSEL4  = TRG_ENABLE(PMD_PROG4);		/* PMD0 TRG4:BProg4 at Sector4	3-shunt */
      ADx->PSEL5  = TRG_ENABLE(PMD_PROG5);		/* PMD0 TRG5:BProg5 at Sector5	3-shunt */
      ADx->PINTS0 = PMD_INTADPDA;			/* End of all prog generate INTADA */
      ADx->PINTS1 = PMD_INTADPDA;
      ADx->PINTS2 = PMD_INTADPDA;
      ADx->PINTS3 = PMD_INTADPDA;
      ADx->PINTS4 = PMD_INTADPDA;
      ADx->PINTS5 = PMD_INTADPDA;
      ADx->PINTS6 = PMD_INTNONE;
      ADx->PINTS7 = PMD_INTNONE;

      /* ADx->PSET5,PSET0	REG0:V, REG1:W, REG2:U, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_V    << 5U) | AIN0_3PHASE_V)) \
              | ((0x80 | (VE_PHASE_W    << 5U) | AIN0_3PHASE_W)  << 8U) \
              | ((0x80 | (VE_PHASE_U    << 5U) | AIN0_3PHASE_U)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN0_VDC) << 24U));
      ADx->PSET5 = data32;
      ADx->PSET0 = data32;

      /* ADx->PSET1,PSET2	REG0:W, REG1:U, REG2:V, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_W    << 5U) | AIN0_3PHASE_W)) \
              | ((0x80 | (VE_PHASE_U    << 5U) | AIN0_3PHASE_U)  << 8U) \
              | ((0x80 | (VE_PHASE_V    << 5U) | AIN0_3PHASE_V)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN0_VDC) << 24U));
      ADx->PSET1 = data32;
      ADx->PSET2 = data32;

      /* ADx->PSET3,PSET4	REG0:U, REG1:V, REG2:W, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_U    << 5U) | AIN0_3PHASE_U)) \
              | ((0x80 | (VE_PHASE_V    << 5U) | AIN0_3PHASE_V)  << 8U) \
              | ((0x80 | (VE_PHASE_W    << 5U) | AIN0_3PHASE_W)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN0_VDC) << 24U));
    ADx->PSET3	= data32;
    ADx->PSET4	= data32;

    ADx->PSET6	= 0;
    ADx->PSET7	= 0;
      break;
          
    case 1:
      ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD1 TRG0:BProg0 at Sector0	3-shunt */
      ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD1 TRG1:BProg1 at Sector1	3-shunt */
      ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD1 TRG2:BProg2 at Sector2	3-shunt */
      ADx->PSEL9  = TRG_ENABLE(PMD_PROG3);		/* PMD1 TRG3:BProg3 at Sector3	3-shunt */
      ADx->PSEL10 = TRG_ENABLE(PMD_PROG4);		/* PMD1 TRG4:BProg4 at Sector4	3-shunt */
      ADx->PSEL11 = TRG_ENABLE(PMD_PROG5);		/* PMD1 TRG5:BProg5 at Sector5	3-shunt */

      ADx->PINTS0 = PMD_INTADPDB;			/* End of all prog generate INTADB */
      ADx->PINTS1 = PMD_INTADPDB;
      ADx->PINTS2 = PMD_INTADPDB;
      ADx->PINTS3 = PMD_INTADPDB;
      ADx->PINTS4 = PMD_INTADPDB;
      ADx->PINTS5 = PMD_INTADPDB;
      ADx->PINTS6 = PMD_INTNONE;
      ADx->PINTS7 = PMD_INTNONE;

      /* ADx->PSET5,PSET0	REG0:V, REG1:W, REG2:U, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_V    << 5U) | AIN1_3PHASE_V)) \
              | ((0x80 | (VE_PHASE_W    << 5U) | AIN1_3PHASE_W)  << 8U) \
              | ((0x80 | (VE_PHASE_U    << 5U) | AIN1_3PHASE_U)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN1_VDC) << 24U));
      ADx->PSET5 = data32;
      ADx->PSET0 = data32;

      /* ADx->PSET1,PSET2	REG0:W, REG1:U, REG2:V, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_W    << 5U) | AIN1_3PHASE_W)) \
              | ((0x80 | (VE_PHASE_U    << 5U) | AIN1_3PHASE_U)  << 8U) \
              | ((0x80 | (VE_PHASE_V    << 5U) | AIN1_3PHASE_V)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN1_VDC) << 24U));
      ADx->PSET1 = data32;
      ADx->PSET2 = data32;

      /* ADx->PSET3,PSET4	REG0:U, REG1:V, REG2:W, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_U    << 5U) | AIN1_3PHASE_U)) \
              | ((0x80 | (VE_PHASE_V    << 5U) | AIN1_3PHASE_V)  << 8U) \
              | ((0x80 | (VE_PHASE_W    << 5U) | AIN1_3PHASE_W)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN1_VDC) << 24U));
    ADx->PSET3	= data32;
    ADx->PSET4	= data32;

    ADx->PSET6	= 0;
    ADx->PSET7	= 0;
    break;
    
    case 2:
      ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD2 TRG0:BProg0 at Sector0	3-shunt */
      ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD2 TRG1:BProg1 at Sector1	3-shunt */
      ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD2 TRG2:BProg2 at Sector2	3-shunt */
      ADx->PSEL9  = TRG_ENABLE(PMD_PROG3);		/* PMD2 TRG3:BProg3 at Sector3	3-shunt */
      ADx->PSEL10 = TRG_ENABLE(PMD_PROG4);		/* PMD2 TRG4:BProg4 at Sector4	3-shunt */
      ADx->PSEL11 = TRG_ENABLE(PMD_PROG5);		/* PMD2 TRG5:BProg5 at Sector5	3-shunt */

      ADx->PINTS0 = PMD_INTADPDB; 					/* End of all prog generate INTADB */
      ADx->PINTS1 = PMD_INTADPDB;
      ADx->PINTS2 = PMD_INTADPDB;
      ADx->PINTS3 = PMD_INTADPDB;
      ADx->PINTS4 = PMD_INTADPDB;
      ADx->PINTS5 = PMD_INTADPDB;
      ADx->PINTS6 = PMD_INTNONE;
      ADx->PINTS7 = PMD_INTNONE;

      /* ADx->PSET5,PSET0	REG0:V, REG1:W, REG2:U, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_V    << 5U) | AIN2_3PHASE_V)) \
              | ((0x80 | (VE_PHASE_W    << 5U) | AIN2_3PHASE_W)  << 8U) \
              | ((0x80 | (VE_PHASE_U    << 5U) | AIN2_3PHASE_U)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN2_VDC) << 24U));
      ADx->PSET5 = data32;
      ADx->PSET0 = data32;

      /* ADx->PSET1,PSET2	REG0:W, REG1:U, REG2:V, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_W    << 5U) | AIN2_3PHASE_W)) \
              | ((0x80 | (VE_PHASE_U    << 5U) | AIN2_3PHASE_U)  << 8U) \
              | ((0x80 | (VE_PHASE_V    << 5U) | AIN2_3PHASE_V)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN2_VDC) << 24U));
      ADx->PSET1 = data32;
      ADx->PSET2 = data32;

      /* ADx->PSET3,PSET4	REG0:U, REG1:V, REG2:W, REG3:Vdc */
      data32 = (((0x80 | (VE_PHASE_U    << 5U) | AIN2_3PHASE_U)) \
              | ((0x80 | (VE_PHASE_V    << 5U) | AIN2_3PHASE_V)  << 8U) \
              | ((0x80 | (VE_PHASE_W    << 5U) | AIN2_3PHASE_W)  << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN2_VDC) << 24U));
    ADx->PSET3	= data32;
    ADx->PSET4	= data32;

    ADx->PSET6	= 0;
    ADx->PSET7	= 0;
      break;   
      
    default:
      assert_param(0);
      break;
    }

    break;

  case CURRENT_SHUNT_1:
    switch (channel_number)
    {
    case 0:
      ADx->PSEL0  = TRG_ENABLE(PMD_PROG0);		/* PMD0 TRG0:BProg0 1-shunt */
      ADx->PSEL1  = TRG_ENABLE(PMD_PROG1);		/* PMD0 TRG1:BProg1 1-shunt */
      ADx->PSEL2  = TRG_ENABLE(PMD_PROG2);		/* PMD0 TRG2:BProg2 - Vdc */
      ADx->PSEL3  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG3:BProg3 */
      ADx->PSEL4  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG4:BProg4 */
      ADx->PSEL5  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG5:BProg5 */
      ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADA */
      ADx->PINTS1 = PMD_INTADPDA;
      ADx->PINTS2 = PMD_INTNONE;
      ADx->PINTS3 = PMD_INTNONE;
      ADx->PINTS4 = PMD_INTNONE;
      ADx->PINTS5 = PMD_INTNONE;
      ADx->PINTS6 = PMD_INTNONE;
      ADx->PINTS7 = PMD_INTNONE;
      
      /* ADx->PSET0	REG0:Idc , REG1: , REG2: , REG3: */
      data32 = (((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE)) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE) << 8U) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE) << 16U) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_VDC) << 24U));
      ADx->PSET0 = data32;

      /* ADx->PSET1	REG0: , REG1: Idc, REG2: , REG3: */
      data32 = (((0x80 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE)) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE) << 8U) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE) << 16U) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_VDC) << 24U));
      ADx->PSET1 = data32;

      /* ADx->PSET2	REG0: , REG1: , REG2: , REG3: Vdc */
      data32 = (((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE)) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE) << 8U) \
              | ((0x00 | (VE_PHASE_NONE << 5U) | AIN0_1PHASE) << 16U) \
              | ((0x80 | (VE_PHASE_NONE << 5U) | AIN0_VDC) << 24U));
      ADx->PSET2 = data32;

      ADx->PSET3 = 0;
      ADx->PSET4 = 0;
      ADx->PSET5 = 0;
      ADx->PSET6 = 0;
      ADx->PSET7 = 0;          
      break;
    case 1:
      ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD1 TRG0:BProg0 1-shunt */
      ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD1 TRG1:BProg1 1-shunt */
      ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD1 TRG2:BProg2 */
      ADx->PSEL9  = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG3:BProg3 */
      ADx->PSEL10 = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG4:BProg4 */
      ADx->PSEL11 = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG5:BProg5 */

      ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADB */
      ADx->PINTS1 = PMD_INTNONE;
      ADx->PINTS2 = PMD_INTADPDB;
      ADx->PINTS3 = PMD_INTNONE;
      ADx->PINTS4 = PMD_INTNONE;
      ADx->PINTS5 = PMD_INTNONE;
      ADx->PINTS6 = PMD_INTNONE;
      ADx->PINTS7 = PMD_INTNONE;
      break;

    case 2:
      ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD2 TRG0:BProg0 1-shunt */
      ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD2 TRG1:BProg1 1-shunt */
      ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD2 TRG2:BProg2 */
      ADx->PSEL9  = TRG_DISABLE(PMD_PROG0);		/* PMD2 TRG3:BProg3 */
      ADx->PSEL10 = TRG_DISABLE(PMD_PROG0);		/* PMD2 TRG4:BProg4 */
      ADx->PSEL11 = TRG_DISABLE(PMD_PROG0);		/* PMD2 TRG5:BProg5 */
      /* Getting AD convert registers address. */

      ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADB */
      ADx->PINTS1 = PMD_INTNONE;
      ADx->PINTS2 = PMD_INTADPDB;
      ADx->PINTS3 = PMD_INTNONE;
      ADx->PINTS4 = PMD_INTNONE;
      ADx->PINTS5 = PMD_INTNONE;
      ADx->PINTS6 = PMD_INTNONE;
      ADx->PINTS7 = PMD_INTNONE;
      break;

    default:
      assert_param(0);
      break;
    }

    ADx->PSET3 = 0;
    ADx->PSET4 = 0;
    ADx->PSET5 = 0;
    ADx->PSET6 = 0;
    ADx->PSET7 = 0;
    break;

  default:
    assert_param(0);
    break;
  }

  /* Set ADC Unit A Clock */
  ADC_SetClk(ADx, ADC_SAMPLING_PERIOD_2, ADC_SCLK_2);

  switch (channel_number)
  {
  case 0:
    ADC_SetSWTrg(ADx, VDC_MEASURE0_REG, TRG_ENABLE(AIN0_VDC));
    ADC_Start(ADx, ADC_TRG_CONTINUE);
    ADx->TSET5 = 0x29;
    break;
  case 1:
    NVIC_SetPriority(INTADBPDB_IRQn, INTERRUPT_PRIORITY_ADC);
    NVIC_ClearPendingIRQ(INTADBPDB_IRQn);
    NVIC_EnableIRQ(INTADBPDB_IRQn);
    break;
  case 2:
    NVIC_SetPriority(INTADCPDB_IRQn, INTERRUPT_PRIORITY_ADC);
    NVIC_ClearPendingIRQ(INTADCPDB_IRQn);
    NVIC_EnableIRQ(INTADCPDB_IRQn);
    break;
  default:
    assert_param(0);
    break;
  }


  /* Enable ADC Unit */
  ADC_Enable(ADx, ADC_CLKMOD_160M, ADC_REFSEL_AIN, ADC_RCUT_NORMAL);
}
