/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdint.h>
#include "config.h"
#include TMPM_HEADER_FILE
#include "hal_cg.h"
#include TMPM_CG_HEADER_FILE

#define CG_FCEN_WRITE0_MASK                  ((uint32_t)0x1C800000)
#define ADC_ST_ADBF_MASK                     ((uint32_t)0x00000080)
#define IS_ADC_UNIT(param)          ((param) == TSB_ADA || (param) == TSB_ADB || (param) == TSB_ADC)

cg_t p_obj = {TSB_CG};

int HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_A clock)
{  
  uint32_t value;
  
  value = 1 << clock;
  
  return cg_middle_clock_a_enable(&p_obj, value);
}

int HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_B clock)
{
  uint32_t value;
  
  value = (1 << clock) + 0x20000000; /* enum to value + bit 29 */
  
  return cg_middle_clock_b_enable(&p_obj, value);
}

/**
   * @brief  Enable or disable supplying clock fc to peripheries
   * @param  Periph: The target peripheral of CG supplies clock
   *   This parameter can be one of the following values or their combination:   
   *   CG_FC_PERIPH_DNF_A, CG_FC_PERIPH_DNF_B, CG_FC_PERIPH_DNF_C, CG_FC_PERIPH_ALL.
   * @param  NewState: New state of clock supply setting.
   *   This parameter can be one of the following values:
   *   DISABLE or ENABLE
   * @retval None
   */
void HAL_CG_SetFcSupplyPeriph(uint32_t Periph, FunctionalState NewState)
{
    uint32_t regval = TSB_CG->FCEN;

    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {   /* write '1' to enable */
        regval |= Periph;
    } else {                    /* clear to '0' to disable */
        regval &= ~Periph;
    }

    regval &= CG_FCEN_WRITE0_MASK;  /* CGFCEN bit 31-29,25-24,22-0 write "0" */
    TSB_CG->FCEN = regval;
}

/**
  * @brief  Enable or disable supplying clock (fc or fplladc) for ADC.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_ADA, TSB_ADB or TSB_ADC.
  * @param  NewState: New state of clock (fc or fplladc) supply setting for ADC.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void HAL_CG_SetAdcClkSupply(TSB_AD_TypeDef * ADx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Check ADC is not during conversion */
    while ((ADx->ST & ADC_ST_ADBF_MASK) != 0U) {
        /* Do nothing */
    }

    /* Set CGSPCLKEN<ADCKEN>  */
    if (NewState == ENABLE) {
		if (ADx == TSB_ADA) {
	        TSB_CG_SPCLKEN_ADCKEN0 = 1U;
		} else if (ADx == TSB_ADB) {
	        TSB_CG_SPCLKEN_ADCKEN1 = 1U;
		} else {
	        TSB_CG_SPCLKEN_ADCKEN2 = 1U;
		}
    } else {
		if (ADx == TSB_ADA) {
	        TSB_CG_SPCLKEN_ADCKEN0 = 0U;
		} else if (ADx == TSB_ADB) {
	        TSB_CG_SPCLKEN_ADCKEN1 = 0U;
		} else {
    	    TSB_CG_SPCLKEN_ADCKEN2 = 0U;
		}
    }
}


