/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#ifdef USE_ENCODER

#include TMPM_HEADER_FILE
#include "hal_gpio.h"
#include "hal_cg.h"

#include <core_cm4.h>

#include "hal_encoder.h"
#include "motor_control.h"
#include "hal_ve.h"
#include "board.h"
#include "txz_driver_def.h"
#include "motion_control.h"

#define ENC_DIVIDER_UP_BORDER    1500
#define ENC_DIVIDER_DOWN_BORDER  4000

#define FIXPOINT_12 0x1000                                                      /* fix point Q12 4096 */
#define FIXPOINT_15 0x8000                                                      /* fix point Q15 32768 */
#define FIXPOINT_16 0x10000                                                     /* fix point Q16 65536 */
#define FIXPOINT_31 0x80000000                                                  /* fix point Q31 */

#ifdef USE_MOTION_CONTROL
extern void MotionControl_IRQ_Callback(uint8_t channel, uint8_t irq_nr);
#endif /* USE_MOTION_CONTROL */

extern EncoderValues  EncoderData[MAX_CHANNEL];                                 /*<! generic encoder result data */
static uint8_t        input_nr[MAX_CHANNEL];
static int8_t         divider_nr[MAX_CHANNEL];
static FRACTIONAL     ENC_DeltaThetaBetweenTicks;
#ifdef USE_MOTION_CONTROL
static uint8_t        tweak_irq_nr[MAX_CHANNEL];
#endif /* USE_MOTION_CONTROL */

struct HallSectorValue
{
  uint8_t value;
  uint16_t angle;
};

struct DivideFactor_s
{
  uint8_t tncr;
  uint8_t factor;
};


static const struct HallSectorValue HallSectorTable[6] =
{
/* Enc Value  Angle of puls */
  {0x1,                 0},
  {0x3,                60},
  {0x2,               120},
  {0x6,               180},
  {0x4,               240},
  {0x5,               300},
};

static const struct DivideFactor_s DivideFactor[8] =
{
  {0x0,   1},
  {0x1,   2},
  {0x2,   4},
  {0x3,   8},
  {0x4,  16},
  {0x5,  32},
  {0x6,  64},
  {0x7, 128},
};

static void ENC_IOInit(uint8_t channel_number)
{
  HAL_GPIO_Port ENCPort= HAL_GPIO_PORT_F;
  uint8_t   ENCMask=0;

  switch (channel_number)
  {
  case 0:
    ENCPort = HAL_GPIO_PORT_N;
    if (input_nr[channel_number] >= 2)
      ENCMask |= HAL_GPIO_BIT_0 | HAL_GPIO_BIT_1;
    if ((input_nr[channel_number] == 1) || (input_nr[channel_number] == 3))
      ENCMask |= HAL_GPIO_BIT_2;
    break;
  case 1:
    ENCPort = HAL_GPIO_PORT_F;
    if (input_nr[channel_number] >= 2)
      ENCMask |= HAL_GPIO_BIT_3 | HAL_GPIO_BIT_4;
    if ((input_nr[channel_number] == 1) || (input_nr[channel_number] == 3))
      ENCMask |= HAL_GPIO_BIT_5;
    break;
  case 2:
    ENCPort = HAL_GPIO_PORT_D;
    if (input_nr[channel_number] >= 2)
      ENCMask |= HAL_GPIO_BIT_3 | HAL_GPIO_BIT_4;
    if ((input_nr[channel_number] == 1) || (input_nr[channel_number] == 3))
      ENCMask |= HAL_GPIO_BIT_5;
    break;
  default:
    assert_param(0);
    break;
  }
  HAL_GPIO_Init(ENCPort, ENCMask, HAL_GPIO_ENC);
}

/*! \brief  Get the Theta value from Encoder
  *
  * @param  channel_number:  channel to measure
  * @retval Angle in degree of sector
*/
uint16_t ENC_GetTheta(unsigned char channel_number)
{
  uint8_t   i=0;
  uint8_t   ENCStatus=0;

  static uint8_t old_status=0xff;

  switch (MotorParameterValues[channel_number].Encoder)
  {
  case ENCODER_HALL_UVW:
  case ENCODER_HALL_UV:
    break;
  default:
    return 0;
  }

  switch (channel_number)
  {
  case 0:
    /* get u-value */
    ENCStatus = HAL_GPIO_GetBit(HAL_GPIO_PORT_N, HAL_GPIO_BIT_0 );
    /* get v-value */
    ENCStatus |= HAL_GPIO_GetBit(HAL_GPIO_PORT_N,HAL_GPIO_BIT_1 ) <<1;
    /* get w-value */
    ENCStatus |= HAL_GPIO_GetBit(HAL_GPIO_PORT_N,HAL_GPIO_BIT_2 ) <<2;
    break;
  case 1:
    /* get u-value */
    ENCStatus = HAL_GPIO_GetBit(HAL_GPIO_PORT_F, HAL_GPIO_BIT_3);
    /* get v-value */
    ENCStatus |= HAL_GPIO_GetBit(HAL_GPIO_PORT_F, HAL_GPIO_BIT_4 ) <<1;
    /* get w-value */
    ENCStatus |= HAL_GPIO_GetBit(HAL_GPIO_PORT_F, HAL_GPIO_BIT_5) <<2;
    break;
  case 2:
    /* get u-value */
    ENCStatus = HAL_GPIO_GetBit(HAL_GPIO_PORT_D, HAL_GPIO_BIT_3);
    /* get v-value */
    ENCStatus |= HAL_GPIO_GetBit(HAL_GPIO_PORT_D, HAL_GPIO_BIT_4) <<1;
    /* get w-value */
    ENCStatus |= HAL_GPIO_GetBit(HAL_GPIO_PORT_D, HAL_GPIO_BIT_5) <<2;
    break;
  default:
    assert_param(0);
    break;
  }

  for(i=0;i<sizeof(HallSectorTable)/(2* sizeof(uint16_t));i++)
    if (HallSectorTable[i].value == ENCStatus)
      break;

  /* Error scenario */
  if (i == (sizeof(HallSectorTable)/(2* sizeof(uint16_t))))
  {
      /* In case of hall sensor error, set angle to 0 */
      i = 0;
  }
  
  if (old_status != ENCStatus)
  {
    old_status = ENCStatus;
    printf("Angle: %d\n",HallSectorTable[i].angle);
  }
  return HallSectorTable[i].angle;
}

int32_t ENC_GetEncCounter(uint8_t channel_number)
{
  TSB_EN_TypeDef* pENCx = NULL;

  switch (channel_number)
  {
  case 0:
    pENCx = TSB_EN0;
    break;
  case 1:
    pENCx = TSB_EN1;
    break;
  case 2:
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }

  return pENCx->CNT;
}

void ENC_ClearEncCounter(uint8_t channel_number)
{
  TSB_EN_TypeDef* pENCx = NULL;

  switch (channel_number)
  {
  case 0:
    pENCx = TSB_EN0;
    break;
  case 1:
    pENCx = TSB_EN1;
    break;
  case 2:
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }
  pENCx->TNCR=pENCx->TNCR | ENC_COUNTER_CLEAR;
}

static void IRQ_Common_enc(uint8_t channel_number)
{
#ifdef USE_MOTION_CONTROL
  MotionControl_IRQ_Callback(channel_number, tweak_irq_nr[channel_number]);
#endif /* USE_MOTION_CONTROL */
}

void ENC_SetEncoderIRQValue(uint8_t channel_number, uint8_t tweak_nr, uint32_t value)
{
#ifdef USE_MOTION_CONTROL
  TSB_EN_TypeDef* pENCx = NULL;

  switch (channel_number)
  {
  case 0:
    pENCx = TSB_EN0;
    break;
  case 1:
    pENCx = TSB_EN1;
    break;
  case 2:
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }

  tweak_irq_nr[channel_number] = tweak_nr;

 
  pENCx->MCMP = value;
#endif /* USE_MOTION_CONTROL */ 
  return;
}

uint32_t        ticksBetweenEvents;

static void IRQ_Common(unsigned char channel_number)
{
  TSB_EN_TypeDef* pENCx = NULL;
  static uint8_t edgesCount =0;

  switch (channel_number)
  {
  case 0:
    pENCx = TSB_EN0;
    break;
  case 1:
    pENCx = TSB_EN1;
    break;
  case 2:
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }

  switch (MotorParameterValues[channel_number].EncoderUsage)
  {
  case ENCODER_USAGE_SPEED:
    switch (MotorParameterValues[channel_number].Encoder)
    {
    case ENCODER_HALL_UVW:
    case ENCODER_HALL_UV:
      {
        static uint8_t  event_cnt =0;
        static uint32_t last_ticksBetweenEvents[3];

        ticksBetweenEvents = pENCx->CNT;

        last_ticksBetweenEvents[event_cnt++] = ticksBetweenEvents;
        if (event_cnt>=3)
          event_cnt=0;

        EncoderData[channel_number].tickSum =   last_ticksBetweenEvents[0]
                                              + last_ticksBetweenEvents[1]
                                              + last_ticksBetweenEvents[2];
        EncoderData[channel_number].ticksBetweenEvents = ticksBetweenEvents;


        EncoderData[channel_number].CW = (pENCx->STS & ENC_DIRECTION) >> 13;

        EncoderData[channel_number].pulses++;

        if (EncoderData[channel_number].CW == 1)
          EncoderData[channel_number].event_nr++;
        else
          EncoderData[channel_number].event_nr--;
      }
      if (EncoderData[channel_number].event_nr > MotorParameterValues[channel_number].PolePairs * 2 * input_nr[channel_number] - 1)
      {
        EncoderData[channel_number].event_nr=0;
        if (MotorControl[channel_number].ActualStage.main != Stage_FOC)
          EncoderData[channel_number].Theta0.value = MotorControl[channel_number].Theta.value;
        else
          MotorControl[channel_number].Theta.value = EncoderData[channel_number].Theta0.value;
      }
      else if (EncoderData[channel_number].event_nr < -MotorParameterValues[channel_number].PolePairs * 2 * input_nr[channel_number] + 1)
      {
        EncoderData[channel_number].event_nr=0;
        if (MotorControl[channel_number].ActualStage.main != Stage_FOC)
          EncoderData[channel_number].Theta0.value = MotorControl[channel_number].Theta.value;
        else
          MotorControl[channel_number].Theta.value = EncoderData[channel_number].Theta0.value;
      }

      break;

    case ENCODER_SINGLE_PULSE:

      EncoderData[channel_number].ticksBetweenEvents = pENCx->CNT;
      pENCx->TNCR=pENCx->TNCR | ENC_COUNTER_CLEAR;

      if (MotorControl[channel_number].ActualStage.main != Stage_FOC)
        EncoderData[channel_number].Theta0.value = MotorControl[channel_number].Theta.value;
      else
        MotorControl[channel_number].Theta.value = EncoderData[channel_number].Theta0.value;
      break;

    case ENCODER_INCENC_AB:
    case ENCODER_INCENC_ABZ:
    case ENCODER_AMS_AS5145H:

      ticksBetweenEvents = pENCx->CNT;

      EncoderData[channel_number].ticksBetweenEvents = ticksBetweenEvents;
      EncoderData[channel_number].CW = (pENCx->STS & ENC_DIRECTION) >> 13;

      edgesCount++;
      if( 4 == edgesCount )
      {
          edgesCount = 0;
      }

      if (EncoderData[channel_number].CW == 1)
        EncoderData[channel_number].event_nr += DivideFactor[divider_nr[channel_number]].factor;
      else
        EncoderData[channel_number].event_nr -= DivideFactor[divider_nr[channel_number]].factor;

      if (EncoderData[channel_number].event_nr > 4 * MotorParameterValues[channel_number].EncoderResolution - 1)
      {
        EncoderData[channel_number].event_nr -= 4 * MotorParameterValues[channel_number].EncoderResolution ;
        EncoderData[channel_number].FullTurns ++;
        if (MotorControl[channel_number].ActualStage.main != Stage_FOC)
         EncoderData[channel_number].Theta0.value = MotorControl[channel_number].Theta.value;
      }
      else if (EncoderData[channel_number].event_nr < -4*MotorParameterValues[channel_number].EncoderResolution -1)
      {
        EncoderData[channel_number].event_nr += 4 * MotorParameterValues[channel_number].EncoderResolution;
        EncoderData[channel_number].FullTurns --;
        if (MotorControl[channel_number].ActualStage.main != Stage_FOC)
          EncoderData[channel_number].Theta0.value = MotorControl[channel_number].Theta.value;
      }
      break;
    }
    break;

  case ENCODER_USAGE_EVENT:
    switch (MotorParameterValues[channel_number].Encoder)
    {
    case ENCODER_HALL_UVW:
    case ENCODER_HALL_UV:
      EncoderData[channel_number].event_nr=pENCx->CNT;
      if (EncoderData[channel_number].event_nr > MotorParameterValues[channel_number].PolePairs * 2 * input_nr[channel_number] - 1)
      {
        EncoderData[channel_number].event_nr=0;
        EncoderData[channel_number].FullTurns++;
        pENCx->TNCR |= ENC_COUNTER_CLEAR;
      }
      else if (EncoderData[channel_number].event_nr < -MotorParameterValues[channel_number].PolePairs * 2 * input_nr[channel_number] + 1)
      {
        EncoderData[channel_number].event_nr=0;
        EncoderData[channel_number].FullTurns--;
        pENCx->TNCR |= ENC_COUNTER_CLEAR;
      }
      break;
    case ENCODER_INCENC_AB:
    case ENCODER_INCENC_ABZ:
    case ENCODER_AMS_AS5145H:
      EncoderData[channel_number].event_nr = pENCx->CNT;

      EncoderData[channel_number].CW = (pENCx->STS & ENC_DIRECTION) >> 13;

      if (EncoderData[channel_number].event_nr==0)
        if (EncoderData[channel_number].CW==1)
          EncoderData[channel_number].FullTurns ++;
        else
          EncoderData[channel_number].FullTurns --;
      break;

    case ENCODER_SINGLE_PULSE:
      break;
    }

    case ENCODER_USAGE_EVENT_IRQ:
#ifdef USE_MOTION_CONTROL
      MotionControl_IRQ_Callback(channel_number, tweak_irq_nr[channel_number]);
#endif /* USE_MOTION_CONTROL */
      break;

    break;
  }
}

/*! \brief  Determine Omega (& Theta) from measured encoder interrupts
  *
  * @param  channel_number:  channel to measure
  * @retval None
*/
  FRACTIONAL      ENC_ThetaCalc;
  int64_t         ENC_OmegaCalc;

  int64_t         ENC_OmegaCalc_PreCalc;

void ENC_Determine_Omega_Theta(unsigned char channel_number)
{
  TSB_EN_TypeDef* pENCx    = NULL;
  uint32_t        ticksSinceLastEvent;

  switch (MotorParameterValues[channel_number].Encoder)
  {
  case ENCODER_HALL_UV:
  case ENCODER_HALL_UVW:

    ENC_OmegaCalc = ENC_OmegaCalc_PreCalc / EncoderData[channel_number].tickSum;

    if (EncoderData[channel_number].CW==0)
      ENC_OmegaCalc*=-1;

    MotorControl[channel_number].Omega.part.reg = ENC_OmegaCalc;

    MotorControl[channel_number].Theta.value += (int32_t) ( MotorControl[channel_number].Omega.part.reg
                                                 *( MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency<<1));
    break;

  case ENCODER_SINGLE_PULSE:
    break;

  case ENCODER_INCENC_AB:
  case ENCODER_INCENC_ABZ:
  case ENCODER_AMS_AS5145H:

    switch (channel_number)
    {
    case 0:
      pENCx = TSB_EN0;
      break;
    case 1:
      pENCx = TSB_EN1;
      break;
    case 2:
      pENCx = TSB_EN2;
      break;
    default:
      assert_param(0);
      break;
    }

    pENCx->TNCR |= ENC_SW_CAPTURE;
    ticksSinceLastEvent = pENCx->CNT;

    ENC_OmegaCalc = ENC_OmegaCalc_PreCalc /  EncoderData[channel_number].ticksBetweenEvents;

    if (EncoderData[channel_number].CW != 1)
      ENC_OmegaCalc *= -1;

    MotorControl[channel_number].Omega.part.reg = ENC_OmegaCalc;

    ENC_ThetaCalc.value =   ENC_DeltaThetaBetweenTicks.value *
                            ( EncoderData[channel_number].event_nr + ticksSinceLastEvent / EncoderData[channel_number].ticksBetweenEvents )
                          + EncoderData[channel_number].Theta0.value;

    MotorControl[channel_number].Theta.value = ENC_ThetaCalc.value;

    break;
  default:
    break;
  }
}

/*! \brief  Interrupt handler of Encoder Channel 0*/
void INTENC00_IRQHandler(void)
{
  IRQ_Common(0);
}

/*! \brief  Interrupt handler of Encoder Channel 1
  *
  * @param  None
  * @retval None
*/
void INTENC10_IRQHandler(void)
{
  IRQ_Common(1);
}

/*! \brief  Interrupt handler of Encoder Channel 2
  *
  * @param  None
  * @retval None
*/
void INTENC20_IRQHandler(void)
{
  IRQ_Common(2);
}

/*! \brief  Interrupt handler of Encoder Channel 0*/
void INTENC01_IRQHandler(void)
{
  IRQ_Common_enc(0);
}

/*! \brief  Interrupt handler of Encoder Channel 1
  *
  * @param  None
  * @retval None
*/
void INTENC11_IRQHandler(void)
{
  IRQ_Common_enc(1);
}

/*! \brief  Interrupt handler of Encoder Channel 2
  *
  * @param  None
  * @retval None
*/
void INTENC21_IRQHandler(void)
{
  IRQ_Common_enc(2);
}

/*! \brief  Interrupt enable for the selected Encoder Channel
  *
  * @param  channel_number: channel to configure
  * @retval None
*/
void ENC_EnableIRQ(unsigned char channel_number)
{
  TSB_EN_TypeDef* pENCx = NULL;

  switch (channel_number)
  {
  case 0:
    pENCx = TSB_EN0;
    break;
  case 1:
    pENCx = TSB_EN1;
    break;
  case 2:
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }

  pENCx->INTCR |= ( ENC_INT_MCMPIE);
}

/*! \brief  Interrupt disable for the selected Encoder Channel
  *
  * @param  channel_number: channel to configure
  * @retval None
*/
void ENC_DisableIRQ(unsigned char channel_number)
{
  TSB_EN_TypeDef* pENCx = NULL;

  switch (channel_number)
  {
  case 0:
    pENCx = TSB_EN0;
    break;
  case 1:
    pENCx = TSB_EN1;
    break;
  case 2:
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }

  pENCx->INTCR &= ~( ENC_INT_MCMPIE);
}

/*! \brief  Initialize the Encoder
  *
  * @param  channel_number: channel to configure
  * @retval None
*/
void ENC_Init(unsigned char channel_number)
{
  TSB_EN_TypeDef* pENCx    = NULL;
  uint32_t        tncr     = 0;
  int64_t         temp;

  if(MotorParameterValues[channel_number].Encoder  == ENCODER_NO_ENCODER)
  {
    switch (channel_number)
    {
    case 0:
      NVIC_DisableIRQ(INTENC00_IRQn);
      break;
    case 1:
      NVIC_DisableIRQ(INTENC10_IRQn);
      break;
    case 2:
      NVIC_DisableIRQ(INTENC20_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    return;
  }

  memset(divider_nr,0,sizeof(divider_nr));

  switch (channel_number)
  {
  case 0:
    /* enable clock for encoder instance */
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_ENC_CH0);
    pENCx = TSB_EN0;
    break;
  case 1:
    /* enable clock for encoder instance */
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_ENC_CH1);
    pENCx = TSB_EN1;
    break;
  case 2:
    /* enable clock for encoder instance */
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_ENC_CH2);
    pENCx = TSB_EN2;
    break;
  default:
    assert_param(0);
    break;
  }

  tncr = ENC_RUN_ENABLE | DivideFactor[divider_nr[channel_number]].tncr;

  switch (MotorParameterValues[channel_number].Encoder)
  {
  case ENCODER_INCENC_ABZ:
    switch (MotorParameterValues[channel_number].EncoderUsage)
    {
    case ENCODER_USAGE_SPEED:
      tncr    |= ENC_MODE_SENSOR_TIME;
      break;

    case ENCODER_USAGE_EVENT:
    case ENCODER_USAGE_EVENT_IRQ:
      tncr |= ENC_MODE_SENSOR_EVENT;
      break;

    default:
      break;
    }
    input_nr[channel_number] = 2;
    break;

  case ENCODER_INCENC_AB:
    switch (MotorParameterValues[channel_number].EncoderUsage)
    {
    case ENCODER_USAGE_SPEED:
      tncr  |= ENC_MODE_SENSOR_TIME;
      break;

    case ENCODER_USAGE_EVENT:
    case ENCODER_USAGE_EVENT_IRQ:
      tncr |= ENC_MODE_SENSOR_EVENT;
      break;

    default:
      break;
    }
    input_nr[channel_number] = 2;
    break;

  case ENCODER_HALL_UVW:
    tncr |= ENC_3PHASE_ENABLE;
    input_nr[channel_number] = 3;
    switch (MotorParameterValues[channel_number].EncoderUsage)
    {
    case ENCODER_USAGE_SPEED:
      tncr |= ENC_MODE_SENSOR_TIME;
      break;

    case ENCODER_USAGE_EVENT:
    case ENCODER_USAGE_EVENT_IRQ:
      tncr |= ENC_MODE_SENSOR_EVENT;
      break;

    default:
      break;
    }
    break;

  case ENCODER_HALL_UV:
    input_nr[channel_number] = 2;
    switch (MotorParameterValues[channel_number].EncoderUsage)
    {
    case ENCODER_USAGE_SPEED:
      tncr |= ENC_MODE_SENSOR_TIME;
      break;

    case ENCODER_USAGE_EVENT:
      tncr |= ENC_MODE_SENSOR_EVENT;
      break;

    default:
      break;
    }
    break;
  case ENCODER_SINGLE_PULSE:
    input_nr[channel_number] = 1;
    break;

  case ENCODER_AMS_AS5145H:
    tncr    |= ENC_MODE_SENSOR_TIME;
    input_nr[channel_number] = 2;
    MotorParameterValues[channel_number].EncoderResolution = 1024;
    break;

  default:
    break;
  }
  pENCx->CLKCR |= 0x03;
  pENCx->INPCR |= 0x5 << 8;

  pENCx->RELOAD                     = 0xFFFFFFFF;
  pENCx->TNCR                       = tncr;

  temp                              = (((int64_t)FIXPOINT_16-1) << 16)
                                      * MotorParameterValues[channel_number].PolePairs
                                      / (4*MotorParameterValues[channel_number].EncoderResolution-1);  ;

  ENC_DeltaThetaBetweenTicks.value  = temp ;

  switch (MotorParameterValues[channel_number].Encoder)
  {
  case ENCODER_HALL_UV:
  case ENCODER_HALL_UVW:
    ENC_OmegaCalc_PreCalc =  (uint64_t)80000000; /* 80 Mhz */
    ENC_OmegaCalc_PreCalc *= FIXPOINT_15;
    ENC_OmegaCalc_PreCalc *= 4;
    ENC_OmegaCalc_PreCalc /= 8;
    ENC_OmegaCalc_PreCalc /= MotorControl[channel_number].HzMax;
    break;
  case ENCODER_INCENC_AB:
  case ENCODER_INCENC_ABZ:
  case ENCODER_AMS_AS5145H:
    ENC_OmegaCalc_PreCalc =  (uint64_t)80000000; /* 80 Mhz */
    ENC_OmegaCalc_PreCalc *= FIXPOINT_15;
    ENC_OmegaCalc_PreCalc /= DivideFactor[divider_nr[channel_number]].factor;
    ENC_OmegaCalc_PreCalc /= 1000;
    ENC_OmegaCalc_PreCalc /= MotorControl[channel_number].HzMax;
    break;
  default:
    break;
  }

  ENC_IOInit(channel_number);                                                   /*! setup IO */

  __DSB();                                                                      /* ! flush the pipeline */

  switch (MotorParameterValues[channel_number].EncoderUsage)
  {
  case ENCODER_USAGE_SPEED:
  case ENCODER_USAGE_EVENT:
    switch (channel_number)
    {
    case 0:
      NVIC_SetPriority(INTENC00_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_EnableIRQ(INTENC00_IRQn);                                               /*! enable the interrupt */
      pENCx->INTCR |= ENC_INT_CAPIE;                                               /*! enable irq in encoder IP */
      break;
    case 1:
      NVIC_SetPriority(INTENC10_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_EnableIRQ(INTENC10_IRQn);                                               /*! enable the interrupt */
      pENCx->INTCR |= ENC_INT_CAPIE;                                               /*! enable irq in encoder IP */
      break;
    case 2:
      NVIC_SetPriority(INTENC20_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_EnableIRQ(INTENC20_IRQn);                                               /*! enable the interrupt */
      pENCx->INTCR |= ENC_INT_CAPIE;                                               /*! enable irq in encoder IP */

      break;
    default:
      assert_param(0);
      break;
    }
  case ENCODER_USAGE_EVENT_IRQ:
    switch (channel_number)
    {
    case 0:
      NVIC_SetPriority(INTENC00_IRQn, INTERRUPT_PRIORITY_ENCODER);                /*! set the encoder interrupt priority */
      NVIC_SetPriority(INTENC01_IRQn, INTERRUPT_PRIORITY_ENCODER);                /*! set the encoder interrupt priority */
      NVIC_EnableIRQ(INTENC00_IRQn);                                              /*! enable the interrupt */
      NVIC_EnableIRQ(INTENC01_IRQn);                                              /*! enable the interrupt */
      break;
    case 1:
      NVIC_SetPriority(INTENC10_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_SetPriority(INTENC11_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_EnableIRQ(INTENC10_IRQn);                                               /*! enable the interrupt */
      NVIC_EnableIRQ(INTENC11_IRQn);                                               /*! enable the interrupt */
      break;
    case 2:
      NVIC_SetPriority(INTENC20_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_SetPriority(INTENC21_IRQn, INTERRUPT_PRIORITY_ENCODER);                 /*! set the encoder interrupt priority */
      NVIC_EnableIRQ(INTENC20_IRQn);                                               /*! enable the interrupt */
      NVIC_EnableIRQ(INTENC21_IRQn);                                               /*! enable the interrupt */
      break;
    default:
      assert_param(0);
      break;
    }
  }

}
#endif /* USE_ENCODER */
