/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include "config.h"
#include "TMPM4KN.h"
#include TMPM_GPIO_HEADER_FILE
#include "hal_gpio.h"
#include "hal_cg.h"

#define BITBAND_SET_BIT(value, bitPos, bitValue) (value|= bitValue << bitPos)

static const HAL_GPIO_InitTypeDef HAL_GPIO_Init_Struct_Output =
{
  HAL_GPIO_OUTPUT_MODE,                                                             /* Enable LEDs as output */
  HAL_GPIO_PULLUP_DISABLE,                                                          /* Enable/disable pull up */
  HAL_GPIO_OPEN_DRAIN_DISABLE,                                                      /* Open drain or CMOS */
  HAL_GPIO_PULLDOWN_DISABLE,                                                        /* Enable/disable pull down */
  HAL_GPIO_IRQ_DISABLE,                                                             /* Enable/disable interrupt */
};

static const HAL_GPIO_InitTypeDef HAL_GPIO_Init_Struct_Input =
{
  HAL_GPIO_INPUT_MODE,
  HAL_GPIO_PULLUP_DISABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_DISABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigTX =
{
  HAL_GPIO_OUTPUT_MODE,
  HAL_GPIO_PULLUP_DISABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_DISABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigRX =
{
  HAL_GPIO_INPUT_MODE,
  HAL_GPIO_PULLUP_DISABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_ENABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigPMDOutput =
{
  HAL_GPIO_OUTPUT_MODE,
  HAL_GPIO_PULLUP_DISABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_DISABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigPMDInput =
{
  HAL_GPIO_INPUT_MODE,
  HAL_GPIO_PULLUP_ENABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_ENABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigENC =
{
  HAL_GPIO_INPUT_MODE,
  HAL_GPIO_PULLUP_DISABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_ENABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigI2C =
{
  HAL_GPIO_OUTPUT_MODE,
  HAL_GPIO_PULLUP_ENABLE,
  HAL_GPIO_OPEN_DRAIN_ENABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_ENABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigSPIOutput =
{
  HAL_GPIO_OUTPUT_MODE,
  HAL_GPIO_PULLUP_ENABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_DISABLE,
};

static const HAL_GPIO_InitTypeDef halPortConfigSPIInput =
{
  HAL_GPIO_INPUT_MODE,
  HAL_GPIO_PULLUP_ENABLE,
  HAL_GPIO_OPEN_DRAIN_DISABLE,
  HAL_GPIO_PULLDOWN_DISABLE,
  HAL_GPIO_IRQ_ENABLE,
};


static gpio_t p_obj={TSB_PA, TSB_PB, TSB_PC, TSB_PD, TSB_PE, TSB_PF, TSB_PG, TSB_PH, TSB_PJ, TSB_PK, TSB_PL, TSB_PM, TSB_PN, TSB_PU, TSB_PV};

/*! \brief Searches in the source for the first enabled bit and returns the bit position
  *
  * @param  source : input value
  *
  * @retval bit position (index)
*/
static uint8_t HAL_GPIO_Bit2BitPos (uint8_t source)
{
  uint8_t loop;

  for (loop =0x0; loop < 8; loop ++)
  {
    if ((1<<loop) & source)
    {
      return (uint8_t)loop;
    }
  }
  return 0;
}

/*! \brief Calculates the port number for the specific device driver from the overall hal supported ports
  *
  * @param  port : port number of the hal-layer
  * @param  pin  : pointer for the result (the driver port number)
  *
  * @retval 0: success, 1: error
*/
static uint8_t HAL_GPIO_GetDrvPort (HAL_GPIO_Port port, uint32_t *drvPort)
{
  if (HAL_GPIO_PORT_I == port)
  {
    /* not supported for TMPKM4KN */
    return -1;
  }
  if (HAL_GPIO_PORT_I > port)
  {
    *drvPort = (uint32_t)port;
  }
  if (HAL_GPIO_PORT_I < port)
  {
    *drvPort = (uint32_t)port -1;
  }
  return 0;
}

/*! \brief Configuration for a specific port pin
  *
  * @param  port : port number of the used port
  * @param  pin  : selection of pin (only one pin, one bit) is be allowed
  * @param  gpioInitTypeDef : structure which contains all information about the configuration of the port pin
  * @param  frValue: value of the selected pin function (only in the range of 0-7)
  *
  * @retval 0: success 1: error
*/
static uint8_t HAL_GPIO_CfgPort(uint32_t port, uint8_t bit, const HAL_GPIO_InitTypeDef *gpioInitTypeDef, uint8_t frValue)
{
  uint8_t bitPos = HAL_GPIO_Bit2BitPos(bit);

    /* always enable clock for used port */
  HAL_CG_EnableMiddleSpeedClockA(((HAL_CG_CLK_A)(port)));

  /* set pull down mode */
  if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_PUP, gpioInitTypeDef->PullUp) != SUCCESS)
  {
    return 1;
  }
  /* set open drain mode */
  if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_OD, gpioInitTypeDef->OpenDrain) != SUCCESS)
  {
    return 1;
  }
  /* set  pull up mode */
  if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_PDN, gpioInitTypeDef->PullDown)!= SUCCESS)
  {
    return 1;
  }

  /* disable interrupt functionality */
  if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_IE, gpioInitTypeDef->Interrupt)!= SUCCESS)
  {
    return 1;
  }

  /* set  c register for GPIO functionlity */
  if (HAL_GPIO_INPUT_MODE == gpioInitTypeDef->IOMode)
  {
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_CR, 0)!= SUCCESS)
    {
      return 1;
    }
  }
  if (HAL_GPIO_OUTPUT_MODE == gpioInitTypeDef->IOMode)
  {
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_CR, 1)!= SUCCESS)
    {
      return 1;
    }
  }
  /* set functionality */
  switch (frValue)
  {
  case 1:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR1, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  case 2:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR2, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  case 3:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR3, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  case 4:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR4, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  case 5:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR5, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  case 6:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR6, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  case 7:
    if ( gpio_write_bit(&p_obj, port, bitPos, GPIO_Mode_FR7, 1)!= SUCCESS)
    {
      return 1;
    }
    break;
  default: /* 0 and other */
    /* do nothing */
    break;
  }

  return 0;
}

uint8_t HAL_GPIO_Init(HAL_GPIO_Port port, uint8_t bit, gpio_function function)
{
  uint8_t fr; // function register to use
  uint32_t loop;
  uint32_t drvPort;

  if (HAL_GPIO_GetDrvPort (port, &drvPort) != 0)
  {
    assert_param(0);
    return 1;
  }
  for (loop =0x01; loop < 0x100; loop = loop << 1)
  {
    if (loop & bit)
    {
      switch (function)
      {
      case HAL_GPIO_OUTPUT:
        HAL_GPIO_CfgPort(drvPort, loop, &HAL_GPIO_Init_Struct_Output, 0);
        break;
      case HAL_GPIO_INPUT:
        HAL_GPIO_CfgPort(drvPort, loop, &HAL_GPIO_Init_Struct_Input, 0);
        break;
      case HAL_GPIO_UART_RX:
        switch (drvPort)
        {
        case GPIO_PORT_C:
          if ((loop == HAL_GPIO_BIT_0) || (loop == HAL_GPIO_BIT_4))
          {
            fr = 2;
          }
          else
          {
            if ((loop == HAL_GPIO_BIT_1) || (loop == HAL_GPIO_BIT_5))

            {
              fr = 1;
            }
            else
            {
              assert_param(0);
              return 1;
            }
          }
          HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigRX, fr);
          break;
        case GPIO_PORT_F:
          if ((loop == HAL_GPIO_BIT_0) || (loop == HAL_GPIO_BIT_3) || (loop == HAL_GPIO_BIT_6))
          {
            fr= 2;
          }
          else
          {
            if ((loop == HAL_GPIO_BIT_1) || (loop == HAL_GPIO_BIT_4) || (loop == HAL_GPIO_BIT_7))
            {
              fr= 1;
            }
            else
            {
              assert_param(0);
              return 1;
            }
          }
          HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigRX, fr);
          break;
        default:
          break;
        }
        break;
      case HAL_GPIO_UART_TX:
        switch (drvPort)
        {
        case GPIO_PORT_C:
          if ((loop == HAL_GPIO_BIT_0) || (loop == HAL_GPIO_BIT_4))
          {
            fr= 1;
          }
          else
          {
            if ((loop == HAL_GPIO_BIT_1) || (loop == HAL_GPIO_BIT_5))
            {
              fr= 2;
            }
            else
            {
              assert_param(0);
              return 1;
            }
          }
          HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigTX, fr);
          break;
        case GPIO_PORT_F:
          if ((loop == HAL_GPIO_BIT_0) || (loop == HAL_GPIO_BIT_3) || (loop == HAL_GPIO_BIT_6))
          {
            fr= 1;
          }
          else
          {
            if ((loop == HAL_GPIO_BIT_1) || (loop == HAL_GPIO_BIT_4) || (loop == HAL_GPIO_BIT_7))
            {
              fr= 2;
            }
            else
            {
              assert_param(0);
              return 1;
            }
          }
          HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigTX, fr);
          break;
        default:
          assert_param(0);
          return 1;
          break;
        }
        break;

      case HAL_GPIO_ENC:
        fr= 0;
        switch (drvPort)
        {
        case GPIO_PORT_D:
        case GPIO_PORT_F:
        case GPIO_PORT_P:
          if ((loop == HAL_GPIO_BIT_3) || (loop == HAL_GPIO_BIT_4) || (loop == HAL_GPIO_BIT_5))
          {
            fr= 6;
          }
          break;
        case GPIO_PORT_N:
          if ((loop == HAL_GPIO_BIT_0) || (loop == HAL_GPIO_BIT_1) || (loop == HAL_GPIO_BIT_2))
          {
            fr= 6;
          }
          break;
        case GPIO_PORT_R:
          if ((loop == HAL_GPIO_BIT_3) || (loop == HAL_GPIO_BIT_4) || (loop == HAL_GPIO_BIT_5))
          {
            fr= 6;
          }
          break;
        case GPIO_PORT_U:
          if ((loop == HAL_GPIO_BIT_3) || (loop == HAL_GPIO_BIT_5) || (loop == HAL_GPIO_BIT_6))
          {
            fr= 6;
          }
          break;
        }
        if (fr != 0)
        {
          HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigENC, fr);
        }
        break;
		
      case HAL_GPIO_PMD_OUTPUT:
        switch (drvPort)
        {
        case GPIO_PORT_B:
            fr= 4;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigPMDOutput, fr);
          break;
        break;
        case GPIO_PORT_E:
            fr= 6;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigPMDOutput, fr);
          break;
        case GPIO_PORT_U:
            fr= 6;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigPMDOutput, fr);
          break;
        default:
          assert_param(0);
        break;
        }
        break;

      case HAL_GPIO_PMD_INPUT:
        switch (drvPort)
        {
        case GPIO_PORT_B:
            fr= 4;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigPMDInput, fr);
          break;
        break;
        case GPIO_PORT_E:
            fr= 6;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigPMDInput, fr);
          break;
        case GPIO_PORT_U:
            fr= 6;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigPMDInput, fr);
          break;
        default:
          assert_param(0);
        break;
        }
        break;
     case HAL_GPIO_I2C_SDA:
     case HAL_GPIO_I2C_SCL:
        switch (drvPort)
        {
        case GPIO_PORT_C:
            fr= 4;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigI2C, fr);
          break;
        break;
        case GPIO_PORT_D:
            fr= 2;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigI2C, fr);
          break;
        case GPIO_PORT_T:
            fr= 2;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigI2C, fr);
          break;
        case GPIO_PORT_U:
            fr= 3;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigI2C, fr);
          break;
        default:
          assert_param(0);
        break;
        }
        break;
     case HAL_GPIO_SPI_INPUT:
        switch (drvPort)
        {
        case GPIO_PORT_A:
        case GPIO_PORT_G:
            fr= 1;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigSPIInput, fr);
          break;
        break;
        case GPIO_PORT_C:
            fr= 3;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigSPIInput, fr);
          break;
        case GPIO_PORT_V:
            fr= 2;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigSPIInput, fr);
          break;
        default:
          assert_param(0);
          break;
        }
        break;
    case HAL_GPIO_SPI_OUTPUT:
        switch (drvPort)
        {
        case GPIO_PORT_A:
        case GPIO_PORT_G:
            fr= 1;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigSPIOutput, fr);
          break;
        break;
        case GPIO_PORT_C:
            fr= 3;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigSPIOutput, fr);
          break;
        case GPIO_PORT_V:
            fr= 2;
            HAL_GPIO_CfgPort(drvPort, loop, &halPortConfigSPIOutput, fr);
          break;
        default:
          assert_param(0);
          break;
        }
        break;
      default:
        assert_param(0);
        /* error GPIO funtion unknown */
        return 1;
        break;
      }
    }
  }
  return 0;
}

uint8_t HAL_GPIO_SetBit(HAL_GPIO_Port port, uint8_t bit, uint8_t value)
{
  uint32_t drvPort;
  uint8_t  bitPos = HAL_GPIO_Bit2BitPos(bit);

  if (HAL_GPIO_GetDrvPort (port, &drvPort) != 0)
  {
    assert_param(0);
    return 1;
  }
  return gpio_write_bit(&p_obj, drvPort, bitPos, GPIO_Mode_DATA, value);
}

uint8_t HAL_GPIO_GetBit(HAL_GPIO_Port port, uint8_t bit)
{
  gpio_pinstate_t pinstate = GPIO_PIN_RESET;
  uint32_t drvPort;
  uint8_t  bitPos = HAL_GPIO_Bit2BitPos(bit);

  if (HAL_GPIO_GetDrvPort (port, &drvPort) != 0)
  {
    assert_param(0);
    return 1;
  }
  gpio_read_bit(&p_obj, drvPort, bitPos, GPIO_Mode_DATA, &pinstate);

  if (pinstate == GPIO_PIN_SET)
  {
    return 1;
  }
  return 0;
}
