/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include "config.h"
#include "hal_timer.h"
#include "hal_cg.h"
#include "txz_driver_def.h"

void (*timer_irq_callback[12])();

static void irq_callback (uint8_t number)
{
  assert_param(timer_irq_callback[number]);
  timer_irq_callback[number]();
}

void INTT32A00AC_IRQHandler(void)
{
  irq_callback(0);
};

void INTT32A00B_IRQHandler(void)
{
  irq_callback(1);
};

void INTT32A01AC_IRQHandler(void)
{
  irq_callback(2);
};

void INTT32A01B_IRQHandler(void)
{
  irq_callback(3);
};

void INTT32A02AC_IRQHandler(void)
{
  irq_callback(4);
};

void INTT32A02B_IRQHandler(void)
{
  irq_callback(5);
};

void INTT32A03AC_IRQHandler(void)
{
  irq_callback(6);
};

void INTT32A03B_IRQHandler(void)
{
  irq_callback(7);
};

void INTT32A04AC_IRQHandler(void)
{
  irq_callback(8);
};

void INTT32A04B_IRQHandler(void)
{
  irq_callback(9);
};

void INTT32A05AC_IRQHandler(void)
{
  irq_callback(10);
};

void INTT32A05B_IRQHandler(void)
{
  irq_callback(11);
};

#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */

void HAL_Timer_Init(uint8_t number,uint32_t cycle, uint32_t duty)
{
  switch (number)
  {
  case 0:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_T32A_CH0);
    TSB_T32A0->MOD  = 0;                      /* 32bit=0, halt=0 */
    TSB_T32A0->CRA  = 0x10000700;             /* PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
    TSB_T32A0->RGA0 = 0x9C40;                 /* (4) */
    TSB_T32A0->RGA1 = 0x9C40;
    TSB_T32A0->RELDA = 0x0;                   /* Reload = 0 */
    TSB_T32A0->IMA  = 1;                      /* IMA0=1 */
    break;
  case 1:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_T32A_CH0);
    TSB_T32A0->MOD  = 0;                      /* 32bit=0, halt=0 */
    TSB_T32A0->CRB  = 0x10000700;             /* PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
    TSB_T32A0->RGB0 = 0x9C40;                 /* (4) */
    TSB_T32A0->RGB1 = 0x9C40;
    TSB_T32A0->RELDB = 0x0;                   /* Reload = 0 */
    TSB_T32A0->IMB  = 1;                      /* IMA0=1 */
    break;
  case 2:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_T32A_CH1);
    TSB_T32A1->MOD  = 0;                      /* 32bit=0, halt=0 */
    TSB_T32A1->CRA  = 0x10000700;             /* PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
    TSB_T32A1->RGA0 = 0x9C40;                 /* (4) */
    TSB_T32A1->RGA1 = 0x9C40;
    TSB_T32A1->RELDA = 0x0;                   /* Reload = 0 */
    TSB_T32A1->IMA  = 1;                      /* IMA0=1 */
    break;
  case 3:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_T32A_CH1);
    TSB_T32A1->MOD  = 0;                      /* 32bit=0, halt=0 */
    TSB_T32A1->CRB  = 0x20000700;             /* PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
    TSB_T32A1->RGB0 = 0x9C40;                 /* (4) */
    TSB_T32A1->RGB1 = 0x9C40;
    TSB_T32A1->RELDB = 0x0;                   /* Reload = 0 */
    TSB_T32A1->IMB  = 1;                      /* IMA0=1 */
    break;
  case 4:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_T32A_CH2);
    TSB_T32A2->MOD  = 0;                      /* 32bit=0, halt=0 */
    TSB_T32A2->CRA  = 0x10000700;             /* PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
    TSB_T32A2->RGA0 = 0x9C40;                 /* (4) */
    TSB_T32A2->RGA1 = 0x9C40;
    TSB_T32A2->RELDA = 0x0;                   /* Reload = 0 */
    TSB_T32A2->IMA  = 1;                      /* IMA0=1 */
    break;
  case 5:
    HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_T32A_CH2);
    TSB_T32A2->MOD  = 0;                      /* 32bit=0, halt=0 */
    TSB_T32A2->CRB  = 0x20000700;             /* PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
    TSB_T32A2->RGB0 = 0x9C40;                 /* (4) */
    TSB_T32A2->RGB1 = 0x9C40;
    TSB_T32A2->RELDB = 0x0;                   /* Reload = 0 */
    TSB_T32A2->IMB  = 1;                      /* IMA0=1 */
    break;

  default:
    assert_param(0);
    break;
  }
}

void HAL_Timer_RegisterIRQCallback (uint8_t number, void (*callback)())
{
  assert_param(callback);
  timer_irq_callback[number] = callback;
}

void HAL_Timer_EnableIRQ(uint8_t number)
{
  switch (number)
  {
  case 0:
    NVIC_EnableIRQ(INTT32A00AC_IRQn);
    break;
  case 1:
    NVIC_EnableIRQ(INTT32A00B_IRQn);
    break;
  case 2:
    NVIC_EnableIRQ(INTT32A01AC_IRQn);
    break;
  case 3:
    NVIC_EnableIRQ(INTT32A01B_IRQn);
    break;
  case 4:
    NVIC_EnableIRQ(INTT32A02AC_IRQn);
    break;
  case 5:
    NVIC_EnableIRQ(INTT32A02B_IRQn);
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_Timer_DisableIRQ(uint8_t number)
{
  switch (number)
  {
  case 0:
    NVIC_DisableIRQ(INTT32A00AC_IRQn);
    break;
  case 1:
    NVIC_DisableIRQ(INTT32A00B_IRQn);
    break;
  case 2:
    NVIC_DisableIRQ(INTT32A01AC_IRQn);
    break;
  case 3:
    NVIC_DisableIRQ(INTT32A01B_IRQn);
    break;
  case 4:
    NVIC_DisableIRQ(INTT32A02AC_IRQn);
    break;
  case 5:
    NVIC_DisableIRQ(INTT32A02B_IRQn);
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_Timer_SetIRQPriority(uint8_t number, uint32_t prio)
{
  switch (number)
  {
  case 0:
    NVIC_SetPriority(INTT32A00AC_IRQn,prio);
    break;
  case 1:
    NVIC_SetPriority(INTT32A00B_IRQn,prio);
    break;
  case 2:
    NVIC_SetPriority(INTT32A01AC_IRQn,prio);
    break;
  case 3:
    NVIC_SetPriority(INTT32A01B_IRQn,prio);
    break;
  case 4:
    NVIC_SetPriority(INTT32A02AC_IRQn,prio);
    break;
  case 5:
    NVIC_SetPriority(INTT32A02B_IRQn,prio);
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_Timer_Stop(uint8_t number)
{
  switch (number)
  {
  case 0:
    TSB_T32A0->RUNA = 0x5;
    break;
  case 1:
    TSB_T32A0->RUNB = 0x5;
    break;
  case 2:
    TSB_T32A1->RUNA = 0x5;
    break;
  case 3:
    TSB_T32A1->RUNB = 0x5;
    break;
  case 4:
    TSB_T32A2->RUNA = 0x5;
    break;
  case 5:
    TSB_T32A2->RUNB = 0x5;
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_Timer_Start(uint8_t number)
{
  switch (number)
  {
  case 0:
    TSB_T32A0->RUNA = 0x3;
    break;
  case 1:
    TSB_T32A0->RUNB = 0x3;
    break;
  case 2:
    TSB_T32A1->RUNA = 0x3;
    break;
  case 3:
    TSB_T32A1->RUNB = 0x3;
    break;
  case 4:
    TSB_T32A2->RUNA = 0x3;
    break;
  case 5:
    TSB_T32A2->RUNB = 0x3;
    break;
  default:
    assert_param(0);
    break;
  }
}

