/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include "config.h"
#include "hal_gpio.h"
#include "hal_cg.h"
#include "hal_uart.h"
#include "txz_driver_def.h"

/**
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define CFG_BAUDRATE                    ((uint32_t)(115200))
#define CFG_BAUDRATE_SET_VAL_115200     ((uint32_t)(0x00A6002B))
#define CFG_BAUDRATE_SET_VAL_5MHZ       ((uint32_t)(0x00000001))

/**
 *  @}
 */ /* End of group BaudRateConfiguration */
/**
 *  @defgroup  TransmitError Transmit Error
 *  @brief    Transmit Error  .
 *  @{
 */
#define ERROR_UART_TRANSMIT       ((uint32_t)(1))
/**
 *  @}
 */ /* End of group TransmitError */

/**
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART_DATA_DIRECTION_LSB         ((uint32_t)(0x00000000))    /*!< LSB first */
#define UART_STOP_BIT_1                 ((uint32_t)(0x0)<<4)    /*!< 1 bit */
#define UART_STOP_BIT_2                 ((uint32_t)(0x0)<<4)    /*!< 2 bit */
#define UART_PARITY_BIT_ODD             ((uint32_t)(0x0)<<3)    /*!< Odd Parity  */
#define UART_PARITY_BIT_EVEN            ((uint32_t)(0x1)<<3)    /*!< Even Parity  */
#define UART_PARITY_DISABLE             ((uint32_t)(0x0)<<2)    /*!< Disable Parity*/
#define UART_PARITY_ENABLE              ((uint32_t)(0x1)<<2)    /*!< Disable Parity*/
#define UART_DATA_LENGTH_7              ((uint32_t)(0x00))      /*!< 8 bit */
#define UART_DATA_LENGTH_8              ((uint32_t)(0x01))      /*!< 8 bit */
#define UART_DATA_LENGTH_9              ((uint32_t)(0x10))      /*!< 9 bit */
 /**
 *  @}
 */ /* End of group UART_DataLength */

/**
 *  @defgroup UART_TxInterrupt  Tx Interrpt
 *  @brief    Available Transmit Interrupt Macro Definisiton.
 *  @{
 */
#define UART_TX_INT_DISABLE             ((uint32_t)0x00000000)    /*!< Not Available. */
#define UART_TX_INT_ENABLE              ((uint32_t)0x00000040)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_TxInterrupt */

/**
 *  @defgroup UART_RxInterrupt  Rx Interrpt
 *  @brief    Available Receive Interrupt Macro Definisiton.
 *  @{
 */
#define UART_RX_INT_DISABLE             ((uint32_t)0x00000000)    /*!< Not Available. */
#define UART_RX_INT_ENABLE              ((uint32_t)0x00000010)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_TxInterrupt */

/**
 *  @defgroup UART_ErrInterrupt  Err Interrpt
 *  @brief    Available Error Interrupt Macro Definisiton.
 *  @{
 */
#define UART_ERR_INT_DISABLE            ((uint32_t)0x00000000)    /*!< Not Available. */
#define UART_ERR_INT_ENABLE             ((uint32_t)0x00000004)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_TxInterrupt */

/**
 *  @defgroup UART_TxState  Tx State
 *  @brief    Transmitting State Macro Definisiton.
 *  @{
 */
#define UART_TX_STATE_RUN               ((uint32_t)0x00008000)    /*!< Run       */
/**
 *  @}
 */ /* End of group UART_TxState */
/**
 *  @defgroup UART_TxDone   Transmitting Done
 *  @brief    Transmitting Done Macro Definisiton.
 *  @{
 */
#define UART_TX_DONE                    ((uint32_t)0x00004000)    /*!< Transmitting Done */
/**
 *  @}
 */ /* End of group UART_TxDone */

 /**
 *  @defgroup UART_TxEnable  Transmitting Enable
 *  @brief    Transmitting Enable Macro Definisiton.
 *  @{
 */
#define UART_TXE_ENABLE                 ((uint32_t)0x00000002)    /*!< Transmitting Enable */
#define UART_TXE_DISABLE                ((uint32_t)0x00000000)    /*!< Transmitting Disable */
/**
 *  @}
 */ /* End of group UART_TxEnable */

 /**
 *  @defgroup UART_RxEnable Receive Enable
 *  @brief    Receive Enable Macro Definisiton.
 *  @{
 */
#define UART_RXE_ENABLE                 ((uint32_t)0x00000001)    /*!< Transmitting Enable */
#define UART_RXE_DISABLE                ((uint32_t)0x00000000)    /*!< Transmitting Disable */
/**
 *  @}
 */ /* End of group UART_TxEnable */

/**
 *  @defgroup UART_RxTx FIFO Clear
 *  @brief    FIFOs Clearing Macro Definisiton.
 *  @{
 */
#define UART_TFCLR                      ((uint32_t)0x00000002)    /*!< TX FIFO Clear */
#define UART_RFCLR                      ((uint32_t)0x00000001)    /*!< RX FIFO Clear */
/**
 *  @}
 */ /* End of group UART_TxEnable */


/**
 *  @defgroup UART_Prescaler  Prescaler
 *  @brief    Prescaler Macro Definisiton.
 *  @{
 */
#define UART_PLESCALER_1                ((uint32_t)0x00000000)    /*!< 1/1   */
/**
 *  @}
 */ /* End of group UART_Prescaler */

 /**
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/**
 *  @}
 */ /* End of group GPIO_EN */
 /**
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/**
 *  @}
 */ /* End of group FSYS_EN */



void (*uart_irq_callback[4][3])();

static void irq_callback (uint8_t channel, uart_direction direction)
{
  assert_param(uart_irq_callback[channel][((uint8_t)direction)>>1]);
  uart_irq_callback[channel][((uint8_t)direction)>>1]();
}

void INTSC0TX_IRQHandler(void)
{
  irq_callback(0,HAL_UART_TX);
}

void INTSC0RX_IRQHandler(void)
{
  irq_callback(0,HAL_UART_RX);
}

void INTSC0ERR_IRQHandler(void)
{
  irq_callback(0,HAL_UART_ERR);
}

void INTSC1TX_IRQHandler(void)
{
  irq_callback(1,HAL_UART_TX);
}

void INTSC1RX_IRQHandler(void)
{
  irq_callback(1,HAL_UART_RX);
}

void INTSC1ERR_IRQHandler(void)
{
  irq_callback(1,HAL_UART_ERR);
}

void INTSC2TX_IRQHandler(void)
{
  irq_callback(2,HAL_UART_TX);
}

void INTSC2RX_IRQHandler(void)
{
  irq_callback(2,HAL_UART_RX);
}

void INTSC2ERR_IRQHandler(void)
{
  irq_callback(2,HAL_UART_ERR);
}

void INTSC3TX_IRQHandler(void)
{
  irq_callback(3,HAL_UART_TX);
}

void INTSC3RX_IRQHandler(void)
{
  irq_callback(3,HAL_UART_RX);
}

void INTSC3ERR_IRQHandler(void)
{
  irq_callback(3,HAL_UART_ERR);
}

/*--------------------------------------------------*/
/**
  * @brief  UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Config(uint8_t channel, speed_mode baud, uint8_t bits, parity_mode parity, uint8_t stop_bits)
{
  TSB_UART_TypeDef *UARTx;

  uint32_t cr0_val = UART_DATA_DIRECTION_LSB;

  switch (channel)
  {
    case 0:
      /* (1) Enable the peripheral clock of UART0  */
      HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_UART_CH0);
      UARTx = TSB_UART0;
      break;
    case 1:
      /* (1) Enable the peripheral clock of UART1  */
      HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_UART_CH1);
      UARTx = TSB_UART1;
      break;
    case 2:
      /* (1) Enable the peripheral clock of UART2  */
      HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_UART_CH2);
      UARTx = TSB_UART2;
      break;
  case 3:
     /* (1) Enable the peripheral clock of UART3  */
     HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_UART_CH3);
     UARTx = TSB_UART3;
     break;
  default:
    return;
  }

  UARTx->SWRST = 0x2U;
  UARTx->SWRST = 0x1U;

  /* Wait reset done */
  while(UARTx->SWRST==  0x80);

  switch (baud)
  {
  case SPEED_115200:
    /* (2) 16 times oversampling by 115200 baud */
    /*  Fsysme = 80MHz, KEN = 1:Enable, N+(64-K)/64 division K:0x26 N:0x2B */
    UARTx->BRD = CFG_BAUDRATE_SET_VAL_115200;
    UARTx->CLK = (UART_PLESCALER_1 << 4);
    break;
  case SPEED_5MHZ:
    /* (2) 16 times oversampling by 5Mhz baud */
    /*  Fsysme = 80MHz, KEN = 0:Enable, N+(64-K)/64 division K:0 N:1 */
    UARTx->BRD = CFG_BAUDRATE_SET_VAL_5MHZ;
    UARTx->CLK = (UART_PLESCALER_1 << 4);
    break;
  default:
    assert_param(0);
    break;
  }

  switch (bits)
  {
  case 7:
    cr0_val |= UART_DATA_LENGTH_7;
    break;
  case 8:
    cr0_val |= UART_DATA_LENGTH_8;
    break;
  case 9:
    cr0_val |= UART_DATA_LENGTH_9;
    break;
  default:
    assert_param(0);
    break;
  }

  switch(parity)
  {
  case NO_PARITY:
    cr0_val |= UART_PARITY_DISABLE;
    break;
  case EVEN_PARITY:
    cr0_val |= UART_PARITY_BIT_EVEN | UART_PARITY_ENABLE;
    break;
  case ODD_PARITY:
    cr0_val |= UART_PARITY_BIT_ODD | UART_PARITY_ENABLE;
    break;
  default:
    assert_param(0);
  }

  switch (stop_bits)
  {
  case 1:
    cr0_val |= UART_STOP_BIT_1;
    break;
  case 2:
    cr0_val |= UART_STOP_BIT_2;
    break;
  default:
    assert_param(0);
    break;
  }

  UARTx->CR0 = cr0_val;

    /* (4) Transmit interrupt Enable    */
    UARTx->CR1 |= (UART_RX_INT_ENABLE|UART_TX_INT_ENABLE|UART_ERR_INT_ENABLE);
    UARTx->TRANS = 0x3;

  switch (channel)
  {
    case 0:
      NVIC_EnableIRQ(INTSC0RX_IRQn); /* Enable UART0_IRQn RX */
      NVIC_EnableIRQ(INTSC0ERR_IRQn); /* Enable UART0_IRQn ERR */
      break;
    case 1:
      NVIC_EnableIRQ(INTSC1RX_IRQn); /* Enable UART1_IRQn RX */
      NVIC_EnableIRQ(INTSC1ERR_IRQn); /* Enable UART1_IRQn ERR */
      break;
    case 2:
      NVIC_EnableIRQ(INTSC2RX_IRQn); /* Enable UART2_IRQn */
      NVIC_EnableIRQ(INTSC2ERR_IRQn); /* Enable UART2_IRQn ERR */
      break;
  case 3:
      NVIC_EnableIRQ(INTSC3RX_IRQn); /* Enable UART3_IRQn */
      NVIC_EnableIRQ(INTSC3ERR_IRQn); /* Enable UART3_IRQn ERR */
      break;
   default:
     return;
  }
}

void HAL_UART_Init(uint8_t channel,
                   uart_direction direction ,
                   speed_mode baud,
                   uint8_t bits,
                   parity_mode parity,
                   uint8_t stop_bits,
                   flow_control flow)
{
  switch (channel)
  {
    case 0:
      HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_0, HAL_GPIO_UART_TX);
      HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_1, HAL_GPIO_UART_RX);
      break;
    case 1:
     HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_4, HAL_GPIO_UART_TX);
     HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_5, HAL_GPIO_UART_RX);
     break;
    case 2:
      /* not used in current baseboard design */
      return;
      break;
    case 3:
      HAL_GPIO_Init(HAL_GPIO_PORT_F, HAL_GPIO_BIT_6, HAL_GPIO_UART_TX);
      HAL_GPIO_Init(HAL_GPIO_PORT_F, HAL_GPIO_BIT_7, HAL_GPIO_UART_RX);
      break;
   default:
     return;
  }

  UART_Config(channel,baud,bits,parity,stop_bits);
}

void HAL_UART_RegisterIRQCallback (uint8_t channel, uart_direction direction, void (*callback)())
{
  assert_param(callback);
  uart_irq_callback[channel][((uint8_t)direction) >> 1] = callback;
}

void HAL_UART_ClearPendingIRQ(uint8_t channel, uart_direction direction)
{
  switch (direction)
  {
  case HAL_UART_TX:
    switch (channel)
    {
    case 0:
      NVIC_ClearPendingIRQ(INTSC0TX_IRQn);
      break;
    case 1:
      NVIC_ClearPendingIRQ(INTSC1TX_IRQn);
      break;
    case 2:
      NVIC_ClearPendingIRQ(INTSC2TX_IRQn);
      break;
    case 3:
      NVIC_ClearPendingIRQ(INTSC3TX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_UART_RX:
    switch (channel)
    {
    case 0:
      NVIC_ClearPendingIRQ(INTSC0RX_IRQn);
      break;
    case 1:
      NVIC_ClearPendingIRQ(INTSC1RX_IRQn);
      break;
    case 2:
      NVIC_ClearPendingIRQ(INTSC2RX_IRQn);
      break;
    case 3:
      NVIC_ClearPendingIRQ(INTSC3RX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_UART_ERR:
    switch (channel)
    {
    case 0:
      NVIC_ClearPendingIRQ(INTSC0ERR_IRQn);
      break;
    case 1:
      NVIC_ClearPendingIRQ(INTSC1ERR_IRQn);
      break;
    case 2:
      NVIC_ClearPendingIRQ(INTSC2ERR_IRQn);
      break;
    case 3:
      NVIC_ClearPendingIRQ(INTSC3ERR_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_UART_EnableIRQ(uint8_t channel, uart_direction direction)
{
  switch (direction)
  {
  case HAL_UART_TX:
    switch (channel)
    {
    case 0:
      NVIC_EnableIRQ(INTSC0TX_IRQn);
      break;
    case 1:
      NVIC_EnableIRQ(INTSC1TX_IRQn);
      break;
    case 2:
      NVIC_EnableIRQ(INTSC2TX_IRQn);
      break;
    case 3:
      NVIC_EnableIRQ(INTSC3TX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_UART_RX:
    switch (channel)
    {
    case 0:
      NVIC_EnableIRQ(INTSC0RX_IRQn);
      break;
    case 1:
      NVIC_EnableIRQ(INTSC1RX_IRQn);
      break;
    case 2:
      NVIC_EnableIRQ(INTSC2RX_IRQn);
      break;
    case 3:
      NVIC_EnableIRQ(INTSC3RX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;

  case HAL_UART_ERR:
    switch (channel)
    {
    case 0:
      NVIC_EnableIRQ(INTSC0ERR_IRQn);
      break;
    case 1:
      NVIC_EnableIRQ(INTSC1ERR_IRQn);
      break;
    case 2:
      NVIC_EnableIRQ(INTSC2ERR_IRQn);
      break;
    case 3:
      NVIC_EnableIRQ(INTSC3ERR_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_UART_DisableIRQ(uint8_t channel, uart_direction direction)
{
  switch (direction)
  {
  case HAL_UART_TX:
    switch (channel)
    {
    case 0:
      NVIC_DisableIRQ(INTSC0TX_IRQn);
      break;
    case 1:
      NVIC_DisableIRQ(INTSC1TX_IRQn);
      break;
    case 2:
      NVIC_DisableIRQ(INTSC2TX_IRQn);
      break;
    case 3:
      NVIC_DisableIRQ(INTSC3TX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_UART_RX:
    switch (channel)
    {
    case 0:
      NVIC_DisableIRQ(INTSC0RX_IRQn);
      break;
    case 1:
      NVIC_DisableIRQ(INTSC1RX_IRQn);
      break;
    case 2:
      NVIC_DisableIRQ(INTSC2RX_IRQn);
      break;
    case 3:
      NVIC_DisableIRQ(INTSC3RX_IRQn);
      break;
    default:
      assert_param(0);
    break;
    }
    break;
  case HAL_UART_ERR:
    switch (channel)
    {
    case 0:
      NVIC_DisableIRQ(INTSC0ERR_IRQn);
      break;
    case 1:
      NVIC_DisableIRQ(INTSC1ERR_IRQn);
      break;
    case 2:
      NVIC_DisableIRQ(INTSC2ERR_IRQn);
      break;
    case 3:
      NVIC_DisableIRQ(INTSC3ERR_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  default:
    assert_param(0);
  }
}

void HAL_UART_SetIRQPriority(uint8_t channel, uart_direction direction, uint32_t prio)
{
  switch (direction)
  {
  case HAL_UART_TX:
    switch (channel)
    {
    case 0:
      NVIC_SetPriority(INTSC0TX_IRQn,prio);
      break;
    case 1:
      NVIC_SetPriority(INTSC1TX_IRQn,prio);
      break;
    case 2:
      NVIC_SetPriority(INTSC2TX_IRQn,prio);
      break;
    case 3:
      NVIC_SetPriority(INTSC3TX_IRQn,prio);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_UART_RX:
    switch (channel)
    {
    case 0:
      NVIC_SetPriority(INTSC0RX_IRQn,prio);
      break;
    case 1:
      NVIC_SetPriority(INTSC1RX_IRQn,prio);
      break;
    case 2:
      NVIC_SetPriority(INTSC2RX_IRQn,prio);
      break;
    case 3:
      NVIC_SetPriority(INTSC3RX_IRQn,prio);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_UART_ERR:
    switch (channel)
    {
    case 0:
      NVIC_SetPriority(INTSC0ERR_IRQn,prio);
      break;
    case 1:
      NVIC_SetPriority(INTSC1ERR_IRQn,prio);
      break;
    case 2:
      NVIC_SetPriority(INTSC2ERR_IRQn,prio);
      break;
    case 3:
      NVIC_SetPriority(INTSC3ERR_IRQn,prio);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_UART_SendData(uint8_t channel, uint32_t data)
{
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    UARTx->DR = data & 0x1FFU;

}

WorkState HAL_UART_GetBufState(uint8_t channel, uart_direction direction)
{
    uint32_t tmp = 0U;
    WorkState retval = BUSY;

    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }


    tmp = ((uint32_t) (UARTx->SR));
    switch (direction) {
    case HAL_UART_TX:
        if ((tmp & 0xf00) == 0) {
            /* Return Tx buffer empty if the flag is set */
            retval = DONE;
        } else if ((tmp & 0xf00) == 0x700){
            retval = BUSY;
        } else
        {
            /* Do nothing */
        }
        break;
    case HAL_UART_RX:
        if ((tmp & 0xf) == 0xf) {
            /* Return Rx buffer full if the flag is set */
            retval = DONE;
        } else {
            /* Do nothing */
        }
        break;
    default:
        /* Do nothing */
        break;
    }

    return retval;
}

uint32_t HAL_UART_GetReceivedData(uint8_t channel)
{
    volatile uint32_t retval = 0U;
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    /* Return received data */
    retval = (UARTx->DR & 0x1FFU);

    return retval;
}

uint32_t HAL_UART_GetError(uint8_t channel)
{
    uint32_t retval = 0U;
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    /* Return received data */
    retval = (UARTx->ERR & 0x1FU);

    return retval;
}

void HAL_UART_ClearError(uint8_t channel, uint32_t error)
{
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    UARTx->ERR |= error;
}



void HAL_UART_SetRxTx(uint8_t channel, uint8_t rxEnable, uint8_t txEnable)
{
    uint32_t val = 0U;
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    if (rxEnable)
    {
      val |= UART_RXE_ENABLE;
    }

    if (txEnable)
    {
      val |= UART_TXE_ENABLE;
    }
    UARTx->TRANS = (UARTx->TRANS & 0xFFFFFFFC) | val;
}

void HAL_UART_ClearRXFIFO(uint8_t channel)
{
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    UARTx->FIFOCLR = UART_RFCLR;
}

void HAL_UART_ClearTXFIFO(uint8_t channel)
{
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    UARTx->FIFOCLR = UART_TFCLR;
}

void HAL_UART_SWRst(uint8_t channel)
{
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    UARTx->SWRST = 0x2;
    UARTx->SWRST = 0x1;

    /* wait for completion */
    while (UARTx->SWRST != 0)
    {;}

}

uint32_t HAL_UART_GetRXFIFOLevel(uint8_t channel)
{
    uint32_t retval = 0U;
    TSB_UART_TypeDef* UARTx;

    switch (channel)
    {
    case 0:
          UARTx = TSB_UART0;
          break;
    case 1:
          UARTx = TSB_UART1;
          break;
    case 2:
          UARTx = TSB_UART2;
          break;
    case 3:
          UARTx = TSB_UART3;
          break;
    }

    retval = UARTx->SR & 0x0000000FU;

    return retval;
}
