/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _HAL_GPIO_H_
#define _HAL_GPIO_H_

typedef enum {
  HAL_GPIO_OUTPUT,
  HAL_GPIO_INPUT,
  HAL_GPIO_UART_TX,
  HAL_GPIO_UART_RX,
  HAL_GPIO_PWM_INPUT,
  HAL_GPIO_I2C_SDA,
  HAL_GPIO_I2C_SCL,
  HAL_GPIO_SPI_INPUT,
  HAL_GPIO_SPI_OUTPUT,
  HAL_GPIO_PMD_OUTPUT,
  HAL_GPIO_PMD_INPUT,
  HAL_GPIO_ENC,
} gpio_function;

typedef struct {
    uint8_t IOMode;         /* Set the port input or output mode       */
    uint8_t PullUp;         /* Enable or disable Pullup function       */
    uint8_t OpenDrain;      /* Enable or disable open drain function */
    uint8_t PullDown;       /* Enable or disable Pulldown function */
    uint8_t Interrupt;      /* Enable or disable interrupt function */
} HAL_GPIO_InitTypeDef;

typedef enum
{
  HAL_GPIO_PORT_A = 0x0, /*!< 0: PA       */
  HAL_GPIO_PORT_B,       /*!< 1: PB       */
  HAL_GPIO_PORT_C,       /*!< 2: PC       */
  HAL_GPIO_PORT_D,       /*!< 3: PD       */
  HAL_GPIO_PORT_E,       /*!< 4: PE       */
  HAL_GPIO_PORT_F,       /*!< 5: PF       */
  HAL_GPIO_PORT_G,       /*!< 6: PG       */
  HAL_GPIO_PORT_H,       /*!< 7: PH       */
  HAL_GPIO_PORT_I,       /*!< 8: PI       */
  HAL_GPIO_PORT_J,       /*!< 9: PJ       */
  HAL_GPIO_PORT_K,       /*!< 10:PK       */
  HAL_GPIO_PORT_L,       /*!< 11:PL       */
  HAL_GPIO_PORT_M,       /*!< 12:PM       */
  HAL_GPIO_PORT_N,       /*!< 13:PN       */
  HAL_GPIO_PORT_P,       /*!< 14:PP       */
  HAL_GPIO_PORT_R,       /*!< 15:PR       */
  HAL_GPIO_PORT_T,       /*!< 16:PT       */
  HAL_GPIO_PORT_U,       /*!< 17:PU       */
  HAL_GPIO_PORT_V,       /*!< 18:PV       */
  HAL_GPIO_PORT_W,       /*!< 19:PW       */
  HAL_GPIO_GROUP_Max     /*!< Max Number   */
}HAL_GPIO_Port;

#define HAL_GPIO_INPUT_MODE                 ((uint8_t)0x00)
#define HAL_GPIO_OUTPUT_MODE                ((uint8_t)0x01)

#define HAL_GPIO_IRQ_DISABLE                ((uint8_t)0x00)
#define HAL_GPIO_IRQ_ENABLE                 ((uint8_t)0x01)

#define HAL_GPIO_PULLUP_DISABLE             ((uint8_t)0x00)
#define HAL_GPIO_PULLUP_ENABLE              ((uint8_t)0x01)

#define HAL_GPIO_PULLDOWN_DISABLE           ((uint8_t)0x00)
#define HAL_GPIO_PULLDOWN_ENABLE            ((uint8_t)0x01)

#define HAL_GPIO_OPEN_DRAIN_DISABLE         ((uint8_t)0x00)
#define HAL_GPIO_OPEN_DRAIN_ENABLE          ((uint8_t)0x01)

#define HAL_GPIO_FUNC_REG_1                 ((uint8_t)0x00)
#define HAL_GPIO_FUNC_REG_2                 ((uint8_t)0x01)
#define HAL_GPIO_FUNC_REG_3                 ((uint8_t)0x02)
#define HAL_GPIO_FUNC_REG_4                 ((uint8_t)0x03)
#define HAL_GPIO_FUNC_REG_5                 ((uint8_t)0x04)
#define HAL_GPIO_FUNC_REG_6                 ((uint8_t)0x05)
#define HAL_GPIO_FUNC_REG_7                 ((uint8_t)0x06)

#define HAL_GPIO_BIT_0                      ((uint8_t)0x01)    
#define HAL_GPIO_BIT_1                      ((uint8_t)0x02)    
#define HAL_GPIO_BIT_2                      ((uint8_t)0x04)
#define HAL_GPIO_BIT_3                      ((uint8_t)0x08)
#define HAL_GPIO_BIT_4                      ((uint8_t)0x10)
#define HAL_GPIO_BIT_5                      ((uint8_t)0x20)
#define HAL_GPIO_BIT_6                      ((uint8_t)0x40)
#define HAL_GPIO_BIT_7                      ((uint8_t)0x80)                    
#define HAL_GPIO_BITS_ALL                   ((uint8_t)0xFF)                    

/*! \brief Configurates the selected Port pins  
  *
  * @param  port : number of the used port
  * @param  pin  : which should be configured
  * @param  function :   
  *
  * @retval 0: success 1: error
*/
uint8_t    HAL_GPIO_Init    (HAL_GPIO_Port port, uint8_t bit, gpio_function function);

/*! \brief Sets the pin level of the specified Port pin (only if configured as output port)
  *
  * @param  port : number of the used port
  * @param  pin  : which should be set
  * @param  value :  1: High level, 0: Low level 
  *
  * @retval 0: success 1: error
*/
uint8_t    HAL_GPIO_SetBit  (HAL_GPIO_Port port, uint8_t bit, uint8_t value);

/*! \brief Gets the output level of the specified Port pin (only if configured as input port)
  *
  * @param  port : number of the used port
  * @param  pin  : used pin 
  *
  * @retval value :  1: High level, 0: Low level 
*/
uint8_t    HAL_GPIO_GetBit  (HAL_GPIO_Port port, uint8_t bit);

/*! \brief Sets the pin level of all pins of the specified Port (only if configured as output port)
  *
  * @param  port : number of the used port
  * @param  pin  : which should be set
  * @param  value :  1: High level, 0: Low level bit coded for every pin 
  *
  * @retval 0: success 1: error
*/
uint8_t    HAL_GPIO_SetValue(HAL_GPIO_Port port, uint8_t value);

/*! \brief Gets the output level of all pins of the specified Port (only if configured as input port)
  *
  * @param  port : number of the used port
  *
  * @retval 1: High level, 0: Low level bit coded for every pin
*/
uint8_t    HAL_GPIO_GetValue(HAL_GPIO_Port port);

#endif