/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _HAL_OFD_H_
#define _HAL_OFD_H_

typedef enum {
    HAL_OFD_PLL_ON = 0U,
    HAL_OFD_PLL_OFF = 1U
} halOfdPllState;


typedef enum {
    HAL_OFD_DISABLE = 0U,
    HAL_OFD_ENABLE = 1U
} halOfdFunctionalState;

/**
  * @brief  Enable or disable the writing of OFDCR2/OFDMNPLLON/OFDMXPLLON/OFDMNPLLOFF/OFDMXPLLOFF.
  * @param  NewState: the new state of enable or disable
  *             This parameter can be one of the following values:
  *             ENABLE or DISABLE
  * @retval None
  */
extern void HAL_OFD_SetRegWriteMode(halOfdFunctionalState newState);

/**
  * @brief  Enables the OFD function.
  * @param  None
  * @retval None
  */
extern void HAL_OFD_Enable(void);

/**
  * @brief  Disables the OFD function.
  * @param  None
  * @retval None
  */
extern void HAL_OFD_Disable(void);

/**
  * @brief  Set the count value of detection frequency.
  * @param  State: State of PLL.
  *             This parameter can be one of the following values:
  *             OFD_PLL_ON or OFD_PLL_OFF
  *         HigherDetectionCount: the count value of higher detection frequency.
  *         LowerDetectionCount: the count value of lower detection frequency.
  * @retval None
  */
extern void HAL_OFD_SetDetectionFrequency(halOfdPllState state, uint32_t higherDetectionCount,
                               uint32_t lowerDetectionCount);


#endif