/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef _SPI_H_
#define _SPI_H_

#if defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__ || defined __TMPM_4KN__

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include TMPM_HEADER_FILE
#include TMPM_SPI_HEADER_FILE
#include "hal_spi.h"


/*! SPI data length */
typedef enum
{
  HAL_SPI_DATA_LENGTH_8   = 0x08,    /*!< 8 bit  */
  HAL_SPI_DATA_LENGTH_9   = 0x09,    /*!< 9 bit  */
  HAL_SPI_DATA_LENGTH_10  = 0x0a,    /*!< 10 bit */
  HAL_SPI_DATA_LENGTH_11  = 0x0b,    /*!< 11 bit */
  HAL_SPI_DATA_LENGTH_12  = 0x0c,    /*!< 12 bit */
  HAL_SPI_DATA_LENGTH_13  = 0x0d,    /*!< 13 bit */
  HAL_SPI_DATA_LENGTH_14  = 0x0e,    /*!< 14 bit */
  HAL_SPI_DATA_LENGTH_15  = 0x0f,    /*!< 15 bit */
  HAL_SPI_DATA_LENGTH_16  = 0x10,    /*!< 16 bit */
  HAL_SPI_DATA_LENGTH_17  = 0x11,    /*!< 17 bit */
  HAL_SPI_DATA_LENGTH_18  = 0x12,    /*!< 18 bit */
  HAL_SPI_DATA_LENGTH_19  = 0x13,    /*!< 19 bit */
  HAL_SPI_DATA_LENGTH_20  = 0x14,    /*!< 20 bit */
  HAL_SPI_DATA_LENGTH_21  = 0x15,    /*!< 21 bit */
  HAL_SPI_DATA_LENGTH_22  = 0x16,    /*!< 22 bit */
  HAL_SPI_DATA_LENGTH_23  = 0x17,    /*!< 23 bit */
  HAL_SPI_DATA_LENGTH_24  = 0x18,    /*!< 24 bit */
  HAL_SPI_DATA_LENGTH_25  = 0x19,    /*!< 25 bit */
  HAL_SPI_DATA_LENGTH_26  = 0x1a,    /*!< 26 bit */
  HAL_SPI_DATA_LENGTH_27  = 0x1b,    /*!< 27 bit */
  HAL_SPI_DATA_LENGTH_28  = 0x1c,    /*!< 28 bit */
  HAL_SPI_DATA_LENGTH_29  = 0x1d,    /*!< 29 bit */
  HAL_SPI_DATA_LENGTH_30  = 0x1e,    /*!< 30 bit */
  HAL_SPI_DATA_LENGTH_31  = 0x1f,    /*!< 31 bit */
  HAL_SPI_DATA_LENGTH_32  = 0x20,    /*!< 32 bit */
} HAL_SPI_DataLength;

typedef enum
{
  HAL_SPI_CK_1ST_EDGE,           /*!< 1st Edge Sampling */
  HAL_SPI_CK_2ND_EDGE,           /*!< 2nd Edge Sampling */
} HAL_SPI_CLK_SAMPLING;

typedef enum
{
    HAL_SPI_CK_IDLE_LOW,            /*!< IDLE LEVEL CLOCK LOW */
    HAL_SPI_CK_IDLE_HI,             /*!< IDLE LEVEL CLOCK HI  */
}HAL_SPI_CLK_IDLE_LEVEL;

/*! SPI speed modes */
typedef enum
{
  HAL_SPI_20_MHZ = 1,
  HAL_SPI_10_MHZ,
  HAL_SPI_5_MHZ,
  HAL_SPI_3_33_MHZ,
  HAL_SPI_2_5_MHZ,
  HAL_SPI_1_MHZ,
  HAL_SPI_500_KHZ,
  HAL_SPI_250_KHZ,
  HAL_SPI_125_KHZ,
}HAL_SPI_Speed;

/*! SPI transmissions */
typedef enum
{
  HAL_SPI_TX = 1<<0,
  HAL_SPI_RX = 1<<1,
} HAL_SPI_Direction;

/*! Function return values */
typedef enum
{
  HAL_SPI_NO_ERR                = 0U,                                           /*!<  success */
  HAL_SPI_TRGERR                ,                                               /*!<  SPI trigger communication error */
  HAL_SPI_UNDERRUN              ,                                               /*!<  SPI underrun error */
  HAL_SPI_OVERRUN               ,                                               /*!<  SPI overrun error */
  HAL_SPI_PARITY_ERR            ,                                               /*!<  SPI parity error */
  HAL_SPI_TRANSMIT_ERR          ,                                               /*!<  SPI transmit error */
  HAL_SPI_RECEIVE_ERR           ,                                               /*!<  SPI receive error */
  HAL_SPI_INIT_ERR              ,                                               /*!<  SPI initialisation error */
  HAL_SPI_INVALID_PARAMETER     ,                                               /*!<  SPI invalid input paraameter */
  HAL_SPI_ERRS                  ,                                               /*!<  SPI general error */
} HAL_SPI_Err;

/**
 * @struct HAL_SPI_TransRec8_t
 * @brief  Transmit data information structure definenition.
 * @brief  When data length definenition is "8bit", use this.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t *p_data;     /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
} HAL_SPI_TransRec8_t;

/*----------------------------------*/
/**
 * @struct HAL_SPI_TransRec16_t
 * @brief  Transmit data information structure definenition.
 * @brief  When data length definenition is "9 - 16 bit", use this.
*/
/*----------------------------------*/
typedef struct
{
    uint16_t *p_data;    /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
} HAL_SPI_TransRec16_t;

/*----------------------------------*/
/**
 * @struct HAL_SPI_TransRec32_t
 * @brief  Transmit data information structure definnition.
 * @brief  When data length definenition is "17 - 32 bit", use this.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t *p_data;    /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
} HAL_SPI_TransRec32_t;

/*----------------------------------*/
/**
 * @struct HAL_SPI_TransRec_t
 * @brief  Transmit data information structure definition.
*/
/*----------------------------------*/
typedef union
{
    HAL_SPI_TransRec8_t  trx8;                                                  /*!< transmit8_t  */
    HAL_SPI_TransRec16_t trx16;                                                 /*!< transmit16_t */
    HAL_SPI_TransRec32_t trx32;                                                 /*!< transmit16_t */
}HAL_SPI_TransRec_t;

/*
 -----------------------------------------------------------------------------------------------------------------------
  @brief SPI Init Structure definition
 -----------------------------------------------------------------------------------------------------------------------
 */
typedef struct
{
  uint8_t   channel;                                                            /*!< */
  uint8_t   chipSelect;                                                         /*!< Chip select signal */
  uint8_t   transferMode;                                                       /*!< This member configures the SPI mode: 1= master, 0 = slave */
  HAL_SPI_Speed  speed;                                                         /*!< Speed for SPI port */
  HAL_SPI_DataLength length;                                                    /*!< Data length */
  HAL_SPI_CLK_SAMPLING clkSampling;                                             /*!< Data sampling*/
  HAL_SPI_CLK_IDLE_LEVEL clkIdleLvl;                                            /*!< Clock idle level */
  void (*transmitHandler)(HAL_SPI_Err error);                                   /*!< callback function only for none blocking transmit transmissions*/
  void (*receiveHandler)(HAL_SPI_Err error, HAL_SPI_TransRec_t *receive);       /*!< callback function only for none blocking receive transmissions*/
} HAL_SPI_CfgData;

/*
 -----------------------------------------------------------------------------------------------------------------------
  @brief SPI Init Structure definition
 -----------------------------------------------------------------------------------------------------------------------
 */
typedef struct
{
  tspi_t  tspi;
  void (*transmitHandler)(HAL_SPI_Err error);
  void (*receiveHandler)(HAL_SPI_Err error, HAL_SPI_TransRec_t *receive);
} HAL_SPI_Data;

/*
 * @} ;
 * End of group SPI_Exported_Types ;
 * @defgroup SPI_Exported_Constants @{
 */
#define SPI0                  TSB_SC0
#define SPI1                  TSB_SC1
#ifdef __TMPM_370__
#define SPI2                  TSB_SC2
#endif
#define SPI3                  TSB_SC3
#ifdef __TMPM_370__
#define IS_SPI_PERIPH(param)  (((param) == SPI0) || ((param) == SPI1) || ((param) == SPI2) || ((param) == SPI3))
#endif
#ifdef __TMPM_372__
#define IS_SPI_PERIPH(param)  (((param) == SPI0) || ((param) == SPI1) || ((param) == SPI3))
#endif
#ifdef __TMPM_373__
#define IS_SPI_PERIPH(param)  (((param) == SPI0) || ((param) == SPI1) || ((param) == SPI3))
#endif
#ifdef __TMPM_374__
#define IS_SPI_PERIPH(param)  (((param) == SPI0) || ((param) == SPI1) || ((param) == SPI3))
#endif
#ifdef __TMPM_375__
#define IS_SPI_PERIPH(param)  (((param) == SPI0))
#endif

#define SPI_ENABLE_RX ((uint32_t) 0x00000020)
#define SPI_ENABLE_TX ((uint32_t) 0x00000010)
#define IS_SPI_MODE(param) \
    (((param) == SPI_ENABLE_RX) || ((param) == SPI_ENABLE_TX) || ((param) == (SPI_ENABLE_TX | SPI_ENABLE_RX)))
#define SPI_RX            ((uint32_t) 0x00000020)
#define SPI_TX            ((uint32_t) 0x00000040)
#define IS_SPI_TRX(param) (((param) == SPI_RX) || ((param) == SPI_TX))

/*! \brief  Enable SPI channel
  *
  * @param  channel:       channel to enable
  * @retval None
*/
void          HAL_SPI_Enable      (uint8_t channel);

/*! \brief  Disable SPI channel
  *
  * @param  channel:       channel to disable
  * @retval None
*/
void          HAL_SPI_Disable     (uint8_t channel);


/*! \brief  Performs a SW reset of SPI channel
  *
  * @param  channel:       channel to reset
  * @retval None
*/
void          HAL_SPI_SWReset     (uint8_t channel);

/*! \brief  Initialise a specific SPI channel
  *
  * @param  HAL_SPI_CfgData  pointer to init structure with parameter for intitialisation
  * @retval HAL_SPI_Err
*/
HAL_SPI_Err   HAL_SPI_Init(const HAL_SPI_CfgData* initStruct);

/*! \brief  Get data from SPI channel
  *
  * @param  channel:       channel to use
  * @retval Received Data
*/
uint32_t HAL_SPI_GetRxData   (uint8_t channel);

/*! \brief  Send Data on SPI channel
  *
  * @param  channel:       channel to use
  * @param  Data:          data to send
  * @retval void
*/
void  HAL_SPI_SetTxData   (uint8_t channel, uint32_t Data);

/*! \brief  Get error state of the SPI channel
  *
  * @param  channel:       channel to configure
  * @retval Enumeration values of enum HAL_SPI_Err
*/
HAL_SPI_Err  HAL_SPI_GetErrState (uint8_t channel);

/*! \brief None Blocking function to send multiple data in master mode
  *
  * @param  channel:            channel to use
  * @param  *write:             Pointer to the writa buffer
  * @retval Enumeration values of enum HAL_SPI_Err
*/
HAL_SPI_Err HAL_SPI_MasterWriteBytes(uint8_t channel, HAL_SPI_TransRec_t *write);

/*! \brief None Blocking function to read multiple data in master mode
  *
  * @param  channel:        Channel to use
  * @param  *read:          Pointer to the read buffer
  * @retval Enumeration values of enum HAL_SPI_Err
*/
HAL_SPI_Err  HAL_SPI_MasterReadBytes (uint8_t channel, const HAL_SPI_TransRec_t *read);

/*! \brief Blocking function to send one buffer in master mode
  *
  * @param  channel:        Channel to use
  * @param  *transfer:      Pointer to data to be send
  * @retval Enumeration values of enum HAL_SPI_Err
*/
HAL_SPI_Err  HAL_SPI_MasterTransfer  (uint8_t channel, const HAL_SPI_TransRec_t *transfer);

/*! \brief Blocking function to read one buffer in master mode
  *
  * @param  channel:        Channel to use
  * @param  *receive:       Pointer to the read buffer
  * @retval Enumeration values of enum HAL_SPI_Err
*/
HAL_SPI_Err  HAL_SPI_MasterReceive   (uint8_t channel, HAL_SPI_TransRec_t *receive);

/*! \brief
  *
  * @param  channel:       Channel to use
  * @param  direction:     Enable Irq for either TX or RX SPI direction
  * @retval void
*/
void HAL_SPI_EnableIRQ(uint8_t channel, HAL_SPI_Direction direction);

/*! \brief
  *
  * @param  channel:       Channel to use
  * @param  direction      Disable Irq for either TX or RX SPI direction
  * @retval void
*/
void HAL_SPI_DisableIRQ(uint8_t channel, HAL_SPI_Direction direction);

/*! \brief
  *
  * @param  channel:       channel to configure
  * @param  direction
  * @param  prio
  * @retval void
*/
void HAL_SPI_SetIRQPriority(uint8_t channel, HAL_SPI_Direction direction, uint32_t prio);


#endif // __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__ || defSined __TMPM_4KN__


#endif /* _SPI_H_ */
