/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _HAL_UART_H_
#define _HAL_UART_H_

#if defined __TMPM_370__ || defined __TMPM_372__ || defined __TMPM_373__ || defined __TMPM_374__ || defined __TMPM_375__ || defined __TMPM_376__
#include TMPM_UART_HEADER_FILE
#else
#include "txz_driver_def.h"
#endif

typedef enum
{
  HAL_UART_TX = 1<<0,
  HAL_UART_RX = 1<<1,
  HAL_UART_RXTX = 0x3,
  HAL_UART_ERR = 1<<2,
} uart_direction;

typedef enum {
  NO_PARITY,
  EVEN_PARITY,
  ODD_PARITY,
} parity_mode;

typedef enum {
  NO_FLOW_CTRL
} flow_control;

typedef enum {
  BERR = 1<<0,
  FERR = 1<<1,
  PERR = 1<<2,
  OVRERR = 1<<3,
  TRGERR = 1<<4,
} uart_errors;

typedef enum {
  SPEED_115200,
  SPEED_5MHZ,
} speed_mode;

void HAL_UART_Init(uint8_t channel, uart_direction direction , speed_mode baud, uint8_t bits, parity_mode parity, uint8_t stop_bits, flow_control flow);
WorkState HAL_UART_GetBufState(uint8_t channel, uart_direction direction);
void HAL_UART_SendData(uint8_t channel, uint32_t data);
uint32_t HAL_UART_GetReceivedData(uint8_t channel);
void HAL_UART_RegisterIRQCallback (uint8_t channel, uart_direction direction, void (*callback)());
void HAL_UART_DisableIRQ(uint8_t channel, uart_direction direction);
void HAL_UART_EnableIRQ(uint8_t channel, uart_direction direction);
void HAL_UART_ClearPendingIRQ(uint8_t channel, uart_direction direction);
void HAL_UART_SetIRQPriority(uint8_t channel, uart_direction direction, uint32_t prio);
void HAL_UART_ClearError(uint8_t channel, uint32_t error);
uint32_t HAL_UART_GetError(uint8_t channel);
void HAL_UART_SetRxTx(uint8_t channel, uint8_t rxEnable, uint8_t txEnable);
void HAL_UART_DisableRX(uint8_t channel);
void HAL_UART_ClearRXFIFO(uint8_t channel);
uint32_t HAL_UART_GetRXFIFOLevel(uint8_t channel);
#endif