/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _VE_DEFINE_H_
#define _VE_DEFINE_H_

#include <stdint.h>
#include "config.h"
#include TMPM_HEADER_FILE
#include "hal_timer.h"

#if 1

#define VE_CHANNEL0_TMRB                 TSB_TB3                                /* Timer for VE Loop */
#define VE_CHANNEL0_TMBR_IRQ             INTTB30_IRQn
#define VE_CHANNEL0_TMBR_IRQ_HANDLER     INTTB30_IRQHandler

#ifndef __TMPM_375__
#define VE_CHANNEL1_TMRB                 TSB_TB1                                /* Timer for VE Loop */
#define VE_CHANNEL1_TMBR_IRQ             INTTB10_IRQn
#define VE_CHANNEL1_TMBR_IRQ_HANDLER     INTTB10_IRQHandler
#else
#define VE_CHANNEL1_TMRB                 TSB_TB0                                /* Timer for VE Loop */
#define VE_CHANNEL1_TMBR_IRQ             INTTB00_IRQn
#define VE_CHANNEL1_TMBR_IRQ_HANDLER     INTTB00_IRQHandler
#endif /* __TMPM_375__ */

#endif

#define PAI_VALUE   31416                                                       /* Pi     * 10000 */
#define PAI_UNIT    -4
#define PAI2_VALUE  2 * PAI_VALUE                                               /* Pi * 2 * 10000 */
#define PAI2_UNIT   -4

#define SECONDS_PER_MINUTE                60                                    /* One minute has 60 seconds */

#define VE_ENABLE                         (1 << 0)                              /* Enable VE */
#define VE_DISABLE                        (0 << 0)                              /* Enable VE */
#define VE_IDLE_ACTIVE                    (1 << 1)                              /* Controls whether or not the clock is supplied to the Vector Engine in IDLE mode */

#define VE_MODE_OFF                       0x00                                  /* Flag VE channel off */
#define VE_OUTPUTENABLE	                  0x04                                  /* Flag VE output enable */
#define VE_EMGRETURN                      0x0C                                  /* Flag VE return from EMG */
#define VE_ZEROCURRENTEN                  0x02                                  /* Flag VE Zero Current detection enable */
#define VE_PHASEINTERPOL                  0x01                                  /* Flag VE phase interpolation enable */


#define VE_TRIGGER_MODE_CHANNEL_0_SHIFT   0
#define VE_TRIGGER_MODE_CHANNEL_1_SHIFT   2
#define VE_TRIGGER_MODE_IGN_BOTH          0x00                                  /* Ignore both INTA0 (unit A) and INTA1 (unit B) */
#define VE_TRIGGER_MODE_START_x0          0x01                                  /* Start by INTx0 */
#define VE_TRIGGER_MODE_START_x1          0x02                                  /* Start by INTx1 */
#define VE_TRIGGER_MODE_START_BOTH        0x03                                  /* Start when both INTA0 (unit A) and INTA1 (unit B) occur */

#define VE_CPURUNTRG_CHANNEL_0            (0x1 << 0)                            /* Starts channel 0 by programming */
#define VE_CPURUNTRG_CHANNEL_1            (0x1 << 1)                            /* Starts channel 1 by programming */

/* FMODE */
#define VE_FMODE_3PHASE_MODULATION        (0 << 0)
#define VE_FMODE_2PHASE_MODULATION        (1 << 0)
#define VE_FMODE_PWM_SHIFT_DISABLE        (0 << 1)
#define VE_FMODE_PWM_SHIFT_ENABLE         (1 << 1)
#define VE_FMODE_CURR_DETECT_3SHUNT       (0x0 << 2)
#define VE_FMODE_CURR_DETECT_2SENSOR      (0x1 << 2)
#define VE_FMODE_CURR_DETECT_1SHUNT_UP    (0x2 << 2)
#define VE_FMODE_CURR_DETECT_1SHUNT_DOWN  (0x3 << 2)
#define VE_FMODE_PMD_CHANNEL_0            (0 << 4)
#define VE_FMODE_PMD_CHANNEL_1            (1 << 4)
#define VE_FMODE_ADCSEL_UNIT_A            (0x0 << 6)
#define VE_FMODE_ADCSEL_UNIT_B            (0x1 << 6)
#define VE_FMODE_ADCSEL_UNIT_AB           (0x2 << 6)
#define VE_FMODE_MREGDIS                  (1 << 9)

/* MODE */
#define VE_MODE_PHASE_INTERPOLATION_EN    (1 << 0)
#define VE_MODE_ZERO_CURRENT_DETECT       (1 << 1)
#define VE_MODE_OUTPUT_OFF                (0x0 << 2)
#define VE_MODE_OUTPUT_ENABLE             (0x1 << 2)
#define VE_MODE_OUTPUT_OFF_EMG_RET        (0x3 << 2)

/* REPTIME */
#define VE_REPTIME_CHANNEL_0_SHIFT        0
#define VE_REPTIME_CHANNEL_1_SHIFT        4
#define VE_REPTIME_NO_SCHEDULE            0
#define VE_REPTIME_1                      0x01
#define VE_REPTIME_2                      0x02
#define VE_REPTIME_3                      0x03
#define VE_REPTIME_4                      0x04
#define VE_REPTIME_5                      0x05
#define VE_REPTIME_6                      0x06
#define VE_REPTIME_7                      0x07
#define VE_REPTIME_8                      0x08
#define VE_REPTIME_9                      0x09
#define VE_REPTIME_10                     0x0a
#define VE_REPTIME_11                     0x0b
#define VE_REPTIME_12                     0x0c
#define VE_REPTIME_13                     0x0d
#define VE_REPTIME_14                     0x0e
#define VE_REPTIME_15                     0x0f

/* TASKAPP */
#define VE_TASKAPP_CLEAN_MASK                         0x0f
#define VE_TASKAPP_OUTPUT_CONTROL                     0x00
#define VE_TASKAPP_TRIGGER_GENERATION                 0x01
#define VE_TASKAPP_INPUT_PROCESSING                   0x02
#define VE_TASKAPP_INPUT_PHASE_CONVERSION             0x03
#define VE_TASKAPP_INPUT_COORDINATE_AXIS_CONVERSION   0x04
#define VE_TASKAPP_CURRENT_CONTROL                    0x05
#define VE_TASKAPP_SIN_COS_COMPUTATION                0x06
#define VE_TASKAPP_OUTPUT_COORDINATE_AXIS_CONVERSION  0x07
#define VE_TASKAPP_OUTPUT_PHASE_CONVERSION            0x08

/* ACTSCH */
#define VE_ACTSCH_CLEAN_MASK                0x0f
#define VE_ACTSCH_INDIVIDUAL_TASK_EXECUTION 0x00
#define VE_ACTSCH_SCHEDULE_0                0x00
#define VE_ACTSCH_SCHEDULE_1                0x01
#define VE_ACTSCH_SCHEDULE_4                0x04
#define VE_ACTSCH_SCHEDULE_9                0x09

typedef struct
{
  __IO uint32_t EN;                                                             /*!< VE enable/disable                            */
  __O  uint32_t CPURUNTRG;                                                      /*!< CPU start trigger selection                  */
  __IO uint32_t TASKAPP;                                                        /*!< Task selection                               */
  __IO uint32_t ACTSCH;                                                         /*!< Operation schedule selection                 */
  __IO uint32_t REPTIME;                                                        /*!< Schedule repeat count                        */
  __IO uint32_t TRGMODE;                                                        /*!< Start trigger mode                           */
  __IO uint32_t ERRINTEN;                                                       /*!< Error interrupt enable/disable               */
  __O  uint32_t COMPEND;                                                        /*!< VE forced termination                        */
  __I  uint32_t ERRDET;                                                         /*!< Error detection                              */
  __I  uint32_t SCHTASKRUN;                                                     /*!< Schedule executing flag/executing task       */
       uint32_t RESERVED0;                                                      /*!< Reserved  register                           */
  __IO uint32_t TMPREG0;                                                        /*!< Temporary register                           */
  __IO uint32_t TMPREG1;                                                        /*!< Temporary register                           */
  __IO uint32_t TMPREG2;                                                        /*!< Temporary register                           */
  __IO uint32_t TMPREG3;                                                        /*!< Temporary register                           */
  __IO uint32_t TMPREG4;                                                        /*!< Temporary register                           */
  __IO uint32_t TMPREG5;                                                        /*!< Temporary register                           */
       uint32_t RESERVED00;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED01;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED02;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED03;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED04;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED05;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED06;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED07;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED08;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED09;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED10;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED11;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED12;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED13;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED14;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED15;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED16;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED17;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED18;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED19;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED20;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED21;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED22;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED23;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED24;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED25;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED26;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED27;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED28;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED29;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED30;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED31;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED32;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED33;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED34;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED35;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED36;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED37;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED38;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED39;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED40;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED41;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED42;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED43;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED44;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED45;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED46;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED47;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED48;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED49;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED50;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED51;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED52;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED53;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED54;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED55;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED56;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED57;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED58;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED59;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED60;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED61;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED62;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED63;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED64;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED65;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED66;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED67;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED68;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED69;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED70;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED71;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED72;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED73;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED74;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED75;                                                     /*!< Placeholder                                  */
       uint32_t RESERVED76;                                                     /*!< Placeholder                                  */
  __IO uint32_t TADC;                                                           /*!< Common ADC conversion time (based on PWM clock) */
} TEE_VEC_TypeDef;

typedef struct
{
  __IO uint32_t MCTLF;                                                          /*!< Status flags                                 */
  __IO uint32_t MODE;                                                           /*!< Task control mode                            */
  __IO uint32_t FMODE;                                                          /*!< Flow control                                 */
  __IO uint32_t TPWM;                                                           /*!< PWM period rate (PWM period [s] \A1\C1 maximum speed*1 \A1\C1 2^16) */
  __IO uint32_t OMEGA;                                                          /*!< Rotation speed (speed [Hz]\A1\C2 maximum speed *1\A1\F91 \A1\C1 2^15) */
  __IO uint32_t THETA;                                                          /*!< Motor phase (motor phase [deg]/360 \A1\C1 2^16) */
  __IO uint32_t IDREF;                                                          /*!< d-axis reference value (current [A] \A1\C2 maximum current*2 \A1\C1 2^15) */
  __IO uint32_t IQREF;                                                          /*!< q-axis reference value (current [A] \A1\C2 maximum current *2 \A1\C1 2^15) */
  __IO uint32_t VD;                                                             /*!< d-axis voltage (voltage [V] \A1\C2 maximum voltage *3 \A1\C1 2^31)*/
  __IO uint32_t VQ;                                                             /*!< q-axis voltage (voltage [V] \A1\C2 maximum voltage *3 \A1\C1 2^31)*/
  __IO uint32_t CIDKI;                                                          /*!< Integral coefficient for PI control of d-axis */
  __IO uint32_t CIDKP;                                                          /*!< Proportional coefficient for PI control of d-axis */
  __IO uint32_t CIQKI;                                                          /*!< Integral coefficient for PI control of q-axis */
  __IO uint32_t CIQKP;                                                          /*!< Proportional coefficient for PI control of q-axis */
  __IO uint32_t VDIH;                                                           /*!< Upper 32 bits of integral term (VDI ) of d-axis voltage */
  __IO uint32_t VDILH;                                                          /*!< Lower 32 bits of integral term (VDI) of d-axis voltage */
  __IO uint32_t VQIH;                                                           /*!< Upper 32 bits of integral term (VQI) of q-axis voltage */
  __IO uint32_t VQILH;                                                          /*!< Lower 32 bits of integral term (VQI) of q-axis voltage */
  __IO uint32_t FPWMCHG;                                                        /*!< Switching speed (for 2-phase modulation and shift PWM) */
  __IO uint32_t MDPRD;                                                          /*!< PWM period (to be set identically with PMD\A1\AFs PWM period) */
  __IO uint32_t MINPLS;                                                         /*!< Minimum pulse width                          */
  __IO uint32_t TRGCRC;                                                         /*!< Synchronizing trigger correction value       */
       uint32_t RESERVED;                                                       /*!< Reserved                                     */
  __IO uint32_t COS;                                                            /*!< Cosine value at THETA for output conversion (Q15 data) */
  __IO uint32_t SIN;                                                            /*!< Sine value at THETA for output conversion (Q15 data) */
  __IO uint32_t COSM;                                                           /*!< Previous cosine value for input processing (Q15 data) */
  __IO uint32_t SINM;                                                           /*!< Previous sine value for input processing (Q15 data) */
  __IO uint32_t SECTOR;                                                         /*!< Sector information (0-11)                    */
  __IO uint32_t SECTORM;                                                        /*!< Previous sector information for input processing (0-11) */
  __IO uint32_t IA0;                                                            /*!< AD conversion result of a-phase zero-current *4 */
  __IO uint32_t IB0;                                                            /*!< AD conversion result of b-phase zero-current *4 */
  __IO uint32_t IC0;                                                            /*!< AD conversion result of c-phase zero-current *4 */
  __IO uint32_t IAADC;                                                          /*!< AD conversion result of a-phase current *4    */
  __IO uint32_t IBADC;                                                          /*!< AD conversion result of b-phase current *4    */
  __IO uint32_t ICADC;                                                          /*!< AD conversion result of c-phase current *4    */
  __IO uint32_t VDC;                                                            /*!< DC supply voltage (voltage [V] \A1\C2 maximum voltage *3 \A1\C1 2^15) */
  __IO uint32_t ID;                                                             /*!< d-axis current (current [A] \A1\C2 maximum current *2 \A1\C1 2^31) */
  __IO uint32_t IQ;                                                             /*!< q-axis current (current [A] \A1\C2 maximum current *2 \A1\C1 2^31) */
} TEE_VE_TypeDef;

typedef struct
{
  __IO uint32_t VCMPU;                                                          /*!< PMD control: CMPU setting                    */
  __IO uint32_t VCMPV;                                                          /*!< PMD control: CMPV setting                    */
  __IO uint32_t VCMPW;                                                          /*!< PMD control: CMPW setting                    */
  __IO uint32_t OUTCR;                                                          /*!< PMD control: Output control (MDOUT)          */
  __IO uint32_t VTRGCMP0;                                                       /*!< PMD control: TRGCMP0 setting                 */
  __IO uint32_t VTRGCMP1;                                                       /*!< PMD control: TRGCMP1 setting                 */
  __IO uint32_t VTRGSEL;                                                        /*!< PMD control: Trigger selection               */
  __O  uint32_t EMGRS;                                                          /*!< PMD control: EMG return (EMGCR[EMGRS])       */
}TEE_VEPMD_TypeDef;

#ifdef __TMPM_4KN__
#define TEE_VEC    (( TEE_VEC_TypeDef *)   TSB_VE0_BASE)
#define TEE_VE0    (( TEE_VE_TypeDef  *)   (TSB_VE0_BASE+0x044))
#define TEE_VEPMD0 (( TEE_VEPMD_TypeDef *) (TSB_VE0_BASE+0x17c))
#else

#ifndef __TMPM_475__
#define TEE_VEC    (( TEE_VEC_TypeDef *)   TSB_VE_BASE)
#define TEE_VE0    (( TEE_VE_TypeDef  *)   (TSB_VE_BASE+0x044))
#define TEE_VE1    (( TEE_VE_TypeDef  *)   (TSB_VE_BASE+0x0dc))
#define TEE_VEPMD0 (( TEE_VEPMD_TypeDef *) (TSB_VE_BASE+0x17c))
#define TEE_VEPMD1 (( TEE_VEPMD_TypeDef *) (TSB_VE_BASE+0x19c))
#else
#define TEE_VEC0   (( TEE_VEC_TypeDef *)   (TSB_VE0_BASE+0x000))
#define TEE_VEC1   (( TEE_VEC_TypeDef *)   (TSB_VE1_BASE+0x000))
#define TEE_VE0    (( TEE_VE_TypeDef  *)   (TSB_VE0_BASE+0x044))
#define TEE_VE1    (( TEE_VE_TypeDef  *)   (TSB_VE1_BASE+0x044))
#define TEE_VEPMD0 (( TEE_VEPMD_TypeDef *) (TSB_VE0_BASE+0x17c))
#define TEE_VEPMD1 (( TEE_VEPMD_TypeDef *) (TSB_VE1_BASE+0x19c))

#define  INTVCNA_IRQn INTVCN0_IRQn
#define  INTVCNB_IRQn INTVCN1_IRQn
#endif
#endif /* __TMPM_4KN__ */

#ifdef FIX_155_PMD_DELAY
extern uint8_t ve_irq_state[3];
#endif

/*! \brief PMD output state
 *
 *  Switch off or on the PMD
 */
typedef enum {
  PMD_OFF,
  PMD_ON,
} VE_PMD_State;

/*! \brief VE interrupt state
 *
 *  Switch off or on the IRQs
 */
typedef enum {
  VE_IRQ_OFF,
  VE_IRQ_ON,
} VE_IRQ_State;

void HAL_VE_PMD_Output(uint8_t channel_number, VE_PMD_State state);
void HAL_VE_IRQ(uint8_t channel_number ,VE_IRQ_State state);
void HAL_VE_InitPISettings(uint8_t channel_number);
void HAL_VE_Init(uint8_t channel_number);
void HAL_VE_PMD_EMGReturn(uint8_t channel_number);

#ifdef FIX_155_PMD_DELAY
void updatePmdRegisterSetting_Ch0();
#endif

#endif /* _VE_DEFINE_H_ */