/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _HAL_WDT_H_
#define _HAL_WDT_H_

#define HAL_WDT_DETECT_TIME_EXP_15          ((uint32_t)0x00000000)
#define HAL_WDT_DETECT_TIME_EXP_17          ((uint32_t)0x00000010)
#define HAL_WDT_DETECT_TIME_EXP_19          ((uint32_t)0x00000020)
#define HAL_WDT_DETECT_TIME_EXP_21          ((uint32_t)0x00000030)
#define HAL_WDT_DETECT_TIME_EXP_23          ((uint32_t)0x00000040)
#define HAL_WDT_DETECT_TIME_EXP_25          ((uint32_t)0x00000050)

#define HAL_WDT_WDOUT                       ((uint32_t)0x00000001)
#define HAL_WDT_NMIINT                      ((uint32_t)0x00000000)


/*! \brief Configure DSO Logging facility response structure
 *
 *  This structure contains all required information for the configuring of the logging of the internal
 *  Vector Engine Registers response
 */
typedef struct {
    uint32_t DetectTime;            /*!< Set WDT detection time. */
    uint32_t OverflowOutput;        /*!< Select "Generates NMI interrupt" or "Connects WDT out to reset". */
} halWdtInitTypeDef;

/**
  * @brief  Initialize WDT.
  * @param  InitStruct: The structure containing basic WD configuration.
  * @retval None
  */
extern void HAL_WDT_Init(const halWdtInitTypeDef * initStruct);

/**
  * @brief  Enable the WDT.
  * @param  None
  * @retval None
  */
extern void HAL_WDT_Enable(void);

/**
  * @brief  Disable the WDT.
  * @param  None
  * @retval None
  */
extern void HAL_WDT_Disable(void);

/**
  * @brief  Clears the watchdog timer.
  * @param  None
  * @retval None
  */
extern void HAL_WDT_Clear(void);

#endif