/*!< THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _UNITS_ENUMS_H_
#define _UNITS_ENUMS_H_

/*! \brief Motor turning possibilities.
 *
 *  This enumeration displays the different rotation possibilities the motor is capeable of.
 */
typedef enum {
  INIT_NOT_DONE                = 0,                                             /*!< Initalisation of system not completed */
  INIT_DONE                    = 1,                                             /*!< Initalisation of system completed */
} INIT_STATE;

/*! \brief Unit value
 *
 * To prevent variable overflow some values are splitted to value and exponent part
 */
typedef __packed struct {
    uint32_t  value;                                                            /* value */
    int32_t   unit;                                                             /* unit */
} UNIT_VALUE;

/*! \brief Fractional representation of values
 *
 *  Some calcalations of register values are done with fractional parts (due to small numbers)
 *  The upper 16bit are representing the register value, while the lower 16bit are representing
 *  the fractional part
 */
typedef union
{
  struct
  {
    int16_t frac;                                                               /* Fractional part of calculation */
    int16_t reg;                                                                /* Register part of calculation */
  } part;                                                                       /* Partial representation */
  int32_t value;                                                                /* calculation as 32 bit */
} FRACTIONAL;

#endif /* _UNITS_ENUMS_H_ */
