/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <string.h>

#include "config.h"
#ifdef USE_HSDSO

#include BOARD_CONFIG_HEADER_FILE

#include "hal_uart.h"

#include "debug.h"
#include "hsdso.h"
#include "dso.h"
#include "crc8.h"
#include "hal_ve.h"
#include "motor_control.h"

static int is_configured = 0;
static int is_enabled = 0;
static int in_progress = 0;

static int user_dummy = 0;

#define MAX_NUM_ENTRIES 32
#define NUM_SIZE_HEADER 5 /* Start-Byte, Checksum, Event, Timestamp */
#define SIZE_XFER_ARRAY (NUM_SIZE_HEADER+(MAX_NUM_ENTRIES*2))
static uint8_t   g_xfer_buf[SIZE_XFER_ARRAY];
static uint8_t   g_capture_buf[(MAX_NUM_ENTRIES*2)];
static int       g_xfer_offset;
static int16_t   *g_log_ptr;
static uint32_t  g_selected_values = 0;
static int       g_count = 0;
static int       g_missed = 0;
static uint16_t  g_timestamp;
static int       g_PwmFrequency;
static uint8_t   invert_current        = 0;                                     /* Invert values for currents when current sensors are used with normal orientationm */
static uint8_t   g_channel = 1;

int HsDSOClock;

void hsdso_tx_handler(void)
{

  assert_param(is_enabled);

  while(1) {

    /* check if fifo is full, if so exit */
    if (HAL_UART_GetBufState(HSDSO_COMMUNICATION_UART_CHANNEL,HAL_UART_TX) == BUSY)
      break;

    /* check if we have transferred all bytes */
    if (g_xfer_offset >= (NUM_SIZE_HEADER+(g_count*2))) {
      /* signal end of transfer when fifo has underrun */
      if (HAL_UART_GetBufState(HSDSO_COMMUNICATION_UART_CHANNEL,HAL_UART_TX) == DONE)
      {
        HAL_UART_DisableIRQ(HSDSO_COMMUNICATION_UART_CHANNEL,HAL_UART_TX);
        in_progress = 0;
      }
      break;
    }

    HAL_UART_SendData(HSDSO_COMMUNICATION_UART_CHANNEL, g_xfer_buf[g_xfer_offset++]);
  }

}

static void configure_hsdso(void)
{
  HAL_UART_Init(HSDSO_COMMUNICATION_UART_CHANNEL,HAL_UART_TX,SPEED_5MHZ,8,NO_PARITY,1,NO_FLOW_CTRL);
  HAL_UART_RegisterIRQCallback (HSDSO_COMMUNICATION_UART_CHANNEL, HAL_UART_TX , hsdso_tx_handler );
}

int HsDSO_Disable(int motor_nr)
{
  if (!is_enabled) {
    dprintf("motor_nr %d already disabled\n", motor_nr);
    return 0;
  }

  is_enabled = 0;
  in_progress = 0;

  dprintf("motor_nr %d\n", motor_nr);
  return 0;
}

int HsDSO_Enable(int motor_nr, uint32_t selected_values, uint8_t spread_factor)
{
  uint32_t cnt;

  invert_current = (  (ChannelValues[motor_nr].MeasurementDirection==CURRENT_MEASUREMENT_INVERTED));

  if (!is_configured) {
    is_configured = 1;
    configure_hsdso();
  }

  if (!selected_values) {
    dprintf("motor_nr %d, !selected_values\n", motor_nr);
    return -1;
  }

  g_selected_values = selected_values;
  g_count = 0;
  g_channel = motor_nr;

  for (cnt = g_selected_values; cnt; cnt >>= 1UL)
    g_count += cnt & 1;

  if (g_count > MAX_NUM_ENTRIES) {
    dprintf("motor_nr %d, g_count %d > MAX_NUM_ENTRIES\n", motor_nr, g_count);
    return -1;
  }

  g_log_ptr = NULL;
  g_PwmFrequency = SystemValues[motor_nr].PwmFrequency;

  is_enabled = 1;

  /* emtpy FIFO */
  dprintf("motor_nr %d, g_selected_values 0x%08lx, g_count %d\n", motor_nr, g_selected_values, g_count);
  return 0;
}

#define LOG_IF_SELECTED(reg,check,shift,invert)    \
        if ((g_selected_values & (uint32_t)check)) \
          if (invert)                              \
            *pos++ = (-1*reg)>>shift;              \
          else                                     \
            *pos++ = reg>>shift;

void HsDSO_Log_PMD(int offset)
{
  int16_t *pos;

  if (!is_enabled)
    return;

  pos = (int16_t *)&g_capture_buf[0];
#if (defined (__KEIL__))
#pragma diag_suppress 68
#endif
#if (defined __TMPM_370__ ) || (defined __TMPM_372__ ) || (defined __TMPM_373__ ) || (defined __TMPM_374__ ) || (defined __TMPM_375__ ) || (defined __TMPM_376__ )
  LOG_IF_SELECTED(TEE_VEC->TMPREG0,             Va,         17, 0             );
  LOG_IF_SELECTED(TEE_VEC->TMPREG1,             Vb,         17, 0             );
  LOG_IF_SELECTED(TEE_VEC->TMPREG2,             Vc,         17, 0             );
  LOG_IF_SELECTED(TEE_VEC->TMPREG3,             Valpha,     16, 0             );
  LOG_IF_SELECTED(TEE_VEC->TMPREG4,             Vbeta,      16, 0             );
#endif
#ifdef __TMPM_475__
  switch (g_channel)
  {
  case 0:
    LOG_IF_SELECTED(TEE_VEC0->TMPREG0,             Va,         17, 0             );
    LOG_IF_SELECTED(TEE_VEC0->TMPREG1,             Vb,         17, 0             );
    LOG_IF_SELECTED(TEE_VEC0->TMPREG2,             Vc,         17, 0             );
    LOG_IF_SELECTED(TEE_VEC0->TMPREG3,             Valpha,     16, 0             );
    LOG_IF_SELECTED(TEE_VEC0->TMPREG4,             Vbeta,      16, 0             );
    break;
  case 1:
    LOG_IF_SELECTED(TEE_VEC1->TMPREG0,             Va,         17, 0             );
    LOG_IF_SELECTED(TEE_VEC1->TMPREG1,             Vb,         17, 0             );
    LOG_IF_SELECTED(TEE_VEC1->TMPREG2,             Vc,         17, 0             );
    LOG_IF_SELECTED(TEE_VEC1->TMPREG3,             Valpha,     16, 0             );
    LOG_IF_SELECTED(TEE_VEC1->TMPREG4,             Vbeta,      16, 0             );
    break;
  }
#endif

#ifdef __TMPM_4KN__
  switch (g_channel)
  {
  case 0:
    LOG_IF_SELECTED(TSB_VE0->VDUTYA,                      Va,         17, 0             );
    LOG_IF_SELECTED(TSB_VE0->VDUTYB,                      Vb,         17, 0             );
    LOG_IF_SELECTED(TSB_VE0->VDUTYC,                      Vc,         17, 0             );
    LOG_IF_SELECTED(TSB_VE0->VALPHA,                      Valpha,     16, 0             );
    LOG_IF_SELECTED(TSB_VE0->VBETA,                       Vbeta,      16, 0             );
    break;
  case 1:
  case 2:
    LOG_IF_SELECTED(MotorControl[g_channel].DutyU,          Va,         0, 0             );
    LOG_IF_SELECTED(MotorControl[g_channel].DutyV,          Vb,         0, 0             );
    LOG_IF_SELECTED(MotorControl[g_channel].DutyW,          Vc,         0, 0             );
    LOG_IF_SELECTED(MotorControl[g_channel].Valpha.value,   Valpha,     16, 0             );
    LOG_IF_SELECTED(MotorControl[g_channel].Vbeta.value,    Vbeta,      16, 0             );
    break;
  }
#endif

#if (defined (__KEIL__))
#pragma diag_default 68
#endif
  g_log_ptr = pos;
}

void HsDSO_Log_VE(int offset)
{
  TSB_VE_TypeDef* pVEx = TSB_VE0;
  int16_t *pos;

  g_timestamp = (g_timestamp + 1) % g_PwmFrequency;

  if (!is_enabled)
    return;

  if (!g_log_ptr)
    return;

  if (in_progress) {
    g_missed++;
    return;
  }

  pos = g_log_ptr;
  g_log_ptr = NULL;

#if (defined (__KEIL__))
#pragma diag_suppress 68
#endif
  LOG_IF_SELECTED(pVEx->ID,                             Id,         16, invert_current);
  LOG_IF_SELECTED(pVEx->IDREF,                          Id_Ref,      0, 0             );
  LOG_IF_SELECTED(pVEx->IQ,                             Iq,         16, invert_current);
  LOG_IF_SELECTED(pVEx->IQREF,                          Iq_Ref,      0, 0             );
  LOG_IF_SELECTED(pVEx->VD,                             Vd,         16, 0             );
  LOG_IF_SELECTED(pVEx->VQ,                             Vq,         16, 0             );
  LOG_IF_SELECTED(pVEx->VDIH,                           Vdi,        16, 0             );
  LOG_IF_SELECTED(pVEx->VQIH,                           Vqi,        16, 0             );
  LOG_IF_SELECTED(pVEx->THETA,                          Theta,       1, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Omega.part.reg, Omega,       0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].OmegaCalc,      OmegaCalc,   0, 0             );
  LOG_IF_SELECTED(pVEx->SIN,                            SIN_Theta,   0, 0             );
  LOG_IF_SELECTED(pVEx->COS,                            COS_Theta,   0, 0             );
  LOG_IF_SELECTED(pVEx->SECTOR,                         Sector,      0, 0             );
  LOG_IF_SELECTED(pVEx->VDC,                            DCLink,      0, 0             );
#ifndef __TMPM_475__
  LOG_IF_SELECTED(pVEx->IA,                             Ia,         16, invert_current);
  LOG_IF_SELECTED(pVEx->IB,                             Ib,         16, invert_current);
  LOG_IF_SELECTED(pVEx->IC,                             Ic,         16, invert_current);
  LOG_IF_SELECTED(pVEx->IALPHA,                         Ialpha,     16, invert_current);
  LOG_IF_SELECTED(pVEx->IBETA,                          Ibeta,      16, invert_current);
#else
  switch (channel)
  {
  case 0:
    LOG_IF_SELECTED(TEE_VEC0->TMPREG0,                  Ia,         16, invert_current);
    LOG_IF_SELECTED(TEE_VEC0->TMPREG1,                  Ib,         16, invert_current);
    LOG_IF_SELECTED(TEE_VEC0->TMPREG2,                  Ic,         16, invert_current);
    LOG_IF_SELECTED(TEE_VEC0->TMPREG3,                  Ialpha,     16, invert_current);
    LOG_IF_SELECTED(TEE_VEC0->TMPREG4,                  Ibeta,      16, invert_current);
    break;
  case 1:
    LOG_IF_SELECTED(TEE_VEC1->TMPREG0,                  Ia,         16, invert_current);
    LOG_IF_SELECTED(TEE_VEC1->TMPREG1,                  Ib,         16, invert_current);
    LOG_IF_SELECTED(TEE_VEC1->TMPREG2,                  Ic,         16, invert_current);
    LOG_IF_SELECTED(TEE_VEC1->TMPREG3,                  Ialpha,     16, invert_current);
    LOG_IF_SELECTED(TEE_VEC1->TMPREG4,                  Ibeta,      16, invert_current);
    break;
  }
#endif
  LOG_IF_SELECTED(MotorControl[g_channel].ActualStage.main,         VE_Stage,    0, 0             );
  LOG_IF_SELECTED(user_dummy,                           User_1,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                           User_2,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                           User_3,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                           User_4,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                           User_5,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                           User_6,      0, 0             );
#if (defined (__KEIL__))
#pragma diag_default 68
#endif

  g_xfer_buf[0] = 0x11;
  g_xfer_buf[2] = g_missed & 0xff;
  g_xfer_buf[3] = g_timestamp & 0xff;
  g_xfer_buf[4] = (g_timestamp >> 8) & 0xff;
  memcpy(&g_xfer_buf[5], g_capture_buf, g_count*2);
  g_xfer_buf[1] = crc_update(0x00, &g_xfer_buf[2], 3+(g_count*2));
  g_missed = 0;
  g_xfer_offset = 0;
  in_progress = 1;

}


void HsDSO_LogSWFOC(int offset)
{
  int16_t *pos=NULL;

  g_timestamp = (g_timestamp + 1) % g_PwmFrequency;

  if (!is_enabled)
    return;

  if (!g_log_ptr)
    return;

  if (in_progress) {
    g_missed++;
    return;
  }

#if (defined (__KEIL__))
#pragma diag_suppress 68
#endif
  LOG_IF_SELECTED(MotorControl[g_channel].Id,                             Id,         16, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].Id_reference,                   Id_Ref,      0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Iq,                             Iq,         16, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].Iq_reference,                   Iq_Ref,      0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Vd.value,                       Vd,         16, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Vq.value,                       Vq,         16, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Vdi,                            Vdi,        48, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Vqi,                            Vqi,        48, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Theta.part.reg,                    Theta,       1, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Omega.part.reg,                 Omega,       0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].OmegaCalc,                      OmegaCalc,   0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           SIN_Theta,   0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           COS_Theta,   0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Sector,                         Sector,      0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Vdc,                            DCLink,      0, 0             );
  LOG_IF_SELECTED(MotorControl[g_channel].Ia,                             Ia,         0, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].Ib,                             Ib,         0, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].Ic,                             Ic,         0, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].Ialpha,                         Ialpha,     0, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].Ibeta,                          Ibeta,      0, invert_current);
  LOG_IF_SELECTED(MotorControl[g_channel].ActualStage.main,               VE_Stage,    0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           User_1,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           User_2,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           User_3,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           User_4,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           User_5,      0, 0             );
  LOG_IF_SELECTED(user_dummy,                                           User_6,      0, 0             );
#if (defined (__KEIL__))
#pragma diag_default 68
#endif

  g_xfer_buf[0] = 0x11;
  g_xfer_buf[2] = g_missed & 0xff;
  g_xfer_buf[3] = g_timestamp & 0xff;
  g_xfer_buf[4] = (g_timestamp >> 8) & 0xff;
  memcpy(&g_xfer_buf[5], g_capture_buf, g_count*2);
  g_xfer_buf[1] = crc_update(0x00, &g_xfer_buf[2], 3+(g_count*2));
  g_missed = 0;
  g_xfer_offset = 0;
  in_progress = 1;

}



#endif

