/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdint.h>
#include <stdlib.h>
#include "motor_control.h"
#include "fwmathlib.h"
#include "hal_encoder.h"
#include "txz_driver_def.h"
#include "estimator.h"
#include "sw_foc.h"

/*! \brief  Calculate Id-Ref and Iq-Ref
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void TorqueController_Calculate_Id_Iq_ref(uint8_t channel_number)
{
  uint16_t increase_current;

  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Stop:
  case Stage_Bootstrap:
  case Stage_Brake:
  case Stage_ZeroCurrentMeasure:
  case Stage_Emergency:
    MotorControl[channel_number].Id_reference = 0;
    MotorControl[channel_number].Iq_reference = 0;
    break;

  case Stage_Initposition:
    MotorControl[channel_number].Id_reference = 0;

    /* set current "Iq" increasing as C-load */
    increase_current = (((abs(MotorControl[channel_number].PreCalculation.IqCurrentForInitposition)<<16))
                           / ((MotorControl[channel_number].PreCalculation.WaitTime_Position)/2))>>16;
    /* check minimized increase */
    if (increase_current==0)
      increase_current=1;

    if (MotorSetValues[channel_number].TargetSpeed >= 0)
      MotorControl[channel_number].Iq_reference += increase_current;
    else
      MotorControl[channel_number].Iq_reference -= increase_current;

    /* normalizing and limiting this values to maximum current */
    if(MotorControl[channel_number].Iq_reference   >= MotorControl[channel_number].PreCalculation.IqCurrentForInitposition)
      MotorControl[channel_number].Iq_reference  = (int16_t)(MotorControl[channel_number].PreCalculation.IqCurrentForInitposition);
    else if(MotorControl[channel_number].Iq_reference <= -MotorControl[channel_number].PreCalculation.IqCurrentForInitposition)
      MotorControl[channel_number].Iq_reference     = -(int16_t)(MotorControl[channel_number].PreCalculation.IqCurrentForInitposition);

    break;

  case Stage_Force:
  case Stage_FOC:
    if (MotorControl[channel_number].Omega_command.part.reg >= 0)
      MotorControl[channel_number].Iq_reference = MotorControl[channel_number].PreCalculation.IqCurrentForInitposition;
    else
      MotorControl[channel_number].Iq_reference = -MotorControl[channel_number].PreCalculation.IqCurrentForInitposition;


    MotorControl[channel_number].Iq_reference_I.part.reg  = MotorControl[channel_number].Iq_reference;
    break;

  default:
    assert_param(0);
    break;
  }
}


int16_t rpm;
uint8_t regulate;
  int16_t  Omega_Deviation;
  int64_t  Temp_P;
  int64_t  Temp_I;

/*! \brief  Calculate or set Omeag and Theta
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void TorqueController_Omega_Theta(uint8_t channel_number)
{
  uint8_t direction;
  static uint8_t initposition_angle_determin = 0;

  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Stop:
  case Stage_Bootstrap:
  case Stage_Brake:
    initposition_angle_determin = 0;
    break;
  case Stage_Initposition:
    if ( initposition_angle_determin == 0)
      initposition_angle_determin = 1;
    {

      direction = MotorControl[channel_number].Omega_Target >= 0 ;
      if (MotorControl[channel_number].LastDirection != direction)
      {
        MotorControl[channel_number].LastDirection = direction;
        if (MotorControl[channel_number].LastDirection == 1)
        {
          MotorControl[channel_number].Theta.part.reg  = (int16_t)((int64_t)-90 * FIXPOINT_16 / 360);
          MotorControl[channel_number].Theta.part.frac = 0;
#ifdef USE_ENCODER         
          if (MotorParameterValues[channel_number].Encoder != ENCODER_NO_ENCODER)
            MotorControl[channel_number].Theta.part.reg = ((ENC_GetTheta(channel_number)) * FIXPOINT_16 / 360);
#endif /* USE_ENCODER */            
        }
        else
        {
          MotorControl[channel_number].Theta.part.reg  = (uint16_t)(((uint64_t)90 * FIXPOINT_16 / 360));
          MotorControl[channel_number].Theta.part.frac = 0;
#ifdef USE_ENCODER          
          if (MotorParameterValues[channel_number].Encoder != ENCODER_NO_ENCODER)
            MotorControl[channel_number].Theta.part.reg = ((ENC_GetTheta(channel_number)) * FIXPOINT_16 / 360);
#endif /* USE_ENCODER */            
        }
      }
      EncoderData[channel_number].Theta0.part.reg = MotorControl[channel_number].Theta.part.reg;
    }
    break;

  case Stage_Force:
    initposition_angle_determin = 0;
    if (MotorControl[channel_number].Omega.part.reg !=0)
    {
      direction = MotorControl[channel_number].Omega_command.part.reg > 0;
      if  (MotorControl[channel_number].LastDirection!=direction)
      {
        MotorControl[channel_number].LastDirection=direction;
        MotorControl[channel_number].Theta.part.reg +=  FIXPOINT_16 / 2;
        if (channel_number == 0)
        {
          long long temp;
          temp = ( ((long long)TSB_VE0->VDIH) << 32) | TSB_VE0->VDILH;
          temp *= -1;
          TSB_VE0->VDIH = (temp >> 32);
          TSB_VE0->VDILH = temp & 0x00000000ffffffffL;
          temp = ( ((long long)TSB_VE0->VQIH) << 32) | TSB_VE0->VQILH;
          temp *= -1;
          TSB_VE0->VQIH = (temp >> 32);
          TSB_VE0->VQILH = temp & 0x00000000ffffffffL;
        }
        else
        {
          SW_FOC_Invert_Current_Intergral_Value(channel_number);
        }
      }
    }
    Estimator_Calculate_Omega_BLDC(channel_number);

    MotorControl[channel_number].Omega.value  = MotorControl[channel_number].Omega_command.value;
    MotorControl[channel_number].Theta.value += (int32_t) ( MotorControl[channel_number].Omega.part.reg
                                             *( MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency<<1));

    break;

  case Stage_FOC:
    initposition_angle_determin = 0;
#ifdef USE_ENCODER
    if ((MotorParameterValues[channel_number].Encoder != ENCODER_NO_ENCODER) && (MotorParameterValues[channel_number].EncoderUsage == ENCODER_USAGE_SPEED))
      ENC_Determine_Omega_Theta(channel_number);
    else
#endif /* USE_ENCODER */
    {
      static int16_t counter;
    Estimator_Calculate_Omega_BLDC(channel_number);
    MotorControl[channel_number].Theta.value += (int32_t) ( MotorControl[channel_number].Omega.part.reg
                                                 *( MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency<<1));

    rpm = ( (MotorControl[channel_number].Omega.part.reg * SECONDS_PER_MINUTE * MotorControl[channel_number].HzMax
              / FIXPOINT_15 / MotorParameterValues[channel_number].PolePairs
                  )  * 2+1)/2;

    rpm -= MotorSetValues[channel_number].TargetSpeed;

    if (counter++>1000)
    {
      counter = 0;


    Temp_I += (int64_t)rpm * MotorControl[channel_number].PreCalculation.SpeedKiPreCalculation / 10;

    if (Temp_I>10)
      Temp_I=10;
    if (Temp_I<-10)
      Temp_I=-10;

    /* proportional part of the control algorithm normalized on base of current */
    Temp_P  = (int64_t)rpm * MotorControl[channel_number].PreCalculation.SpeedKpPreCalculation / 1000;

    if (Temp_P>10)
      Temp_P=10;
    if (Temp_P<-10)
      Temp_P=-10;

    if( 0 != MotorControl[channel_number].Omega_Target) {
    /* add the proportional part and and divide for correction */
    MotorControl[channel_number].Omega_Target -= (int16_t)((Temp_I + Temp_P));
    }
    }

    }
    break;

  default:
    assert_param(0);
    break;
  }
}
