/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <string.h>

#include "config.h"
#ifdef USE_INTERNAL_MOTOR_PARAMS

#include "motor_control.h"

/*! \brief  Setup the internal Motor Parameters
  *
  * If the motor parameters are compiled into the Firmware
  * this is setting up the values to Ram
  *
  * @param  None
  * @retval None
*/
void SetupInternalMotorParams(void)
{
  /* Setup compiled in values for Motor 0 */
#ifdef MOTOR_CHANNEL_0
#include MOTOR_CHANNEL_0
  MotorParameterValues[0].PolePairs          = MOTOR_POLE_PAIRS;
  MotorParameterValues[0].Direction          = MOTOR_DIRECTION;
  MotorParameterValues[0].Encoder            = MOTOR_ENCODER_TYPE;
  MotorParameterValues[0].EncoderUsage       = MOTOR_ENCODER_USAGE;
  MotorParameterValues[0].EncoderResolution  = MOTOR_ENCODER_COUNT;
  MotorParameterValues[0].EncoderMinPos      = MOTOR_ENCODER_MIN_COUNT;
  MotorParameterValues[0].EncoderStartPos    = MOTOR_ENCODER_START_COUNT;
  MotorParameterValues[0].EncoderMaxPos      = MOTOR_ENCODER_MAX_COUNT;
  MotorParameterValues[0].EncoderLMApproach  = MOTOR_ENCODER_LM_APPROACH_RPM;
  MotorParameterValues[0].EncoderGearFactor  = MOTOR_GEAR_FACTOR;
  MotorParameterValues[0].EncoderReceiver    = MOTOR_ENCODER_RECEIVER;
  MotorParameterValues[0].EncoderReceiverMode= MOTOR_ENCODER_RECEIVER_MODE;
  MotorParameterValues[0].MaxAngAcc.value    = MOTOR_ANGULAR_ACC_MAX_VALUE;
  MotorParameterValues[0].MaxAngAcc.unit     = MOTOR_ANGULAR_ACC_MAX_UNIT;
  MotorParameterValues[0].TorqueFactor.value = MOTOR_TORQUE_FACTOR_VALUE;
  MotorParameterValues[0].TorqueFactor.unit  = MOTOR_TORQUE_FACTOR_UNIT;
  MotorParameterValues[0].Resistance.value   = MOTOR_RESISTANCE_VALUE;
  MotorParameterValues[0].Resistance.unit    = MOTOR_RESISTANCE_UNIT;
  MotorParameterValues[0].Inductance.value   = MOTOR_INDUCTANCE_VALUE;
  MotorParameterValues[0].Inductance.unit    = MOTOR_INDUCTANCE_UNIT;
  MotorParameterValues[0].SpeedLimit.value   = MOTOR_SPEED_LIMIT_VALUE;
  MotorParameterValues[0].SpeedLimit.unit    = MOTOR_SPEED_LIMIT_UNIT;
  MotorParameterValues[0].SpeedChange.value  = MOTOR_SPEED_CHANGE_VALUE;
  MotorParameterValues[0].SpeedChange.unit   = MOTOR_SPEED_CHANGE_UNIT;
  MotorParameterValues[0].PositionDelay      = MOTOR_POSITION_DELAY_VALUE;
  MotorParameterValues[0].IqStart.value      = MOTOR_IQ_START_VALUE;
  MotorParameterValues[0].IqStart.unit       = MOTOR_IQ_START_UNIT;
  MotorParameterValues[0].IdStart.value      = MOTOR_ID_START_VALUE;
  MotorParameterValues[0].IdStart.unit       = MOTOR_ID_START_UNIT;
  MotorParameterValues[0].IqLimit.value      = MOTOR_IQ_LIM_VALUE;
  MotorParameterValues[0].IqLimit.unit       = MOTOR_IQ_LIM_UNIT;
  MotorParameterValues[0].IdLimit.value      = MOTOR_ID_LIM_VALUE;
  MotorParameterValues[0].IdLimit.unit       = MOTOR_ID_LIM_UNIT;
  memset (&MotorParameterValues[0].MotorId,0,sizeof(MOTORID));
  memcpy (&MotorParameterValues[0].MotorId,MOTORID,sizeof(MOTORID));

  PIControl[0].PiIdKi                        = CONTROL_ID_KI_VALUE;
  PIControl[0].PiIdKp                        = CONTROL_ID_KP_VALUE;
  PIControl[0].PiIqKi                        = CONTROL_IQ_KI_VALUE;
  PIControl[0].PiIqKp                        = CONTROL_IQ_KP_VALUE;
  PIControl[0].PositionKi                    = CONTROL_POSITION_KI_VALUE;
  PIControl[0].PositionKp                    = CONTROL_POSITION_KP_VALUE;
  PIControl[0].SpeedKi                       = CONTROL_SPEED_KI_VALUE;
  PIControl[0].SpeedKp                       = CONTROL_SPEED_KP_VALUE;

  SystemValues[0].PwmFrequency               = SYSTEM_PWM_FREQUENCY_VALUE;
  SystemValues[0].ShutdownMode               = SYSTEM_SHUTDOWN_MODE;
  SystemValues[0].Braketime                  = SYSTEM_BRAKE_TIME_VALUE;
  SystemValues[0].RestartMode                = SYSTEM_RESTART_MODE;
  SystemValues[0].StalldetectValue           = SYSTEM_STALL_DETECT_VALUE;
  SystemValues[0].Overtemperature            = SYSTEM_OVERTEMPERATURE_VALUE;
  SystemValues[0].ExternalSpeedControl       = SYSTEM_SPEED_CONTROL_MODE;
  SystemValues[0].SwOvervoltage              = SYSTEM_SW_OVERVOLTAGE_VALUE;
  SystemValues[0].SwUndervoltage             = SYSTEM_SW_UNDERVOLTAGE_VALUE;
  SystemValues[0].SwOvercurrent.value        = SYSTEM_SW_OVERCURRENT_VALUE;
  SystemValues[0].SwOvercurrent.unit         = SYSTEM_SW_OVERCURRENT_UNIT;
  SystemValues[0].SpeedReduction             = SYSTEM_SPEED_REDUCTION_VALUE;

  /* A little bit tricky - undef all defines to be able to load the same
    define again, or ause an other define, but having the same naming */
#include "motor_undefine.h"
#endif

/* Setup compiled in values for Motor 1 */
#ifdef MOTOR_CHANNEL_1
#include MOTOR_CHANNEL_1

  MotorParameterValues[1].PolePairs          = MOTOR_POLE_PAIRS;
  MotorParameterValues[1].Direction          = MOTOR_DIRECTION;
  MotorParameterValues[1].Encoder            = MOTOR_ENCODER_TYPE;
  MotorParameterValues[1].EncoderUsage       = MOTOR_ENCODER_USAGE;
  MotorParameterValues[1].EncoderResolution  = MOTOR_ENCODER_COUNT;
  MotorParameterValues[1].EncoderMinPos      = MOTOR_ENCODER_MIN_COUNT;
  MotorParameterValues[1].EncoderStartPos    = MOTOR_ENCODER_START_COUNT;
  MotorParameterValues[1].EncoderMaxPos      = MOTOR_ENCODER_MAX_COUNT;
  MotorParameterValues[1].EncoderLMApproach  = MOTOR_ENCODER_LM_APPROACH_RPM;
  MotorParameterValues[1].EncoderGearFactor  = MOTOR_GEAR_FACTOR;
  MotorParameterValues[1].EncoderReceiver    = MOTOR_ENCODER_RECEIVER;
  MotorParameterValues[1].EncoderReceiverMode= MOTOR_ENCODER_RECEIVER_MODE;
  MotorParameterValues[1].MaxAngAcc.value    = MOTOR_ANGULAR_ACC_MAX_VALUE;
  MotorParameterValues[1].MaxAngAcc.unit     = MOTOR_ANGULAR_ACC_MAX_UNIT;
  MotorParameterValues[1].TorqueFactor.value = MOTOR_TORQUE_FACTOR_VALUE;
  MotorParameterValues[1].TorqueFactor.unit  = MOTOR_TORQUE_FACTOR_UNIT;
  MotorParameterValues[1].Resistance.value   = MOTOR_RESISTANCE_VALUE;
  MotorParameterValues[1].Resistance.unit    = MOTOR_RESISTANCE_UNIT;
  MotorParameterValues[1].Inductance.value   = MOTOR_INDUCTANCE_VALUE;
  MotorParameterValues[1].Inductance.unit    = MOTOR_INDUCTANCE_UNIT;
  MotorParameterValues[1].SpeedLimit.value   = MOTOR_SPEED_LIMIT_VALUE;
  MotorParameterValues[1].SpeedLimit.unit    = MOTOR_SPEED_LIMIT_UNIT;
  MotorParameterValues[1].SpeedChange.value  = MOTOR_SPEED_CHANGE_VALUE;
  MotorParameterValues[1].SpeedChange.unit   = MOTOR_SPEED_CHANGE_UNIT;
  MotorParameterValues[1].PositionDelay      = MOTOR_POSITION_DELAY_VALUE;
  MotorParameterValues[1].IqStart.value      = MOTOR_IQ_START_VALUE;
  MotorParameterValues[1].IqStart.unit       = MOTOR_IQ_START_UNIT;
  MotorParameterValues[1].IdStart.value      = MOTOR_ID_START_VALUE;
  MotorParameterValues[1].IdStart.unit       = MOTOR_ID_START_UNIT;
  MotorParameterValues[1].IqLimit.value      = MOTOR_IQ_LIM_VALUE;
  MotorParameterValues[1].IqLimit.unit       = MOTOR_IQ_LIM_UNIT;
  MotorParameterValues[1].IdLimit.value      = MOTOR_ID_LIM_VALUE;
  MotorParameterValues[1].IdLimit.unit       = MOTOR_ID_LIM_UNIT;
  memset (&MotorParameterValues[1].MotorId,0,sizeof(MOTORID));
  memcpy (&MotorParameterValues[1].MotorId,MOTORID,sizeof(MOTORID));

  PIControl[1].PiIdKi                        = CONTROL_ID_KI_VALUE;
  PIControl[1].PiIdKp                        = CONTROL_ID_KP_VALUE;
  PIControl[1].PiIqKi                        = CONTROL_IQ_KI_VALUE;
  PIControl[1].PiIqKp                        = CONTROL_IQ_KP_VALUE;
  PIControl[1].PositionKi                    = CONTROL_POSITION_KI_VALUE;
  PIControl[1].PositionKp                    = CONTROL_POSITION_KP_VALUE;
  PIControl[1].SpeedKi                       = CONTROL_SPEED_KI_VALUE;
  PIControl[1].SpeedKp                       = CONTROL_SPEED_KP_VALUE;

  SystemValues[1].PwmFrequency               = SYSTEM_PWM_FREQUENCY_VALUE;
  SystemValues[1].ShutdownMode               = SYSTEM_SHUTDOWN_MODE;
  SystemValues[1].Braketime                  = SYSTEM_BRAKE_TIME_VALUE;
  SystemValues[1].RestartMode                = SYSTEM_RESTART_MODE;
  SystemValues[1].StalldetectValue           = SYSTEM_STALL_DETECT_VALUE;
  SystemValues[1].Overtemperature            = SYSTEM_OVERTEMPERATURE_VALUE;
  SystemValues[1].ExternalSpeedControl       = SYSTEM_SPEED_CONTROL_MODE;
  SystemValues[1].SwOvervoltage              = SYSTEM_SW_OVERVOLTAGE_VALUE;
  SystemValues[1].SwUndervoltage             = SYSTEM_SW_UNDERVOLTAGE_VALUE;
  SystemValues[1].SwOvercurrent.value        = SYSTEM_SW_OVERCURRENT_VALUE;
  SystemValues[1].SwOvercurrent.unit         = SYSTEM_SW_OVERCURRENT_UNIT;
  SystemValues[1].SpeedReduction             = SYSTEM_SPEED_REDUCTION_VALUE;

#include "motor_undefine.h"
#endif

  /* Setup compiled in values for Motor 2 */
#ifdef MOTOR_CHANNEL_2
#include MOTOR_CHANNEL_2
  MotorParameterValues[2].PolePairs          = MOTOR_POLE_PAIRS;
  MotorParameterValues[2].Direction          = MOTOR_DIRECTION;
  MotorParameterValues[2].Encoder            = MOTOR_ENCODER_TYPE;
  MotorParameterValues[2].EncoderUsage       = MOTOR_ENCODER_USAGE;
  MotorParameterValues[2].EncoderResolution  = MOTOR_ENCODER_COUNT;
  MotorParameterValues[2].EncoderMinPos      = MOTOR_ENCODER_MIN_COUNT;
  MotorParameterValues[2].EncoderStartPos    = MOTOR_ENCODER_START_COUNT;
  MotorParameterValues[2].EncoderMaxPos      = MOTOR_ENCODER_MAX_COUNT;
  MotorParameterValues[2].EncoderLMApproach  = MOTOR_ENCODER_LM_APPROACH_RPM;
  MotorParameterValues[2].EncoderGearFactor  = MOTOR_GEAR_FACTOR;
  MotorParameterValues[2].EncoderReceiver    = MOTOR_ENCODER_RECEIVER;
  MotorParameterValues[2].EncoderReceiverMode= MOTOR_ENCODER_RECEIVER_MODE;
  MotorParameterValues[2].MaxAngAcc.value    = MOTOR_ANGULAR_ACC_MAX_VALUE;
  MotorParameterValues[2].MaxAngAcc.unit     = MOTOR_ANGULAR_ACC_MAX_UNIT;
  MotorParameterValues[2].TorqueFactor.value = MOTOR_TORQUE_FACTOR_VALUE;
  MotorParameterValues[2].TorqueFactor.unit  = MOTOR_TORQUE_FACTOR_UNIT;
  MotorParameterValues[2].Resistance.value   = MOTOR_RESISTANCE_VALUE;
  MotorParameterValues[2].Resistance.unit    = MOTOR_RESISTANCE_UNIT;
  MotorParameterValues[2].Inductance.value   = MOTOR_INDUCTANCE_VALUE;
  MotorParameterValues[2].Inductance.unit    = MOTOR_INDUCTANCE_UNIT;
  MotorParameterValues[2].SpeedLimit.value   = MOTOR_SPEED_LIMIT_VALUE;
  MotorParameterValues[2].SpeedLimit.unit    = MOTOR_SPEED_LIMIT_UNIT;
  MotorParameterValues[2].SpeedChange.value  = MOTOR_SPEED_CHANGE_VALUE;
  MotorParameterValues[2].SpeedChange.unit   = MOTOR_SPEED_CHANGE_UNIT;
  MotorParameterValues[2].PositionDelay      = MOTOR_POSITION_DELAY_VALUE;
  MotorParameterValues[2].IqStart.value      = MOTOR_IQ_START_VALUE;
  MotorParameterValues[2].IqStart.unit       = MOTOR_IQ_START_UNIT;
  MotorParameterValues[2].IdStart.value      = MOTOR_ID_START_VALUE;
  MotorParameterValues[2].IdStart.unit       = MOTOR_ID_START_UNIT;
  MotorParameterValues[2].IqLimit.value      = MOTOR_IQ_LIM_VALUE;
  MotorParameterValues[2].IqLimit.unit       = MOTOR_IQ_LIM_UNIT;
  MotorParameterValues[2].IdLimit.value      = MOTOR_ID_LIM_VALUE;
  MotorParameterValues[2].IdLimit.unit       = MOTOR_ID_LIM_UNIT;
  memset (&MotorParameterValues[2].MotorId,0,sizeof(MOTORID));
  memcpy (&MotorParameterValues[2].MotorId,MOTORID,sizeof(MOTORID));

  PIControl[2].PiIdKi                        = CONTROL_ID_KI_VALUE;
  PIControl[2].PiIdKp                        = CONTROL_ID_KP_VALUE;
  PIControl[2].PiIqKi                        = CONTROL_IQ_KI_VALUE;
  PIControl[2].PiIqKp                        = CONTROL_IQ_KP_VALUE;
  PIControl[2].PositionKi                    = CONTROL_POSITION_KI_VALUE;
  PIControl[2].PositionKp                    = CONTROL_POSITION_KP_VALUE;
  PIControl[2].SpeedKi                       = CONTROL_SPEED_KI_VALUE;
  PIControl[2].SpeedKp                       = CONTROL_SPEED_KP_VALUE;

  SystemValues[2].PwmFrequency               = SYSTEM_PWM_FREQUENCY_VALUE;
  SystemValues[2].ShutdownMode               = SYSTEM_SHUTDOWN_MODE;
  SystemValues[2].Braketime                  = SYSTEM_BRAKE_TIME_VALUE;
  SystemValues[2].RestartMode                = SYSTEM_RESTART_MODE;
  SystemValues[2].StalldetectValue           = SYSTEM_STALL_DETECT_VALUE;
  SystemValues[2].Overtemperature            = SYSTEM_OVERTEMPERATURE_VALUE;
  SystemValues[2].ExternalSpeedControl       = SYSTEM_SPEED_CONTROL_MODE;
  SystemValues[2].SwOvervoltage              = SYSTEM_SW_OVERVOLTAGE_VALUE;
  SystemValues[2].SwUndervoltage             = SYSTEM_SW_UNDERVOLTAGE_VALUE;
  SystemValues[2].SwOvercurrent.value        = SYSTEM_SW_OVERCURRENT_VALUE;
  SystemValues[2].SwOvercurrent.unit         = SYSTEM_SW_OVERCURRENT_UNIT;
  SystemValues[2].SpeedReduction             = SYSTEM_SPEED_REDUCTION_VALUE;

  /* A little bit tricky - undef all defines to be able to load the same
    define again, or ause an other define, but having the same naming */
#include "motor_undefine.h"
#endif

}
#endif














