/* THE SOURCE CODE AND ITS
* RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
* WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
* AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
* INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
* DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
* INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
* MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
* TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
* LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
* FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
* LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
* OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
* ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
* ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
* FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
* OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
*/

#include <string.h>
#include <stdio.h>

#include "user_config.h"
#include "config.h"
#include BOARD_CONFIG_HEADER_FILE

#include "led.h"

#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_0 */

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_1 */

#include "FreeRTOS.h"
#include "task.h"

#include "debug.h"
#include "load_statistics.h"
#include "api.h"
#include "api_internal.h"
#include "hal_ve.h"
#include "hal_adc.h"
#include "hal_pmd.h"
#include "dso.h"
#include "hsdso.h"
#include "turn_control.h"
#include "motion_control.h"
#include "hal_encoder.h"
#include "config_storage.h"
#include "temperature_control.h"
#include "board.h"
#include "hv_serial_communication.h"
#include "fwmathlib.h"
#include "protocol.h"
#include "motor_control.h"
#include "peripherals_control.h"
#include "txz_driver_def.h"


static uint8_t hasReset = 1;
static UNIT_VALUE  IqStartStorage[MAX_CHANNEL];

int StartMotor(struct StartMotorReq* req, struct StartMotorRsp* rsp)
{
  dprintf("%s(%d) called\n", __func__,req->motor_nr);

  memset(rsp, 0, sizeof(*rsp));

  MotorExtendedStateValues[req->motor_nr].ControlMethod=MOTORCONTROL_CONTROL_BY_SPEED;

  IqStartStorage[req->motor_nr].value = MotorParameterValues[req->motor_nr].IqStart.value;
  IqStartStorage[req->motor_nr].unit  = MotorParameterValues[req->motor_nr].IqStart.unit;

  return MotorControl_Start(req->motor_nr);
}

#ifdef USE_CONFIG_STORAGE
extern uint8_t g_config_storage[CONFIG_STORAGE_PAGE_SIZE];
#else
#ifdef DEBUG
uint8_t printout_mem[2048];
#endif
#endif /* USE_CONFIG_STORAGE */

int StopMotor(struct StopMotorReq* req, struct StopMotorRsp* rsp)
{
  dprintf("%s(%d) called\n", __func__,req->motor_nr);

  memset(rsp, 0, sizeof(*rsp));

#if ((defined USE_CONFIG_STORAGE) && (defined DEBUG))

  memset(&g_config_storage[0], 0xcc, CONFIG_STORAGE_PAGE_SIZE);
  vTaskList((char*)&g_config_storage[0]);
  printf("Name          State   Prio  StackLeft  Number\n*********************************************\n");
#if (defined(__ICCARM__)) /* IAR Compiler - */
#pragma diag_suppress = Pe167
#elif defined __KEIL__
#pragma diag_suppress = 167
#endif
  puts(&g_config_storage[0]);
#if (defined(__ICCARM__)) /* IAR Compiler - */
#pragma diag_suppress = Pe167
#elif defined __KEIL__
#pragma diag_suppress = 167
#endif

#else
#if ((!defined USE_CONFIG_STORAGE) && (defined DEBUG))
  memset(&printout_mem[0], 0xcc, sizeof(printout_mem));
  vTaskList((char*)&printout_mem[0]);
  printf("Name          State   Prio  StackLeft  Number\n*********************************************\n");
#endif

#if (defined(__ICCARM__)) /* IAR Compiler - */
#pragma diag_suppress = Pe167
#elif defined __KEIL__
#pragma diag_suppress = 167
#endif
#if ((!defined USE_CONFIG_STORAGE) && (defined DEBUG))
  puts(&printout_mem[0]);
#endif
#if (defined(__ICCARM__)) /* IAR Compiler - */
#pragma diag_suppress = Pe167
#elif defined __KEIL__
#pragma diag_suppress = 167
#endif

#endif

  if (IqStartStorage[req->motor_nr].value!=0)
  {
    MotorParameterValues[req->motor_nr].IqStart.value = IqStartStorage[req->motor_nr].value;
    MotorParameterValues[req->motor_nr].IqStart.unit = IqStartStorage[req->motor_nr].unit;
  }

  return MotorControl_Stop(req->motor_nr);
}

/*! \brief  Set Motor Speed
*
* Set the Target Speed of the Motor
*
*/
static void SetSpeed(int motor_nr, int32_t speed)
{
  /* todo: probably MotorSetSettings can be eliminated */
  MotorSetSettings s;
  s.TargetSpeed = speed;

  memcpy(&MotorSetValues[motor_nr], &s, sizeof(MotorSetValues[motor_nr]));
#ifdef USE_LOAD_DEPENDANT_SPEED_REDUCTION
  MotorControl[motor_nr].TargetSpeed_Backup  = MotorSetValues[motor_nr].TargetSpeed;
  MotorErrorField[motor_nr].Error &= ~VE_SPEEDREDUCTION;
#endif /* USE_LOAD_DEPENDANT_SPEED_REDUCTION */
}


/*! \brief  Get Motor State
*
* Get the current control state of the motor
*
*/
int GetMotorState       (struct GetMotorStateReq* req,       struct GetMotorStateRsp* rsp)
{

  switch (MotorExtendedStateValues[req->motor_nr].ControlMethod)
  {
  case MOTORCONTROL_CONTROL_BY_SPEED:
    rsp->State.ActualSpeed = MotorExtendedStateValues[req->motor_nr].ActualControlledParameter;
    rsp->State.Current     = MotorExtendedStateValues[req->motor_nr].Current;
    rsp->State.TargetSpeed = MotorExtendedStateValues[req->motor_nr].TargetControlledParameter;
    rsp->State.Torque      = MotorExtendedStateValues[req->motor_nr].NonControlParameter;
    break;

  case MOTORCONTROL_CONTROL_BY_TORQUE:
    rsp->State.ActualSpeed = MotorExtendedStateValues[req->motor_nr].NonControlParameter;
    rsp->State.Current     = MotorExtendedStateValues[req->motor_nr].Current;
    rsp->State.TargetSpeed = 0;
    rsp->State.Torque      = MotorExtendedStateValues[req->motor_nr].ActualControlledParameter;
    break;

  default:
    assert_param(0);
  }

  rsp->State.Timestamp = xTaskGetTickCount();

  return 0;
}

int StoreParameters     (struct StoreParametersReq* req,     struct StoreParametersRsp* rsp)
{
  int ret = 0;

  dprintf("%s():\n", __func__);

  memset(rsp, 0, sizeof(*rsp));
#ifdef USE_CONFIG_STORAGE
  ret = config_storage_save_config();
#endif

  return ret;
}

int ClearParameters     (struct ClearParametersReq* req,     struct ClearParametersRsp* rsp)
{
  int ret = 0;

  dprintf("%s():\n", __func__);

  memset(rsp, 0, sizeof(*rsp));
#ifdef USE_CONFIG_STORAGE
  ret = config_storage_clear_config();
#endif

  return ret;
}

int ConfigDsoLog        (struct ConfigDsoLogReq* req,        struct ConfigDsoLogRsp* rsp)
{
  dprintf("%s(%d) called\n", __func__,req->motor_nr);

#ifdef USE_DSO
  memset(rsp, 0, sizeof(*rsp));
  rsp->logsteps=DSO_Configure_Continuous_Mode(req->motor_nr,
                                            req->selected_values,
                                            req->trigger_on_value,
                                            req->trigger_mode,
                                            req->trigger_level,
                                            req->spread_factor);
  return 0;
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif // USE_DSO
}

int GetDsoLogData       (struct GetDsoLogDataReq* req,       struct GetDsoLogDataRsp* rsp)
{

#ifdef USE_DSO
  memset(rsp,   0, sizeof(*rsp));
  return DSO_GetLogData((int16_t*)rsp->values.value);
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif // USE_DSO
}

int ConfigureHsDso       (struct ConfigureHsDsoReq* req,     struct ConfigureHsDsoRsp* rsp)
{
  dprintf("%s() called, enable %d\n", __func__, req->enable);

  memset(rsp, 0, sizeof(*rsp));

#ifdef USE_HSDSO
  if (req->enable)
    return HsDSO_Enable(req->motor_nr, req->selected_values, req->spread_factor);
  else
    return HsDSO_Disable(req->motor_nr);
#else
  return 0;
#endif
}

int GetFWVersion        (struct GetFWVersionReq* req,        struct GetFWVersionRsp* rsp)
{
  dprintf("%s() called\n", __func__);

  memset(rsp, 0, sizeof(*rsp));

  rsp->values.fw_version[0]= FW_VERSION_MAJOR;
  rsp->values.fw_version[1]= FW_VERSION_MINOR;

  return 0;
}

int DoTurn              (struct DoTurnReq* req,              struct DoTurnRsp* rsp)
{
  dprintf("%s() called\n", __func__);

#ifdef USE_TURN_CONTROL
  memset(rsp, 0, sizeof(*rsp));
  CalculateTurnRampAndTurn(req->motor_nr,
                           req->turns,
                           req->additional_angle,
                           req->max_speed);
  return 0;
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif
}

/* section for functions of motion control */

/*! \brief  DoLinearMotion
*
* linear motion control for motor to absolute position
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int DoLinearMotion    (struct DoLinearMotionReq* req, struct DoLinearMotionRsp* rsp)
{

#ifdef USE_MOTION_CONTROL
  memset(rsp, 0, sizeof(*rsp));
  CalculatePulsesRampAndTurn(req->motor_nr,
                           req->abs_postion,
                           req->max_speed);
  return 0;
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif
}

/*! \brief  GetAbsolutePosition
*
* Retrieve the curren position of the rotor in terms of pulses
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int GetAbsolutePosition(struct GetAbsolutePositionReq* req, struct GetAbsolutePositionRsp* rsp)
{

#ifdef USE_MOTION_CONTROL
  memset(rsp, 0, sizeof(*rsp));
  rsp->current_position = GetPosition(req->motor_nr);
  return 0;
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif
}

/*! \brief  AbortLinearMotion
*
* linear motion control for motor to absolute position
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int AbortLinearMotion    (struct AbortLinearMotionReq* req, struct AbortLinearMotionRsp* rsp)
{
  dprintf("%s() called\n", __func__);

#ifdef USE_MOTION_CONTROL
  memset(rsp, 0, sizeof(*rsp));
  return AbortLinearMotionCommand(req->motor_nr);
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif
}

/*! \brief  GetMotorControlMethod
*
* Retrive the currently used control method
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int GetMotorControlMethod    (struct GetMotorControlMethodReq* req, struct GetMotorControlMethodRsp* rsp)
{

  memset(rsp, 0, sizeof(*rsp));
  rsp->method = GetControlMethodCommand(req->motor_nr);

  dprintf("Control Method 2: %d\n",  rsp->method);

  return 0;
}

/*! \brief  SetMotorControlMethod
*
* Retrive the currently used control method
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int SetMotorControlMethod    (struct SetMotorControlMethodReq* req, struct SetMotorControlMethodRsp* rsp)
{
  memset(rsp, 0, sizeof(*rsp));
  rsp->error = SetControlMethodCommand(req->motor_nr, req->method);

  return 0;
}

/*! \brief  GetExtendedMotorState
*
* Retrive the currently used control method, speed, torque, set control target (speed or torque), used current and timestamp
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int GetExtendedMotorState       (struct GetExtendedMotorStateReq* req,       struct GetExtendedMotorStateRsp* rsp)
{
  memcpy(&rsp->State, &MotorExtendedStateValues[req->motor_nr], sizeof(rsp->State));
  rsp->State.Timestamp = xTaskGetTickCount();
  rsp->State.ControlMethod = GetControlMethodCommand(req->motor_nr);

  return 0;
}

/*! \brief  Start Motor with torque
  *
  * @param[in]  q Question from Serial Protocol
  * @param[out] a Answer   for  Serial Protocol
  * @retval 0 Success
  * @retval -1 Failure
*/
int StartMotorTorqueCtrl(struct StartMotorTorqueCtrlReq* req, struct StartMotorTorqueCtrlRsp* rsp)
{
  MotorExtendedStateValues[req->motor_nr].ControlMethod=MOTORCONTROL_CONTROL_BY_TORQUE;

  return MotorControl_Start(req->motor_nr);
}

/*! \brief   SetDemoState
*
* Change to a new demo state
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int  SetDemoState    (struct  SetDemoStateReq* req, struct SetDemoStateRsp* rsp)
{

#ifdef USE_DEMO_CONTROL
  memset(rsp, 0, sizeof(*rsp));
  Demo((demo_status)req->new_state);
  return 0;
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif /* USE_DEMO_CONTROL */
}

/*! \brief   GetDemoState
*
* retrieve the current demo state
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int  GetDemoState    (struct  GetDemoStateReq* req, struct GetDemoStateRsp* rsp)
{

#ifdef USE_DEMO_CONTROL
  memset(rsp, 0, sizeof(*rsp));
  rsp->curr_state = DemoState();
  return 0;
#else
  memset(rsp, -1, sizeof(*rsp));
  return -1;
#endif /* USE_DEMO_CONTROL */
}

/*! \brief  GetEncoderCount Value
*
*
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
int GetEncoderCounter(struct GetEncoderCounterReq* req, struct GetEncoderCounterRsp* rsp)
{
  memset(rsp, 0, sizeof(*rsp));
#ifdef USE_ENCODER
  rsp->counterValue = ENC_GetEncCounter(req->encoder_nr);
#else
  rsp->counterValue = 0;
#endif /* USE_ENCODER */
  return 0;
}

/* section for functions of motion control */


/*! \brief  Get DC-Link Voltage
*
* Read out the actual DC-Link voltage of the board
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
static int GetDcLinkVoltageHelper(int motor_nr, uint16_t *Voltage)
{
  uint32_t voltage = 0;
#ifdef USE_HV_COMMUNICATION
  voltage =   10
    * HV_Communication_GetValue(0)
      * 5
        * pow10(-ChannelValues[motor_nr].SensitivityVoltageMeasure.unit)
          / ChannelValues[motor_nr].SensitivityVoltageMeasure.value
            / 0x3ff;
#else
  if( CHANNEL_IS_USED == ChannelUsageConfiguration[motor_nr] )
  {
    voltage = HAL_ADC_GetVDC(motor_nr)
      * 10
        * 5
          * pow10(-ChannelValues[motor_nr].SensitivityVoltageMeasure.unit)
            / ChannelValues[motor_nr].SensitivityVoltageMeasure.value
              / 0xfff;
  }
#endif

  *Voltage = voltage;

#ifdef BOARD_VDC_CHANNEL_0_VALUE
  if (motor_nr == 0)
    *Voltage = 10 * BOARD_VDC_CHANNEL_0_VALUE / pow10(-BOARD_VDC_CHANNEL_0_UNIT);
#endif

#ifdef BOARD_VDC_CHANNEL_1_VALUE
  if (motor_nr == 1)
    *Voltage = 10 * BOARD_VDC_CHANNEL_1_VALUE / pow10(-BOARD_VDC_CHANNEL_1_UNIT);
#endif

#ifdef BOARD_VDC_CHANNEL_2_VALUE
  if (motor_nr == 2)
    *Voltage = 10 * BOARD_VDC_CHANNEL_2_VALUE / pow10(-BOARD_VDC_CHANNEL_2_UNIT);
#endif


  return 0;
}

static uint8_t xxx_temp_string_memory[MAX_LENGTH_MOTOR_ID];

static int get_board_system_setting(struct GetEntryReq *req, struct GetEntryRsp *e)
{
  int id = req->id;
  memset(e, 0, sizeof(*e));

  switch(id) {
  case MPC_FirmwareFeatures:
#ifdef USE_DSO
    e->value |= FEATURE_HAS_DSO;
#endif /* USE_DSO */
#ifdef USE_LOAD_STATISTICS
    e->value |= FEATURE_HAS_LOAD_STATISTIC;
#endif /* USE_LOAD_STATISTICS */
#ifdef USE_TURN_CONTROL
    e->value |= FEATURE_HAS_TURN_CONTROL;
#endif /* USE_TURN_CONTROL */
#ifdef USE_MOTION_CONTROL
    e->value |= FEATURE_HAS_MOTION_CONTROL;
#endif /* USE_MOTION_CONTROL */
#ifdef USE_DEMO_CONTROL
    e->value |= FEATURE_HAS_DEMO;
#endif /* USE_DEMO_CONTROL */
#ifdef USE_TORQUE_CONTROL
    e->value |= FEATURE_HAS_TORQUE_CONTROL;
#endif /* USE_TORQUE_CONTROL */
#ifdef USE_TEMPERATURE_CONTROL
    e->value |= FEATURE_HAS_TEMPERATURE_CONTROL;
#endif /* USE_TEMPERATURE_CONTROL */
#ifdef USE_HSDSO
    e->value |= FEATURE_HAS_HSDSO;
#endif /* USE_HSDSO */
#ifdef USE_RW_BOARD_SETTINGS
    e->value |= FEATURE_HAS_RW_BOARD_SETTINGS;
#endif /* USE_RW_BOARD_SETTINGS */
#ifdef USE_EXTERNAL_SPEED_CONTROL
    e->value |= FEATURE_HAS_EXTERNAL_SPEED_CONTROL;
#endif /* USE_EXTERNAL_SPEED_CONTROL */
#ifdef USE_SW_OVER_UNDER_VOLTAGE_DETECTION
    e->value |= FEATURE_HAS_SW_OVER_UNDER_VOLTAGE_DETECTION;
#endif /* USE_SW_OVER_UNDER_VOLTAGE_DETECTION */
#ifdef USE_MOTOR_DISCONNECT_DETECTION
    e->value |= FEATURE_HAS_MOTOR_DISCONNECT_DETECTION;
#endif /* USE_MOTOR_DISCONNECT_DETECTION */
#ifdef USE_SW_OVERCURRENT_DETECTION
    e->value |= FEATURE_HAS_SW_OVERCURRENT_DETECTION;
#endif /* USE_SW_OVERCURRENT_DETECTION */
#ifdef USE_LOAD_DEPENDANT_SPEED_REDUCTION
    e->value |= FEATURE_HAS_LOAD_DEPENDANT_SPEED_REDUCTION;
#endif /* USE_LOAD_DEPENDANT_SPEED_REDUCTION */
#ifdef USE_CAN
    e->value |= FEATURE_HAS_CAN;
#endif /* USE_CAN */
#ifdef USE_STALL_DETECT
    e->value |= FEATURE_HAS_STALL_DETECT;
#endif /* USE_STALL_DETECT */

    e->value |= FEATURE_HAS_GET_DC_LINK_VOLTAGE;
    e->value |= FEATURE_HAS_GET_MOTOR_STAGE;

    dprintf("FirmwareFeatures: value %d\n", e->value);

    break;
  case MPC_BoardRevision:
    e->value = BoardRevision;
    dprintf("BoardRevision: value %d\n", e->value);
    break;
  case MPC_Channels:
    e->value = BOARD_AVAILABLE_CHANNELS;
    dprintf("Channels: value %d\n", e->value);
    break;
  case MPC_DsoSize:
    e->value = BOARD_DSO_SIZE;
    dprintf("DsoSize: value %d\n", e->value);
    break;
  case MPC_ParameterTransferCount:
    e->value = MAX_NUM_CONFIG_VALUES;
    dprintf("ParameterTransferCount: value %d\n", e->value);
    break;
  case MPC_BoardName:
    {
      const uint8_t *s = &xxx_temp_string_memory[0];
      memset(&xxx_temp_string_memory[0], 0, sizeof(xxx_temp_string_memory));
      memcpy(&xxx_temp_string_memory[0], BOARD_NAME, sizeof(BOARD_NAME));
			e->value =  s[req->offset*4+0]<< 0
        |s[req->offset*4+1]<< 8
          |s[req->offset*4+2]<<16
            |s[req->offset*4+3]<<24;
      break;
    }
  case MPC_BoardNamePWR0:
#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
    {
      const uint8_t *s = &xxx_temp_string_memory[0];
      memset(&xxx_temp_string_memory[0], 0, sizeof(xxx_temp_string_memory));
      memcpy(&xxx_temp_string_memory[0], BOARD_NAME_PWR, sizeof(BOARD_NAME_PWR));
			e->value =  s[req->offset*4+0]<< 0
        |s[req->offset*4+1]<< 8
          |s[req->offset*4+2]<<16
            |s[req->offset*4+3]<<24;
    }
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_0 */
    break;
  case MPC_BoardNamePWR1:
#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
    {
      const uint8_t *s = &xxx_temp_string_memory[0];
      memset(&xxx_temp_string_memory[0], 0, sizeof(xxx_temp_string_memory));
      memcpy(&xxx_temp_string_memory[0], BOARD_NAME_PWR, sizeof(BOARD_NAME_PWR));
			e->value =  s[req->offset*4+0]<< 0
        |s[req->offset*4+1]<< 8
          |s[req->offset*4+2]<<16
            |s[req->offset*4+3]<<24;
    }
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_1 */
    break;

  case MPC_BoardNamePWR2:
#ifdef BOARD_PWR_HEADER_FILE_2
#include BOARD_PWR_HEADER_FILE_2
    {
      const uint8_t *s = &xxx_temp_string_memory[0];
      memset(&xxx_temp_string_memory[0], 0, sizeof(xxx_temp_string_memory));
      memcpy(&xxx_temp_string_memory[0], BOARD_NAME_PWR, sizeof(BOARD_NAME_PWR));
			e->value =  s[req->offset*4+0]<< 0
        |s[req->offset*4+1]<< 8
          |s[req->offset*4+2]<<16
            |s[req->offset*4+3]<<24;
    }
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_1 */
    break;
  case MSS_HsDSOClock:
    {
      int T0;
      const uint32_t a = 2U;
      SystemCoreClockUpdate();
      T0 = SystemCoreClock / (a << ((TSB_CG->SYSCR >> 8U) & 7U));

      if (SystemCoreClock <=80*1000*1000)
        e->value = SystemCoreClock/16;
      else
        e->value = T0/16;
    }
    break;
  case MSS_FirmwareVersion:
    {
      e->value = (FW_VERSION_MAJOR << 8) | FW_VERSION_MINOR;
      break;
    }
  case MSS_SystemLoad:
    {
#ifdef USE_LOAD_STATISTICS
      e->value = SYSTEMLoad_get();
#else
      return -1;
#endif
      break;
    }
  case MSS_ResetStatus:
    e->value = hasReset;
    hasReset = 0;
    break;
  default:
    dprintf("unknown id %d\n", id);
    return -1;
  }
  return 0;
}

static int GetParameterHelper(int motor_nr, struct GetEntryReq *req, struct GetEntryRsp *rsp)
{
  MotorParameters *m;
  PIControlSettings *p;
  SystemDependandValues *s;
  ChannelDependandValues *c;

  m = &MotorParameterValues[motor_nr];
  p = &PIControl[motor_nr];
  s = &SystemValues[motor_nr];
  c = &ChannelValues[motor_nr];

#define MAKE_ENTRY_G(m,x) \
case MPC_##x: \
  rsp->value = m->x; \
    rsp->unit = 0; \
      break;

#define MAKE_ENTRY_GU(m,x) \
case MPC_##x: \
  rsp->value = m->x.value; \
    rsp->unit = m->x.unit; \
      break;

      switch(req->id) {
        MAKE_ENTRY_G(m, PolePairs);
        MAKE_ENTRY_G(m, Direction);
        MAKE_ENTRY_GU(m, MaxAngAcc);
        MAKE_ENTRY_GU(m, TorqueFactor);
        MAKE_ENTRY_GU(m, Resistance);
        MAKE_ENTRY_GU(m, Inductance);
        MAKE_ENTRY_GU(m, SpeedLimit);
        MAKE_ENTRY_GU(m, SpeedChange);
        MAKE_ENTRY_G(m, PositionDelay);
        MAKE_ENTRY_GU(m, IqStart);
        MAKE_ENTRY_GU(m, IdStart);
        MAKE_ENTRY_GU(m, IqLimit);
        MAKE_ENTRY_GU(m, IdLimit);
      case MPC_MotorId:
	   rsp->value =  m->MotorId[req->offset*4+0]<< 0
          |m->MotorId[req->offset*4+1]<< 8
            |m->MotorId[req->offset*4+2]<<16
              |m->MotorId[req->offset*4+3]<<24;

        break;
        MAKE_ENTRY_G(m, Encoder);
        MAKE_ENTRY_G(m, EncoderUsage);
        MAKE_ENTRY_G(m, EncoderResolution);
        MAKE_ENTRY_G(m, EncoderMinPos);
        MAKE_ENTRY_G(m, EncoderStartPos);
        MAKE_ENTRY_G(m, EncoderMaxPos);
        MAKE_ENTRY_G(m, EncoderLMApproach);
        MAKE_ENTRY_G(m, EncoderGearFactor);
        MAKE_ENTRY_G(m, EncoderReceiver); 
        MAKE_ENTRY_G(m, EncoderReceiverMode); 
		
        MAKE_ENTRY_G(p, PiIdKi);
        MAKE_ENTRY_G(p, PiIdKp);
        MAKE_ENTRY_G(p, PiIqKi);
        MAKE_ENTRY_G(p, PiIqKp);
        MAKE_ENTRY_G(p, PositionKi);
        MAKE_ENTRY_G(p, PositionKp);
        MAKE_ENTRY_G(p, SpeedKi);
        MAKE_ENTRY_G(p, SpeedKp);

        MAKE_ENTRY_G(s, PwmFrequency);
        MAKE_ENTRY_G(s, ShutdownMode);
        MAKE_ENTRY_G(s, Braketime);
        MAKE_ENTRY_G(s, Brakepercentage);
        MAKE_ENTRY_G(s, RestartMode);
        MAKE_ENTRY_G(s, StalldetectValue);
        MAKE_ENTRY_G(s, Overtemperature);
        MAKE_ENTRY_G(s, CanId);
        MAKE_ENTRY_G(s, ExternalSpeedControl);
        MAKE_ENTRY_G(s, SwOvervoltage);
        MAKE_ENTRY_G(s, SwUndervoltage);
        MAKE_ENTRY_GU(s, SwOvercurrent);
        MAKE_ENTRY_G(s, SpeedReduction);

        MAKE_ENTRY_G(c, Deadtime);
        MAKE_ENTRY_G(c, BootstrapDelay);
        MAKE_ENTRY_GU(c, SensitivityCurrentMeasure);
        MAKE_ENTRY_GU(c, SensitivityVoltageMeasure);
        MAKE_ENTRY_G(c, GainCurrentMeasure);
        MAKE_ENTRY_G(c, MeasurementType);
        MAKE_ENTRY_G(c, MeasurementDirection);
        MAKE_ENTRY_G(c, PolL);
        MAKE_ENTRY_G(c, PolH);

      case MMS_Temperature:
#ifdef USE_TEMPERATURE_CONTROL
        rsp->value = TEMPERATURE_GetActualTemperature(motor_nr);
        break;
#else
        return -1;
#endif
      case MMS_DcLinkVoltage:
        {
          uint16_t voltage;

          if (GetDcLinkVoltageHelper(motor_nr, &voltage))
            return -1;

          rsp->value = voltage;
          break;
        }
      case MMS_ErrorState:
        rsp->value = MotorErrorField[motor_nr].Error;
        break;
      case MMS_MotorStage:
        rsp->value = MotorControl[motor_nr].ActualStage.main;
        break;
      case MMS_TurnNumber:
#ifdef USE_TURN_CONTROL
        rsp->value = EncoderData[motor_nr].FullTurns;
        /* Convert electrical to physical turns */
        if( 0 != MotorParameterValues[motor_nr].PolePairs )
        {
          rsp->value /= MotorParameterValues[motor_nr].PolePairs;
        }
        break;
#else
        return -1;
#endif
      default:
        dprintf("unknown id %d\n", req->id);
        return -1;
      }

      return 0;
}

int GetOneMotorParameter(struct GetOneMotorParameterReq* req,  struct GetOneMotorParameterRsp* rsp)
{
  int ret;

  memset(rsp, 0, sizeof(*rsp));

  if (req->motor_nr == 255) {
    ret = get_board_system_setting(&req->entry, &rsp->entry);
    if (ret) {
      dprintf("unknown id %d\n", req->entry.id);
      return -1;
    }
    return 0;
  }

  /* todo: fix check for motor id */
  if (req->motor_nr > MAX_CHANNEL) {
    dprintf("invalid motor id %d\n", req->motor_nr);
    return -1;
  }

  ret = GetParameterHelper(req->motor_nr, &req->entry, &rsp->entry);
  if (ret)
    return ret;

  return 0;
}

int GetMotorParameters  (struct GetMotorParametersReq* req,  struct GetMotorParametersRsp* rsp)
{
  int ret;
  int i;


  if (req->entries > MAX_NUM_CONFIG_VALUES)
    return -1;

  memset(rsp, 0, sizeof(*rsp));

  if (req->motor_nr == 255) {
    for (i = 0; i < req->entries; i++) {
      struct GetEntryReq *qe = &req->entry[i];
      struct GetEntryRsp *ae = &rsp->entry[i];

      ret = get_board_system_setting(qe, ae);
      if (ret) {
        dprintf("unknown id %d\n", qe->id);
        return -1;
      }
    }
    return 0;
  }

  if (req->motor_nr > MAX_CHANNEL) {
    dprintf("invalid motor id %d\n", req->motor_nr);
    return -1;
  }

  for (i = 0; i < req->entries; i++) {
    struct GetEntryRsp *ae = &rsp->entry[i];
    struct GetEntryReq *qe = &req->entry[i];
    ret = GetParameterHelper(req->motor_nr, qe, ae);
    if (ret)
      return ret;
  }

  return 0;
}

static int SetParameterHelper(int motor_nr, struct SetEntry *e)
{
  MotorParameters *m;
  PIControlSettings *p;
  SystemDependandValues *s;
  ChannelDependandValues *c;

  m = &MotorParameterValues[motor_nr];
  p = &PIControl[motor_nr];
  s = &SystemValues[motor_nr];
  c = &ChannelValues[motor_nr];

#define MAKE_ENTRY_S(m,x,param) \
case MPC_##x: \
  m->x = e->value; \
    if (param) \
      MotorControl[motor_nr].ParameterChange |= param; \
        break;

#define MAKE_ENTRY_SU(m,x,param) \
case MPC_##x: \
  m->x.value = e->value; \
    m->x.unit = e->unit; \
      if (param)  \
        MotorControl[motor_nr].ParameterChange |= param; \
          break;

          switch(e->id)
          {
            MAKE_ENTRY_S (m, PolePairs                 ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, Direction                 ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, MaxAngAcc                 ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, TorqueFactor              ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, Resistance                ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, Inductance                ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, SpeedLimit                ,0);                           /* This value can not changed while motor is turning */
            MAKE_ENTRY_SU(m, SpeedChange               ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, PositionDelay             ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, IqStart                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, IdStart                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, IqLimit                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_SU(m, IdLimit                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
          case MPC_MotorId:
            m->MotorId[e->unit*4+0] =  (e->value >>  0) & 0xff;
            m->MotorId[e->unit*4+1] =  (e->value >>  8) & 0xff;
            m->MotorId[e->unit*4+2] =  (e->value >> 16) & 0xff;
            m->MotorId[e->unit*4+3] =  (e->value >> 24) & 0xff;
            break;
			
            MAKE_ENTRY_S (m, Encoder                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderUsage              ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderResolution         ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderMinPos             ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderStartPos           ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderMaxPos             ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderLMApproach         ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (m, EncoderGearFactor         ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);            
            MAKE_ENTRY_S (m, EncoderReceiver           ,MOTORCONTROL_CHANGE_MOTOR_PARAMS); 
            MAKE_ENTRY_S (m, EncoderReceiverMode       ,MOTORCONTROL_CHANGE_MOTOR_PARAMS); 

            MAKE_ENTRY_S (p, PiIdKi                    ,MOTORCONTROL_CHANGE_PI_PARAMS);
            MAKE_ENTRY_S (p, PiIdKp                    ,MOTORCONTROL_CHANGE_PI_PARAMS);
            MAKE_ENTRY_S (p, PiIqKi                    ,MOTORCONTROL_CHANGE_PI_PARAMS);
            MAKE_ENTRY_S (p, PiIqKp                    ,MOTORCONTROL_CHANGE_PI_PARAMS);
            MAKE_ENTRY_S (p, PositionKi                ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (p, PositionKp                ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (p, SpeedKi                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (p, SpeedKp                   ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);

            MAKE_ENTRY_S (s, PwmFrequency              ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE | MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD);
            MAKE_ENTRY_S (s, ShutdownMode              ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE);
            MAKE_ENTRY_S (s, Braketime                 ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE); /* todo: add unit? */
            MAKE_ENTRY_S (s, Brakepercentage           ,0);
            MAKE_ENTRY_S (s, RestartMode               ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE);
            MAKE_ENTRY_S (s, StalldetectValue          ,0);                           /* This value is checked constantly - no dynamic update needed */
            MAKE_ENTRY_S (s, Overtemperature           ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE);
            MAKE_ENTRY_S (s, CanId                     ,0);                           /* This value can not be changed during runtime */
            MAKE_ENTRY_S (s, ExternalSpeedControl      ,0);                           /* This value is checked constantly - no dynamic update needed */
            MAKE_ENTRY_S (s, SwOvervoltage             ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE);
            MAKE_ENTRY_S (s, SwUndervoltage            ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE);
            MAKE_ENTRY_SU(s, SwOvercurrent             ,MOTORCONTROL_CHANGE_MOTOR_PARAMS);
            MAKE_ENTRY_S (s, SpeedReduction            ,MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE);

            MAKE_ENTRY_S (c, Deadtime                  ,MOTORCONTROL_CHANGE_BOARD_PARAMS_PMD);
            MAKE_ENTRY_S (c, BootstrapDelay            ,MOTORCONTROL_CHANGE_BOARD_PARAMS_VE);
            MAKE_ENTRY_SU(c, SensitivityCurrentMeasure ,MOTORCONTROL_CHANGE_BOARD_PARAMS_VE);
            MAKE_ENTRY_SU(c, SensitivityVoltageMeasure ,MOTORCONTROL_CHANGE_BOARD_PARAMS_VE);
            MAKE_ENTRY_S (c, GainCurrentMeasure        ,MOTORCONTROL_CHANGE_BOARD_PARAMS_VE);
            MAKE_ENTRY_S (c, MeasurementType           ,MOTORCONTROL_CHANGE_BOARD_PARAMS_VE);
            MAKE_ENTRY_S (c, MeasurementDirection      ,MOTORCONTROL_CHANGE_BOARD_PARAMS_VE);
            MAKE_ENTRY_S (c, PolL                      ,MOTORCONTROL_CHANGE_BOARD_PARAMS_PMD);
            MAKE_ENTRY_S (c, PolH                      ,MOTORCONTROL_CHANGE_BOARD_PARAMS_PMD);

          case MMS_Speed:
            SetSpeed(motor_nr, e->value);
            break;

          case MMS_Torque:

            dprintf("MMS_Torque: Motor: %d Value:%d\n",motor_nr,e->value);
            IqStartStorage[motor_nr].value = MotorParameterValues[motor_nr].IqStart.value;
            IqStartStorage[motor_nr].unit  = MotorParameterValues[motor_nr].IqStart.unit;

            MotorParameterValues[motor_nr].IqStart.value      = 1000 * e->value /  MotorParameterValues[motor_nr].TorqueFactor.value;
            MotorParameterValues[motor_nr].IqStart.unit       = e->unit - 3;

            MotorControl[motor_nr].ParameterChange |= MOTORCONTROL_CHANGE_MOTOR_PARAMS;
            break;

          default:
            dprintf("unknown id %d\n", e->id);
            break;
          }

          return 0;
}

int SetOneMotorParameter(struct SetOneMotorParameterReq* req,  struct SetOneMotorParameterRsp* rsp)
{
  int ret;

    dprintf("motor_nr %d, entries %d\n", req->motor_nr, req->entry.id);
  /* todo: fix check for motor id */
  if (req->motor_nr > MAX_CHANNEL) {
    dprintf("invalid motor id %d\n", req->motor_nr);
    return -1;
  }

  memset(rsp, 0, sizeof(*rsp));

  ret = SetParameterHelper(req->motor_nr, &req->entry);
  if (ret)
    return -1;

  return 0;
}

int SetMotorParameters  (struct SetMotorParametersReq* req,  struct SetMotorParametersRsp* rsp)
{
  int i;

    dprintf("motor_nr %d, entries %d\n", req->motor_nr, req->entries);

  if (req->entries > MAX_NUM_CONFIG_VALUES)
    return -1;

  /* todo: fix check for motor id */
  if (req->motor_nr > MAX_CHANNEL) {
    dprintf("invalid motor id %d\n", req->motor_nr);
    return -1;
  }

  memset(rsp, 0, sizeof(*rsp));

  for (i = 0; i < req->entries; i++)
  {
    struct SetEntry *e = &req->entry[i];
    if (SetParameterHelper(req->motor_nr, e))
      return -1;
  }

  return 0;
}

int SetPwmChannelParameter(struct SetPwmChannelParameterReq* req,  struct SetPwmChannelParameterRsp* rsp)
{
  memset(rsp, -1, sizeof(*rsp));
  return -1;
}

