/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Toshiba Electronics Europe GmbH
*   European LSI Design and Engineering Center (ELDEC)
****************************************************************************/

/**
* @file api.h
*
* @brief Contains definitions and function declarations for Motor Control API.
*
* All functions may return the following, common return codes.
*
* All functions may return the following, common return codes.
*
* @return RESULT_SUCCESS Function completed succesfully
* @return RESULT_ERROR Error
*
*/
#ifndef __API_H__
#define __API_H__

#include "api_internal.h"
#include "hal_ve.h"
#ifdef USE_DEMO_CONTROL
#include "DemoApplication.h"
#endif /* USE_DEMO_CONTROL */

/* #include "DemoApplication.h" */
#define MAX_LENGTH_MOTOR_ID     32                                              /*!< Number of characters for Motor ID */
#define MAX_LENGTH_BOARD_NAME   32                                              /*!< Number of characters for Board Name */

typedef enum
{
  RPC_StartMotor                  =0x00,
  RPC_StopMotor                   =0x01,
  RPC_GetMotorState               =0x02,
  RPC_StoreParameters             =0x03,
  RPC_ClearParameters             =0x04,
  RPC_ConfigDSOLog                =0x05,
  RPC_GetDSOLogData               =0x06,
  RPC_ConfigureHsDSO              =0x07,
  RPC_DoTurn                      =0x08,
  RPC_GetOneMotorParameter        =0x09,
  RPC_SetOneMotorParameter        =0x0a,
  RPC_GetMotorParameters          =0x0b,
  RPC_SetMotorParameters          =0x0c,
  RPC_DoLinearMotion              =0x0d,
  RPC_GetAbsolutePosition         =0x0e,
  RPC_AbortLinearMotion           =0x0f,
  RPC_StartMotorTorqueCtrl        =0x10,
  RPC_GetMotorControlMethod       =0x11,
  RPC_SetMotorControlMethod       =0x12,
  RPC_GetEncoderCounter           =0x13,
  RPC_GetFWVersion                =0x14,
  RPC_SetDemoState                =0x21,
  RPC_GetDemoState                =0x22,
  RPC_GetExtendedMotorState       =0x23,
  RPC_Dummy                       =0xff,
} RemotePrcocedureCall;


typedef enum
{
  MPC_PolePairs                   =0x10,
  MPC_Direction                   =0x11,
  MPC_MaxAngAcc                   =0x12,
  MPC_TorqueFactor                =0x13,
  MPC_Resistance                  =0x14,
  MPC_Inductance                  =0x15,
  MPC_SpeedLimit                  =0x16,
  MPC_SpeedChange                 =0x17,
  MPC_PositionDelay               =0x18,
  MPC_IqStart                     =0x19,
  MPC_IdStart                     =0x1a,
  MPC_IqLimit                     =0x1b,
  MPC_IdLimit                     =0x1c,
  MPC_MotorId                     =0x1d,

  MPC_Encoder                     =0x30,
  MPC_EncoderUsage                =0x31,
  MPC_EncoderResolution           =0x32,
  MPC_EncoderMinPos               =0x33,
  MPC_EncoderStartPos             =0x34,
  MPC_EncoderMaxPos               =0x35,
  MPC_EncoderLMApproach           =0x36,
  MPC_EncoderGearFactor           =0x37,
  MPC_EncoderReceiver             =0x38, 
  MPC_EncoderReceiverMode         =0x39, 

  MPC_PiIdKi                      =0x40,
  MPC_PiIdKp                      =0x41,
  MPC_PiIqKi                      =0x42,
  MPC_PiIqKp                      =0x43,
  MPC_PositionKi                  =0x44,
  MPC_PositionKp                  =0x45,
  MPC_SpeedKi                     =0x46,
  MPC_SpeedKp                     =0x47,

  MPC_PwmFrequency                =0x50,
  MPC_ShutdownMode                =0x51,
  MPC_Braketime                   =0x52,
  MPC_Brakepercentage             =0x53,
  MPC_RestartMode                 =0x54,
  MPC_StalldetectValue            =0x55,
  MPC_Overtemperature             =0x56,
  MPC_CanId                       =0x57,
  MPC_ExternalSpeedControl        =0x58,
  MPC_SwOvervoltage               =0x59,
  MPC_SwUndervoltage              =0x5a,
  MPC_SwOvercurrent               =0x5b,
  MPC_SpeedReduction              =0x5c,

  MPC_Deadtime                    =0x70,
  MPC_BootstrapDelay              =0x71,
  MPC_SensitivityCurrentMeasure   =0x72,
  MPC_SensitivityVoltageMeasure   =0x73,
  MPC_GainCurrentMeasure          =0x74,
  MPC_MeasurementType             =0x75,
  MPC_MeasurementDirection        =0x76,
  MPC_PolL                        =0x77,
  MPC_PolH                        =0x78,

  MPC_FirmwareFeatures            =0x90,
  MPC_BoardRevision               =0x91,
  MPC_Channels                    =0x92,
  MPC_DsoSize                     =0x93,
  MPC_ParameterTransferCount      =0x94,
  MPC_BoardName                   =0x95,
  MPC_BoardNamePWR0               =0x96,
  MPC_BoardNamePWR1               =0x97,
  MPC_BoardNamePWR2               =0x98,
} MotorParameterCall;

typedef enum
{
  MSS_FirmwareVersion             =0xb0,
  MSS_ResetStatus                 =0xb1,
  MSS_SystemLoad                  =0xb2,
  MSS_HsDSOClock                  =0xb3,
} MotorSystemSetting;

typedef enum
{
  MMS_Temperature                 =0xe0,
  MMS_ErrorState                  =0xe1,
  MMS_DcLinkVoltage               =0xe2,
  MMS_MotorStage                  =0xe3,
  MMS_TurnNumber                  =0xe4,
  MMS_Speed                       =0xe5,
  MMS_Torque                      =0xe6,
} MotorManagementCall;

/*! \brief Motor turning possibilities.
 *
 *  This enumeration displays the different rotation possibilities the motor is capeable of.
 */
typedef enum {
  MOTOR_CW_ONLY                = 0,                                             /*!< Motor can turn clock-wise only */
  MOTOR_CCW_ONLY               = 1,                                             /*!< Motor can turn counter-clock-wise only */
  MOTOR_CW_CCW                 = 2,                                             /*!< Motor can turn clock-wise and counter-clock wise */
} MOTOR_DIRECTION;

/*! \brief Motor hardware encoder types.
 *
 *  This enumeration selects the encoder type which could be connected between encoder and MCU
 */
typedef enum {
  ENCODER_NO_RECEIVER            = 0,                                          /*!< Motor encoder is directly connected to MCU */
  ENCODER_RESOLVER_RECEIVER      = 1,                                          /*!< Motor encoder is connected via resolver encoder to the MCU */
  ENCODER_DIFFERENTIAL_RECEIVER  = 2,                                          /*!< Motor encoder is connected via differential encoder to the MCU */
  ENCODER_SINGLE_ENDED_RECEIVER  = 3,                                          /*!< Motor encoder is connected via single ended encoder to the MCU */
} ENCODER_RECEIVER;

/*! \brief Differential/Single Ended encoder operation
 *
 *  This enumeration selects the operation mode for the signals (A, B and Z) of the differential/Single Ended encoder
 */
typedef enum {
  ENCODER_D_HTL              = 0,                                          /*!< Operation mode of differential encoder is differential HTL  */
  ENCODER_RS_422             = 1,                                          /*!< Operation mode of differential encoder is RS-242  */
  ENCODER_SE_HTL             = 2,                                          /*!< Operation mode of differential encoder is single ended HTL  */
  ENCODER_TTL                = 3,                                          /*!< Operation mode of differential encoder is TTL  */
} ENCODER_RECEIVER_MODE;

/*! \brief Motor encoder type.
 *
 *  This enumeration selects the encoder usage for the Motor
 */
typedef enum {
  ENCODER_NO_ENCODER           = 0,                                             /*!< Motor has NO Encoder - use SW emulation for event counting */
  ENCODER_HALL_UVW             = 1,                                             /*!< Motor is using 3 Hall Sensors Phase U,V,W   */
  ENCODER_HALL_UV              = 2,                                             /*!< Motor is using 2 Hall Sensors Phase U and V */
  ENCODER_INCENC_ABZ           = 3,                                             /*!< Motor is using encoder inputs A,B,Z         */
  ENCODER_INCENC_AB            = 4,                                             /*!< Motor is using encoder inputs A,B           */
  ENCODER_SINGLE_PULSE         = 5,                                             /*!< Motor is using single pulse                 */
  ENCODER_AMS_AS5145H          = 6,                                             /*!< Motor is using AMS - AS5145H                */
} ENCODER_TYPE;

/*! \brief Motor encoder usage type.
 *
 *  This enumeration selects the encoder usage for the Motor
 */
typedef enum {
  ENCODER_USAGE_NONE           = 0,                                             /*!< Encoder is not used */
  ENCODER_USAGE_SPEED          = 1,                                             /*!< Encoder is used for speed control */
  ENCODER_USAGE_EVENT          = 2,                                             /*!< Encoder is used for event counting */
  ENCODER_USAGE_EVENT_IRQ      = 3,                                             /*!< Encoder is used for IRQ on count pulse */
} ENCODER_USAGE;

/*! \brief External Speed control flags.
 *
 *  This enumeration is for the different external speed control modes
 */
typedef enum {
  EXTERNAL_SPEED_NONE                = 0,                                       /*!< No External Speed Control */
  EXTERNAL_SPEED_ADC                 = 1,                                       /*!< External Speed Control via ADC */
  EXTERNAL_SPEED_PWM                 = 2,                                       /*!< External Speed Control via PWM */
  EXTERNAL_SPEED_SERVO_NEGMAX_TO_MAX = 3,                                       /*!< External Speed Control via Servo Signal */
  EXTERNAL_SPEED_SERVO_0_TO_MAX      = 4,                                       /*!< External Speed Control via Servo Signal */
} EXTERNAL_SPEED;

/*! \brief Speed unit
 *
 *  This enumeration is for the different units of speed
 */
typedef enum {
  HZ                           = 0,                                             /*!< Speed value is given in Hz */
  RPM                          = 1,                                             /*!< Speed value is given in RPM */
} SPEED;

/*! \brief Accelleration unit
 *
 *  This enumeration is for the different units of acceleration
 */
typedef enum {
  RAD_BY_S2                    = 0,                                             /*!< acceleration is given as rad/s^2 */
  HZ_BY_S                      = 1,                                             /*!< acceleration is given as Hz/s    */
  RPM_BY_S                     = 2,                                             /*!< acceleration is given as RPM/s   */
  RPM_BY_MIN                   = 3,                                             /*!< acceleration is given as RMP/min */
} ACCELERATION;

/*! \brief Torque factor unit
 *
 *  This enumeration is for the different units of the torque factor
 */
typedef enum {
  MNM_BY_A                     = -3,                                            /*!< torque factor given as mNm/A */
  NM_BY_A                      =  0,                                            /*!< torque factor given as Nm/A */
  MNM_BY_MA                    =  0,                                            /*!< torque factor given as mNm/mA */
} TORQUE;

/*! \brief resistance unit
 *
 *  This enumeration is for the different units of the resistance
 */
typedef enum {
  OHM                          =  0,                                            /*!< resistance given in Ohm */
  MOHM                         = -3,                                            /*!< resistance given as mOhm */
  UOHM                         = -6,                                            /*!< resistance given as uOhm */
} RESISTANCE;

/*! \brief inductance unit
 *
 *  This enumeration is for the different units of the inductance
 */
typedef enum {
  MH                           = -3,                                            /*!< inductance given in mH */
  UH                           = -6,                                            /*!< inductance given in uH */
  NH                           = -9,                                            /*!< inductance given in nH */
} INDUCTANCE;

/*! \brief current unit
 *
 *  This enumeration is for the different units of the currents
 */
typedef enum {
  AMPERE                      =  0,                                            /*!< current given in A */
  MAMPERE                      = -3,                                            /*!< current given in mA */
} CURRENT;

/*! \brief Shutdown mode
 *
 *  Selects how the motor should behave in case of stop command
 */
typedef enum {
  SHUTDOWN_DISABLE_OUTPUT      = 0,                                             /*!< Shutdown motor by switching off signals */
  SHUTDOWN_SHORT_BRAKE         = 1,                                             /*!< Shutdown motor by short lines */
  SHUTDOWN_GENTLE              = 2,                                             /*!< Shutdown motor by speeding down to 0 */
} SHUTDOWN;

/*! \brief Restart mode
 *
 *  Selects how the motor should behave in case of stalled electromagnetic field
 */
typedef enum {
  SWITCH_OFF_MOTOR             = 0,                                             /*!< Switch off motor */
  RESTART_MOTOR                = 1,                                             /*!< Restart motor */
} RESTART;

/*! \brief sensitivity of current measurement
 *
 *  This enumeration is for the different units of the current measurement
 */
typedef enum {
  MV_BY_A                      = -3,                                            /*!< mV/A */
  UV_BY_A                      = -6,                                            /*!< uV/A */
} SENSITIVITY_CURRENT;

/*! \brief sensitivity of voltage measurement
 *
 *  This enumeration is for the different units of the voltageement
 */
typedef enum {
  MV_BY_V                      = -3,                                            /*!< mV/V */
} SENSITIVITY_VOLTAGE;

/*! \brief clamping valtage unit
 *
 *  This enumeration is for the different units of the voltageement
 */
typedef enum {
  V                            = 0,                                             /*!< mV/V */
} VOLTAGE;


/*! \brief Shunt type be used
 *
 *  This enumaration is for selecting between the usage of a 1-shunt or 3-shunt
 *  solution
 */
typedef enum
{
  CURRENT_SHUNT_1              = 1,                                             /*!< Channel is using 1-shunt solution */
  CURRENT_SHUNT_3              = 3,                                             /*!< Channel is using 3-shunt solution */
  CURRENT_SENSOR_2             = 2,                                             /*!< Channel is using 2 sensor solution */
} CURRENT_MEASUREMENT;

/*! \brief Sensor orientation
 *
 *  This enumaration is for selecting the current sensor orientation
 *  solution
 */
typedef enum
{
  CURRENT_MEASUREMENT_NORMAL        = 0,                                        /*!< Channel is using normal orientation */
  CURRENT_MEASUREMENT_INVERTED      = 1,                                        /*!< Channel is using inverted orientation */
} CURRENT_MEASUREMENT_ORIENTATION;

/*! \brief Sensor orientation
 *
 *  This enumaration is for selecting the current sensor orientation
 *  solution
 */
typedef enum
{
  POLARISATION_LOW             = 0,                                             /*!< Driving the FETs by low signal */
  POLARISATION_HIGH            = 1,                                             /*!< Driving the FETs by high signal */
} POLARISATION;

                                                                                     /*! \brief Motor Error Flags.
 *
 *  This enumeration is for the different error flags in motor state
 */
typedef enum {
  VE_ERROR_NONE       = 0,                                                      /*!< No error */
  VE_OVERTEMPERATURE  = (1<<0),                                                 /*!< Over temperature detected */
  VE_EMERGENCY        = (1<<1),                                                 /*!< Emergency Signal Detected */
  VE_OVERVOLTAGE      = (1<<2),                                                 /*!< Overvoltage Signal Detected */
  VE_NOMOTOR          = (1<<3),                                                 /*!< Motor Disconnection detected */
  VE_SWUNDERVOLTAGE   = (1<<4),                                                 /*!< Undervoltage detected (software solution) */
  VE_SWOVERVOLTAGE    = (1<<5),                                                 /*!< Overvoltage detected (software solution) */
  VE_SWOVERCURRENT    = (1<<6),                                                 /*!< Overcurrent detected (software solution) */
  VE_WRONGDIRECTION   = (1<<7),                                                 /*!< Wrong Motor direction requested */
  VE_SPEEDREDUCTION   = (1<<8),                                                 /*!< Target speed reduced due to load */
  VE_STALLDETECTED    = (1<<9),                                                 /*!< Field stall detected */
} VE_ERROR;


/*! \brief Motor Parameters.
 *
 *  All parameters needed to describe a motor fully
 */
typedef __packed struct
{
  uint8_t     PolePairs;                                                        /*!< [NONE]           - Half of Poles */
  uint8_t     Direction;                                                        /*!< [NONE]           - Enum Type (MOTOR_CW_ONLY / MOTOR_CCW_ONLY / MOTOR_CW_CCW) */
  uint8_t     Encoder;                                                          /*!< [NONE]           - Enum Type (MOTOR_NO_SENSOR / MOTOR_HALL_SENSOR / MOTOR_ENCODER) */
  uint8_t     EncoderUsage;                                                     /*!< [NONE]           - Enum Type (MOTOR_NO_SENSOR / MOTOR_HALL_SENSOR / MOTOR_ENCODER) */
  uint16_t    EncoderResolution;                                                /*!< [NONE]           - Incremental rotary encoder counts per revolution*/
  uint32_t    EncoderMinPos;                                                    /*!< [NONE]           - Minimal position for the linear motion in terms of encoder pulses */
  uint32_t    EncoderStartPos;                                                  /*!< [NONE]           - Start position for the linear motion in terms of encoder pulses */
  uint32_t    EncoderMaxPos;                                                    /*!< [NONE]           - Maximal position for the linear motion in terms of encoder pulses */
  uint16_t    EncoderLMApproach;                                                /*!< [RPM]            - Absolute approach speed in RPM */
  uint16_t    EncoderGearFactor;                                                /*!< [NONE]           - Gear factro 1:xxx */
  uint8_t     EncoderReceiver;                                                  /*!< [NONE]           - Enum Type (ENCODER_NO_RECEIVER / ENCODER_RESOLVER_RECEIVER / ENCODER_DIFFERENTIAL_RECEIVER) */
  uint8_t     EncoderReceiverMode;                                              /*!< [NONE]           - Enum Type (ENCODER_DIFF_D_HTL / ENCODER_DIFF_RS_422 / ENCODER_DIFF_SE_HTL / ENCODER_DIFF_TTL) */
  UNIT_VALUE  MaxAngAcc;                                                        /*!< [rad/sec^2]      - max angular velocity */
  UNIT_VALUE  TorqueFactor;                                                     /*!< [mNm/A]          - max torgue value * 10 */
  UNIT_VALUE  Resistance;                                                       /*!< [mOhm]           - Winding resistance */
  UNIT_VALUE  Inductance;                                                       /*!< [uH]             - Winding inductance */
  UNIT_VALUE  SpeedLimit;                                                       /*!< [Hz or RPM]      - Limitation speed of motor */
  UNIT_VALUE  SpeedChange;                                                      /*!< [Hz or RPM]      - Max Forced speed of motor (no FOC) */
  uint16_t    PositionDelay;                                                    /*!< [ms]             - Time of Positioning Stage */
  UNIT_VALUE  IqStart;                                                          /*!< [mA]             - Start current (Iq) */
  UNIT_VALUE  IdStart;                                                          /*!< [mA]             - Start current (Id) */
  UNIT_VALUE  IqLimit;                                                          /*!< [mA]             - q-axis limitation current */
  UNIT_VALUE  IdLimit;                                                          /*!< [mA]             - d-axis limitation current */
  uint8_t     MotorId[MAX_LENGTH_MOTOR_ID];                                     /*!< [NONE]             - name of the Motor device */
} __attribute__((packed)) MotorParameters;


/*! \brief PI Control settings.
 *
 *  Settings for the PI Control Loop (also at the different stages)
 */
typedef __packed struct
{
  uint32_t PiIdKi;                                                              /*!< [V/As]           - d-axis current control Integral gain */
  uint32_t PiIdKp;                                                              /*!< [V/A]            - d-axis current control Proportional gain */
  uint32_t PiIqKi;                                                              /*!< [V/As]           - q-axis current control Integral gain */
  uint32_t PiIqKp;                                                              /*!< [V/A]            - q-axis current control Proportional gain */
  uint32_t PositionKi;                                                          /*!< [Hz/Vs] * 1000   - Position estimation Integral gain (Ki) * 1000 */
  uint32_t PositionKp;                                                          /*!< [Hz/V] * 1000    - Position estimation Proportional gain (Kp) * 1000 */
  uint32_t SpeedKi;                                                             /*!< [mA/Hz*s]        - Speed Control Integral gain */
  uint32_t SpeedKp;                                                             /*!< [mA/Hz]          - Speed Control Proportional gain */
} __attribute__((packed)) PIControlSettings;

/*! \brief channel dependend values.
 *
 *  Values that are purely channel dependend
 */
typedef __packed struct
{
  uint16_t    Deadtime;                                                         /*!< [ns]             - Dead time of output switching polarity unit (steps of 100 ns) */
  uint16_t    BootstrapDelay;                                                   /*!< [ms]             - Bootstrap time */
  UNIT_VALUE  SensitivityCurrentMeasure;                                        /*!< [mV/A]           - Input current(Iabc) at ADC =0xFFF0 */
  UNIT_VALUE  SensitivityVoltageMeasure;                                        /*!< [mV/V]           - Input voltage(Vdc)   at ADC =0xFFF0 */
  uint8_t     GainCurrentMeasure;                                               /*!< [NONE]           - Gain setting for current measurement with 3 input lines (3Shunt ro 2Sensor) */
  uint8_t     MeasurementType;                                                  /*!< [NONE]           - Enum Type (SHUNT_TYPE_1 = 1,SHUNT_TYPE_3 = 3) */
  uint8_t     MeasurementDirection;                                             /*!< [NONE]           - Direction of the current measurement */
  uint8_t     PolL;                                                             /*!< [NONE]           - Active state for lower side FETs */
  uint8_t     PolH;                                                             /*!< [NONE]           - Active state for lower side FETs */
} __attribute__((packed)) ChannelDependandValues;

/*! \brief System settings.
 *
 *  Values that are dependend of the combination of Board and Motor
 */
typedef __packed struct
{
  uint16_t   PwmFrequency;                                                      /*!< [Hz]             - PWM Frequency */
  uint8_t    ShutdownMode;                                                      /*!< [NONE]           - Shutdown Mode for the VE */
  uint16_t   Braketime;                                                         /*!< [ms]             - Time for braking when Short brake is selected */
  uint8_t    Brakepercentage;                                                   /*!< [ms]       - Parcentage of braking mode when Short brake is selected */
  uint8_t    RestartMode;                                                       /*!< [NONE]           - Restart behaviour */
  uint16_t   StalldetectValue;                                                  /*!< [NONE]           - vqi value for stall detection */
  int8_t     Overtemperature;                                                   /*!< [Degree Celsius] - Shutdown if temperature exceeds this temperature setting */
  uint8_t    CanId;                                                             /*!< [NONE]           - CAN ID of Board */
  uint8_t    ExternalSpeedControl;                                              /*!< [NONE]           - Enum Type (ESC_[NONE] = 0, ESC_ADC = 1, ESC_PWM = 2) */
  uint16_t   SwOvervoltage;                                                     /*!< [V]              - Voltage for SW Overvolage detection */
  uint16_t   SwUndervoltage;                                                    /*!< [V]              - Voltage for SW Undervoltage detection */
  UNIT_VALUE SwOvercurrent;                                                     /*!< [mA]             - Current for SW Overcurrent detection */
  uint8_t    SpeedReduction;                                                    /*!< [%]              - Percentage of IqLimitit when Speed Reduction shall be used */
} __attribute__((packed)) SystemDependandValues;


/*! \brief Board info values.
 *
 *  Values that give inforamtion about the board
 */
typedef __packed struct
{
  uint8_t   channels;                                                           /*!< [NONE]             - Bitfield for determine the availabble motor channels */
  uint16_t  DSOSize;                                                            /*!< [NONE]             - Number of Data Samples for DSO functionality */
  uint8_t   BoardName[MAX_LENGTH_BOARD_NAME];                                   /*!< [NONE]             - name of the board */
  uint8_t   BoardNamePWR0[MAX_LENGTH_BOARD_NAME];                               /*!< [NONE]             - name of the power board channel 0*/
  uint8_t   BoardNamePWR1[MAX_LENGTH_BOARD_NAME];                               /*!< [NONE]             - name of the power board channel 1*/
  uint8_t   BoardRevision;                                                      /*!< [NONE]             - revision of the board */
  uint16_t  FirmwareFeatures;                                                   /*!< [NONE]             - features compiled into firmware */
} __attribute__((packed)) BoardInfoValues;

/*! \brief Firmware Version values.
 *
 *  Values that give inforamtion about the board
 */
typedef __packed struct
{
  uint8_t   fw_version[2];                                                      /*!< [NONE]             - Version information [Major][Minor] */
} __attribute__((packed)) FWVersion;


/*! \brief Motor Error.
 *
 *  Signaling different errors that could have appeared
 */
typedef __packed struct
{
  uint16_t  Error;                                                              /*!< Error field */
} __attribute__((packed)) MotorError;

/*! \brief DSO Logging data set.
 *
 *  One data set of sampled DSO data during logging.
 */
typedef __packed struct
{
  int16_t  value[MAX_DSO_VALUE];                                                /*!< DSO logged data value */
} __attribute__((packed)) DSOValues;

/*! \brief Motor settings.
 *
 *  Target speed Speed and target direction for the Motor.
 */
typedef __packed struct
{
  int32_t TargetSpeed;                                                          /*!< [RPM]            - Target Rotation speed of Motor - negative is CCW*/
} __attribute__((packed)) MotorSetSettings;

/*! \brief Motor state.
 *
 *  Actual speed, direction, used current and used torque at the time of request
 */
typedef __packed struct
{
  int32_t   ActualSpeed;                                                        /*!< [RPM]              - Actual Rotation speed of Motor - negative is CCW*/
  int32_t   TargetSpeed;                                                        /*!< [RPM]              - Target Rotation speed of Motor - negative is CCW*/
  uint32_t  Current;                                                            /*!< [mA]               - Current of motor */
  uint32_t  Torque;                                                             /*!< [Ncm]              - Torque of Motor */
  uint32_t  Timestamp;                                                          /*!< [systicks]         - systick counter of system */
} __attribute__((packed)) MotorStateSettings;

/*! \brief Motor state.
 *
 *  Actual control parameter value, target control value, used current and actual non-control value at the time of request
 */
typedef __packed struct
{
  int32_t   ActualControlledParameter;                                          /*!< [RPM] or [Ncm]  - Actual Rotation speed or Torque of the Motor - negative speed is CCW */
  int32_t   TargetControlledParameter;                                          /*!< [RPM] or [Ncm]  - Target Rotation speed of Torque of the Motor - negative speed is CCW */
  uint32_t  Current;                                                            /*!< [mA]            - Current used to drive the motor */
  uint32_t  NonControlParameter;                                                /*!< [Ncm] or [RPM]  - Current Torque or Speed of the Motor */
  uint32_t  Timestamp;                                                          /*!< [systicks]      - Systick counter of system */
  uint8_t   ControlMethod;                                                      /*!< [NONE]          - Idle, Control by Speed, Control by Torque */
} __attribute__((packed)) MotorExtendedStateSettings;



typedef __packed struct
{
  int32_t  ActualCalculatedEncoderValue;                                        /*!< [RPM]              - Actual Rotation speed of Motor - negative is CCW */
  int32_t  CalculatedEncoderValue;                                              /*!< [lines]            - Calcualted number of lines */
  int32_t  HallEncoderValue;                                                    /*!< [pulses]           - Current number of HAL sensor pulses */
  int32_t  LineEncoderValue;                                                    /*!< [lines]            - Current number of line encoder lines */
  uint32_t Timestamp;                                                           /*!< [systicks]         - systick counter of system */
} __attribute__((packed)) TurnStateSettings;


typedef struct
{
	int32_t			filter;				/* [---] current filter */
	int32_t			max_value;			/* [---] compensation maximum value (duty ratio register value) */
	int16_t			current;			/* [A]   compensation control threshold */
} Deadtime_InitTypeDef;

typedef struct
{
	int32_t			filter;				/* [---] current filter */
	int32_t			max_value;			/* [---] compensation maximum value (duty ratio register value) */
	int16_t			current;			/* [A]   compensation control threshold */
	FRACTIONAL			ia_ave;				/* [A]   U-phase current average work */
	FRACTIONAL			ib_ave;				/* [A]   V-phase current average work */
	FRACTIONAL			ic_ave;				/* [A]   W-phase current average work */
	int16_t			iu_dt;				/* [A]   U-phase current average */
	int16_t			iv_dt;				/* [A]   V-phase current average */
	int16_t			iw_dt;				/* [A]   W-phase current average */
	int32_t			u_duty;				/* [---] U-phase duty ratio compensation value */
	int32_t			v_duty;				/* [---] V-phase duty ratio compensation value */
	int32_t			w_duty;				/* [---] W-phase duty ratio compensation value */
} deadtime_t;


/*===================================================================*
	Constant Definition
 *===================================================================*/
/* Constant data for deadtime control(Value is example) */
#define	cDEADTIME_FILTER_CH1		(1)			/* [---] */
#define	cDEADTIME_FET_RT_CH1		(0.5f)		/* [us] */
#define	cDEADTIME_FET_FT_CH1		(0.5f)		/* [us] */
#define	cDEADTIME_DTIME_CH1			(1.0f)		/* [us] */
#define	cDEADTIME_CUR_MAX_CH1		(0.2f)		/* [A] */

#define	cDEADTIME_FILTER_CH0		(1)			/* [---] */
#define	cDEADTIME_FET_RT_CH0		(0.5f)		/* [us] */
#define	cDEADTIME_FET_FT_CH0		(0.5f)		/* [us] */
#define	cDEADTIME_DTIME_CH0			(1.0f)		/* [us] */
#define	cDEADTIME_CUR_MAX_CH0		(0.2f)		/* [A] */

/*! \brief Start Motor request structure
 *
 *  This structure contains all required information for a start request of a motor
 */
__packed struct StartMotorReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
}__attribute__((packed));


/*! \brief Stop Motor request structure
 *
 *  This structure contains all required information for a start response of a motor
 */
__packed struct StartMotorRsp
{
  uint8_t motor_nr;                                                              /*!< Motor number */
}__attribute__((packed));


/*! \brief Stop Motor request structure
 *
 *  This structure contains all required information for a stop request of a motor
 */
__packed struct StopMotorReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
}__attribute__((packed));

/*! \brief Stop Motor request structure
 *
 *  This structure contains all required information for a stop response of a motor
 */
__packed struct StopMotorRsp
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Get actual Motor State request structure
 *
 *  This structure contains all required information for a get motor state request of a motor
 */
__packed struct GetMotorStateReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
}__attribute__((packed));

/*! \brief Get actual Motor State request structure
 *
 *  This structure contains all required information for a get motor state response of a motor
 */
__packed struct GetMotorStateRsp
{
  MotorStateSettings State;                                                     /*!< actual state of Motor */
}__attribute__((packed));

/*! \brief Get actual Extended Motor State request structure
 *
 *  This structure contains all required information for a get motor state request of a motor
 */
__packed struct GetExtendedMotorStateReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
}__attribute__((packed));

/*! \brief Get actual Extended Motor State request structure
 *
 *  This structure contains all required information for a get motor state response of a motor
 */
__packed struct GetExtendedMotorStateRsp
{
  MotorExtendedStateSettings State;                                             /*!< actual state of Motor */
}__attribute__((packed));

/*! \brief Start Motor with torque Question
 *
 * Load data of protocol communication for start of motor with torque (answer to host)
 */
__packed struct StartMotorTorqueCtrlReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
};

/*! \brief Start Motor with torque Answer
 */
__packed struct StartMotorTorqueCtrlRsp
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
};


/*! \brief Store Parameters (Motor/PI/System) to EEProm request structure
 *
 *  This structure contains all required information for a store parameter request of a motor
 */
__packed struct StoreParametersReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
}__attribute__((packed));

/*! \brief Store Parameters (Motor/PI/System) to EEProm response structure
 *
 *  This structure contains all required information for a store parameter response of a motor
 */
__packed struct StoreParametersRsp
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Clears EEProm content request structure
 *
 *  This structure contains all required information for a clear parameter request of a motor
 */
__packed struct ClearParametersReq
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Clears EEProm content response structure
 *
 *  This structure contains all required information for a clear parameter response of a motor
 */
__packed struct ClearParametersRsp
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Configure DSO Logging facility request structure
 *
 *  This structure contains all required information for the configuring of the logging of the internal
 *  Vector Engine Registers request
 */
__packed struct ConfigDsoLogReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number */
  uint32_t selected_values;                                                     /*!< Bitfield with DSO_Selection enumerated values for selecting the signals to be logged */
  uint32_t trigger_on_value;                                                    /*!< Bitfield with DSO_Selection enumerated values for selecting the signals to trigger on */
  uint8_t  trigger_mode;                                                        /*!< Bitfield with TriggerMode enumerated values to select the trigger mode of the DSO */
  uint16_t trigger_level;                                                       /*!< Trigger level for the DSO */
  uint8_t  spread_factor;                                                       /*!< Factor of which the logging is spread */
}__attribute__((packed));

/*! \brief Configure DSO Logging facility response structure
 *
 *  This structure contains all required information for the configuring of the logging of the internal
 *  Vector Engine Registers response
 */
__packed struct ConfigDsoLogRsp
{
  uint16_t logsteps;                                                            /*!< Number of data-sets that will be logged */
}__attribute__((packed));

/*! \brief request the DSO Log Data request structure
 *
 *  This structure contains all required information for requesting the data that has been logged by the
 *  DOS after the DSO has been triggered.
 */
__packed struct GetDsoLogDataReq
{
  uint8_t  dummy;                                                               /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief response the DSO Log Data request structure
 *
 *  This structure contains all required information for getting the data that has been logged by the
 *  DOS after the DSO has been triggered.
 */
__packed struct GetDsoLogDataRsp
{
  DSOValues values;                                                             /*!< Dataset of logged DSO data */
}__attribute__((packed));

/*! \brief Configure highspeed DSO request structure
 *
 *  This structure contains all required information for configuration of the DSO request
 */
__packed struct ConfigureHsDsoReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number */
  uint8_t  enable;                                                              /*!< enable the HSDSO  */
  uint32_t selected_values;                                                     /*!< Bitfield with DSO_Selection enumerated values for selecting the signals to be logged */
  uint8_t  spread_factor;                                                       /*!< Factor of which the logging is spread */
}__attribute__((packed));

/*! \brief Configure highspeed DSO response structure
 *
 *  This structure contains all required information for configuration of the DSO response
 */
__packed struct ConfigureHsDsoRsp
{
  uint8_t  dummy;                                                               /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Get Firmware Version request structure
 *
 *  This structure contains all required information for getting the FW version request
 */
__packed struct GetFWVersionReq
{
  uint8_t  dummy;                                                               /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Get Firmware Version response structure
 *
 *  This structure contains all required information for getting the FW version response
 */
__packed struct GetFWVersionRsp
{
  FWVersion  values;                                                            /*!< FW Version  */
}__attribute__((packed));

/*! \brief Do Turn request structure
 *
 *  This structure contains all required information for a do turn request of a motor
 */
__packed struct DoTurnReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number: */
  int16_t  turns;                                                               /*!< Number of turns to perform */
  uint16_t additional_angle;                                                    /*!< Additional Angle in 1/10 degree in same direction as turns */
  uint16_t max_speed;                                                           /*!< Maximum rpm while turning */
}__attribute__((packed));

/*! \brief Do Turn response structure
 *
 *  This structure contains all required information for a do turn response of a motor
 */
__packed struct DoTurnRsp
{
  uint8_t  dummy;                                                               /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Do Linear Motion request structure
 *
 *  This structure contains all required information for a do linear motion request of a motor
 */
__packed struct DoLinearMotionReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number: */
  uint32_t abs_postion;                                                         /*!< New absolute position in terms of encoder pulses */
  uint16_t max_speed;                                                           /*!< Maximum rpm while turning */
}__attribute__((packed));

/*! \brief Do Linear Motion response structure
 *
 *  This structure contains all required information for a do linear motion response of a motor
 */
__packed struct DoLinearMotionRsp
{
  uint8_t  error;                                                               /*!< Error value  */
}__attribute__((packed));

/*! \brief request the Get Absolute Position request structure
 *
 *  This structure contains all required information for requesting the absolute postion
 *  parameter set of the motor in number of encoder pulses
 */
__packed struct GetAbsolutePositionReq
{
  uint8_t motor_nr;                                                             /*!< Motor number */
}__attribute__((packed));

/*! \brief response the Get Absolute Position request structure
 *
 *  This structure contains all required information for getting the absolute position set of the
 *  motor including the min and max limits, the start position and the actual aboslute position
 *
 */
__packed struct GetAbsolutePositionRsp
{
  uint32_t current_position;                                                    /*!< Current absolute position in number of encoder pulses */
}__attribute__((packed));

/*! \brief Abort Linear Motion request structure
 *
 *  This structure contains all required information for a do linear motion request of a motor
 */
__packed struct AbortLinearMotionReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number: */
}__attribute__((packed));

/*! \brief Abort Linear Motion response structure
 *
 *  This structure contains all required information for a do linear motion response of a motor
 */
__packed struct AbortLinearMotionRsp
{
  uint8_t  error;                                                               /*!< Error value  */
}__attribute__((packed));

/*! \brief Get Motor Control Method request structure
 *
 *  This structure contains all required information for a a get control method request of a motor
 */
__packed struct GetMotorControlMethodReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number: */
}__attribute__((packed));

/*! \brief Get Motor Control Method response structure
 *
 *  This structure contains all required information for a get control method response of a motor
 */
__packed struct GetMotorControlMethodRsp
{
  uint8_t  method;                                                              /*!< Current control method  */
}__attribute__((packed));

/*! \brief Set Motor Control Method request structure
 *
 *  This structure contains all required information for a a get control method request of a motor
 */
__packed struct SetMotorControlMethodReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number: */
  uint8_t  method;                                                              /*!< Requested control method  */
}__attribute__((packed));

/*! \brief Set Motor Control Method response structure
 *
 *  This structure contains all required information for a get control method response of a motor
 */
__packed struct SetMotorControlMethodRsp
{
  uint8_t  error;                                                              /*!< Error value */
}__attribute__((packed));

/*! \brief SetDemoState request structure
 *
 *  This structure contains all required information for execution of a single demo state
 */
__packed struct SetDemoStateReq
{
  uint8_t new_state;                                                            /*!< New state */
}__attribute__((packed));

/*! \brief SetDemoState response structure
 *
 *  This structure contains all required information for a single demo step response
 */
__packed struct SetDemoStateRsp
{
  uint8_t  error;                                                               /*!< Error value  */
}__attribute__((packed));

/*! \brief GetDemoState request structure
 *
 *  This structure contains all required information for execution of a single demo state
 */
__packed struct GetDemoStateReq
{
  uint8_t  dummy;                                                               /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief GetDemoState response structure
 *
 *  This structure contains all required information for a single demo step response
 */
__packed struct GetDemoStateRsp
{
  uint8_t curr_state;                                                        /*!< New state */
}__attribute__((packed));

/*! \brief GetEncoderCounter request structure
 *
 *  This structure contains all required information for a encoder counter request
 */
__packed struct GetEncoderCounterReq
{
  uint8_t  encoder_nr;                                                            /*!< Encoder number: */
}__attribute__((packed));

/*! \brief GetEncoderCounter response structure
 *
 *  This structure contains all required information for a encoder counter response
 */
__packed struct GetEncoderCounterRsp
{
  int32_t  counterValue;                                                        /*!< Encoder Counter Value */
}__attribute__((packed));

/*! \brief max. size of entries allowed for SetMotorParametersReq and GetMotorParametersReq */
#define MAX_NUM_CONFIG_VALUES 8

/*! \brief Get entry structure
 *
 *  This structure contains all required information for getting a parameter request entry
 */
__packed struct GetEntryReq
{
    uint8_t id;                                                                 /*!< replace */
    uint8_t offset;                                                             /*!< replace */
}__attribute__((packed));

/*! \brief Get one motor parameters request structure
 *
 *  This structure contains all required information for the request of getting one specific parameter set for a motor
 */
__packed struct GetOneMotorParameterReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number */
  struct GetEntryReq entry;                                                     /*!< replace */
}__attribute__((packed));

/*! \brief Get motor parameters request
 *
 *  This structure contains all required information for the request of getting specific motor paramaters for multiple set for a motor
 */
__packed struct GetMotorParametersReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number */
  uint8_t  entries;                                                             /*!< Size of entries. Max Numbers allowed: MAX_NUM_CONFIG_VALUES */
  struct GetEntryReq entry[MAX_NUM_CONFIG_VALUES];                              /*!< Entry structure */
}__attribute__((packed));

/*! \brief Get entry structure for response
 *
 *  This structure contains all required information for a parameter response entry
 */
__packed struct GetEntryRsp
{
    int32_t value;                                                              /*!< replace */
     int8_t unit;                                                               /*!< replace */
}__attribute__((packed));

/*! \brief Get one motor parameters set response structure
 *
 *  This structure contains all required information for the reponse of getting one specific parameter set for a motor
 */
__packed struct GetOneMotorParameterRsp
{
  struct GetEntryRsp entry;                                                     /*!< Get entry response structure fro one parameters set */
}__attribute__((packed));

/*! \brief Get multiple motor parameters sets response structure
 *
 *  This structure contains all required information for the reponse of getting multiple specific parameter sets for a motor
 */
__packed struct GetMotorParametersRsp
{
  uint8_t  error;                                                               /*!< replace */
  uint8_t  offset;                                                              /*!< replace */
  struct GetEntryRsp entry[MAX_NUM_CONFIG_VALUES];                              /*!< array of paramters sets response entries. Maximal size :  MAX_NUM_CONFIG_VALUES */
}__attribute__((packed));

/*! \brief Set entry structure
 *
 *  This structure contains all required information for setting a parameter request entry
 */
__packed struct SetEntry
{
    uint8_t id;                                                                 /*!< replace */
    int8_t  unit;                                                               /*!< replace */
    int32_t value;                                                              /*!< replace */
}__attribute__((packed));

/*! \brief Set one motor parameters request structure
 *
 *  This structure contains all required information for the request of setting one specific parameter set for a motor
 */
__packed struct SetOneMotorParameterReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number */
  struct SetEntry entry;                                                        /*!< replace */
}__attribute__((packed));

/*! \brief Set multiple motor parameters sets request structure
 *
 *  This structure contains all required information for the request of setting multiple parameters sets for a motor
 */
__packed struct SetMotorParametersReq
{
  uint8_t  motor_nr;                                                            /*!< Motor number */
  uint8_t  entries;                                                             /*!< number of parameters sets entries */
  struct SetEntry entry[MAX_NUM_CONFIG_VALUES];                                 /*!< parameters sets entries */
}__attribute__((packed));

/*! \brief Set one motor parameters response structure
 *
 *  This structure contains all required information for the reponse of setting one specific parameter set for a motor
 */
__packed struct SetOneMotorParameterRsp
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));

/*! \brief Set multiple motor parameters sets response structure
 *
 *  This structure contains all required information for the reponse of setting multiple specific parameters sets for a motor
 */
__packed struct SetMotorParametersRsp
{
  uint8_t error;                                                                /*!< replace */
  uint8_t offset;                                                               /*!< replace */
}__attribute__((packed));

/*! \brief Set PWM settings for one channel request structure
 *
 *  This structure contains all required information for the request of setting the PWM paramter for one channel
 */
__packed struct SetPwmChannelParameterReq
{
  uint8_t   pwmChannel;                                                         /*!< channel number */
  uint16_t  onTime;                                                             /*!< time that the output is asserted (valid values from 0-4096) */
  uint16_t  offTime;                                                            /*!< time until the output is negated (valid values from 0-4096) */
}__attribute__((packed));

/*! \brief Set one motor parameters response structure
 *
 *  This structure contains all required information for the reponse of setting one specific parameter set for a motor
 */
__packed struct SetPwmChannelParameterRsp
{
  uint8_t dummy;                                                                /*!< Dummy value - due to protocol specification  */
}__attribute__((packed));



/*! \brief  Start Motor
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int StartMotor     (struct StartMotorReq* req,            struct StartMotorRsp* rsp);

/*! \brief  Stop Motor
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int StopMotor             (struct StopMotorReq* req,             struct StopMotorRsp* rsp);

/*! \brief  Get actual Motor State
*
* Read out the current status of the motor (actual speed, current, torque)
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int GetMotorState         (struct GetMotorStateReq* req,         struct GetMotorStateRsp* rsp);

/*! \brief  Store all Parameters
*
* Store all Parameters to Non-Volatile Memory
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int StoreParameters       (struct StoreParametersReq* req,       struct StoreParametersRsp* rsp);

/*! \brief  Clear all Parameters
*
* Clear the Non-Volatile Memory
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int ClearParameters       (struct ClearParametersReq* req,       struct ClearParametersRsp* rsp);

/*! \brief  Configure DSO Log
*
* Configure which values shall be captured by the DSO
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int ConfigDsoLog          (struct ConfigDsoLogReq* req,          struct ConfigDsoLogRsp* rsp);

/*! \brief  Get DSO Log Data
*
* Read out one set of log data after DSO has finished
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int GetDsoLogData         (struct GetDsoLogDataReq* req,         struct GetDsoLogDataRsp* rsp);

/*! \brief  Configure High Speed DSO
*
* Configure which values shall be transmitted by the DSO
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int ConfigureHsDso        (struct ConfigureHsDsoReq* req,        struct ConfigureHsDsoRsp* rsp);

/*! \brief  Do turn
*
* Turns the motor a number of times
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int DoTurn                (struct DoTurnReq* req,                struct DoTurnRsp* rsp);

/*! \brief  Gets one specific parameters sets for a motor
*
* Turns the motor a number of times
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int GetOneMotorParameter  (struct GetOneMotorParameterReq* req,  struct GetOneMotorParameterRsp* rsp);

/*! \brief  Sets one specific parameters sets for a motor
*
*
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int SetOneMotorParameter  (struct SetOneMotorParameterReq* req,  struct SetOneMotorParameterRsp* rsp);

/*! \brief  Gets multiple specific parameters sets for a motor
*
*
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int GetMotorParameters    (struct GetMotorParametersReq* req,    struct GetMotorParametersRsp* rsp);

/*! \brief  Sets multiple specific parameters sets for a motor
*
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int SetMotorParameters    (struct SetMotorParametersReq* req,    struct SetMotorParametersRsp* rsp);

/*! \brief  Do linear motion
*
* Turns the motor a number of times
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int DoLinearMotion         (struct DoLinearMotionReq* req,                struct DoLinearMotionRsp* rsp);

/*! \brief  Get absolute position
*
* Gets the absolute postion infomration
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int GetAbsolutePosition         (struct GetAbsolutePositionReq* req,                struct GetAbsolutePositionRsp* rsp);


/*! \brief  Abort linear motion
*
* Turns the motor a number of times
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int AbortLinearMotion         (struct AbortLinearMotionReq* req,                struct AbortLinearMotionRsp* rsp);

/*! \brief  StartMotorTorqueCtrl
*
* Set the currently used control method to torque
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
extern int StartMotorTorqueCtrl    (struct StartMotorTorqueCtrlReq* req, struct StartMotorTorqueCtrlRsp* rsp);

/*! \brief  GetMotorControlMethod
*
* Retrive the currently used control method
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
extern int GetMotorControlMethod    (struct GetMotorControlMethodReq* req, struct GetMotorControlMethodRsp* rsp);

/*! \brief  SetMotorControlMethod
*
* Set the desired control method
*
* @param  q: Question from Serial Protocol
* @param  a: Answer   for  Serial Protocol
* @retval Success
*/
extern int SetMotorControlMethod    (struct SetMotorControlMethodReq* req, struct SetMotorControlMethodRsp* rsp);

/*! \brief  Get actual Motor State
*
* Read out the current status of the motor (actual speed, current, torque, control method)
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
int GetExtendedMotorState       (struct GetExtendedMotorStateReq* req,       struct GetExtendedMotorStateRsp* rsp);

/*! \brief  Get firmware version
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int GetFWVersion          (struct GetFWVersionReq* req,          struct GetFWVersionRsp* rsp);

/*! \brief  SetPwmChannel
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int SetPwmChannelParameter(struct SetPwmChannelParameterReq* req,  struct SetPwmChannelParameterRsp* rsp);

/*! \brief  SetDemoState
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int  SetDemoState(struct  SetDemoStateReq* req, struct SetDemoStateRsp* rsp);

/*! \brief  GetDemoState
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
extern int  GetDemoState(struct  GetDemoStateReq* req, struct GetDemoStateRsp* rsp);


/*! \brief  GetEncoderCounter
*
* @param  *req: Pointer to the request structure
* @param  *rsp: Pointer to the response structure
* @retval Success
*/
int GetEncoderCounter(struct GetEncoderCounterReq* req, struct GetEncoderCounterRsp* rsp);
#endif

/************************** BEGIN of DISCLAIMER   **************************

- TOSHIBA is continually working to improve the quality and reliability of
  its products. Nevertheless, semiconductor devices in general can
  malfunction or fail due to their inherent electrical sensitivity and
  vulnerability to physical stress. It is the responsibility of the buyer,
  when utilizing TOSHIBA products, to comply with the standards of safety
  in making a safe design for the entire system, and to avoid situations in
  which a malfunction or failure of such TOSHIBA products could cause loss of
  human life, bodily injury or damage to property.

  In developing your designs, please ensure that TOSHIBA products are used
  within specified operating ranges as set forth in the most recent TOSHIBA
  products specifications.
  Also, please keep in mind the precautions and conditions set forth in the
  "Handling Guide for Semiconductor Devices," or
  "TOSHIBA Semiconductor Reliability Handbook" etc..

- The Toshiba products listed in this document are intended for usage in
  general electronics applications (computer, personal equipment, office
  equipment, measuring equipment, industrial robotics, domestic appliances,
  etc.).
  These Toshiba products are neither intended nor warranted for usage in
  equipment that requires extraordinarily high quality and/or reliability or
  a malfunction or failure of which may cause loss of human life or bodily
  injury ("Unintended Usage"). Unintended Usage include atomic energy control
  instruments, airplane or spaceship instruments, transportation instruments,
  traffic signal instruments, combustion control instruments, medical
  instruments, all types of safety devices, etc..
  Unintended Usage of Toshiba products listed in this document shall be
  made at the customer's own risk.

- Toshiba assumes no liability for any damage or losses (including but not
  limited to, loss of business profit,business interruption, loss of business
  information,and other pecuniary losses) occurring
  from the use of, or inability to use, this product.

- The products described in this document are subject to the foreign exchange
  and foreign trade laws.

- The products described in this document contain components made in the
  United States and subject to export control of the U.S. authorities.
  Diversion contrary to the U.S. law is prohibited.

- The information contained herein is presented only as a guide for the
  applications of our products.
  No responsibility is assumed by TOSHIBA CORPORATION for any infringements
  of intellectual property or other rights of the third parties which may
  result from its use.
  No license is granted by implication or otherwise under any intellectual
  property or other rights of TOSHIBA CORPORATION or others.

- The information contained herein is subject to change without notice.

****************************  END of DISCLAIMER ****************************/
