/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Toshiba Electronics Europe GmbH
*   European LSI Design and Engineering Center (ELDEC)
****************************************************************************/

/**
* @file api_internal.h
*
* @brief Contains internal definitions and function declarations for Motor Control API.
*
*/
#ifndef __API_INTERNAL_H__
#define __API_INTERNAL_H__

#include "dso.h"

/* this hack is used to calculate the maximum size of any of the structures.
if you add a new function with new structures, don't forget to change this
macro accordingly. */
#define FIND_MAX_STRUCT(x) \
__packed union max_ ## x \
{ \
  struct StartMotor ## x                 i0;  \
  struct StopMotor ## x                  i1;  \
  struct GetMotorState ## x              i2;  \
  struct StoreParameters ## x            i3;  \
  struct ClearParameters ## x            i4;  \
  struct ConfigDsoLog ## x               i5;  \
  struct GetDsoLogData ## x              i6;  \
  struct ConfigureHsDso ## x             i7;  \
  struct DoTurn ## x                     i8;  \
  struct GetOneMotorParameter ## x       i9;  \
  struct SetOneMotorParameter ## x       i10; \
  struct GetMotorParameters ## x         i11; \
  struct SetMotorParameters ## x         i12; \
  struct DoLinearMotion ## x             i13; \
  struct GetAbsolutePosition## x         i14; \
  struct AbortLinearMotion ## x          i15; \
  struct StartMotorTorqueCtrl ## x       i16; \
  struct GetMotorControlMethod ## x      i17; \
  struct SetMotorControlMethod ## x      i18; \
  struct GetEncoderCounter ## x          i19; \
  struct GetFWVersion ## x               i20; \
  struct GetDemoState ## x               i21; \
  struct SetDemoState ## x               i22; \
  struct GetExtendedMotorState ## x      i23; \
} __attribute__((packed))

#define MAX_SIZE_REQ  sizeof(FIND_MAX_STRUCT(Req))
#define MAX_SIZE_RSP  sizeof(FIND_MAX_STRUCT(Rsp))

enum function_error
{
  FUNCTION_ERROR_NONE,
  FUNCTION_ERROR_TIMEOUT,
  FUNCTION_ERROR_CRC,
  FUNCTION_ERROR_EINVAL,
};

enum feature_list
{
  FEATURE_HAS_DSO                             = (1<< 0),
  FEATURE_HAS_LOAD_STATISTIC                  = (1<< 1),
  FEATURE_HAS_TURN_CONTROL                    = (1<< 2),
  FEATURE_HAS_TEMPERATURE_CONTROL             = (1<< 3),
  FEATURE_HAS_HSDSO                           = (1<< 4),
  FEATURE_HAS_RW_BOARD_SETTINGS               = (1<< 5),
  FEATURE_HAS_EXTERNAL_SPEED_CONTROL          = (1<< 6),
  FEATURE_HAS_SW_OVER_UNDER_VOLTAGE_DETECTION = (1<< 7),
  FEATURE_HAS_MOTOR_DISCONNECT_DETECTION      = (1<< 8),
  FEATURE_HAS_SW_OVERCURRENT_DETECTION        = (1<< 9),
  FEATURE_HAS_LOAD_DEPENDANT_SPEED_REDUCTION  = (1<<10),
  FEATURE_HAS_CAN                             = (1<<11),
  FEATURE_HAS_STALL_DETECT                    = (1<<12),
  FEATURE_HAS_MOTION_CONTROL                  = (1<<13),
  FEATURE_HAS_GET_DC_LINK_VOLTAGE             = (1<<14),
  FEATURE_HAS_GET_MOTOR_STAGE                 = (1<<15),
  FEATURE_HAS_DEMO                            = (1<<16),
  FEATURE_HAS_TORQUE_CONTROL                  = (1<<17)                
};

#endif

/************************** BEGIN of DISCLAIMER   **************************

- TOSHIBA is continually working to improve the quality and reliability of
  its products. Nevertheless, semiconductor devices in general can
  malfunction or fail due to their inherent electrical sensitivity and
  vulnerability to physical stress. It is the responsibility of the buyer,
  when utilizing TOSHIBA products, to comply with the standards of safety
  in making a safe design for the entire system, and to avoid situations in
  which a malfunction or failure of such TOSHIBA products could cause loss of
  human life, bodily injury or damage to property.

  In developing your designs, please ensure that TOSHIBA products are used
  within specified operating ranges as set forth in the most recent TOSHIBA
  products specifications.
  Also, please keep in mind the precautions and conditions set forth in the
  "Handling Guide for Semiconductor Devices," or
  "TOSHIBA Semiconductor Reliability Handbook" etc..

- The Toshiba products listed in this document are intended for usage in
  general electronics applications (computer, personal equipment, office
  equipment, measuring equipment, industrial robotics, domestic appliances,
  etc.).
  These Toshiba products are neither intended nor warranted for usage in
  equipment that requires extraordinarily high quality and/or reliability or
  a malfunction or failure of which may cause loss of human life or bodily
  injury ("Unintended Usage"). Unintended Usage include atomic energy control
  instruments, airplane or spaceship instruments, transportation instruments,
  traffic signal instruments, combustion control instruments, medical
  instruments, all types of safety devices, etc..
  Unintended Usage of Toshiba products listed in this document shall be
  made at the customer's own risk.

- Toshiba assumes no liability for any damage or losses (including but not
  limited to, loss of business profit,business interruption, loss of business
  information,and other pecuniary losses) occurring
  from the use of, or inability to use, this product.

- The products described in this document are subject to the foreign exchange
  and foreign trade laws.

- The products described in this document contain components made in the
  United States and subject to export control of the U.S. authorities.
  Diversion contrary to the U.S. law is prohibited.

- The information contained herein is presented only as a guide for the
  applications of our products.
  No responsibility is assumed by TOSHIBA CORPORATION for any infringements
  of intellectual property or other rights of the third parties which may
  result from its use.
  No license is granted by implication or otherwise under any intellectual
  property or other rights of TOSHIBA CORPORATION or others.

- The information contained herein is subject to change without notice.

****************************  END of DISCLAIMER ****************************/
