/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#ifndef _FWMATHLIB_H_
#define _FWMATHLIB_H_
#include <stdint.h>

#define FIXPOINT_12             (0x1000L)                                       /* fix point Q12 4096 */
#define FIXPOINT_15             (0x8000L)                                       /* fix point Q15 32768 */
#define FIXPOINT_16             (0x10000L)                                      /* fix point Q16 65536 */
#define FIXPOINT_31             (0x80000000LL)                                  /* fix point Q31 */
#define FIXPOINT_32             (0x100000000LL)                                 /* fix point Q32 */
#define ELE_DEG(x)              (uint16_t)(FIXPOINT_16 * (x) / 360)             /* Electric Angle for 16bit */
#define ELE_DEG32(x)            (uint32_t)(FIXPOINT_32 * (x) / 360)             /* Electric Angle for 32bit */
#define VE_DIV_Q15_SQRT3        (18919)                                         /* 1 / ROOT3 * 32768 */

#define ROUND_BIT15             (0x00008000)                                    /* Rounding off */

/* fix point12 sine values */
#define cSine1          (int16_t) (FIXPOINT_12 * 3.140625)
#define cSine2          (int16_t) (FIXPOINT_12 * 0.02026367)
#define cSine3          (int16_t) (FIXPOINT_12 * -5.325196)
#define cSine4          (int16_t) (FIXPOINT_12 * 0.5446778)
#define cSine5          (int16_t) (FIXPOINT_12 * 1.800293)

uint32_t        pow10(uint8_t power);
int16_t         Sine(int16_t theta);
void            ClarkeTransformation(int16_t _iu, int16_t _iv, int16_t _iw, int16_t* _ialpha, int16_t* _ibeta);
void            ParkTransformation(int16_t _ialpha, int16_t _ibeta, uint16_t _theta, int16_t* _id, int16_t* _iq);
void            InvParkTransformation(int32_t _d, int32_t _q, uint16_t _theta, int32_t* _alpha, int32_t* _beta);
void            SpaceVectorModulation(int32_t _valpha, int32_t _vbeta, int16_t _vdc, uint8_t _sector, uint8_t _modul, uint16_t* _p_vu, uint16_t* _p_vv, uint16_t* _p_vw);
int32_t         MediumOf3(int32_t _a, int32_t _b, int32_t _c);
int32_t         MinimumOf3(int32_t _a, int32_t _b, int32_t _c);
int32_t         MaximumOf3(int32_t _a, int32_t _b, int32_t _c);
void            SortOf3(int32_t* _n1, int32_t* _n2, int32_t* _n3);

#endif /* _FWMATHLIB_H_ */
